rem Don't call this directly.
@rem
@rem Updated:
@rem   22-August-2022 Upgraded for IM v7.
@rem

if "%TMPDIR%"=="" exit /B 1



:: Extrapolate outwards to get four corners.

set FMT=^
X0b=%%[fx:%X0%-(%X1%-%X0%)/4]\n^
X1b=%%[fx:%X1%+(%X1%-%X0%)/4]\n^
X3b=%%[fx:%X3%-(%X2%-%X3%)/4]\n^
X2b=%%[fx:%X2%+(%X2%-%X3%)/4]\n^
Y0b=%%[fx:%Y0%-(%Y1%-%Y0%)/4]\n^
Y3b=%%[fx:%Y3%-(%Y2%-%Y3%)/4]\n^
Y1b=%%[fx:%Y1%+(%Y1%-%Y0%)/4]\n^
Y2b=%%[fx:%Y2%+(%Y2%-%Y3%)/4]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

set FMT=^
X0c=%%[fx:%X0b%-(%X3b%-%X0b%)/2]\n^
X1c=%%[fx:%X1b%-(%X2b%-%X1b%)/2]\n^
X3c=%%[fx:%X3b%+(%X3b%-%X0b%)/2]\n^
X2c=%%[fx:%X2b%+(%X2b%-%X1b%)/2]\n^
Y0c=%%[fx:%Y0b%-(%Y3b%-%Y0b%)/2]\n^
Y3c=%%[fx:%Y3b%+(%Y3b%-%Y0b%)/2]\n^
Y1c=%%[fx:%Y1b%-(%Y2b%-%Y1b%)/2]\n^
Y2c=%%[fx:%Y2b%+(%Y2b%-%Y1b%)/2]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L


:: Distort so the four corners become a rectangle.

set FMT=^
left=%%[fx:(%X0c%+%X3c%)/2]\n^
top=%%[fx:(%Y0c%+%Y1c%)/2]\n^
right=%%[fx:(%X1c%+%X2c%)/2]\n^
bot=%%[fx:(%Y2c%+%Y3c%)/2]\n

set FMT=^
left=%%[fx:min(min(min(%X0c%,%X1c%),%X2c%),%X3c%)]\n^
top=%%[fx:min(min(min(%Y0c%,%Y1c%),%Y2c%),%Y3c%)]\n^
right=%%[fx:max(max(max(%X0c%,%X1c%),%X2c%),%X3c%)]\n^
bot=%%[fx:max(max(max(%Y0c%,%Y1c%),%Y2c%),%Y3c%)]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

set PERSP_PARAMS=^
%X0c%,%Y0c%,%left%,%top%,^
%X1c%,%Y1c%,%right%,%top%,^
%X2c%,%Y2c%,%right%,%bot%,^
%X3c%,%Y3c%,%left%,%bot%


set FMT=^
LL=%%[fx:floor(%left%+0.5)]\n^
TT=%%[fx:floor(%top%+0.5)]\n^
WW=%%[fx:floor(%right%-%left%+0.5)]\n^
HH=%%[fx:floor(%bot%-%top%+0.5)]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

set OKAY=1

:: For deugging, we can record the calculated patches.

if "%DEBUG%" GTR "1" (
  set WR_CARD=+write %TMPDIR%24c_card.tiff
  set WR_PATHCHS=+write %TMPDIR%24c_x6.png
) else (
  set WR_CARD=
  set WR_PATHCHS=
)


echo %0: viewport [%WW%x%HH%+%LL%+%TT%]
echo %0: persp [%PERSP_PARAMS%]

set cnt=0
for /F "usebackq tokens=1,2" %%A in (`%IMG7%magick ^
  %TMP_ROT% ^
  -define "distort:viewport=%WW%x%HH%+%LL%+%TT%" ^
  -distort Perspective "%PERSP_PARAMS%" ^
  +repage ^
  %WR_CARD% ^
  -crop 6x4@ +repage ^
  -gravity Center ^
  -crop 75x75%%+0+0 +repage ^
  -format "SDOK %%[fx:standard_deviation<0.03?1:0]\n" +write info: ^
  +depth ^
  %WR_PATHCHS% ^
  -scale "1x1^!" ^
  +append +repage ^
  -crop 6x1 +repage -append +repage ^
  +write %OUT_MAT% ^
  -scale "600x400^" ^
  +write %OUT_SCL% ^
  NULL:`) do (
  if "%%A"=="SDOK" if "%%B"=="0" (
    echo %0: cnt=!cnt! Bad patch
    set OKAY=0
  )
  set /A cnt+=1
)

if not %cnt%==24 (
  echo %0: didn't find 24 patches.
  exit /B 1
)

echo %0: Created %OUT_MAT% and %OUT_SCL%

