rem From image %1 that contains a 24-colour card,
rem make image files %2, must contain XX.
rem %3 if 1, debug mode.
rem %4 shrink size. 0 means no shrink. Default 1000.
rem 
@rem
@rem Updated:
@rem   22-August-2022 Upgraded for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 24c

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set DEBUG=%3
if "%DEBUG%"=="." set DEBUG=
if "%DEBUG%"=="" set DEBUG=0

set SHR_SIZE=%4
if "%SHR_SIZE%"=="." set SHR_SIZE=
if "%SHR_SIZE%"=="" set SHR_SIZE=1000

:: FIXME: TRY_ROT is never set.
if "%TRY_ROT%"=="" set TRY_ROT=0


set LOGFILE=24card.log

echo %0: %1 %OUTFILE% %DEBUG% %SHR_SIZE%

set OUTEXT=%OUTFILE:*.=%

set OUT_MAT=%OUTFILE:XX=mat%
set OUT_SCL=%OUTFILE:XX=scl%
set OUT_DBG=%OUTFILE:XX=dbg%

if "%OUT_MAT%"=="%OUTFILE%" (
  echo %0: Outfile needs XX
  exit /B 1
)

del %OUT_MAT% 2>nul
del %OUT_SCL% 2>nul
del %OUT_DBG% 2>nul

set TMPDIR=\temp\

set TMP_IN=%OUTFILE:XX=tmpin%
set TMP_IN=%TMPDIR%!TMP_IN:%OUTEXT%=tiff!

set TMP_ROT=%OUTFILE:XX=tmprot%
set TMP_ROT=%TMPDIR%!TMP_ROT:%OUTEXT%=tiff!

echo %0: TMP_IN=%TMP_IN%  TMP_ROT=%TMP_ROT%



if "%COLS%"=="" set /A COLS=15

:: Resize down only.
::
if "%SHR_SIZE%"=="0" (
  set sRES=
) else (
  set sRES=-resize "%SHR_SIZE%^>"
)


%IMG7%magick ^
  %INFILE% ^
  -auto-orient ^
  %sRES% ^
  +depth ^
  +write %TMP_IN% ^
  %TMP_ROT%

set sTRANS=-auto-orient %sRES%

if "%TRY_ROT%"=="1" (
  call %PICTBAT%24cardRot %TMP_ROT% %FIND0%

  if ERRORLEVEL 1 exit /B 1

  set sROT=

  if not "!24crBEST_ANG!"==0 (
    set sROT=-virtual-pixel Black +distort SRT 1,!24crBEST_ANG! +repage
  )

  %IMG7%magick ^
    %TMP_ROT%
    !sROT! ^
    +depth ^
    +write %TMP_IN% ^
    %TMP_ROT%

  set sTRANS=%sTRANS% !sROT!
)

if ERRORLEVEL 1 (
  echo %0: %1 magick failed  >>%LOGFILE%
  exit /B 1
)

%IMG7%magick identify %TMP_ROT%




set SRCHPROC=-process 'rmsealpha ms md 15 adj 1.0 so'


set prev_ang=0

call :Find12 %TMP_ROT%
if ERRORLEVEL 1 exit /B 1

:: Crop to the card, plus a generous margin.

set FMT=^
diag=%%[fx:hypot((%Y0%-%Y1%),(%X1%-%X0%))]\n

echo %0: FMT=%FMT%

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

set FMT=^
diagOk=%%[fx:%diag%^>120]\n^
XL=%%[fx:floor((%X0%^<%X1%?%X0%:%X1%)-%diag%)]\n^
WH=%%[fx:floor(3*%diag%+2)]\n^
YT=%%[fx:floor((%Y0%^<%Y1%?%Y0%:%Y1%)-%diag%)]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

if %XL% LSS 0 set XL=0
if %YT% LSS 0 set YT=0

set sCROP=%WH%x%WH%+%XL%+%YT%

echo %0: diag=%diag%  diagOk=%diagOk% crop to %sCROP%

if %diagOk%==0 (
  set OKAY=0
  goto mkDebug
)

%IMG7%magick ^
  %TMP_ROT% ^
  -crop %sCROP% +repage ^
  +write %TMP_IN% ^
  %TMP_ROT%

set sTRANS=%sTRANS% -crop %sCROP% +repage


:: We have found top two points.
:: This gives an angle for rotation, so rotate it, and find other two points.

set CNT=3
:loop

call :Find12 %TMP_ROT%
if ERRORLEVEL 1 exit /B 1

set FMT=^
rotang=%%[fx:%prev_ang%+atan2((%Y0%-%Y1%),(%X1%-%X0%))*180/PI]\n

:: We accept +1 or -1 for dy.
set /A dy=%Y1%-%Y0%
if %dy% LSS 0 set /A dy=-%dy%

if %cnt% GTR 0 if %dy% GTR 1 (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "%FMT%" ^
    xc:`) do set %%L

  set FMT=^
angOkay=%%[fx:abs^(!rotang!^)^>45?0:1]\n

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "!FMT!" ^
    xc:`) do set %%L

  echo %0: cnt=%cnt% rotang=!rotang! angOkay=!angOkay!

  if !angOkay!==0 (
    set OKAY=0
    goto mkDebug
  )

  %IMG7%magick ^
    %TMP_IN% ^
    -distort SRT 1,!rotang! +repage ^
    %TMP_ROT%

  set prev_ang=!rotang!

  set /A cnt-=1

  goto loop
)

if %cnt%==0 (
  echo %0: %1 magick failed >>%LOGFILE%
  echo %0: Loop bust: cnt==0.
  set OKAY=0
  goto mkDebug
)

if not %prev_ang%==0 (
  set sTRANS=%sTRANS% -distort SRT 1,%prev_ang% +repage 
)

echo Found X0,Y0 X1,Y1

for /F "usebackq tokens=1-4 delims=@, " %%A in (`%IM7DEV%magick ^
  %TMP_ROT% ^
  %FIND2% ^
  %SRCHPROC% ^
  NULL:`) do (
  if "%%A"=="rmsealpha:" (
    set score2=%%B
    set /A X2=%%C+%COLS%+%SEMI_SP%
    set /A Y2=%%D+%COLS%+%SEMI_SP%
  )
)

echo %0 2: %score2% %X2% %Y2%

for /F "usebackq tokens=1-4 delims=@, " %%A in (`%IM7DEV%magick ^
  %TMP_ROT% ^
  %FIND3%
  %SRCHPROC% ^
  NULL:`) do (
  if "%%A"=="rmsealpha:" (
    set score3=%%B
    set /A X3=%%C+%COLS%+%SEMI_SP%
    set /A Y3=%%D+%COLS%+%SEMI_SP%
  )
)

echo %0 3: %score3% %X3% %Y3%

set X0b=
set X0c=

set OKAY=1

if %X0% GTR %X1% set OKAY=0
if %X3% GTR %X2% set OKAY=0
if %Y0% GTR %Y3% set OKAY=0
if %Y1% GTR %Y2% set OKAY=0

if %OKAY%==0 (
  echo %0: Bad find.
  echo %0: %1 Bad find. >>%LOGFILE%
  set OKAY=0
  goto mkDebug
)

set /A dx=%X1%-%X0%
if %dx% LSS 90 (
  echo %0: Too small.
  echo %0: dx=%dx% Too small. Don't shrink so much?  >>%LOGFILE%
  set OKAY=0
  goto mkDebug
)

set EIGHTNUM=%X0%,%Y0% %X1%,%Y1% %X2%,%Y2% %X3%,%Y3%



call %PICTBAT%24cardChop

if ERRORLEVEL 1 (
  echo 24cardChop failed
  exit /B 1
)

goto skipChop
::--- Move following

:: Extrapolate outwards to get four corners.

set FMT=^
X0b=%%[fx:%X0%-(%X1%-%X0%)/4]\n^
X1b=%%[fx:%X1%+(%X1%-%X0%)/4]\n^
X3b=%%[fx:%X3%-(%X2%-%X3%)/4]\n^
X2b=%%[fx:%X2%+(%X2%-%X3%)/4]\n^
Y0b=%%[fx:%Y0%-(%Y1%-%Y0%)/4]\n^
Y3b=%%[fx:%Y3%-(%Y2%-%Y3%)/4]\n^
Y1b=%%[fx:%Y1%+(%Y1%-%Y0%)/4]\n^
Y2b=%%[fx:%Y2%+(%Y2%-%Y3%)/4]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

set FMT=^
X0c=%%[fx:%X0b%-(%X3b%-%X0b%)/2]\n^
X1c=%%[fx:%X1b%-(%X2b%-%X1b%)/2]\n^
X3c=%%[fx:%X3b%+(%X3b%-%X0b%)/2]\n^
X2c=%%[fx:%X2b%+(%X2b%-%X1b%)/2]\n^
Y0c=%%[fx:%Y0b%-(%Y3b%-%Y0b%)/2]\n^
Y3c=%%[fx:%Y3b%+(%Y3b%-%Y0b%)/2]\n^
Y1c=%%[fx:%Y1b%-(%Y2b%-%Y1b%)/2]\n^
Y2c=%%[fx:%Y2b%+(%Y2b%-%Y1b%)/2]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L


:: Distort so the four corners become a rectangle.

set FMT=^
left=%%[fx:(%X0c%+%X3c%)/2]\n^
top=%%[fx:(%Y0c%+%Y1c%)/2]\n^
right=%%[fx:(%X1c%+%X2c%)/2]\n^
bot=%%[fx:(%Y2c%+%Y3c%)/2]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

set PERSP_PARAMS=^
%X0c%,%Y0c%,%left%,%top%,^
%X1c%,%Y1c%,%right%,%top%,^
%X2c%,%Y2c%,%right%,%bot%,^
%X3c%,%Y3c%,%left%,%bot%


set FMT=^
LL=%%[fx:floor(%left%+0.5)]\n^
TT=%%[fx:floor(%top%+0.5)]\n^
WW=%%[fx:floor(%right%-%left%+0.5)]\n^
HH=%%[fx:floor(%bot%-%top%+0.5)]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

set OKAY=1

:: For deugging, we can record the calculated patches.

if "%DEBUG%" GTR "1" (
  set WR_CARD=+write %TMPDIR%24c_card.tiff
  set WR_PATHCHS=+write %TMPDIR%24c_x6.png
) else (
  set WR_CARD=
  set WR_PATHCHS=
)


echo %0: viewport [%WW%x%HH%+%LL%+%TT%]
echo %0: persp [%PERSP_PARAMS%]

set cnt=0
for /F "usebackq tokens=1,2" %%A in (`%IMG7%magick ^
  %TMP_ROT% ^
  -define "distort:viewport=%WW%x%HH%+%LL%+%TT%" ^
  -distort Perspective "%PERSP_PARAMS%" ^
  +repage ^
  %WR_CARD% ^
  -crop 6x4@ +repage ^
  -gravity Center ^
  -crop 75x75%%+0+0 +repage ^
  -format "SDOK %%[fx:standard_deviation<0.03?1:0]\n" +write info: ^
  +depth ^
  %WR_PATHCHS% ^
  -scale "1x1^!" ^
  +append +repage ^
  -crop 6x1 -append +repage ^
  +write %OUT_MAT% ^
  -scale "600x400^" ^
  +write %OUT_SCL% ^
  NULL:`) do (
  if "%%A"=="SDOK" if "%%B"=="0" (
    echo %0: cnt=!cnt! Bad patch
    set OKAY=0
  )
  set /A cnt+=1
)

echo %0: Created %OUT_MAT% and %OUT_SCL%

::--- End Move following
:skipChop



:mkDebug


call %PICTBAT%24cardDbg

if ERRORLEVEL 1 (
  echo 24cardDbg failed
  exit /B 1
)

goto skipDbg
::--- Move following

if %OKAY%==0 set DEBUG=1

set CIRC=circle 0,0 0,10

del %OUT_DBG% 2>nul

if "%DEBUG%" GTR "0" (

  set CIRCS1b=
  set CIRCS2=
  set CIRCS3=

  if not "%X0b%"=="" (
    set CIRCS2=^
    -draw "translate %X0b%,%Y0b% %CIRC%" ^
    -draw "translate %X1b%,%Y1b% %CIRC%" ^
    -draw "translate %X2b%,%Y2b% %CIRC%" ^
    -draw "translate %X3b%,%Y3b% %CIRC%"
  )

  if not "%X0c%"=="" (
    set CIRCS3=^
    -draw "translate %X0c%,%Y0c% %CIRC%" ^
    -draw "translate %X1c%,%Y1c% %CIRC%" ^
    -draw "translate %X2c%,%Y2c% %CIRC%" ^
    -draw "translate %X3c%,%Y3c% %CIRC%"
  )

  if not "%X2%"=="" (
    set CIRCS1b=^
    -draw "translate %X2%,%Y2% %CIRC%" ^
    -draw "translate %X3%,%Y3% %CIRC%"
  )

  %IMG7%magick ^
    %TMP_ROT% ^
    -fill None ^
    -stroke Orange ^
    -draw "translate %X0%,%Y0% %CIRC%" ^
    -draw "translate %X1%,%Y1% %CIRC%" ^
    !CIRCS1b! ^
    -stroke Cyan !CIRCS2! ^
    -stroke Lime !CIRCS3! ^
    +depth ^
    %OUT_DBG%

  echo %0: Created %OUT_DBG%
)

::--- End Move following
:skipDbg


call echoRestore

echo %0: OKAY=%OKAY%

endlocal & set OUTFILE24c=%OUTFILE%& set OKAY24c=%OKAY%& set sTRANS24c=%sTRANS%& set EIGHTNUM24c=%EIGHTNUM%

@exit /B 0

::---------------------------------------------------
:: Subroutines

:Find12

set SUB_IN=%1

set X0=
for /F "usebackq tokens=1-4 delims=@, " %%A in (`%IM7DEV%magick ^
  %SUB_IN% ^
  %FIND0% ^
  %SRCHPROC% ^
  NULL:`) do (
  if "%%A"=="rmsealpha:" (
    set score0=%%B
    set /A X0=%%C+%COLS%+%SEMI_SP%
    set /A Y0=%%D+%COLS%+%SEMI_SP%
  )
)

echo %0 0: score=%score0% X0=%X0% Y0=%Y0%

if "%X0%"=="" exit /B 1

set X1=
for /F "usebackq tokens=1-4 delims=@, " %%A in (`%IM7DEV%magick ^
  %SUB_IN% ^
  %FIND1% ^
  %SRCHPROC% ^
  NULL:`) do (
  if "%%A"=="rmsealpha:" (
    set score1=%%B
    set /A X1=%%C+%COLS%+%SEMI_SP%
    set /A Y1=%%D+%COLS%+%SEMI_SP%
  )
)

echo %0 1: score=%score1% X1=%X1% Y1=%Y1%

if "%X1%"=="" exit /B 1


exit /B 0
