/* Updated:
     5-April-2018 for v7.0.7-28.
*/

// From an image, make a same-size absolute displacement map.

static void SetArgs (
  double * args, int nArg,
  ssize_t x, ssize_t y,
  double r, double g, double b)
{
  int i = nArg*5;
  args[i++] = x;
  args[i++] = y;
  args[i++] = r;
  args[i++] = g;
  args[i++] = b;
}

static Image * mIdentAbsDispMap (
  Image * image,
  ExceptionInfo *exception)
{
#define NUM_SPARSE_ARGS 20
  double args[NUM_SPARSE_ARGS];

  SetArgs (args, 0, 0,                0,             0, 0, 0);
  SetArgs (args, 1, image->columns-1, 0,             1, 0, 0);
  SetArgs (args, 2, 0,                image->rows-1, 0, 1, 0);
  SetArgs (args, 3, image->columns-1, image->rows-1, 1, 1, 0);

  // V7 SparseColorImage is different.
  // Set the traits of image to RGB only.


#if IMV6OR7==6
  Image * new_img = SparseColorImage (
    image,
    RedChannel | GreenChannel | BlueChannel,
    BilinearColorInterpolate,
    NUM_SPARSE_ARGS, args, exception);
#else
  ChannelType channel_mask = SetImageChannelMask (
    image,
    RedChannel | GreenChannel | BlueChannel);

  Image * new_img = SparseColorImage (
    image,
    BilinearColorInterpolate,
    NUM_SPARSE_ARGS, args, exception);

  SetImageChannelMask (image, channel_mask);
#endif

  if (!new_img) {
    fprintf (stderr, "mIdentAbsDispMap: SparseColorImage failed\n");
    return NULL;
  }

  return (new_img);
}


