rem From image %1,
rem makes output %2 blurred most at edges, or most at non-edges.
rem %3 E=blur at the edges, or N=blur at non-edges.
rem %4 edge detector sigma
rem %5 blur sigma
rem %6 blend percentage: 0=no effect, 100=blur, -100=sharpen
rem %7 colorspace
@rem
@rem Also uses:
@rem   absWR_MASK if not blank, writes mask to this file
@rem   absRD_MASK if not blank, reads mask from this file
@rem   absMSK_PROC
@rem   absMSK_PRE
@rem
@rem Updated:
@rem   16-December-2017 Corrected PREV_COLSP bug.
@rem   29-August-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave

call %PICTBAT%setInOut %1 abs

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set BLR_AT=%3
if "%BLR_AT%"=="." set BLR_AT=
if "%BLR_AT%"=="" set BLR_AT=N

set EDGE_SIG=%4
if "%EDGE_SIG%"=="." set EDGE_SIG=
if "%EDGE_SIG%"=="" set EDGE_SIG=10

set BLUR_SIG=%5
if "%BLUR_SIG%"=="." set BLUR_SIG=
if "%BLUR_SIG%"=="" set /A BLUR_SIG=%EDGE_SIG%/2

set BLEND_PC=%6
if "%BLEND_PC%"=="." set BLEND_PC=
if "%BLEND_PC%"=="" set BLEND_PC=100

set COLSP=%7
if "%COLSP%"=="." set COLSP=

set TMPDIR=\temp\
set TMPGRAY=%TMPDIR%abs_gray.miff
set TMPMAG=%TMPDIR%abs_gray_mag.miff

if /I "%BLR_AT%"=="E" (
  set sNEG=
) else if /I "%BLR_AT%"=="N" (
  set sNEG=-channel RGB -negate +channel
) else (
  echo Bad BLR_AT [%BLR_AT%] should be E or N.
  exit /B 1
)

if "%absMSK_PROC%"=="" set absMSK_PROC=-contrast-stretch 45%%x45%%
if "%absMSK_PROC%"=="." set absMSK_PROC=

if "%absRD_MASK%"=="" (
  set sRD_MASK=%TMPMAG% -auto-level %absMSK_PROC%
) else (
  set sRD_MASK=%absRD_MASK%
)

if "%absWR_MASK%"=="" (
  set sWR_MASK=
) else (
  set sWR_MASK=+write %absWR_MASK%
)


set PREV_COLSP=
for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -format "PREV_COLSP=%%[colorspace]" +write info: ^
  -colorspace Gray ^
  %absMSK_PRE% ^
  %TMPGRAY%`) do set %%L
if "%PREV_COLSP%"=="" exit /B 1
echo PREV_COLSP=%PREV_COLSP%

set COLSP_IN=
set COLSP_OUT=
if "%COLSP%" NEQ "" if /I "%COLSP%" NEQ "%PREV_COLSP%" (
  set COLSP_IN=-colorspace %COLSP% -set colorspace sRGB
  set COLSP_OUT=-set colorspace %COLSP% -colorspace %PREV_COLSP%
)

call %PICTBAT%slopeXYblMag ^
  %TMPGRAY% %TMPMAG% %EDGE_SIG%

%IMG7%magick ^
  %INFILE% ^
  %COLSP_IN% ^
  ( +clone ^
    ( %sRD_MASK% ^
      %sWR_MASK% ^
      %sNEG% ^
    ) ^
    -compose Blur ^
    -set option:compose:args %BLUR_SIG%x%BLUR_SIG% ^
    -composite ^
  ) ^
  -compose Blend -define compose:args=%BLEND_PC% -composite ^
  %COLSP_OUT% ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1


call echoRestore

@endlocal & set absOUTFILE=%OUTFILE%
