rem From image %1,
rem makes output %2
rem blurred or sharpened parallel to contours
rem with most effect at edges, or most at non-edges.
rem %3 E=blur at the edges, or N=blur at non-edges.
rem %4 edge detector sigma
rem %5 blur sigma
rem %6 blend percentage: 0=no effect, 100=blur, -100=sharpen
rem %7 colorspace
rem %8 parallel or perpendicular
@rem
@rem Also uses:
@rem   acbsWR_MASK if not blank, writes mask to this file
@rem   acbsRD_MASK if not blank, reads mask from this file
@rem   acbsMSK_PROC
@rem   acbsMSK_PRE
@rem
@rem Updated:
@rem   16-December-2017 Corrected PREV_COLSP bug.
@rem   29-August-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave

call %PICTBAT%setInOut %1 acbs

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set BLR_AT=%3
if "%BLR_AT%"=="." set BLR_AT=
if "%BLR_AT%"=="" set BLR_AT=N

set EDGE_SIG=%4
if "%EDGE_SIG%"=="." set EDGE_SIG=
if "%EDGE_SIG%"=="" set EDGE_SIG=10

set BLUR_SIG=%5
if "%BLUR_SIG%"=="." set BLUR_SIG=
if "%BLUR_SIG%"=="" set /A BLUR_SIG=%EDGE_SIG%/2

set BLEND_PC=%6
if "%BLEND_PC%"=="." set BLEND_PC=
if "%BLEND_PC%"=="" set BLEND_PC=100

set COLSP=%7
if "%COLSP%"=="." set COLSP=

set PARPERP=%8
if "%PARPERP%"=="." set PARPERP=
if "%PARPERP%"=="" set PARPERP=parallel

set TMPDIR=\temp\
set TMPGRAY=%TMPDIR%acbs_gray.miff
set TMPDIRN=%TMPDIR%acbs_gray_dirn.miff
set TMPMAG=%TMPDIR%acbs_gray_mag.miff
set TMPMASK=%TMPDIR%acbs_gray_mask.miff

if /I "%BLR_AT%"=="E" (
  set sNEG=
) else if /I "%BLR_AT%"=="N" (
  set sNEG=-channel RG -negate +channel
) else (
  echo Bad BLR_AT [%BLR_AT%] should be E or N.
  exit /B 1
)

:: FIXME: One of these is wrong
if /I "%PARPERP%"=="Parallel" (
  set sBLRARG=-set option:compose:args %BLUR_SIG%x0+0+360
) else if /I "%PARPERP%"=="Perpendicular" (
  set sBLRARG=-set option:compose:args 0x%BLUR_SIG%+0+360
) else (
  echo Bad PARPERP [%PARPERP%] should be Parallel or Perpendicular.
  exit /B 1
)

if "%acbsMSK_PROC%"=="" set acbsMSK_PROC=-contrast-stretch 45%%x45%%
if "%acbsMSK_PROC%"=="." set acbsMSK_PROC=

if "%acbsRD_MASK%"=="" (
  set sRD_MASK=%TMPMAG% -auto-level %acbsMSK_PROC%
  set sRD_MASK=%TMPMASK% -auto-level %acbsMSK_PROC%
) else (
  set sRD_MASK=%acbsRD_MASK%
)

if "%acbsWR_MASK%"=="" (
  set sWR_MASK=
) else (
  set sWR_MASK=+write %acbsWR_MASK%
)


set PREV_COLSP=
for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -format "PREV_COLSP=%%[colorspace]" +write info: ^
  -colorspace Gray ^
  %acbsMSK_PRE% ^
  %TMPGRAY%`) do set %%L
if "%PREV_COLSP%"=="" exit /B 1
echo PREV_COLSP=%PREV_COLSP%

set COLSP_IN=
set COLSP_OUT=
if "%COLSP%" NEQ "" if /I "%COLSP%" NEQ "%PREV_COLSP%" (
  set COLSP_IN=-colorspace %COLSP% -set colorspace sRGB
  set COLSP_OUT=-set colorspace %COLSP% -colorspace %PREV_COLSP%
)

call %PICTBAT%slopeXYbl ^
  %TMPGRAY% %TMPGRAY% %EDGE_SIG%

call %PICTBAT%slopeXYmag ^
  %TMPGRAY% %TMPMAG%

call %PICTBAT%slopeXYdirn ^
  %TMPGRAY% %TMPDIRN% ^
  SWAP . 75

%IMG7%magick ^
  %TMPMAG% -auto-level %acbsMSK_PROC% ^
  ( +clone ) ^
  %TMPDIRN% ^
  -combine ^
  -set colorspace sRGB ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  %TMPMASK%

%IMG7%magick ^
  %INFILE% ^
  %COLSP_IN% ^
  ( +clone ^
    ( %sRD_MASK% ^
      %sWR_MASK% ^
      %sNEG% ^
    ) ^
    -compose Blur ^
    %sBLRARG% ^
    -composite ^
  ) ^
  -compose Blend -define compose:args=%BLEND_PC% -composite ^
  %COLSP_OUT% ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1


call echoRestore

@endlocal & set acbsOUTFILE=%OUTFILE%
