#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vsn_defines.h"

// The next function corresponds in style to functions in transform.c
// It takes one image, and returns an image.
//
static Image *CopyBlack(const Image *image,
  ExceptionInfo *exception)
{
  Image
    *new_image;

  assert(image != (Image *) NULL);
  assert(image->signature == MAGICK_CORE_SIG);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);


  // Make a clone of this image, same size but undefined pixel values:
  //
  new_image=CloneImage(image, image->columns, image->rows, MagickTrue, exception);
  if (new_image == (Image *) NULL)
    return(new_image);

#if IMV6OR7==6
//  if (SetImageStorageClass(new_image,DirectClass) == MagickFalse)
//    return (Image *)NULL;

//  GetMagickPixelPacket (new_image, &mppBlack);
//  (void)SetImageColor(new_image, &mppBlack);
#else
//  if (SetImageStorageClass(new_image,DirectClass, exception) == MagickFalse)
//    return (Image *)NULL;

//  GetPixelInfo (new_image, &mppBlack);
//  (void)SetImageColor(new_image, &mppBlack, exception);
#endif

  if (SetNoPalette (new_image, exception) == MagickFalse)
    return (Image *)NULL;

  SetAllBlack (new_image, exception);

  return (new_image);
}



ModuleExport size_t addendImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image,
    *new_image;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  (void) argc;
  (void) argv;

  // Point to the last image in the list
  //
  image=GetLastImageInList(*images);
  if (image == (Image *) NULL) {
    fprintf (stderr, "addend: no images in list\n");
    return (-1);
  }

  new_image = CopyBlack(image,exception);
  if (new_image == (Image *) NULL)
    return(-1);

  // Add the new image to the end of the list:
  AppendImageToList(images,new_image);

  return(MagickImageFilterSignature);
}
