rem Applies auto level and auto gamma as appropriate for four tiles,
rem   blending between the four quarters.
rem %2 is optional output filename.
@rem
@rem Can also use:
@rem   algqMASK_CURVE eg -sigmoidal-contrast 5x50%
@rem   algqLINEAR if 1, composes in linear colourspace.
@rem   algqDO_MIN if 0, ignores min values so doesn't auto-level shadows.
@rem   algqDO_MAX if 0, ignores max values so doesn't auto-level highlights.
@rem   algqDO_GAM if 0, ignores mean values so doesn't auto-gamma.
@rem   algqNEW_MEAN default 0.5
@rem
@rem See also algTile.bat
@rem
@rem Updated:
@rem   27-July-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 algq

if not "%2"=="" set OUTFILE=%2

if "%algqNEW_MEAN%"=="" set algqNEW_MEAN=0.5

set N=0

for /F "tokens=1-3 usebackq" %%A in (`%IMG7%magick ^
  %INFILE% ^
  -crop 2x2@ +repage ^
  -precision 15 ^
  -format "%%[fx:minima*100] %%[fx:maxima*100] %%[fx:ln(mean)/ln(minima+%algqNEW_MEAN%*(maxima-minima))]\n" ^
  info:`) do (
  set eqqMin.!N!=%%A
  set eqqMax.!N!=%%B
  set eqqGam.!N!=%%C

  rem gamma was ln(mean)/ln(0.5)
  rem gamma was ln(mean)/ln((minima+maxima)/2)

  if "%algqDO_MIN%"=="0" set eqqMin.!N!=0
  if "%algqDO_MAX%"=="0" set eqqMax.!N!=100
  if "%algqDO_GAM%"=="0" set eqqGam.!N!=1

  set /A N+=1
)

:: set eqq

%IMG7%magick ^
  %INFILE% ^
  -alpha off ^
  ( -clone 0 -level %eqqMin.0%%%,%eqqMax.0%%%,%eqqGam.0% %algqMASK_CURVE% ) ^
  ( -clone 0 -level %eqqMin.1%%%,%eqqMax.1%%%,%eqqGam.1% %algqMASK_CURVE% ) ^
  ( -clone 0 -level %eqqMin.2%%%,%eqqMax.2%%%,%eqqGam.2% %algqMASK_CURVE% ) ^
  ( -clone 0 -level %eqqMin.3%%%,%eqqMax.3%%%,%eqqGam.3% %algqMASK_CURVE% ) ^
  ( -clone 0 -sparse-color Bilinear 0,0,#000,%%[fx:w-1],0,#fff ) ^
  -delete 0 ^
  ( -clone 0,1,4 -composite ) ^
  ( -clone 2,3,4 -composite ) ^
  -delete 0-4 ^
  ( -clone 0 -sparse-color Bilinear 0,0,#000,0,%%[fx:h-1],#fff ) ^
  -composite ^
  +depth ^
  %OUTFILE%


call echoRestore

@endlocal & set algqOUTFILE=%OUTFILE%
