rem Given image %1, effectively divided into %2 x %3 tiles,
rem applies auto level and auto gamma as appropriate for each tile,
rem and merges the results.
rem %4 is optional output filename.
@rem
@rem Can also use:
@rem   algtMASK_CURVE eg -sigmoidal-contrast 5x50%
@rem   algtLINEAR if 1, composes in linear colourspace.
@rem   algtCLEAN if 0, does not delete temporary files.
@rem   algtDO_MIN if 0, ignores min values so doesn't auto-level shadows.
@rem   algtDO_MAX if 0, ignores max values so doesn't auto-level highlights.
@rem   algtDO_GAM if 0, ignores mean values so doesn't auto-gamma.
@rem
@rem Updated:
@rem   27-July-2022 for IM v7.
@rem


@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 algt


set VSN_LIST=%TEMP%\algtVsn.lis
del %VSN_LIST% 2>nul

set N_COL=%2
set N_ROW=%3
if not "%4"=="" set OUTFILE=%4

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -crop "%N_COL%x%N_ROW%@" +repage ^
  -precision 15 ^
  -format "MIN.%%p=%%[fx:100*minima]\nMAX.%%p=%%[fx:100*maxima]\nMEAN.%%p=%%[fx:100*mean]\nGAMM.%%p=%%[fx:ln(mean)/ln((minima+maxima)/2)]\nMAX_N=%%p\n" ^
  INFO:`) do @set %%L

rem set MIN
rem set MAX
rem set MEAN

set /A COL_X_ROWm1=%N_COL%*%N_ROW%-1

if not %MAX_N%==%COL_X_ROWm1% (
  echo %0: Bug: %MAX_N% neq %COL_X_ROWm1%
  exit /B 1
)



set PART_COM=
for /L %%i in (0,1,%MAX_N%) do (
  if "%algtDO_MIN%"=="0" set MIN.%%i=0
  if "%algtDO_MAX%"=="0" set MAX.%%i=100
  if "%algtDO_GAM%"=="0" set GAMM.%%i=1

  set VSN_FILE=algtVsn_%%i.png
  set PART_COM=!PART_COM! ^( +clone -level !MIN.%%i!%%,!MAX.%%i!%%,!GAMM.%%i! %algtMASK_CURVE% +write !VSN_FILE! +delete ^)
  echo !VSN_FILE!>>%VSN_LIST%
)
rem echo %PART_COM%

%IMG7%magick %INFILE% +depth %PART_COM% NULL:

rem type %VSN_LIST%

set btLINEAR=%algtLINEAR%

call %PICTBAT%blendTile %VSN_LIST% %N_COL% %N_ROW% %OUTFILE%
if ERRORLEVEL 1 exit /B 1

if not "algtCLEAN"=="0" (
  del %VSN_LIST%

  for /L %%i in (0,1,%MAX_N%) do (
    set VSN_FILE=algtVsn_%%i.png
    del !VSN_FILE!
  )
)


call echoRestore

@endlocal & set algtOUTFILE=%OUTFILE%
