rem %1 is image A, %2 is image B,
rem %3 is CSV file with list of coords from image A.
rem
rem Returns (affine or perspective transformation from B to A?)
rem    list of coordinate pairs xA,yA,xB,yB:
rem Returns environment variables:
rem   aaCOORDS list of coords
rem   aaCOORDS_CSV name of CSV file containing the coords
@rem
@rem Updated:
@rem   22-August-2022 Upgraded for IM v7.
@rem

@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 aa

if "%aaDIM%"=="" set aaDIM=100

if not exist %1 exit /B 1
if not exist %2 exit /B 1
if not exist %3 exit /B 1

set SRC_A=%1
set SRC_B=%2
set ALI_COORDS=%3

set ncDEBUG=
call %PICTBAT%nCrop %SRC_A% %ALI_COORDS% %aaDIM% %aaDIM%
if ERRORLEVEL 1 exit /B 1

type %ncLIST_FILE%

set MATCH_LIS=%TEMP%\aa_match.lis
del %MATCH_LIS% 2>nul
set MATCH_LIS2=%TEMP%\aa_match2.lis

set TMP_B=%BASENAME%_aa_tmpb.miff

%IMG7%magick %SRC_B% +repage %TMP_B%

rem echo %0: CropNum,Score,X,Y &gt;%MATCH_LIS%
set CROP_NUM=0
for /F %%F in (%ncLIST_FILE%) do (
  echo %0: %%F

  call %PICTBAT%srchImg %TMP_B% %%F
  if ERRORLEVEL 1 exit /B 1

  echo !siCENT_X!,!siCENT_Y! >>%MATCH_LIS%

  set /A CROP_NUM+=1
)


rem cProject /i%MATCH_LIS% /oaa_match3.lis /h /kScore,X,Y
rem cHead /iaa_match3.lis /h1 /x
set COORD_PAIRS=%TEMP%\aa_coord_pairs.lis
call %PICTBAT%appendLines %MATCH_LIS% %ALI_COORDS% >%COORD_PAIRS%

rem %COORD_PAIRS% has: xB,yB,xA,yA

set MATCH_MTTE=%TEMP%\aa_matchMTTE.lis
rem call %PICTBAT%extendCoordPairs %COORD_PAIRS% %MATCH_MTTE%

rem From CSV file %COORD_PAIRS%, assumed to contain X0,Y0,X1,Y1
rem writes %MATCH_MTTE% containing X0,Y0,X1,Y1,dX,dY,r
rem where:
rem   dX = X1 - X0
rem   dY = Y1 - Y0
rem   r = sqrt (dX^2 + dY^2)
rem
rem Files have no headers.

del %MATCH_MTTE% 2>nul

set sCOORD=
for /F "tokens=1-5 delims=, " %%A in (%COORD_PAIRS%) do (
  set /A dx=%%C-%%A
  set /A dy=%%D-%%B
  set /A dx2=!dx!*!dx!
  set /A dy2=!dy!*!dy!
  set /A r2=!dx2!+!dy2!

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 9 ^
    -format "r=%%[fx:sqrt(!r2!)]" ^
    xc:`) do set %%L

  echo %%A,%%B,%%C,%%D,!dx!,!dy!,!r! >>%MATCH_MTTE%
)



set rcoAPPEND_NUM=1
set rcoDEBUG_FILE=aa_rco_debug.lis
del %rcoDEBUG_FILE% 2>nul
call removeCsvOutliers %MATCH_MTTE% aa_matchMTTE_good.lis 5 1 6 1 7 1
if ERRORLEVEL 1 exit /B 1

rem type aa_matchMTTE_good.lis


rem Read coords into env var.
set sCOORD=
for /F "tokens=1-5 delims=, " %%A in (aa_matchMTTE_good.lis) do (
  set sCOORD=!sCOORD! %%A,%%B,%%C,%%D
)

@call echoRestore

@endlocal & set aaCOORDS=%sCOORD%& set aaCOORDS_CSV=aa_matchMTTE_good.lis

echo %0: Written env var aaCOORDS and file aaCOORDS_CSV [%aaCOORDS_CSV%]

