rem Find x-y alignment of two same-size images %1 and %2 by brute force comparison,
rem where fully-transparent pixels are wildcard.
rem Uses process module rmsealpha.

@rem We find the position that scores the best RMSE.
@rem Fairly fast when WW*HH is small, or search space is small.
@rem
@rem Optional:
@rem   %2 is width of both
@rem   %3 is height of both
@rem   %5,%6 top-left to try
@rem   %7,%8 botton-right to try
@rem
@rem Returns name of a file containing CSV data.
@rem If best is at an extreme, probably dodgy.
@rem
@rem Updated:
@rem   22-August-2022 Upgraded for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave

call %PICTBAT%setInOut %1 abfa


set SCR_FILE=%sioCODE%.scr
set DATA_FILE=%sioCODE%_data.csv
set DATA_FILE2=%sioCODE%_data2.csv
set DATA_FILE3=%sioCODE%_data3.csv

set SRC1=%1
set SRC2=%2

set WW=%3
set HH=%4

if "%WW%"=="." set WW=
if "%HH%"=="." set HH=

set FIND_DIMS=0
if "%WW%"=="" set FIND_DIMS=1
if "%HH%"=="" set FIND_DIMS=1

if %FIND_DIMS%==1 for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h" ^
  %SRC1%`) do set %%L

if "%WW%"=="" (
  echo %0: can't identify SRC1 [%SRC1%]
  exit /B 1
)

echo %SRC1% %SRC2% >%SCR_FILE%

set /A WWm1=%WW%-1
set /A HHm1=%HH%-1

set FirstX=%5
set FirstY=%6
set LastX=%7
set LastY=%8

if "%FirstX%"=="." set /A FirstX=
if "%FirstY%"=="." set /A FirstY=
if "%LastX%"=="." set /A LastX=
if "%LastY%"=="." set /A LastY=

if "%FirstX%"=="" set /A FirstX=-%WWm1%
if "%FirstY%"=="" set /A FirstY=-%HHm1%

if "%LastX%"=="" set /A LastX=%WWm1%
if "%LastY%"=="" set /A LastY=%HHm1%

echo %~n0: FirstX=%FirstX% FirstY=%FirstY% LastX=%LastX% LastY=%LastY%

set MIN_COMP=999

rem Optional -auto-level -auto-gamma  to each crop?
for /L %%y in (%FirstY%,1,%LastY%) do (

  if %%y GEQ 0 (
    set /A H=%HH%-%%y
    set /A Y1=%%y
    set /A Y2=0
  ) else (
    set /A H=%HH%+%%y
    set /A Y1=0
    set /A Y2=-%%y
  )

  for /L %%x in (%FirstX%,1,%LastX%) do (

    if %%x GEQ 0 (
      set /A W=%WW%-%%x
      set /A X1=%%x
      set /A X2=0
    ) else (
      set /A W=%WW%+%%x
      set /A X1=0
      set /A X2=-%%x
    )

    ( echo ^( -clone 0-1
      echo   ^( -clone 0 -crop !W!x!H!+!X1!+!Y1! +repage -auto-level ^)
      echo   ^( -clone 1 -crop !W!x!H!+!X2!+!Y2! +repage -auto-level ^)
      echo   -delete 0-1
      echo   -process 'rmsealpha just_score stdout'
      echo   -format ",%%x,%%y,!W!,!H!,!X1!,!Y1!,!X2!,!Y2!\n"
      echo   -delete 0
      echo   +write info:
      echo   -delete 0
      echo ^)
      echo -exit
    ) >>%SCR_FILE%
  )
)

%IM7DEV%magick @%SCR_FILE% >%DATA_FILE% 2^>^&1
if ERRORLEVEL 1 exit /B 1

:: Get the row with the smallest value in the first column.
cGroup  /i%DATA_FILE% /n0 /o%DATA_FILE2% /u
cJoin   /i%DATA_FILE2%,%DATA_FILE% /o%DATA_FILE3% /k0 /K0 /n /c2

for /F "tokens=1-9 delims=," %%A in (%DATA_FILE3%) do (
  set COMP=%%A
  set BESTX=%%B
  set BESTY=%%C
  set W=%%D
  set H=%%E
  set X1=%%F
  set Y1=%%G
  set X2=%%H
  set Y2=%%I
)

if "%COMP%"=="" exit /B 1

set DODGY=1
if not "%BESTX%"=="%FirstX%" if not "%BESTX%"=="%LastX%" if not "%BESTY%"=="%FirstY%" if not "%BESTY%"=="%LastY%" set DODGY=0

echo %~n0: %COMP% %X% %Y% X1=%X1% Y1=%Y1% X2=%X2% Y2=%Y2%


call echoRestore

endlocal & set abfaDATAFILE=%DATA_FILE3%& set abfaDODGY=%DODGY%
