rem %1 is CSV file with list of coords from image A.
rem %2 is CSV file with list of coords from image B.
rem Some points in A correspond to some points in B.
rem Returns affine or perspective transformation from B to A.
@rem
@rem Updated
@rem   3-May-2017 Weirdly, needed set lstA and B adding.
@rem   3-October-2022 for IM v7.
@rem


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 ap

set lstA=%1
set lstB=%2

if not exist %lstA% exit /B 1
if not exist %lstB% exit /B 1

set TEMPLIS=%TEMP%\ap_%~n1.csv

set alSep=,
call %PICTBAT%appendLines %lstA% %lstB% >ap_corrPnts.csv
call %PICTBAT%readCoordPairs ap_corrPnts.csv ap_corrPnts 1 2 3 4

set /A nMin1=%ap_corrPnts.nPAIRS%-1
set /A nMin2=%ap_corrPnts.nPAIRS%-2

rem i,j are indexes into image A
rem k,l are indexes into image B

set XYLIST=%TEMP%\ap_xylist.csv
del %XYLIST% 2>nul

rem Visit every possible alignment.
rem For each one, calculate the sum of the absolute TX and TY parameters.
rem
for /L %%i in (0,1,%nMin2%) do (
  set /A j0=%%i+1
  for /L %%j in (!j0!,1,%nMin1%) do (
    echo %0: i=%%i j=%%j

    for /L %%k in (0,1,%nMin1%) do (
      set /A l0=%%k+1
      for /L %%l in (0,1,%nMin1%) do (

        if not %%k==%%l (
          rem echo ijkl = %%i %%j %%k %%l
          set TwoPairs=!ap_corrPnts[%%i].A.x!,!ap_corrPnts[%%i].A.y!,!ap_corrPnts[%%k].B.x!,!ap_corrPnts[%%k].B.y! !ap_corrPnts[%%j].A.x!,!ap_corrPnts[%%j].A.y!,!ap_corrPnts[%%l].B.x!,!ap_corrPnts[%%l].B.y!
          rem echo !TwoPairs!

          call %PICTBAT%getPerspParams TwoPairs pp

          for /F "usebackq" %%L in (`%IMG7%magick identify ^
            -format "sumAbs=%%[fx:abs(!pp.TX!)+abs(!pp.TY!)]" ^
            xc:`) do set %%L

          echo !sumAbs!,!pp.TX!,!pp.TY!, %%i,%%j, %%k,%%l >>%XYLIST%
        )
      )
    )
  )
)

cSort /p0 /i%XYLIST% /o%XYLIST% /k0,1

set APCOMP=%TEMP%\ap_comp.csv
cHead /i%XYLIST% /o%TEMP%\ap_xtm1.lis /h1 /x
call %PICTBAT%appendLines %XYLIST% %TEMP%\ap_xtm1.lis >%APCOMP%


rem Add the absolute values of the differences of:
rem   the sum of the absolute TX and TY parameters
rem   the TX parameters
rem   the TY parameters
rem
rem We assume that the smallest value gives the best pair of alignments.
rem
rem IsOk was:
rem   IsOk=%%[fx:(%%D==%%K)==(%%E==%%L)&&(%%F==%%M)==(%%G==%%N)?1:0]
rem 

set COMP_SRT=%TEMP%\ap_comp_srt.csv
del %COMP_SRT% 2>nul
for /F "tokens=1-14 delims=, " %%A in (%APCOMP%) do (

  if "%%N"=="" (
    echo %0: not enough tokens in APCOMP [%APCOMP%]
  ) else (

    for /F "usebackq" %%L in (`%IMG7%magick identify ^
      -format "delta=%%[fx:abs(%%H-(%%A))+abs(%%I-(%%B))+abs(%%J-(%%C))]\nIsOk=%%[fx:(%%D==%%K)==(%%F==%%M)&&(%%D==%%L)==(%%F==%%N)&&(%%E==%%K)==(%%G==%%M)&&(%%E==%%L)==(%%G==%%N)]" ^
      xc:`) do set %%L

    if !IsOk!==1 echo !delta!, %%A,%%B,%%C,%%D,%%E,%%F,%%G, %%H,%%I,%%J,%%K,%%L,%%M,%%N >>%COMP_SRT%
  )
)

if not exist %COMP_SRT% (
  echo %0: No entries in COMP_SRT, %COMP_SRT%
  exit /B 1
)

cSort /p0 /i%COMP_SRT% /o%COMP_SRT% /k0
set COMP_SRT1=%TEMP%\ap_comp_srt1.csv
cHead /i%COMP_SRT% /h1 /o%COMP_SRT1%


rem Get the numbers of the matching points.
rem There will be 3 or 4 matches.
rem Either of the last two may be equal to the first two.

for /F "tokens=5-8,12-15 delims=, " %%A in (%COMP_SRT1%) do (
  echo %%A,%%B,%%C,%%D, %%E,%%F,%%G,%%H

  set ap_corrNdx[0].A=%%A
  set ap_corrNdx[1].A=%%B
  set ap_corrNdx[0].B=%%C
  set ap_corrNdx[1].B=%%D

  set ap_corrNdx.nPAIRS=2

  set /A diff2=^(%%E-%%A^)*^(%%G-%%C^)*^(%%E-%%B^)*^(%%G-%%D^)
  set /A diff3=^(%%F-%%A^)*^(%%H-%%C^)*^(%%F-%%B^)*^(%%H-%%D^)

  if not !diff2!==0 (
    set ap_corrNdx[2].A=%%E
    set ap_corrNdx[2].B=%%G
    set ap_corrNdx.nPAIRS=3
  )

  if not !diff3!==0 (
    set ap_corrNdx[!ap_corrNdx.nPAIRS!].A=%%F
    set ap_corrNdx[!ap_corrNdx.nPAIRS!].B=%%H
    set /A ap_corrNdx.nPAIRS+=1
  )

  echo diff2=!diff2! diff3=!diff3!
)

set ap_corrNdx


rem Get the coords of the 3 or 4 points.
set /A nLAST=ap_corrNdx.nPAIRS-1

set sCOORD=
for /L %%i in (0,1,%nLAST%) do (
  set /A ndxA=ap_corrNdx[%%i].A
  set /A ndxB=ap_corrNdx[%%i].B
  set /A ax=ap_corrPnts[!ndxA!].A.x
  set /A ay=ap_corrPnts[!ndxA!].A.y
  set /A bx=ap_corrPnts[!ndxB!].B.x
  set /A by=ap_corrPnts[!ndxB!].B.y
  set sCOORD=!sCOORD! !ax!,!ay!,!bx!,!by!
)
echo %0: sCOORD=%sCOORD%




@call echoRestore

@endlocal & set apCOORDS=%sCOORD%
