rem Creates frames for animation of a cartoon and texture.
rem
rem %1 image to be turned onto cartoon + texture
rem %2 prefix and extension for frame files. Must _not_ include XX.
rem %3 number of frames [default 10].
rem %4 sigma for blur at start [default 0.5].
rem %5 sigma for blur at end [default 10].
@rem
@rem Updated:
@rem   6-August-2022 for IM v7
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 actx

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

for /F %%F in ("%OUTFILE%") do (
  set OUT_PREF=%%~dpnF
  set OUT_EXT=%%~xF
)

echo OUT_PREF=%OUT_PREF% OUT_EXT=%OUT_EXT%

set nFRAMES=%3
if "%nFRAMES%"=="." set nFRAMES=
if "%nFRAMES%"=="" set nFRAMES=10

set SIG_START=%4
if "%SIG_START%"=="." set SIG_START=
if "%SIG_START%"=="" set SIG_START=0.5

set SIG_END=%5
if "%SIG_END%"=="." set SIG_END=
if "%SIG_END%"=="" set SIG_END=10

call :clean

set SIG=%SIG_START%
for /L %%I in (1,1,%nFRAMES%) do (
  echo %%I !SIG!

  set LZ=000000%%I
  set LZ=!LZ:~-6!

  call %PICTBAT%cartTextEep ^
    %INFILE% ^
    %OUT_PREF%_XX_!LZ!%EXT% !SIG!

  %IMG7%magick ^
    %OUT_PREF%_cart_!LZ!*%EXT% ^
    ^( %OUT_PREF%_edges_!LZ!*%EXT% ^
       -negate -transparent White ^
    ^) ^
    -compose Over -composite ^
    %OUT_PREF%_over_!LZ!%EXT%

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 19 ^
    -format "SIG=%%[fx:!SIG!*pow(%SIG_END%/%SIG_START%,1/(%nFRAMES%-1))]" ^
    xc:`) do set %%L
)


%IMG7%magick ^
  -loop 0 -delay 10 %OUT_PREF%_cart*%EXT% -reverse -duplicate 10 ^
  %OUT_PREF%_cart.gif

%IMG7%magick ^
  -loop 0 -delay 10 %OUT_PREF%_skel*%EXT% -reverse -duplicate 10 ^
  %OUT_PREF%_skel.gif

%IMG7%magick ^
  -loop 0 -delay 10 %OUT_PREF%_text*%EXT% -duplicate 10 ^
  %OUT_PREF%_text.gif

%IMG7%magick ^
  -loop 0 -delay 10 %OUT_PREF%_edges*%EXT% -reverse -duplicate 10 ^
  %OUT_PREF%_edges.gif

%IMG7%magick ^
  -loop 0 -delay 10 %OUT_PREF%_over*%EXT% -reverse -duplicate 10 ^
  %OUT_PREF%_over.gif

call :clean

call echoRestore

@endlocal

exit /B 0

:: -------------------------------------------------
:: Subroutine:

:clean
del %OUT_PREF%_skel_*%OUT_EXT%
del %OUT_PREF%_edges_*%OUT_EXT%
del %OUT_PREF%_cart_*%OUT_EXT%
del %OUT_PREF%_text_*%OUT_EXT%
del %OUT_PREF%_over_*%OUT_EXT%

exit /B 0