/* Updated:
     3-April-2018 for v7.0.7-28
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "vsn_defines.h"
#include "avgconcrings.inc"


static void usage (void)
{
  printf ("Usage: -process 'avgconcrings [OPTION]...'\n");
  printf ("Finds mean of concentric rings for a range of scales.\n");
  printf ("\n");
  printf ("  s0, minScale number  minimum scale\n");
  printf ("  s1, maxScale number  maximum scale\n");
  printf ("  n,  nScales integer  number of scales\n");
  printf ("  m,  method string    'slow' or 'quick'\n");
  printf ("  v,  verbose          write text information to stdout\n");
  printf ("  v2, verbose2         more text information to stdout\n");
  printf ("\nOutput has radii on x-axis, with centre at left, scales on y-axis, lowest at top.\n");
  printf ("\n");
}


static MagickBooleanType IsArg (char * pa, char * ShtOpt, char * LongOpt)
{
  if ((LocaleCompare(pa, ShtOpt)==0) || (LocaleCompare(pa, LongOpt)==0))
    return MagickTrue;

  return MagickFalse;
}


#define NEXTARG \
  if (i < argc-1) i++; \
  else { \
    fprintf (stderr, "paintpatches: ERROR: [%s] needs argument\n", pa); \
    status = MagickFalse; \
  }

static MagickBooleanType menu(
  const int argc,
  const char **argv,
  ConcRingsT * pcr
)
// Returns MagickTrue if okay.
{
  int
    i;

  MagickBooleanType
    status;

  status = MagickTrue;

  for (i=0; i < argc; i++) {
    char * pa = (char *)argv[i];

    if (IsArg (pa, "s0", "minScale")==MagickTrue) {
      NEXTARG;
      pcr->minScale = atof(argv[i]);
    } else if (IsArg (pa, "s1", "maxScale")==MagickTrue) {
      NEXTARG;
      pcr->maxScale = atof(argv[i]);
    } else if (IsArg (pa, "n", "nScales")==MagickTrue) {
      NEXTARG;
      pcr->nScales = atoi(argv[i]);
    } else if (IsArg (pa, "m", "method")==MagickTrue) {
      NEXTARG;
      if (LocaleCompare(argv[i], "slow")==0) pcr->ConcRingsMethod = crmSlow;
      else if (LocaleCompare(argv[i], "quick")==0) pcr->ConcRingsMethod = crmQuick;
      else {
        fprintf (stderr, "avgconcrings: ERROR: unknown method [%s]\n", argv[i]);
        status = MagickFalse;
      }
    } else if (IsArg (pa, "v", "verbose")==MagickTrue) {
      pcr->verbose = 1;
    } else if (IsArg (pa, "v2", "verbose2")==MagickTrue) {
      pcr->verbose = 2;
    } else {
      fprintf (stderr, "avgconcrings: ERROR: unknown option\n");
      status = MagickFalse;
    }
  }

  if (pcr->verbose) {
    fprintf (stderr, "avgconcrings options:");
    fprintf (stderr, "  maxScale %g", pcr->maxScale);
    fprintf (stderr, "  minScale %g", pcr->minScale);
    fprintf (stderr, "  nScales %i", pcr->nScales);
    fprintf (stderr, "  method ");
    switch (pcr->ConcRingsMethod) {
      case crmSlow:  fprintf (stderr, "slow"); break;
      case crmQuick: fprintf (stderr, "quick"); break;
    }
    if (pcr->verbose > 1) fprintf (stderr, "  verbose%i ", pcr->verbose);
    else if (pcr->verbose) fprintf (stderr, "  verbose ");
    fprintf (stderr, "\n");
  }

  if (status == MagickFalse)
    usage ();

  return (status);
}


ModuleExport size_t avgconcringsImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image,
    *new_image;

  MagickBooleanType
    status;

  ConcRingsT
    cr;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  InitConcRings (&cr);

  status = menu (argc, argv, &cr);
  if (status == MagickFalse)
    return (-1);

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    new_image = avgconcrings (image, &cr, exception);
    if (new_image == (Image *) NULL) return (-1);

    ReplaceImageInList(&image,new_image);
    *images=GetFirstImageInList(image);

    ReInitConcRings (&cr);
  }

  DeInitConcRings (&cr);

  return(MagickImageFilterSignature);
}
