rem From %1, a text file list of n*m images, one image per line,
rem   %2 is number of columns (n),
rem   %3 is number of rows (m),
rem   makes a blended image.
rem %4 is optional output filename.
@rem
@rem Can also use:
@rem   btMASK_CURVE eg -sigmoidal-contrast 5x50%
@rem   btLINEAR if 1, composes in linear colourspace.
@rem
@rem Updated:
@rem   27-July-2022 for IM v7.
@rem

@rem FIXME: Input could be TIFF or MIFF with multiple images?


@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 bt


set N_COL=%2
set N_ROW=%3
if not "%4"=="" set OUTFILE=%4

if "%btLINEAR%"=="1" (
  set IN_COL_SP=-colorspace RGB -set colorspace sRGB
  set OUT_COL_SP=-set colorspace RGB -colorspace sRGB
) else (
  set IN_COL_SP=
  set OUT_COL_SP=
)

if not exist %INFILE% (
  echo %0: Can't find input file %INFILE%
  exit /B 1
)

set TMP_CMD=%TEMP%\bt_tmp_cmd.bat
del /q %TMP_CMD% 2>nul

set FNUM=0
set J=0
set I=0
for /F "tokens=*" %%F in (%INFILE%) do (
  rem set BT_IMAGE_!FNUM!=%%F
  set BT_IMAGE_!I!_!J!=%%F
  set /A FNUM+=1
  set /A I+=1
  if !I!==%N_COL% (
    set /A J+=1
    set I=0
  )
)

rem set BT_IMAGE

set /A COL_X_ROW=%N_COL%*%N_ROW%

if not %FNUM%==%COL_X_ROW% (
  echo %0: Bug: %FNUM% neq %COL_X_ROW%
  exit /B 1
)

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nWG=%%[fx:int(w/%N_COL%+0.5)]\nHG=%%[fx:int(h/%N_ROW%+0.5)]" ^
  %BT_IMAGE_0_0%`) do set %%L
if "%WW%"=="" exit /B 1

rem echo WW=%WW% HH=%HH% WG=%WG% HG=%HG%

set /A N_COLm1=%N_COL%-1
set /A N_ROWm1=%N_ROW%-1

echo %IMG7%magick >%TMP_CMD%

set FNUM=0
for /L %%j in (0,1,%N_ROWm1%) do (
  echo ^( >>%TMP_CMD%
  for /L %%i in (0,1,%N_COLm1%) do (
    echo ^( !BT_IMAGE_%%i_%%j! %IN_COL_SP% ^) >>%TMP_CMD%
    if not %%i==0 (
      set /A LEN=%%i*%WG%+%WG%/2
      echo ^( -size 1x%WG% gradient: -rotate 90 >>%TMP_CMD%
      if not "%btMASK_CURVE%"=="" echo %btMASK_CURVE% >>%TMP_CMD%
      echo -background #000 -gravity East -extent !LEN!x1 >>%TMP_CMD%
      echo -background #fff -gravity West -extent %WW%x1 >>%TMP_CMD%
      echo -scale "%WW%x%HH%^^^!" >>%TMP_CMD%
      echo ^) -composite >>%TMP_CMD%
    )
    set /A FNUM+=1
  )
  echo ^) >>%TMP_CMD%
  if not %%j==0 (
    set /A LEN=%%j*%HG%+%HG%/2
    echo ^( -size 1x%HG% gradient:#000-#fff >>%TMP_CMD%
    if not "%btMASK_CURVE%"=="" echo %btMASK_CURVE% >>%TMP_CMD%
    echo -background #000 -gravity South -extent 1x!LEN! >>%TMP_CMD%
    echo -background #fff -gravity North -extent 1x%HH% >>%TMP_CMD%
    echo -scale "%WW%x%HH%^^^!" >>%TMP_CMD%
    echo ^) -composite >>%TMP_CMD%
  )
)
if not "%OUT_COL_SP%"=="" echo %OUT_COL_SP% >>%TMP_CMD%
cPrefix /p0 /i%TMP_CMD% /r"^"
echo %OUTFILE% >>%TMP_CMD%

call %TMP_CMD%

call echoRestore

@endlocal & set btOUTFILE=%OUTFILE%
