rem From image %1, enlarges pixels into blocks.
rem Optional %2 is output filename.
@rem 
@rem Also regards:
@rem   bpSCALE muliplier for pixel size. Default 24.
@rem   bpGAP pixels gap between block pixels. Default 5.
@rem   bpGAPCOL colour for gap between pixels. Default None.
@rem   bpDO_SHAD 0=no shadows, 1=make shadows. Deafult 1.
@rem   bpDO_BACK_CHECK whether to put checkerboard behind blocks. Default 0.
@rem   bpDO_CIRCLE whether to draw a circle on a pixel. Default 0.
@rem     bpCIRC_X, bpCIRC_Y coords of pixel to draw circle on
@rem   bpDO_CROSS whether to draw a diagonal cross on a pixel. Default 0.
@rem     bpCROSS_X, bpCROSS_Y coords of pixel to draw cross on
@rem   bpTEXT Text to append at bottom. Default none.
@rem
@rem Updated:
@rem   28-July-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

call echoOffSave

call %PICTBAT%setInOut %1 bp

if "%bpSCALE%"=="" set bpSCALE=24
if "%bpGAP%"=="" set bpGAP=5
if "%bpGAPCOL%"=="" set bpGAPCOL=none
if "%bpDO_SHAD%"=="" set bpDO_SHAD=1
if "%bpDO_BACK_CHECK%"=="" set bpDO_BACK_CHECK=0

if not "%2"=="" set OUTFILE=%2


for /F %%i in ('%IMG7%magick identify ^
  -format "wd2=%%[fx:%bpSCALE%/2-0.5]\nhd2=%%[fx:%bpSCALE%/2-0.5]\nsm1=%%[fx:%bpSCALE%-1]\nsm2=%%[fx:%bpSCALE%-2]" ^
  xc:') do set %%i

if "%bpDO_CIRCLE%"=="1" (
  if "%bpCIRC_X%"=="" set CIRC_X=1
  if "%bpCIRC_Y%"=="" set CIRC_Y=1

  for /F %%i in ('%IMG7%magick identify ^
  -format "ox=%%[fx:!bpCIRC_X!*%bpSCALE%]\noy=%%[fx:!bpCIRC_Y!*%bpSCALE%]" ^
  xc:') do set %%i

  set CIRC=-fill none ^
    -stroke white -draw "translate !ox!,!oy! circle %wd2%,%hd2%,1,%hd2%" ^
    -stroke black -draw "translate !ox!,!oy! circle %wd2%,%hd2%,2,%hd2%"
) else (
  set CIRC=
)

if "%bpDO_CROSS%"=="1" (
  if "%bpCROSS_X%"=="" set CROSS_X=2
  if "%bpCROSS_Y%"=="" set CROSS_Y=1

  for /F %%i in ('%IMG7%magick identify ^
  -format "cx=%%[fx:!bpCROSS_X!*%bpSCALE%]\ncy=%%[fx:!bpCROSS_Y!*%bpSCALE%]" ^
  xc:') do set %%i

  set LINE=-stroke black -draw "translate !cx!,!cy! line 0,0,%sm1%,%sm1% line 0,%sm1%,%sm1%,0" ^
           -stroke white -draw "translate !cx!,!cy! line 1,0,%sm1%,%sm2% line 0,%sm2%,%sm2%,0"

) else (
  set LINE=
)

if not "%bpTEXT%"=="" (
  set TEXT=^( +clone ^
    -alpha off ^
    -fill white -colorize 100 ^
    -gravity South -pointsize 18 -fill Black ^
    -annotate 0 "%bpTEXT%" ^
    -bordercolor White ^
    -border 1 -trim +repage ^
    -border 4 +repage ^) ^
    -append +repage

) else (
  set TEXT=
)

if %bpDO_BACK_CHECK%==1 (
  set BACK_CHK=^( +clone ^
    -fill pattern:checkerboard -draw "color 0,0 reset" ^) ^
  +swap ^
  -compose Over -composite
) else (
  set BACK_CHK=
)

if "%bpDO_SHAD%"=="1" (
  set SHAD_COL=rgb^^^(0,0,75%%^^^)
  set SHAD_COL=black
  set /A SHAD_X=%bpGAP%/2
  set /A SHAD_Y=%bpGAP%/2

  set SHAD=^( +clone -background !SHAD_COL! -shadow 100x2+!SHAD_X!+!SHAD_Y! ^) ^
    +swap -background None -compose Over -layers merge
) else (
  set SHAD=
)

%IMG7%magick ^
  %INFILE% ^
  -scale %bpSCALE%00%% ^
  %CIRC% ^
  %LINE% ^
  -background %bpGAPCOL% ^
  -crop 0x%bpSCALE% -splice 0x%bpGAP% -append +repage -chop 0x%bpGAP% ^
  -crop %bpSCALE%x0 -splice %bpGAP%x0 +append +repage -chop %bpGAP%x0 ^
  -bordercolor %bpGAPCOL% -compose Copy -border %bpGAP% ^
  %SHAD% ^
  +repage ^
  %TEXT% ^
  %BACK_CHK% ^
  +repage ^
  %OUTFILE%

call echoRestore & set bpOUTFILE=%OUTFILE%
