rem Given %1 is an image file,
rem sets environment variable %2 to "camera value", Cv,
rem where Cv = log2 (T*F/A^2)
rem where T is exposure time in seconds,
rem       F is ISO film speed,
rem       A is aperture f-number.
@rem
@rem Updated:
@rem   24-August-2022 for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INFILE=%1
set ENVVAR=%2

set ShutterSpeed=
set Aperture=
set ISO=

for /F "usebackq" %%L in (`exiftool ^
  -S -ShutterSpeed -Aperture -ISO %INFILE% ^| ^
  sed -e "s/ //" -e s/:/^=/`) do set %%L

set Okay=1

if "%ShutterSpeed%"=="" (
  echo No ShutterSpeed
  set Okay=0
)

if "%Aperture%"=="" (
  echo No Aperture
  set Okay=0
)

if "%ISO%"=="" (
  echo No ISO
  set Okay=0
)

if %Okay%==0 exit /B 1

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "Cv=%%[fx:log((%ShutterSpeed%)*%ISO%/%Aperture%/%Aperture%)/log(2)]" ^
  xc:`) do set %%L

echo %0: %ShutterSpeed%s f/%Aperture% ISO %ISO% Cv=%Cv%

call echoRestore

@endlocal & set %ENVVAR%=%Cv%
