rem Given %1 is an image file,
rem appends CSV values to %2.
rem where Cv = log2 (T*F/A^2)
rem where T is exposure time in seconds,
rem       F is ISO film speed,
rem       A is aperture f-number.
@rem
@rem Updated:
@rem   3-October-2022 for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INFILE=%1
set OUTCSV=%2

if "%OUTCSV%"=="." set OUTCSV=
if "%OUTCSV%"=="" set OUTCSV=cvc.csv

set ShutterSpeed=
set Aperture=
set ISO=

for /F "usebackq" %%L in (`exiftool ^
  -S -ShutterSpeed -Aperture -LightValue -ISO %INFILE% ^| ^
  sed -e "s/ //" -e s/:/^=/`) do set %%L

set Okay=1

if "%ShutterSpeed%"=="" (
  echo No ShutterSpeed
  set Okay=0
)

if "%Aperture%"=="" (
  echo No Aperture
  set Okay=0
)

if "%ISO%"=="" (
  echo No ISO
  set Okay=0
)

if %Okay%==0 exit /B 1

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "Cv=%%[fx:log((%ShutterSpeed%)*%ISO%/%Aperture%/%Aperture%)/log(2)]" ^
  xc:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "Lv2=%%[fx:log(100/%ISO%*%Aperture%*%Aperture%/(%ShutterSpeed%))/log(2)]" ^
  xc:`) do set %%L

echo %ISO%, %ShutterSpeed%, %Aperture%, %LightValue%, %Lv2% >>%2

call echoRestore

@endlocal
