#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vsn_defines.h"

/*
    Find coordinate of the lightest pixel.

    If there is more than one with equal lightness,
    finds the one that is nearest the centroid of those pixels.
*/


// The next function corresponds in style to functions in enhance.c
// It takes one image, and returns a status.
//
static MagickBooleanType centroid(const Image *image,
  ExceptionInfo *exception)
{
  MagickBooleanType
    status;

  CacheView
    *image_view;

  ssize_t
    y;


  assert(image != (Image *) NULL);
  assert(image->signature == MAGICK_CORE_SIG);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  image_view = AcquireVirtualCacheView(image,exception);

  status = MagickTrue;

//  valLightest = 0;
//  xLightest = yLightest = 0;
//  int nFound = 0;

//  int sigX=0, sigY=0;

  long double sigW = 0.0, sigWX = 0.0, sigWY = 0.0;

  for (y=0; y < (ssize_t) image->rows; y++)
  {
    VIEW_PIX_PTR const *p;

    register ssize_t x;

    long double val;

    if (status == MagickFalse) continue;

    p=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
    if (p == (const VIEW_PIX_PTR *) NULL) status=MagickFalse;

    for (x=0; x < (ssize_t) image->columns; x++) {
      val = GetPixelIntensity(image, p);

      sigW += val;
      sigWX += x * val;
      sigWY += y * val;

      p += Inc_ViewPixPtr (image);
    }
  }

  if (!status) return MagickFalse;

  int precision = GetMagickPrecision();

  if (sigW > 0) {
    fprintf (stderr, "centroid: %.*g,%.*g\n",
             precision, (double)(sigWX/sigW),
             precision, (double)(sigWY/sigW));
  } else {
    fprintf (stderr, "centroid: %.*g,%.*g\n",
             precision, (double)((image->columns-1.0)/2.0),
             precision, (double)((image->rows-1.0)/2.0));
  }

  return MagickTrue;
}


ModuleExport size_t centroidImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  MagickBooleanType
    status;

  (void) argc;
  (void) argv;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    status = centroid(image, exception);

    if (status == MagickFalse)
      continue;
  }

  assert (status == MagickTrue);

  return(MagickImageFilterSignature);
}
