#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <ctype.h>
#include "vsn_defines.h"
#include "centsmcrop.inc"


typedef struct {
  MagickBooleanType
    verbose;
} cscT;


static void usage (void)
{
  printf ("Usage: -process 'centsmcrop [OPTION]...'\n");
  printf ("Crops both images to dimensions of the smallest.\n");
  printf ("\n");
  printf ("  v, verbose          write text information to stdout\n");
  printf ("\n");
}


static MagickBooleanType IsArg (char * pa, char * ShtOpt, char * LongOpt)
{
  if ((LocaleCompare(pa, ShtOpt)==0) || (LocaleCompare(pa, LongOpt)==0))
    return MagickTrue;

  return MagickFalse;
}


static MagickBooleanType menu(
  const int argc,
  const char **argv,
  cscT * pCsc
)
// Returns MagickTrue if okay.
{
  int
    i;

  MagickBooleanType
    status;

  status = MagickTrue;

  pCsc->verbose = MagickFalse;

  for (i=0; i < argc; i++) {
    char * pa = (char *)argv[i];

    if (IsArg (pa, "v", "verbose")==MagickTrue) {
      pCsc->verbose = MagickTrue;
    } else {
      fprintf (stderr, "centsmcrop: ERROR: unknown option [%s]\n", pa);
      status = MagickFalse;
    }
  }

  if (pCsc->verbose) {
    fprintf (stderr, "centsmcrop options:");

    if (pCsc->verbose) fprintf (stderr, "  verbose");

    fprintf (stderr, "\n");
  }

  if (status == MagickFalse)
    usage ();

  return (status);
}





ModuleExport size_t centsmcropImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image1,
    *image2;

  cscT
    csc;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  // Replace each pair of images with atan2(u,v).
  // The images must be the same size.

  int ListLen = (int)GetImageListLength(*images);
  if (ListLen !=2) {
    fprintf (stderr, "centsmcrop needs 2 images\n");
    return (-1);
  }

  MagickBooleanType status = menu (argc, argv, &csc);
  if (status == MagickFalse)
    return (-1);

  image1 = (*images);
  image2 = GetNextImageInList (image1);

  if (!CentralSmallestCrop (&image1, &image2, csc.verbose, exception))
    return -1;

  // Replace messes up the images pointer. Make it good:
  *images = GetFirstImageInList (image1);

  return (MagickImageFilterSignature);
}
