#ifndef CENTSMCROP_INC
#define CENTSMCROP_INC

#include "resetpage.inc"
#include "cropchk.inc"


static MagickBooleanType CentralSmallestCrop (
  Image ** iA,
  Image ** iB,
  MagickBooleanType verbose,
  ExceptionInfo *exception)
// Returns both images cropped, if required, to same dimensions,
// being the smallest of the inputs in each direction.
{
  ssize_t smX = (*iA)->columns;
  if (smX > (*iB)->columns) smX = (*iB)->columns;

  ssize_t smY = (*iA)->rows;
  if (smY > (*iB)->rows) smY = (*iB)->rows;

  if (verbose) fprintf (stderr,
    "CentralSmallestCrop: %lix%li %lix%li %lix%li\n",
    (*iA)->columns, (*iA)->rows,
    (*iB)->columns, (*iB)->rows,
    smX, smY);

  if ((*iA)->columns > smX || (*iA)->rows > smY) {
    RectangleInfo crpRect;
    crpRect.width  = smX;
    crpRect.height = smY;
    crpRect.x = ((*iA)->columns - smX) / 2;
    crpRect.y = ((*iA)->rows    - smY) / 2;

    ResetPage (*iA);
    Image * smA = CropCheck (*iA, &crpRect, exception);
    if (!smA) return MagickFalse;
    ResetPage (smA);

    ReplaceImageInList (iA, smA);
  }

  if ((*iB)->columns > smX || (*iB)->rows > smY) {
    RectangleInfo crpRect;
    crpRect.width  = smX;
    crpRect.height = smY;
    crpRect.x = ((*iB)->columns - smX) / 2;
    crpRect.y = ((*iB)->rows    - smY) / 2;

    ResetPage (*iB);
    Image * smB = CropCheck (*iB, &crpRect, exception);
    if (!smB) return MagickFalse;
    ResetPage (smB);

    ReplaceImageInList (iB, smB);
  }
  return MagickTrue;
}

#endif
