rem %1 Prefix for first input chainset.
rem %2 Prefix for second input chainset.
rem %3 Prefix for output chainset.
@rem
@rem Assumes frame names are in lexicographical order.

@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave


set InPref1=%1
set InPref2=%2
set OutPref=%3

call %PICTBAT%chainRdEnv %InPref1%
if ERRORLEVEL 1 exit /B 1

call %PICTBAT%chainRdEnv %InPref2%
if ERRORLEVEL 1 exit /B 1

rem FIXME? Maybe we should check inputs have same directory and same extension.

set cnfInPref=%~n1
set cnfOutPref=%~n3

call %PICTBAT%chainClean %OutPref%

rem Both inputs should be pivoted around first.
rem So the _bwd lists contain one entry each, and can be ignored.

rem Check the backwards lists have exactly one entry.

call readArray %InPref1%_bwd.lis InBwd
if ERRORLEVEL 1 exit /B 1
if %InBwd_CNT% NEQ 1 (
  echo %0: %InPref1%_bwd.lis doesn't have exactly one entry.
  exit /B 1
)

call readArray %InPref2%_bwd.lis InBwd
if ERRORLEVEL 1 exit /B 1
if %InBwd_CNT% NEQ 1 (
  echo %0: %InPref2%_bwd.lis doesn't have exactly one entry.
  exit /B 1
)

rem Read the input frame names.

call readArray %InPref1%_fwd.lis InImgs1
if ERRORLEVEL 1 exit /B 1

call readArray %InPref2%_fwd.lis InImgs2
if ERRORLEVEL 1 exit /B 1

:: Merge the frame name lists into FrmFrom.

set ndx1=0
set ndx2=0
set ndxOut=0

set End1=0
set End2=0

:: FrmFrom is:
::   n is frame number
::   FrmFrom[n] is
::     1 (frame was just in input 1), or
::     2 (frame was just in input 2), or
::     3 (frame was in both inputs).
::   FrmFrom[n][f] f is the source, 1 or 2.
::   FrmFrom[n][f] is index of frame in the input chainset.

:loop

if %ndx1% GTR %InImgs1_CNTm1% (
  set End1=1
)

if %ndx2% GTR %InImgs2_CNTm1% (
  set End2=1
)

if %End1%==1 if %End2%==1 goto break

if %End1%==1 (
  echo from 2: %ndx1% %ndx2% %ndxOut%
  set FrmFrom[!ndxOut!]=2
  set FrmFrom[!ndxOut!][1]=
  set FrmFrom[!ndxOut!][2]=%ndx2%
  set OutImgs[!ndxOut!]=!InImgs2[%ndx2%]!
  set /A ndx2+=1
  set /A ndxOut+=1
) else if %End2%==1 (
  echo from 1: %ndx1% %ndx2% %ndxOut%
  set FrmFrom[!ndxOut!]=1
  set FrmFrom[!ndxOut!][1]=%ndx1%
  set FrmFrom[!ndxOut!][2]=
  set OutImgs[!ndxOut!]=!InImgs1[%ndx1%]!
  set /A ndx1+=1
  set /A ndxOut+=1
) else if !InImgs1[%ndx1%]! LSS !InImgs2[%ndx2%]! (
  echo from 1: %ndx1% %ndx2% %ndxOut%
  set FrmFrom[!ndxOut!]=1
  set FrmFrom[!ndxOut!][1]=%ndx1%
  set FrmFrom[!ndxOut!][2]=
  set OutImgs[!ndxOut!]=!InImgs1[%ndx1%]!
  set /A ndx1+=1
  set /A ndxOut+=1
) else if !InImgs1[%ndx1%]! GTR !InImgs2[%ndx2%]! (
  echo from 2: %ndx1% %ndx2% %ndxOut%
  set FrmFrom[!ndxOut!]=2
  set FrmFrom[!ndxOut!][1]=
  set FrmFrom[!ndxOut!][2]=%ndx2%
  set OutImgs[!ndxOut!]=!InImgs2[%ndx2%]!
  set /A ndx2+=1
  set /A ndxOut+=1
) else (
  echo from both: %ndx1% %ndx2% %ndxOut%
  set FrmFrom[!ndxOut!]=3
  set FrmFrom[!ndxOut!][1]=%ndx1%
  set FrmFrom[!ndxOut!][2]=%ndx2%
  set OutImgs[!ndxOut!]=!InImgs1[%ndx1%]!
  set /A ndx1+=1
  set /A ndx2+=1
  set /A ndxOut+=1
)
goto loop

:break

set FrmFrom_CNT=%ndxOut%
set /A FrmFrom_CNTm1=%ndxOut%-1

rem set FrmFrom

echo %0: number of frames: %ndxOut%

:: Write the frame name lists.

( echo !OutImgs[0]! )> %OutPref%_bwd.lis

(
  for /L %%I in (0,1,%FrmFrom_CNTm1%) do (
    echo !OutImgs[%%I]!
  )
) >%OutPref%_fwd.lis

(
  for /L %%I in (0,1,%FrmFrom_CNTm1%) do (
    echo !OutImgs[%%I]!
  )
) >%OutPref%_all.lis

:: Write the coordinate lists.

:: First, for the features that came from Input 1.
call %PICTBAT%csaNumFeat %InPref1%
set nFeat1m1=!%cnfPref%.NumFeatM1!
for /L %%I in (0,1,%nFeat1m1%) do (
  echo pcI=%%I
  echo %0: Copy from %InPref1% feature %%I

  call readArray %InPref1%_coords_f%%I_fwd.lis inCoords
  if ERRORLEVEL 1 exit /B 1

  rem Loop through the frames.
  for /L %%F in (0,1,%FrmFrom_CNTm1%) do (
    if FrmFrom[%%F]==2 (
      set coords=missing,missing
    ) else (
      set coords=!inCoords[%%F]!
    )
    rem echo frame %%F !coords!
    if %%F==0 (
      ( echo !coords! )>>%OutPref%_coords_f%%I_bwd.lis
    )
    ( echo !coords! )>>%OutPref%_coords_f%%I_fwd.lis
  )
)

set Offset=!%cnfPref%.NumFeat!

:: Now for the features that came from Input 2.

call %PICTBAT%csaNumFeat %InPref2%
set nFeat2m1=!%cnfPref%.NumFeatM1!
for /L %%I in (0,1,%nFeat2m1%) do (
  set /A NewNum=%%I+%Offset%
  echo %0: Copy from %InPref2% feature %%I becomes !NewNum!

  call readArray %InPref2%_coords_f%%I_fwd.lis inCoords
  if ERRORLEVEL 1 exit /B 1

  rem Loop through the frames.
  for /L %%F in (0,1,%FrmFrom_CNTm1%) do (
    if FrmFrom[%%F]==1 (
      set coords=missing,missing
    ) else (
      set coords=!inCoords[%%F]!
    )
    rem echo frame %%F !coords!
    if %%F==0 (
      ( echo !coords! )>>%OutPref%_coords_f!NewNum!_bwd.lis
    )
    ( echo !coords! )>>%OutPref%_coords_f!NewNum!_fwd.lis
  )
)

set %cnfOutPref%.InFrames=!%cnfInPref%.InFrames!
set %cnfOutPref%.InExt=!%cnfInPref%.InExt!
set /A %cnfOutPref%.NumFeat=%nFeat1m1%+%nFeat1m1%+2
set /A %cnfOutPref%.NumFeatM1=%nFeat1m1%+%nFeat1m1%+1
@call %PICTBAT%chainWrEnv %OutPref%
if ERRORLEVEL 1 exit /B 1

set InFrm=!%cnfInPref%.InFrames!
set InExt=!%cnfInPref%.InExt!

call echoRestore

@endlocal & set %cnfOutPref%.InFrames=%InFrm%& set %cnfOutPref%.InExt=%InExt%

@exit /B 0

