rem %1 is prefix for input environment variables:
rem   prefix.InFrames directory containing input frames
rem   prefix.InExt extension of those frames, eg jpg or png or miff.
rem %2 is text file containing a list of input images that are in %csa.InFrames%
rem %3,%4 is coords of a pixel in the first image in list
rem %5 output text file
rem %6 is WxH dimensions of subimage to be found eg 21x21
rem %7 is WxH of area to be searched eg 135x135
rem   %7 must be at least as large as %6.
rem %8 is threshold, 0.0 to 1.0 [default 1.0].
rem   If RMSE greater than threshold, output value is "missing,missing".
rem %9 is bad_threshold, 0.0 to 1.0 [default 1.0].
rem   If RMSE greater than bad_threshold, all following output values are "missing,missing".
@rem
@rem Writes coords of corresponding pixel in all images to output text file.
@rem First entry in output will be %2,%3.
@rem
@rem This tracks the movement of the centres of the feature.
@rem
@rem Also requires:
@rem   csa.InFrames directory containing input frames
@rem   csa.InExt extension of input frames
@rem
@rem Also uses:
@rem   csSUPSAMP factor for supersampling, eg 1 or 2 or 4 or 100.
@rem   csDEBUG if non-blank, prefix for debug files:
@rem     copies cropped areas from the searched input images, drawing a red rectangle.

@if "%4"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set cnfPref=%~n1

if "!%cnfPref%.InFrames!"=="" (
  echo %0: %cnfPref%.InFrames not set
  exit /B 1
)

set INIMGS=%2
set InX=%3
set InY=%4

set OUTTEXT=%5
if "%OUTTEXT%"=="." set OUTTEXT=
if "%OUTTEXT%"=="" set OUTTEXT=cs_coordlist.txt

set SUB_WH=%6
if "%SUB_WH%"=="." set SUB_WH=
if "%SUB_WH%"=="" set SUB_WH=21x21

set SRCH_WH=%7
if "%SRCH_WH%"=="." set SRCH_WH=
if "%SRCH_WH%"=="" set SRCH_WH=135x135

set THRESH=%8
if "%THRESH%"=="." set THRESH=
if "%THRESH%"=="" set THRESH=1

set BAD_THRESH=%9
if "%BAD_THRESH%"=="." set BAD_THRESH=
if "%BAD_THRESH%"=="" set BAD_THRESH=1

echo %0: THRESH=%THRESH% BAD_THRESH=%BAD_THRESH%

call parseXxY 0 0 %SUB_WH%
set SUB_W=%pxxyX%
set SUB_H=%pxxyY%

call parseXxY 0 0 %SRCH_WH%
set SRCH_W=%pxxyX%
set SRCH_H=%pxxyY%

set /A SUB_W2=(%SUB_W%-1)/2
set /A SUB_H2=(%SUB_H%-1)/2
set /A SRCH_W2=(%SRCH_W%-1)/2
set /A SRCH_H2=(%SRCH_H%-1)/2

set TMP_CROP=\temp\tmp_crop.miff

if "%csSUPSAMP%"=="" set csSUPSAMP=1

set SSDIV=%csSUPSAMP%

if %csSUPSAMP%==1 (
  rem set SSRES=
  set SUBSAMP=
) else (
  rem set /A SSPC=%csSUPSAMP%*100
  rem set SSRES=-resize !SSPC!%%
  set SUBSAMP=superSample %csSUPSAMP%
)

if "%csDEBUG%"=="" (
  set WR_DBG=
) else (
  set WR_DBG=+write %TMP_CROP%
)

echo WR_DBG=%WR_DBG%

echo !InX!,!InY!>%OUTTEXT%

set HasFirst=0
set PrevFile=
set NUM=1
set NUM_MISS=0
set LARGEST_DIFF=0
set ALL_MISS=0

for /F "tokens=*" %%F in (%INIMGS%) do (
  rem echo %%F

  if !HasFirst!==0 (
    if not "%csDEBUG%"=="" (
      echo Writing first debug
      %IMG7%magick ^
        !%cnfPref%.InFrames!%%F.!%cnfPref%.InExt! ^
        -fill None -stroke Red ^
        -draw "translate %%[fx:!InX!-%SUB_W2%],%%[fx:!InY!-%SUB_H2%] rectangle 0,0,%SUB_W%,%SUB_H%" ^
        -crop %SRCH_W%x%SRCH_H%+%%[fx:!InX!-%SRCH_W2%]+%%[fx:!InY!-%SRCH_H2%] +repage ^
        %csDEBUG%_000000.png
    )

    set PrevFile=%%F
    set HasFirst=1
  ) else (
    call :srchLink !%cnfPref%.InFrames!!PrevFile! !%cnfPref%.InFrames!%%F !InX! !InY!
    if ERRORLEVEL 1 exit /B 1

    set /A NUM+=1

    echo !ThisX!,!ThisY!>>%OUTTEXT%

    set PrevFile=%%F
  )
)

echo %0: NUM=%NUM% NUM_MISS=%NUM_MISS% LARGEST_DIFF=%LARGEST_DIFF%

call echoRestore

@endlocal & set csNUM=%NUM%& set csNUM_MISS=%NUM_MISS%& set csLARGEST_DIFF=%LARGEST_DIFF%

@exit /B 0

::---------------------------------------------------------

rem Search one link in a search chain.

rem %1 is an image from which a rectangle will be cropped,
rem %2 is an image to be searched.
rem %3,%4 coords of a point in %1
rem Returns InX and InY, corresponding point in %2.

:srchLink

echo %0: %1 %2 %3 %4

set IMG1=%1.!%cnfPref%.InExt!
set IMG2=%2.!%cnfPref%.InExt!

set GuessX=%3
set GuessY=%4

rem set /A SX=%GuessX%-%SRCH_W2%
rem set /A SY=%GuessY%-%SRCH_H2%

for /F "usebackq" %%L in (`%IM7DEV%magick identify ^
  -format "SX=%%[fx:%GuessX%-%SRCH_W2%]\nSY=%%[fx:%GuessY%-%SRCH_H2%]\n" ^
  xc:`) do set %%L

if %SX% LSS 0 (
  echo %0: Error: SX=%SX%
  exit /B 1
)

if %SY% LSS 0 (
  echo %0: Error: SY=%SY%
  exit /B 1
)

set COMP_XW=

for /F "usebackq tokens=1-3 delims=@, " %%A in (`%IM7DEV%magick ^
  ^( %IMG2% ^
     -crop %SRCH_W%x%SRCH_H%+%SX%+%SY% +repage ^
     %WR_DBG% ^
  ^) ^
  ^( %IMG1% ^
     -crop %SUB_W%x%SUB_H%+%%[fx:%GuessX%-%SUB_W2%]+%%[fx:%GuessY%-%SUB_H2%] +repage ^
  ^) ^
  -process 'srchimg %SUBSAMP% noCrop f stdout' ^
  NULL:`) do (
  set DIFF=%%A
  set COMP_XW=%%B
  set COMP_YW=%%C
)

if "%COMP_XW%"=="" exit /B 1

rem for /F "usebackq" %%L in (`%IM7DEV%magick identify ^
rem   -format "COMP_XW=%%[fx:%COMP_XW%/%SSDIV%]\nCOMP_YW=%%[fx:%COMP_YW%/%SSDIV%]\n" ^
rem   xc:`) do set %%L

set FMT=^
MISSING=%%[fx:%DIFF%^>%THRESH%?1:0]\n^
BAD_MISSING=%%[fx:%DIFF%^>%BAD_THRESH%?1:0]\n^
LARGEST_DIFF=%%[fx:%DIFF%^>%LARGEST_DIFF%?%DIFF%:%LARGEST_DIFF%]\n

for /F "usebackq" %%L in (`%IM7DEV%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

if %BAD_MISSING%==1 set ALL_MISS=1

if not "%csDEBUG%"=="" (
  set LZ=000000%NUM%
  set LZ=!LZ:~-6!

  %IMG7%magick ^
    %TMP_CROP% ^
    -fill None -stroke Red ^
    -draw "translate %COMP_XW%,%COMP_YW% rectangle 0,0,%SUB_W%,%SUB_H%" ^
    %csDEBUG%_!LZ!.png
)

if %ALL_MISS%==1 (
  set ThisX=missing
  set ThisY=missing
  set /A NUM_MISS+=1
) else if %MISSING%==1 (
  set ThisX=missing
  set ThisY=missing
  set /A NUM_MISS+=1
) else (
  rem set /A InX=%SX%+%COMP_XW%+%SUB_W2%
  rem set /A InY=%SY%+%COMP_YW%+%SUB_H2%

  for /F "usebackq" %%L in (`%IM7DEV%magick identify ^
    -format "InX=%%[fx:%SX%+%COMP_XW%+%SUB_W2%]\nInY=%%[fx:%SY%+%COMP_YW%+%SUB_H2%]\n" ^
    xc:`) do set %%L

  set ThisX=!InX!
  set ThisY=!InY!
)

echo DIFF=%DIFF% InX=%InX% InY=%InY% MISSING=%MISSING% BAD_MISSING=%BAD_MISSING%

exit /B 0
