rem %1 is input text file containing list of coordinate pairs,
rem   possibly with missing values.
rem %2 is output text file containing list of coordinate pairs,
rem   with missing values replaced with numbers.
rem   Default: as %1.

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INFILE=%1
set OUTFILE=%2

if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=%INFILE%

rem Linear interpolation and extrapolation:
rem
rem Read into arrays of X and Y.
rem Walk the array.
rem At transitions from known to missing,
rem   note the index and last known value.
rem At transitions from missing to known,
rem   we can calculate the missing values.
rem
rem (If any missing at start or end, we need nearest two known values.)
rem
rem Write the new arrays.
rem
rem chainSrch should not write "missing" at start.
rem "missing" at end maybe should be treated as overlapping chain.

call readArrayXY %INFILE% csm
if ERRORLEVEL 1 exit /B 1

set csm

if %csm_CNT%==0 (
  echo %0: csm_CNT==0
  exit /B 1
)

:: state: 0=start, 1=known, 2=missing
set state=0

:: TR is transition: 0=no change, +1=becomes known, -1 becomes missing.

set PrevKnownNdx=0
for %%J in (X Y) do (
  for /L %%I in (0,1,%csm_CNTm1%) do (
    set known=1
    if !csm[%%I][%%J]!==missing set known=0
    set TR=0
    if !state!==0 (
      if !known!==0 (
        set state=2
      ) else (
        set state=1
      )
    ) else (
      if !state!==1 if !known!==0 set TR=-1
      if !state!==2 if !known!==1 set TR=1
    )
    if !TR!==-1 (
      set state=2
      set /A PrevKnownNdx=%%I-1
    )
    if !TR!==1 (
      set state=1
      echo PrevKnownNdx=!PrevKnownNdx!
      set /A FirstMiss=!PrevKnownNdx!+1
      set /A LastMiss=%%I-1
      set /A nDiv=%%I-!PrevKnownNdx!
      call :getVal !PrevKnownNdx! %%J
      set Val0=!Val!
      set Val1=!csm[%%I][%%J]!
      set /A DiffVal=!Val1!-!Val0!
      echo Fill missing from !FirstMiss! to !LastMiss! n=!nDiv!
      echo Val0=!Val0!  Val1=!Val1!
      for /L %%A in (!FirstMiss!,1,!LastMiss!) do (
        set /A "VAL=!Val0!+(%%A-!FirstMiss!+1)*!DiffVal!/!nDiv!"
        echo Fill %%A  !VAL!
        set csm[%%A][%%J]=!VAL!
      )
    )
    echo %%J %%I !csm[%%I][%%J]!  !state!  !TR!
  )
)

set csm

(
  for /L %%I in (0,1,%csm_CNTm1%) do (
    echo !csm[%%I][X]!,!csm[%%I][Y]!
  )
) >%OUTFILE%

type %OUTFILE%

call echoRestore

@endlocal

exit /B 0

::-------------------------------------
:: Subroutines

:getVal
echo getVal %1 %2
set Val=!csm[%1][%2]!
exit /B 0