rem %1 is prefix for input text files containing a list of images, _fwd.lis and _bkwd.lis
rem   (as given to csFiles.bat)
rem   Also used for output text files, named prefix_coords_F0_fwd, etc.
rem %2 is input text file containing list of coordinate pairs in central image.
rem %3 is WxH dimensions of subimage to be found eg 21x21
rem %4 is WxH of area to be searched eg 135x135
rem   %4 must be at least as large as %3.
rem %5 is threshold, 0.0 to 1.0.
rem   If RMSE greater than threshold, output value is "missing,missing".
rem %6 is bad_threshold, 0.0 to 1.0 [default 1.0].
rem   If RMSE greater than bad_threshold, all following output values are "missing,missing".
@rem
@rem Also uses:
@rem   csmDEBUG Prefix for debug files. Default: blank, so no debug.
@rem


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INPREF=%1
set INCOORDS=%2

@call %PICTBAT%chainRdEnv %INPREF%
if ERRORLEVEL 1 exit /B 1

set OUTPREF=%INPREF%

set cnfPref=%~n1

set SUB_WH=%3
if "%SUB_WH%"=="" set SUB_WH=.

set SRCH_WH=%4
if "%SRCH_WH%"=="" set SRCH_WH=.

set THRESH=%5
if "%THRESH%"=="" set THRESH=.

set BAD_THRESH=%6
if "%BAD_THRESH%"=="" set BAD_THRESH=.

set IN_BWD=%INPREF%_bwd.lis
set IN_FWD=%INPREF%_fwd.lis

if not exist %IN_BWD% (
  echo %0: File %IN_BWD% doesn't exist.
  exit /B 1
)

if not exist %IN_FWD% (
  echo %0: File %IN_FWD% doesn't exist.
  exit /B 1
)

set NUM=0
set csmNUM=0
set csmNUM_MISS=0
set csmLARGEST_DIFF=0
for /F "tokens=1,2 delims=, " %%X in (%INCOORDS%) do (
  echo %%X,%%Y

  if "%csmDEBUG%"=="" (
    set csDEBUG=
  ) else (
    set csDEBUG=%csmDEBUG%_!NUM!
  )

  call %PICTBAT%chainSrch ^
    %cnfPref% ^
    %IN_FWD% %%X %%Y %OUTPREF%_coords_f!NUM!_fwd.lis ^
    %SUB_WH% %SRCH_WH% %THRESH% %BAD_THRESH%
  if ERRORLEVEL 1 exit /B 1

  set /A csmNUM+=!csNUM!
  set /A csmNUM_MISS+=!csNUM_MISS!

  for /F "usebackq" %%L in (`%IM7DEV%magick identify ^
    -format "csmLARGEST_DIFF=%%[fx:!csLARGEST_DIFF!>!csmLARGEST_DIFF!?!csLARGEST_DIFF!:!csmLARGEST_DIFF!]\n" ^
    xc:`) do set %%L

  call %PICTBAT%chainSrch ^
    %cnfPref% ^
    %IN_BWD% %%X %%Y %OUTPREF%_coords_f!NUM!_bwd.lis ^
    %SUB_WH% %SRCH_WH% %THRESH% %BAD_THRESH%
  if ERRORLEVEL 1 exit /B 1

  set /A csmNUM+=!csNUM!
  set /A csmNUM_MISS+=!csNUM_MISS!

  for /F "usebackq" %%L in (`%IM7DEV%magick identify ^
    -format "csmLARGEST_DIFF=%%[fx:!csLARGEST_DIFF!>!csmLARGEST_DIFF!?!csLARGEST_DIFF!:!csmLARGEST_DIFF!]\n" ^
    xc:`) do set %%L

  set /A NUM+=1
)

echo %0: NUM=%NUM% csmNUM=%csmNUM% csmNUM_MISS=%csmNUM_MISS% csmLARGEST_DIFF=%csmLARGEST_DIFF%

set %cnfPref%.NumFeat=%NUM%
set /A %cnfPref%.NumFeatM1=%NUM%-1
@call %PICTBAT%chainWrEnv %INPREF%
if ERRORLEVEL 1 exit /B 1

call echoRestore

@endlocal & set %cnfPref%.NumFeat=%NUM%& set /A %cnfPref%.NumFeatM1=%NUM%-1& set csmNUM=%csmNUM%& set csmNUM_MISS=%csmNUM_MISS%& set csmLARGEST_DIFF=%csmLARGEST_DIFF%
