/* cm_coffeemug.inc
 * 2015 Alan Gibson (snibgo applied arts).
 *
 * Derived from:
 *   coffeemug.pov version 2.0 by Richard Callwood III.
 *
 */

#include "strings.inc"
#include "colors.inc"
#include "woods.inc"
#include "shapes.inc"
#include "glass.inc"
#include "coffeemug.inc"

#ifndef (cm_extra_inc) #declare cm_extra_inc = 0; #end

#if (cm_extra_inc = 1)
#include "coffeemug_extra.inc"
#end

#ifndef (Ph) #declare Ph = off; #end // photons
#ifndef (Rad) #declare Rad = off; #end // radiosity
#ifndef (Soft) #declare Soft = off; #end // soft shadows

#ifndef (Texture) #declare Texture = 1; #end

#ifndef (FocalBlur) #declare FocalBlur = 0; #end

#declare C_SKY = rgb 0.01;

#ifndef (MugColR) #declare MugColR = 1; #end
#ifndef (MugColG) #declare MugColG = 1; #end
#ifndef (MugColB) #declare MugColB = 1; #end

#declare MugCol = rgb <MugColR, MugColG, MugColB>;

#default // for wood texture only
{ finish
  { reflection { 0 0.2 fresnel } conserve_energy
    ambient (Rad? 0: C_SKY)
  }
}

#ifndef (MugRad) #declare MugRad = 1.5; #end
#ifndef (MugHt) #declare MugHt = 3.75; #end

#ifndef (Image_File) #declare Image_File = "cm_3.png"; #end
#ifndef (Image_Mult) #declare Image_Mult = 1; #end

//=============================== ENVIRONMENT ==================================

global_settings
{ assumed_gamma 1
  #if (Rad)
    radiosity
    { count 100
      error_bound 0.5
      pretrace_start 0.08
      pretrace_end 0.01
      recursion_limit 2
    }
  #end
  #if (Ph) photons { spacing 0.005 autostop 0 } #end
  max_trace_level 10
}

camera
// { location <1, 13.5, -18>
{ location <2, 10, -18>
  look_at <1, MugHt/2, 0>
  #if (FocalBlur=1)
    aperture 0.7
    blur_samples 20
    focal_point <0, MugHt/2, -MugRad>
  #end
  angle 20
}

#declare LIGHT_POSITION = <-0.5, 1, -1> * 60;
light_source
{ LIGHT_POSITION, rgb 1.2
  fade_power 2 fade_distance vlength (LIGHT_POSITION)
  spotlight point_at <0, MugHt/2, 0> radius 5 falloff 10
  #if (Soft) area_light 2.5 * x, 2.5 * z, 9, 9 adaptive 1 circular orient #end
}

//----- A little ambience -----
background { C_SKY }

//----- Estimate ambient light -----
#declare Illumination = vdot (vnormalize (LIGHT_POSITION), y);
#declare c_Table = rgb <0.295, 0.163, 0.118>; // sampled from T_Wood12
#declare LIT = 0.8; // estimate of contribution of spotlit portion of the table
#declare DIFFUSE = 0.6; // this scene's diffuse (which is the POV-Ray default)
#declare Ambience =
( Rad?
  0:
  (C_SKY + c_Table * (C_SKY + Illumination * DIFFUSE * LIT)) / 2
);
#default { finish { ambient Ambience } }

//----- The table -----
#declare Floor =
plane {
  y, 0
  texture {
    T_Wood12
    rotate <0, 5, 0>
    scale 2
  }
  interior { ior 1.47 }
}

#declare Chq_Floor =
plane {
  y, 0
  pigment {
    checker color Black, color White
  }
}

#declare White_Floor =
plane {
  y, 0
  pigment { rgb 1 }
}

#declare Black_Floor =
plane {
  y, 0
  pigment { rgb 0 }
}

//=============================== MUG TEXTURES =================================

//------------------- Texture elements ---------------------

#declare Scale_Image = 1.4 * Image_Mult;

#declare p_Image = pigment
{// Note: Image_File should be gamma=1.
  image_map { png Image_File once interpolate 2 }
  translate -0.5 // must be centered on the origin
  scale Scale_Image * <6, 2, 1>
}

#declare o_Pic_area = box {-Scale_Image * <3, 1, 1>, Scale_Image * <3, 1, 1>}

#declare f_Gloss = finish
{ specular 4.24 roughness 0.002
  reflection { 0.05 0.4 fresnel } conserve_energy
}

#declare m_Glass = material
{
  texture {
    pigment { Col_Glass_Green }
    finish { F_Glass5 }
  }
  interior { I_Glass1 }
}


//-------------------- Final textures ----------------------

#declare p_Design = CoffeeMug_Pigment
( MugRad, 0, MugHt, MUG_HEIGHT, o_Pic_area, MugCol, p_Image
)


//============================= COFFEE MATERIAL ================================

#declare m_Coffee = material
{ texture
  { pigment { rgbf <0.9, 0.7, 0.4, 1> }
    finish
    { specular 0.64 roughness 0.01
      reflection { 0, 1 fresnel } conserve_energy
    }
  }
  interior
  { ior 1.34
    media
    { absorption <0.25, 0.4, 0.75>
      scattering { 3, rgb 1 }
    }
  }
}

#declare m_Water = material
{ texture
  { pigment { rgbt <0.9, 0.9, 0.9, 1> }
    finish
    { specular 0.64 roughness 0.01
      reflection { 0, 1 fresnel } conserve_energy
    }
  }
  interior {
    ior 1.333
    fade_distance 5
    fade_power 1001.0
    fade_color <0.02, 0.20, 0.06>
  }
}

//================================ SHOWPIECE ===================================

#declare Coffee_Mug =
object
{ CoffeeMug_Mug (MugRad, MugHt, MUG_HEIGHT, no)
  pigment { p_Design }
  finish { f_Gloss }
  interior { ior 1.6 }
  #if (Ph) photons { target reflection on } #end
}

#declare Coffee_Mug_Trans =
object
{ CoffeeMug_Mug (MugRad, MugHt, MUG_HEIGHT, yes)
  pigment { rgbt <1, 0.5, 0.5, 1> }
  finish { f_Gloss }
  interior { ior 1.6 }
  #if (Ph) photons { target reflection on } #end
}

#declare Coffee_Mug_Plain =
object
{ CoffeeMug_Mug (MugRad, MugHt, MUG_HEIGHT, yes)
  pigment { rgb <MugColR, MugColG, MugColB> }
  finish { f_Gloss }
}

#declare Bev_Coffee =
object {
  CoffeeMug_Beverage (MugRad, 0, 8, MUG_INCH, 0.001)
  hollow
  material { m_Coffee }
}

#declare Bev_Water =
object {
  CoffeeMug_Beverage (MugRad, 0, 8, MUG_INCH, 0.001)
  hollow
  material { m_Water }
  #if (Ph) photons { target refraction on } #end
}

#declare MugImage =
object
{ CoffeeMug_Mug (MugRad, MugHt, MUG_HEIGHT, yes)
  CoffeeMug_Pigment
  ( MugRad, 0, MugHt, MUG_HEIGHT, o_Pic_area, rgbt <MugColR, MugColG, MugColB,1>, p_Image
  )
  finish { f_Gloss }
}

// end of coffeemug.pov
