rem From image %1,
rem write %2 that has given crop gray-balanced by multiplying channels.
rem %3, %4, %5 multipliers for R, G and B.
rem %6 method, one of:
rem   exact     use given factors, even if that causes clipping
rem   noClip    if factors would clip, reduce by same proportion so none clip
rem   maxNoClip multiply or divide factors by same proportion so one or more channels is at maximum
rem   white     choose factors so largest value in each channel is at maximum. (NYI)
@rem
@rem Note: could also have option to choose factors to make lightest pixel white,
@rem but this could clip channels in other pixels.
@rem
@rem Updated:
@rem   25-September-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave

call %PICTBAT%setInOut %1 cbm

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set factR=%3
set factG=%4
set factB=%5

if "%factR%"=="." set factR=
if "%factG%"=="." set factG=
if "%factB%"=="." set factB=
if "%factR%"=="" set factR=1
if "%factG%"=="" set factG=1
if "%factB%"=="" set factB=1

set method=%6
if "%method%"=="." set method=
if "%method%"=="" set method=maxNoClip

if /I %method% NEQ exact if /I %method% NEQ noClip if /I %method% NEQ maxNoClip if /I %method% NEQ white (
  echo %0: Invalid method [%method%]
  exit /B 1
)

if /I %method%==exact goto DoIt


set sFMT=^
maxR=%%[fx:maxima.r]\n^
maxG=%%[fx:maxima.g]\n^
maxB=%%[fx:maxima.b]\n^
newMaxR=%%[fx:maxima.r*%factR%]\n^
newMaxG=%%[fx:maxima.g*%factG%]\n^
newMaxB=%%[fx:maxima.b*%factB%]\n

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -precision 16 ^
  -format "%sFMT%" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 16 ^
  -format "newMaxMax=%%[fx:%newMaxR%>%newMaxG%?%newMaxR%:%newMaxG%]\n" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 14 ^
  -format "newMaxMax=%%[fx:%newMaxMax%>%newMaxB%?%newMaxMax%:%newMaxB%]\n" ^
  info:`) do set %%L

if /I %method%==maxNoClip (
  for /F "usebackq" %%L in (`%IMG7%magick ^
    xc: ^
    -precision 14 ^
    -format "factR=%%[fx:%factR%/%newMaxMax%]\nfactG=%%[fx:%factG%/%newMaxMax%]\nfactB=%%[fx:%factB%/%newMaxMax%]\n" ^
    info:`) do set %%L
  goto DoIt
)

if /I %method%==noClip (
  for /F "usebackq" %%L in (`%IMG7%magick ^
    xc: ^
    -format "DoesClip=%%[fx:%newMaxMax%>1?1:0]\n" ^
    info:`) do set %%L

  if %DoesClip%==1 (
    for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 14 ^
      -format "factR=%%[fx:%factR%/%newMaxMax%]\nfactG=%%[fx:%factG%/%newMaxMax%]\nfactB=%%[fx:%factB%/%newMaxMax%]\n" ^
      info:`) do set %%L
  )

  goto DoIt
)


:DoIt

echo %0: factR=%factR% factG=%factG% factB=%factB%

%IMG7%magick ^
  %INFILE% ^
  -channel R -evaluate Multiply %factR% ^
  -channel G -evaluate Multiply %factG% ^
  -channel B -evaluate Multiply %factB% ^
  +channel ^
  %OUTFILE%


call echoRestore

@endlocal & set cbmOUTFILE=%OUTFILE%& set cbmFactR=%factR%& set cbmFactG=%factG%& set cbmFactB=%factB%
