rem Finds proportions of colours in image %1.
rem   using Cygwin sort.exe.
rem Output to %2, a CSV file, columns:
rem   a number, from 0
rem   percentage of pixels that have this colour
rem   cumulative percentage so far
rem   a colour, beginning with #.
rem
rem Stops when any of these is true:
rem   individual PC less than a limit %3 [default no limit];
rem   cumulative PC reaches threshold %4 [default no limit];
rem   number of output colours reaches %5 [default 999999].
rem
rem %6 Optional output image file for map of the found colours.
rem %7 Optional pre-processing.
@rem
@rem Updated:
@rem   2-August-2017 Added %7.
@rem   6-August-2022 for IM v7
@rem   4-January-2023 added +repage after append; add "-precision".



@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 cp

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set LIM_PC=%3
if "%LIM_PC%"=="." set LIM_PC=
if "%LIM_PC%"=="" set LIM_PC=0

set LIM_CUM=%4
if "%LIM_CUM%"=="." set LIM_CUM=
if "%LIM_CUM%"=="" set LIM_CUM=101

set LIM_CNT=%5
if "%LIM_CNT%"=="." set LIM_CNT=
if "%LIM_CNT%"=="" set LIM_CNT=999999

set OUT_MAP=%6
if "%OUT_MAP%"=="." set OUT_MAP=

set PRE_PROC=%~7
if "%PRE_PROC%"=="." set PRE_PROC=

set TMP_HIST_1=%TEMP%\cp_hist_1.txt
set TMP_HIST_2=%TEMP%\cp_hist_2.txt

:: Get a histogram of all the colours present.

set WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 15 ^
  -format "WW=%%w\nHH=%%h\nWH=%%[fx:w*h]" ^
  %INFILE%`) do set %%L
if "%WW%"=="" exit /B 1

echo %0: WH=%WH%


%IMG7%magick ^
  %INFILE% ^
  %PRE_PROC% ^
  -precision 15 ^
  -define histogram:unique-colors=true -format "%%c" ^
  histogram:info:%TMP_HIST_1%

sort -r <%TMP_HIST_1% >%TMP_HIST_2%

set CUM_PC=0
set CNT=0
set ENDIT=0

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 19 ^
  -format "PCWH=%%[fx:100/%WH%]" ^
  xc:`) do set %%L

echo num,PC,CUM_PC,COLOUR >%OUTFILE%

:: FIXME: for v7, we might have single number within the parenthesis.
(
  for /F "tokens=1,3 delims=:)" %%A in (%TMP_HIST_2%) do (
    if !ENDIT!==0 (

      set FMT=^
PC=%%[fx:%%A*%PCWH%]\n^
CUM_PC=%%[fx:!CUM_PC!+%%A*%PCWH%]\n^
ENDIT=%%[fx:!CNT!^>=%LIM_CNT%^|^|%%A*%PCWH%^<=%LIM_PC%^|^|!CUM_PC!+%%A*%PCWH%^>=%LIM_CUM%?1:0]

      for /F "usebackq" %%L in (`%IMG7%magick identify ^
        -precision 15 ^
        -format "!FMT!" ^
        xc:`) do set %%L

      if !ENDIT!==0 (
        echo !CNT!,!PC!,!CUM_PC!,%%B

        if not "%OUT_MAP%"=="" (
          for /F "delims= " %%H in ("%%B") do set HASH=%%H

          if !CNT!==0 (
            %IMG7%magick xc:!HASH! %OUT_MAP%
          ) else (
            %IMG7%magick %OUT_MAP% xc:!HASH! +append +repage %OUT_MAP%
          )
        )

        set /A CNT+=1
      )
    )
  )
) >>%OUTFILE%

echo %0: CNT=%CNT% PC=%PC% CUM_PC=%CUM_PC%

call echoRestore

endlocal & set cpOUTFILE=%OUTFILE%& set /A cpCNT=%CNT%
