rem Given images %1 (source) and %2 (kernel),
rem convolves %1 with %2 by "-morphology convolve".
rem Output to %3.
rem Uses HDRI, so images may contain negative values.
rem %4, %5 kernel origin [default: central coordinate]
rem %6 is one of
rem   none: no normalisation of output
rem   knl: normalises output by dividing by sum of (colours times alphas) in kernel.
rem   src: normalises output by dividing by sum of (colours times alphas) in source.
@rem
@rem Updated:
@rem   25-September-2022 for IM v7.
@rem

@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 cmc

set KERNEL=%2

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set KO_X=%4
if "%KO_X%"=="." set KO_X=

set KO_Y=%5
if "%KO_Y%"=="." set KO_Y=

rem echo KO_X=[%KO_X%] KO_Y=[%KO_Y%]

set NORMALISE=%6
if "%NORMALISE%"=="." set NORMALISE=
if "%NORMALISE%"=="" set NORMALISE=none

set nNORM=99
if /i %NORMALISE%==none set nNORM=0
if /i %NORMALISE%==knl set nNORM=1
if /i %NORMALISE%==src set nNORM=2
if %nNORM%==99 (
  echo Bad normalise [%NORMALISE%]
  exit /B 1
)

set sSCALE_K=
if %nNORM%==1 (
  set sSCALE_K=-define "convolve:scale=^^^!"
)

echo sSCALE_K=[%sSCALE_K%]

set TMP_EXT=.miff
set TMP_IMG=cmc%TMP_EXT%
set TMP_PXL=cmc_pix%TMP_EXT%
set TMP_KNL=cmc_knl%TMP_EXT%
set TMP_PXL_C=cmc_pix_c%TMP_EXT%
set TMP_PXL_A=cmc_pix_a%TMP_EXT%


set IMG_WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "IMG_WW=%%w\nIMG_HH=%%h" ^
  %INFILE%`) do set %%L
if "%IMG_WW%"=="" exit /B 1

set K_WW=
for /F "usebackq" %%L in (`%IMG7%magick ^
  %KERNEL% ^
  +write %TMP_KNL% ^
  ^( +clone -background Black -alpha Background -alpha off ^
     -scale "1x1^!" ^
     +write %TMP_PXL_C% +delete ^) ^
  ^( +clone -alpha extract ^
     -scale "1x1^!" ^
     +write %TMP_PXL_A% +delete ^) ^
  -format "K_WW=%%w\nK_HH=%%h\nK_Wm1=%%[fx:w-1]\nK_Hm1=%%[fx:h-1]" ^
  info:`) do set %%L
if "%K_WW%"=="" exit /B 1

:: FIXME: for even-numbers, more sensible to simply divide by 2?
if "%KO_X%"=="" set /A KO_X=(%K_WW%-1)/2
if "%KO_Y%"=="" set /A KO_Y=(%K_HH%-1)/2

set sX=
set sY=
if %KO_X% GEQ 0 set sX=+
if %KO_Y% GEQ 0 set sY=+
set sORIG=%sX%%KO_X%%sY%%KO_Y%

call %PICTBAT%img2knl4f %KERNEL% cmc_knl.dat . %sORIG%

%IMG7%magick ^
  %INFILE% -alpha off ^
  ( -clone 0 ^
    -channel R -separate +channel ^
    %sSCALE_K% ^
    -morphology Convolve @cmc_knl_R.dat ) ^
  ( -clone 0 ^
    -channel G -separate +channel ^
    %sSCALE_K% ^
    -morphology Convolve @cmc_knl_G.dat ) ^
  ( -clone 0 ^
    -channel B -separate +channel ^
    %sSCALE_K% ^
    -morphology Convolve @cmc_knl_B.dat ) ^
  ( -clone 0 ^
    -channel A -separate +channel ^
    %sSCALE_K% ^
    -morphology Convolve @cmc_knl_A.dat ) ^
  -delete 0 ^
  -channel RGBA -combine ^
  %OUTFILE%

call echoRestore

@endlocal & set cmcOUTFILE=%OUTFILE%& set cmcKO_X=%KO_X%& set cmcKO_Y=%KO_Y%
