rem Given images %1 (source) and %2 (kernel),
rem convolves %1 with %2 by shift-multiply-add.
rem Output to %3.
rem Uses HDRI, so images may contain negative values.
rem %4, %5 kernel origin [default: central coordinate]
rem %6 is one of
rem   none: no normalisation of output
rem   knl: normalises output by dividing by sum of (colours times alphas) in kernel.
rem   src: normalises output by dividing by sum of (colours times alphas) in source.
rem %7 is one of
rem   none  no extension of source
rem   str   where str is an IM virtual-pixel setting, first extend the source.
@rem
@rem Updated:
@rem   25-September-2022 for IM v7.
@rem

rem Bias?
rem Option to rotate.

rem FIXME: this is wrongly named.
rem the convolution rotates the kernel by 180 degrees.
rem So this is the correlation. ??

@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 csma

set KERNEL=%2

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set KO_X=%4
if "%KO_X%"=="." set KO_X=

set KO_Y=%5
if "%KO_Y%"=="." set KO_Y=

rem echo KO_X=[%KO_X%] KO_Y=[%KO_Y%]

set NORMALISE=%6
if "%NORMALISE%"=="." set NORMALISE=
if "%NORMALISE%"=="" set NORMALISE=none

set EXT_METH=%7
if "%EXT_METH%"=="." set EXT_METH=
if "%EXT_METH%"=="" set EXT_METH=none


set nNORM=99
if /i %NORMALISE%==none set nNORM=0
if /i %NORMALISE%==knl set nNORM=1
if /i %NORMALISE%==src set nNORM=2
if %nNORM%==99 (
  echo Bad normalise [%NORMALISE%]
  exit /B 1
)

set TMP_EXT=.miff
set TMP_IN=csma_in%TMP_EXT%
set TMP_OUT=csma%TMP_EXT%
set TMP_PXL=csma_pix%TMP_EXT%
set TMP_KNL=csma_knl%TMP_EXT%
set TMP_PXL_C=csma_pix_c%TMP_EXT%
set TMP_PXL_A=csma_pix_a%TMP_EXT%

set SRC_WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "SRC_WW=%%w\nSRC_HH=%%h" ^
  %INFILE%`) do set %%L
if "%SRC_WW%"=="" exit /B 1

set K_WW=
for /F "usebackq" %%L in (`%IMG7%magick ^
  %KERNEL% ^
  +write %TMP_KNL% ^
  ^( +clone -background Black -alpha Background -alpha off ^
     -scale "1x1^!" ^
     +write %TMP_PXL_C% +delete ^) ^
  ^( +clone -alpha extract ^
     -scale "1x1^!" ^
     +write %TMP_PXL_A% +delete ^) ^
  -format "K_WW=%%w\nK_HH=%%h\nK_WH=%%[fx:w*h]\nK_Wm1=%%[fx:w-1]\nK_Hm1=%%[fx:h-1]" ^
  info:`) do set %%L
if "%K_WW%"=="" exit /B 1


%IMG7%magick %TMP_PXL_C% txt:

:: FIXME: for even-numbers, more sensible to simply divide by 2?
if "%KO_X%"=="" set /A KO_X=(%K_WW%-1)/2
if "%KO_Y%"=="" set /A KO_Y=(%K_HH%-1)/2

set /A ROLL_X=-%KO_X%
set /A ROLL_Y=-%KO_Y%

set sX=
set sY=
if %ROLL_X% GEQ 0 set sX=+
if %ROLL_Y% GEQ 0 set sY=+
set sROLL=-roll %sX%%ROLL_X%%sY%%ROLL_Y%

if "%sROLL%"=="-roll +0+0" set sROLL=

echo %0: sROLL=%sROLL%

:: FIXME: We should normalise here.

if not "%sROLL%"=="" %IMG7%magick ^
  %TMP_KNL% ^
  %sROLL% ^
  %TMP_KNL%

set sEXT=

if /I not "%EXT_METH%"=="none" (

  set sFMT=sVP=^
%%[fx:%SRC_WW%+2*%K_WW%]x^
%%[fx:%SRC_HH%+2*%K_HH%]-^
%K_WW%-^
%K_HH%

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "!sFMT!" ^
    xc:`) do set %%L

  echo %0: sVP=!sVP!

  set sEXT=^
-virtual-pixel %EXT_METH% ^
-define distort:viewport=!sVP! ^
-distort SRT 0 +repage

  echo %0: sEXT=!sEXT!
)

%IMG7%magick ^
  %INFILE% ^
  %sEXT% ^
  %TMP_IN%

if ERRORLEVEL 1 exit /B 1

%IMG7%magick identify %TMP_IN%

set REAL_HDRI=-define compose:clamp=off -define quantum:format=floating-point +depth

%IMG7%magick ^
  -size %SRC_WW%x%SRC_HH% xc:None ^
  %REAL_HDRI% ^
  %TMP_OUT%


:: TMP_PXL is a running total of the sum of the kernel pixels.
::
%IMG7%magick ^
  -size 1x1 xc:None ^
  %REAL_HDRI% ^
  %TMP_PXL%

for /L %%J in (0,1,%K_Hm1%) ^
do for /L %%I in (0,1,%K_Wm1%) ^
do %IMG7%magick ^
      -define compose:clamp=off ^
      %TMP_OUT% ^
      ^( %TMP_IN% ^
        ^( %TMP_KNL% ^
          -crop 1x1+%%I+%%J +repage ^
          ^( +clone ^
             %TMP_PXL% ^
             %REAL_HDRI% ^
             -compose Plus -composite ^
             +write %TMP_PXL% ^
             +delete ^
          ^) ^
          -scale "%SRC_WW%x%SRC_HH%^!" ^
        ^) ^
        %REAL_HDRI% ^
        -channel RGBA ^
        -compose Multiply -composite ^
        +channel ^
        -repage +%%I+%%J ^
      ^) ^
      %REAL_HDRI% ^
      -background Black ^
      -compose Plus -layers merge ^
      +repage ^
      %TMP_OUT%

%IMG7%magick identify %TMP_OUT%

set OUT_WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "OUT_WW=%%w\nOUT_HH=%%h" ^
  %TMP_OUT%`) do set %%L
if "%OUT_WW%"=="" exit /B 1


:: FIXME: instead of normalising at the end, quicker to normalise before extending.

if %nNORM%==1 (
  %IMG7%magick ^
    %TMP_OUT% ^
    ^( %TMP_PXL% +write txt: -scale "%OUT_WW%x%OUT_HH%^!" ^) ^
    -channel RGB ^
    %REAL_HDRI% ^
    -compose DivideSrc -composite ^
    +channel ^
    %OUTFILE%
) else if %nNORM%==2 (
  echo nNORM==2 NYI
  exit /B 1
) else (
  %IMG7%magick ^
    %TMP_OUT% ^
    %OUTFILE%
)


call echoRestore

@endlocal & set csmaOUTFILE=%OUTFILE%& set csmaKO_X=%KO_X%& set csmaKO_Y=%KO_Y%
