rem From %1 an ordinary image,
rem makes %2 cropped to detail of %1.
rem %3 is required dimensions, WxH.
rem %4 force the use of this threshold value.
@rem If width and height are both given, the job is easy.
@rem If neither width nor height are given, we use a threshold.
@rem If only one is given, it iterates to a solution.
@rem   This may not be exactly the specified dimension.
@rem   If not, recursively called with that specified dimension
@rem   and calculated dimension.
@rem
@rem Also uses:
@rem   ctdBLR_SIG: sigma for blur of slope magnitude [0 or 20]
@rem   ctdUSE_CONN_COMP 0 or 1:
@rem     whether to use only largest connected component. [1]
@rem   ctdDO_RECURSE 0 or 1: whether to recursively call. [1]
@rem   ctdDBG_IMG is not blank, writes a debug image to this.
@rem
@rem   ctdREUSE_SM if 1, use existing slope magnitude image (don't make a new one).
@rem     (This is used internally, at recursive calls.)
@rem
@rem Reference: http://im.snibgo.com/crop2det.htm
@rem
@rem Last updated:
@rem   9-July-2017 Use connCompCropWhite.bat; %3 format WxH.
@rem   12-July-2017 Numerous.
@rem   2-August-2020 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

echo %0: %1 %2 %3 %4
echo %0: ctdBLR_SIG=%ctdBLR_SIG% ctdUSE_CONN_COMP=%ctdUSE_CONN_COMP% ctdDO_RECURSE=%ctdDO_RECURSE%

call %PICTBAT%setInOut %1 ctd

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set DIMS=%3
if "%DIMS%"=="." set DIMS=
if "%DIMS%"=="" set DIMS=0x0

set FORCE_TH=%4
if "%FORCE_TH%"=="." set FORCE_TH=

if "%ctdUSE_CONN_COMP%"=="" set ctdUSE_CONN_COMP=1

if "%ctdDO_RECURSE%"=="" set ctdDO_RECURSE=1

rem   Ensure required dims don't exceed image dims.

set WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nDIAG_FACT=%%[fx:hypot(w,h)/664]" ^
  %INFILE%`) do set %%L
if "%WW%"=="" exit /B 1

call parseXxY2i %WW% %HH% DIMS %DIMS%

set R_WI=%DIMS_X%
set R_HT=%DIMS_Y%

if not "%R_WI%"=="0" (
  if %R_WI% GTR %WW% set R_WI=%WW%
)

if not "%R_HT%"=="0" (
  if %R_HT% GTR %HH% set R_HT=%HH%
)

set HAS_BOTH=0
if not "%R_WI%"=="0" if not "%R_HT%"=="0" if "%FORCE_TH%"=="" set HAS_BOTH=1

if "%R_WI%"=="0" if "%R_HT%"=="0" if "%FORCE_TH%"=="" (
  set FORCE_TH=85
  if "%ctdUSE_CONN_COMP%"=="0" set FORCE_TH=90
)

if "%ctdBLR_SIG%"=="" (
  set ctdBLR_SIG=0
  if %ctdUSE_CONN_COMP%==0 set ctdBLR_SIG=20

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "ctdBLR_SIG=%%[fx:!ctdBLR_SIG!*%DIAG_FACT%]\n" ^
    xc:`) do set %%L
)

echo %0: WW=%WW% HH=%HH% req:%R_WI%x%R_HT% ctdBLR_SIG=%ctdBLR_SIG% ctdUSE_CONN_COMP=%ctdUSE_CONN_COMP% ctdDO_RECURSE=%ctdDO_RECURSE% DIAG_FACT=%DIAG_FACT% FORCE_TH=%FORCE_TH%

rem For method 1, COMET_BLR=1*DIAG_FACT.
rem for method 11, try 2*...

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "COMET_BLR=%%[fx:0.5*%DIAG_FACT%]\ncccwAREA_TH=%%[fx:int(%DIAG_FACT%*%DIAG_FACT%*2+1)]\n" ^
  xc:`) do set %%L

set TMPDIR=\temp\
set SM_IMG=%TMPDIR%ctd_sm.miff
set SM_IMG_B=%TMPDIR%ctd_sm_b.miff
set SM_IMG_C=%TMPDIR%ctd_sm_c.miff
set SM_IMG_D=%TMPDIR%ctd_sm_d.miff
set WHT_IMG=%TMPDIR%ctd_white.miff

rem FIXME: make SM_METH a variable
rem FIXME: make one script to do any method?

set SCORE=999999

if "%ctdREUSE_SM%"=="1" goto made_sm

set SM_METH=11

if %SM_METH%==0 (
  call %PICTBAT%slopeMagF %INFILE% %SM_IMG%
) else if %SM_METH%==1 (
  echo %0: COMET_BLR=%COMET_BLR%

  call %PICTBAT%slopeXYblMag ^
    %INFILE% %SM_IMG% %COMET_BLR% ^
    "-grayscale Brightness -auto-level"

  if ERRORLEVEL 1 exit /B 1

) else if %SM_METH%==11 (
  echo %0: gsm BLR=%COMET_BLR%

  call %PICTBAT%gaussSlpMag ^
    %INFILE% %SM_IMG% %COMET_BLR% ^
    "-grayscale Brightness -auto-level"

  if ERRORLEVEL 1 exit /B 1

) else if %SM_METH%==2 (

  call %PICTBAT%gaussStdDev ^
    %INFILE% ^
    %SM_IMG% ^
    "-blur 0x3" ^
    "-separate -evaluate-sequence Max -auto-level"

) else if %SM_METH%==3 (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "METH_SIG=%%[fx:%DIAG_FACT%]\n" ^
    xc:`) do set %%L

  echo METH_SIG=!METH_SIG!

  call %PICTBAT%gaussStdDev ^
    %INFILE% ^
    %SM_IMG% ^
    "-blur 0x!METH_SIG!" ^
    "-separate -evaluate-sequence Max"

) else if %SM_METH%==3XX (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "METH__SIG=%%[fx:2*%DIAG_FACT%]\n" ^
    xc:`) do set %%L

  echo METH_SIG=!METH_SIG!

  call %PICTBAT%gaussStdDev ^
    %INFILE% ^
    %SM_IMG% ^
    "-blur 0x!METH_SIG!" ^
    "-separate -evaluate-sequence Max"

) else if %SM_METH%==4 (

  call %PICTBAT%slopeXY ^
    %INFILE% ^
    s.miff

  call %PICTBAT%slopeXYdiv ^
    s.miff ^
    %SM_IMG%

  %IMG7%magick ^
    %SM_IMG% ^
    -separate -evaluate-sequence Max ^
    %SM_IMG%
)
if ERRORLEVEL 1 exit /B 1

:made_sm

if %HAS_BOTH%==1 (
  %IMG7%magick -size %R_WI%x%R_HT% xc:white %WHT_IMG%
  if ERRORLEVEL 1 exit /B 1

  rem Next for method 1:
  rem %IM%convert %SM_IMG% -sigmoidal-contrast 20,50%% ctd_hack.miff

  rem Next for method 111:
  %IMG7%magick %SM_IMG% -sigmoidal-contrast 20,50%% ctd_hack.miff

  call %PICTBAT%srchImg ctd_hack.miff %WHT_IMG%
  if ERRORLEVEL 1 exit /B 1

  echo %0: crop is !siCOMP_CROP! score is !siCOMP_FLT!

  if /I "%OUTFILE%" NEQ "null:" (
    %IMG7%magick ^
      %INFILE% ^
      -crop !siCOMP_CROP! +repage ^
      %OUTFILE%
    if ERRORLEVEL 1 exit /B 1
  )

  set SCORE=!siCOMP_FLT!
  set CROP4=!siCOMP_CROP!

  goto end
)

if "%R_WI%"=="0" (
  set REQ_DIM=%R_HT%
) else (
  set REQ_DIM=%R_WI%
)

if "%ctdBLR_SIG%"=="0" (
  set sBLUR=
) else (
  set sBLUR=-blur 0x%ctdBLR_SIG%
)

echo %0: %sBLUR% equalize

%IMG7%magick ^
  %SM_IMG% ^
  %sBLUR% ^
  -equalize ^
  %SM_IMG_B%

if not "%FORCE_TH%"=="" (
  set BUST=0
  call :calcCrop %FORCE_TH%
  if ERRORLEVEL 1 exit /B 1

  goto foundCrop
)

set FINISHED=0
set T0=10
set T2=99.99
set T2=100

echo %0: T0=%T0% T2=%T2%

call :calcCrop %T0%
if ERRORLEVEL 1 exit /B 1
set D0=%CALC_DIM%

call :calcCrop %T2%
if ERRORLEVEL 1 set CALC_DIM=0
set D2=%CALC_DIM%

if %D0% LSS %REQ_DIM% (
  echo Bust: REQ_DIM=%REQ_DIM% D0=%D0% 
  exit /B 1
)

if %D2% GTR %REQ_DIM% (
  echo Bust: REQ_DIM=%REQ_DIM% D1=%D1% 
  exit /B 1
)

set BUST=0
set PREV_DIFF=999999
:loop

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 9 ^
  -format "T1=%%[fx:sqrt(%T0%*%T2%)]"
  xc:`) do set %%L

call :calcCrop %T1%
if ERRORLEVEL 1 exit /B 1
set D1=%CALC_DIM%

set /A DIFF=%CALC_DIM%-%REQ_DIM%
if %DIFF% LSS 0 set /A DIFF=-%DIFF%

if %DIFF% LSS %PREV_DIFF% (
  set PREV_DIFF=%DIFF%
  set BW=%CW%
  set BH=%CH%
  set BX=%CX%
  set BY=%CY%
)

if %T0%==%T1% set BUST=1
if %T2%==%T1% set BUST=1

if %BUST%==1 set FINISHED=1

if %FINISHED%==0 (
  if %D1%==%REQ_DIM% (
    echo %0: Found T1=%T1% D1=%D1%
    set FINISHED=1
  ) else if %D1% GTR %REQ_DIM% (
    rem Solution is between 1 and 2
    set T0=%T1%
    set D0=%D1%
  ) else (
    rem Solution is between 0 and 1
    set T2=%T1%
    set D2=%D1%
  )
  goto loop
)


:foundCrop

echo %0: last crop is %CW%x%CH%+%CX%+%CY%

if %BUST%==1 (
  echo %0: ** BUST!!
  echo %0: best crop is %BW%x%BH%+%BX%+%BY%

  set CW=%BW%
  set CH=%BH%
  set CX=%BX%
  set CY=%BY%

  if "%R_WI%"=="0" (
    set /A CW=!CW!*%R_HT%/!CH!

    set CH=%R_HT%
  ) else (
    set /A CH=!CH!*%R_WI%/!CW!

    set CW=%R_WI%
  )
  rem FIXME: Should we also change the other dimension, in proportion?
)

rem Now we know both required dimensions, so maybe recursive call.

if %ctdDO_RECURSE%==0 (
  set CROP4=%CW%x%CH%+%CX%+%CY%
  if /I "%OUTFILE%" NEQ "null:" (
    %IMG7%magick ^
      %INFILE% ^
      -crop !CROP4! +repage ^
      %OUTFILE%
    if ERRORLEVEL 1 exit /B 1
  )
) else (
  set ctdREUSE_SM=1
  call %PICTBAT%cropToDetailSml %INFILE% %OUTFILE% %CW%x%CH%
  if ERRORLEVEL 1 exit /B 1
  set ctdREUSE_SM=

  set CROP4=!ctdCROP!
  set SCORE=!ctdSCORE!
)

rem FIXME: possibly make a debug image.

if not "%ctdDBG_IMG%"=="" %IMG7%magick ^
  %SM_IMG% ^
  -region %CROP4% ^
  +level 33,67%% ^
  %ctdDBG_IMG%

:end

echo %0: OUTFILE=%OUTFILE% ctdSCORE=%SCORE%

call echoRestore

endlocal & set ctdOUTFILE=%OUTFILE%& set ctdSCORE=%SCORE%& set ctdCROP=%CROP4%

@exit /B 0


::--------------------------------------------------
:: Subroutines.

:: calcCrop: Given a threshold %1 percent, find the crop params.
:: Returns just the dimension that was required in CALC_DIM.

:calcCrop

set CW=
if %ctdUSE_CONN_COMP%==1 (

  %IMG7%magick ^
    %SM_IMG_B% ^
    -threshold %1%% ^
    %SM_IMG_C%

  call %PICTBAT%connCompCropWhite %SM_IMG_C%

  if ERRORLEVEL 1 exit /B 1

  set /A AREA=!CW!*!CH!
) else (

  for /F "usebackq tokens=1-4 delims=+x" %%A in (`%IMG7%magick ^
    %SM_IMG_B% ^
    -threshold %1%% ^
    -bordercolor Black -border 1 ^
    -format %%@\n ^
    info:`) do (
    set CW=%%A
    set CH=%%B
    set /A CX=%%C-1
    set /A CY=%%D-1
    set /A AREA=%%A*%%B
  )
)

if "%CW%"=="" exit /B 1

set CALC_DIM=%CW%
if "%R_WI%"=="0" set CALC_DIM=%CH%

echo %0: thresh=%1  %CW%x%CH%+%CX%+%CY%  REQ_DIM=%REQ_DIM% CALC_DIM=%CALC_DIM%

if %AREA%==0 exit /B 1

exit /B 0
