rem From image %1
rem and profile %2 (white line on black background),
rem creates output %3, curving the image around the extruded profile.
rem %4 is tilt of object. >0 tilts up (ie camera tilt down).
rem %5 is percentage perspective narrowing (0=no perspective).
rem %6 is percentage perspective shortening
@rem
@rem  Updated:
@rem    20-August-2022 Upgraded for IM v7.
@rem


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 c3d


set LINE_FILE=%2
set LINE_BASE=%~n2

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set ALPHA=%4
if "%ALPHA%"=="." set ALPHA=
if "%ALPHA%"=="" set ALPHA=0

set NARROW_PC=%5
if "%NARROW_PC%"=="." set NARROW_PC=
if "%NARROW_PC%"=="" set NARROW_PC=0

set SHORTEN_PC=%6
if "%SHORTEN_PC%"=="." set SHORTEN_PC=
if "%SHORTEN_PC%"=="" set SHORTEN_PC=0

:: FIXME if alpha less than 0, different perspective.

call %PICTBAT%colTrim %LINE_FILE% Black

echo ctRECT=%ctRECT%  ctW=%ctW%  ctH=%ctH%

:: Grad file is named after the line, not the first input file.

set GRAD_FILE=%LINE_BASE%_c3d_grad.miff
set TMP_INPUT=c3d_tmp_inp.miff
set FILL_FILE=c3d_fill.miff
set XDISP_TRIAL=c3d_xdisp_trial.miff
set Y_PERSP=c3d_y_persp.miff

echo GRAD_FILE=%GRAD_FILE%

:: For better performance, don't delete GRAD_FILE.
::
del %GRAD_FILE% 2>nul

set line_len=
if exist %GRAD_FILE% (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format %%[line:*] ^
    %GRAD_FILE%`) do (
    set LINE=%%L
    set !LINE::=_!
  )
)

if "%line_len%"=="" (
  call %PICTBAT%dp2grad ^
    %LINE_FILE% ^
    %GRAD_FILE%

  %IMG7%magick ^
    %GRAD_FILE% ^
    -crop %ctRECT% +repage ^
    %GRAD_FILE%

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format %%[line:*] ^
    %GRAD_FILE%`) do (
    set LINE=%%L
    set !LINE::=_!
  )
)

echo line_len=%line_len% dp2gERR=%dp2gERR%
if "%line_len%"=="" exit /B 1

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -resize %line_len%x ^
  -format "INP_WW=%%w\nINP_HH=%%h" ^
  +write info: ^
  %TMP_INPUT%`) do set %%L

echo INP_WW=%INP_WW% INP_HH=%INP_HH% 


call %PICTBAT%shiftFill ^
  %GRAD_FILE% 0x %FILL_FILE%

if ERRORLEVEL 1 exit /B 1

%IMG7%magick ^
  %FILL_FILE% ^
  -crop 0x1+0+0 +repage ^
  -scale "x%INP_HH%^!" ^
  -gravity West ^
  -background White ^
  -extent %INP_WW%x%INP_HH% ^
  ( -clone 0 ^
    -sparse-color bilinear ^
      "0,0 Black 0,%%[fx:h-1] White" ^
  ) ^
  ( -clone 0 ^
    -fill gray(50%%) -colorize 100 ^
  ) ^
  -combine ^
  -set colorspace sRGB ^
  %TMP_INPUT% ^
  +swap ^
  -compose Distort ^
  -set option:compose:args 100%%x100%% ^
  -composite ^
  -crop %ctW%x+0+0 +repage ^
  %XDISP_TRIAL%


set ALPH_RAD=(%ALPHA%*PI/180)

for /F "useback" %%L in (`%IMG7%magick identify ^
  -format "inHcosA=%%[fx:%INP_HH% * cos(%ALPH_RAD%)]\nctHsinA=%%[fx:%ctH% * abs(sin(%ALPH_RAD%))]"
  xc:`) do set %%L

for /F "useback" %%L in (`%IMG7%magick identify ^
  -format "OUT_HH=%%[fx:%inHcosA% + %ctHsinA%]\nctHsinA_2=%%[fx:%ctHsinA%/2]\nIS_NEG=%%[fx:%ALPHA%<0?1:0]"
  xc:`) do set %%L

echo inHcosA=%inHcosA% ctHsinA=%ctHsinA% ctHsinA_2=%ctHsinA_2% OUT_HH=%OUT_HH% IS_NEG=%IS_NEG%

set dNarrow=w*%NARROW_PC%/100

if %IS_NEG%==1 (

  echo Angle is negative: %ANGLE%

  set sNEG_Y=-negate

  set PERSP=^
0,0,0,0,^
%%[fx:w-1],0,%%[fx:w-1],0,^
0,%%[fx:h-1],%%[fx:%dNarrow%],%%[fx:h-1],^
%%[fx:w-1],%%[fx:h-1],%%[fx:w-1-%dNarrow%],%%[fx:h-1]

) else (

  echo Angle is positive: %ANGLE%

  set sNEG_Y=

  set PERSP=^
0,0,%%[fx:%dNarrow%],0,^
%%[fx:w-1],0,%%[fx:w-1-%dNarrow%],0,^
0,%%[fx:h-1],0,%%[fx:h-1],^
%%[fx:w-1],%%[fx:h-1],%%[fx:w-1],%%[fx:h-1]

)

%IMG7%magick ^
  %LINE_FILE% ^
  -crop %ctRECT% +repage ^
  -morphology dilate rectangle:1x%ctH%+0+0 ^
  -scale "x1^!" ^
  ( -clone 0 ^
    -fill gray(50%%) -colorize 100 ^
  ) ^
  ( -clone 1 ) ^
  -swap 0,1 ^
  -combine ^
  -set colorspace sRGB ^
  -scale "x%OUT_HH%^!" ^
  %sNEG_Y% ^
  ( +clone ^
    +level %SHORTEN_PC%%% ^
  ) ^
  ( +clone ^
    -sparse-color bilinear "0,0,White 0,%%[fx:h-1],Black" ^
  ) ^
  -composite ^
  %Y_PERSP%

%IMG7%magick ^
  %XDISP_TRIAL% ^
  -scale "x%inHcosA%^!" ^
  -gravity Center -background None ^
  -extent "x%OUT_HH%^!" ^
  %Y_PERSP% ^
  -virtual-pixel None ^
  -compose Displace ^
  -set option:compose:args 0x%ctHsinA_2% ^
  -composite ^
  -virtual-pixel None ^
  -distort Perspective "%PERSP%" ^
  %OUTFILE%


call echoRestore

endlocal & set c3dOUTFILE=%OUTFILE%
