rem Transforms from cylinder to plane.
rem %1 is image.
rem %2 is tilt angle, degrees. Default=0=no tilt. Positive is with camera above cylinder, looking down.
@rem
@rem Also uses:
@rem   pcStartAng. Default -90.
@rem   pcEndAng. Default 90.
@rem   pcDebug. Default 0.
@rem   pcBackCol. Default None.
@rem
@rem Updated:
@rem   5-August-2022 for IM v7.
@rem
	
@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 c2p


set tiltAng=%2
if "%tiltAng%"=="" set tiltAng=0

call %PICTBAT%planeCylSetup
if ERRORLEVEL 1 exit /B 1

if %pcForceRecalc%==1 (
  del %fC2P_HORIZ%
  del %fC2P_VERT%
  del %fC2P_COMB%
)


if "%pcDebug%"=="1" echo %0: farZAng=%farZAng%

rem plHH=%%[fx:int((h-w*%sinT%/2)/%cosT%+0.5)]
rem tiltFact: opposite sign to Plane2Cylinder.
FOR /F "usebackq" %%L ^
IN (`%CALC% "cylWW=%%w\nextendV=%%h\ncylHH=%%[fx:int(h-w*abs(%sinT%)/2+0.5)]\nplWWxX=%%[fx:int(w*pi/2+0.5)]\ncylWW_2=%%[fx:int(w/2+0.5)]\ntiltFact=%%[fx:%sinT%/2]\nplHH=%%[fx:int((h-abs(w/2*%sinT%))/%cosT%+0.5)]" %INFILE%`) ^
DO set %%L

if "%pcDebug%"=="1" echo %0: extendV=%extendV%

if %ZeroAng%==1 (
  set cylRad=0
  set plWW=%cylWW%
) else (
  FOR /F "usebackq" %%L ^
IN (`%CALC% "cylRad=%%[fx:%cylWW%/(sin(-(%pcStartAng%*pi/180))+sin(%pcEndAng%*pi/180))]" xc:`) ^
DO set %%L

  FOR /F "usebackq" %%L ^
IN (`%CALC% "plWW=%%[fx:!cylRad!*pi*(%pcEndAng%-(%pcStartAng%))/180]\nIsSouth=%%[fx:%tiltAng%<0?1:0]" xc:`) ^
DO set %%L
)

if %IsSouth%==1 (
  set GRAV=South
) else (
  set GRAV=North
)

if "%pcDebug%"=="1" echo %0: extendV=%extendV% cylWW=%cylWW% cylHH=%cylHH% plWW=%plWW% plHH=%plHH% cylWW_2=%cylWW_2%

goto skipSelfSinus
@rem %IM%convert ^
@rem   -size 1x%cylWW% gradient:gray(%pc180end%%%)-gray(%pc180start%%%) -rotate 90 ^
@rem   ( +clone -function sinusoid 0.5,-90,0.5,0.5 ) ^
@rem   -evaluate Divide 2 ^
@rem   -compose ModulusSubtract -composite ^
@rem   -evaluate AddModulus 50%% ^
@rem   cp_sinus.png

%IMG7%magick ^
  -size 1x%cylWW% gradient:gray(%pc180end%%%)-gray(%pc180start%%%) -rotate 90 ^
  -function sinusoid 0.5,-90,0.5,0.5 ^
  cp_sinus.png

FOR /F "usebackq" %%L ^
IN (`%IMG7%magick cp_sinus.png -format "COL_0=%%[pixel:p{0,0}]\nCOL_1=%%[pixel:p{w-1,h-1}]" info:`) ^
DO set %%L

%IMG7%magick ^
  ( -size 1x%cylWW% gradient:%COL_1%-%COL_0% -rotate 90 ) ^
  cp_sinus.png ^
  -evaluate Divide 2 ^
  -compose ModulusSubtract -composite ^
  -evaluate AddModulus 50%% ^
  cp_sinus.png

:skipSelfSinus

rem Forget what we've just done. Invert the plane2cyl version.

if not exist %fC2P_HORIZ% (
  if not exist %fP2C_HORIZ% call %PICTBAT%p2cHoriz.bat

  call %PICTBAT%invRelClut %fP2C_HORIZ%
  rem %IM%convert cp_asin_irc.png cp_sinus.png
)

if "%pcDebug%"=="1" (
  %IMG7%magick %fC2P_HORIZ% cp_sinus.png
  call %PICTBAT%graph1d cp_sinus.png
)

if not exist %fC2P_VERT% %IMG7%magick ^
  -size 1x%cylWW% ^
  gradient:gray(%pc180end%%%)-gray(%pc180start%%%) -rotate 90 ^
  -function Polynomial -4,4,0 -evaluate Pow 0.5 ^
  -function Polynomial %tiltFact%,0.5 ^
  %fC2P_VERT%

if "%pcDebug%"=="1" (
  %IMG7%magick %fC2P_VERT% cp_v_c2p.png
  call %PICTBAT%graph1d cp_v_c2p.png
)


goto skipJunk

goto skipComb
rem Combine the displacement maps
%IMG7%magick ^
  %fC2P_HORIZ% ^
  %fC2P_VERT% ^
  -size %cylWW%x1 xc:gray50 ^
  -combine -set colorspace sRGB ^
  cp_map_c2p.png


:: In v6, this had "-interpolate bicubic" before "-compose".
%IMG7%magick ^
  %INFILE% ^
  ( cp_map_c2p.png -scale "%cylWW%x%extendV%^!" ) ^
  -background lightgreen -virtual-pixel background ^
  -compose displace -set option:compose:args %cylWW%x%cylWW_2% -composite ^
  %OUTFILE%
:skipComb

:: In v6, this had "-interpolate bicubic" before "-compose".
%IMG7%magick ^
  %INFILE% ^
  ( cp_v_c2p.png -scale "%cylWW%x%extendV%^!" ) ^
  -background lightgreen -virtual-pixel background ^
  -compose displace -set option:compose:args 0x%cylWW_2% -composite ^
  ( %fC2P_HORIZ% -scale "%cylWW%x%extendV%^!" ) ^
  -compose displace -set option:compose:args %cylWW%x0 -composite ^
  -crop %cylWW%x%cylHH%+0+0 +repage ^
  -resize "%plWW%x%plHH%^!" ^
  %OUTFILE%

:skipJunk

if %pcBackCol%==None (
  set VIRT_PIX=-virtual-pixel None
) else (
  set VIRT_PIX=-background %pcBackCol% -virtual-pixel background
)

rem Width was cylWW
:: In v6, this had "-interpolate bicubic" before "-compose".
%IMG7%magick ^
  %INFILE% ^
  %VIRT_PIX% ^
  ( cp_v_c2p.png -scale "%cylWW%x%extendV%^!" ) ^
  -compose displace -set option:compose:args 0x%cylWW_2% -composite ^
  ( cp_sinus.png -scale "%cylWW%x%extendV%^!" ) ^
  -compose displace -set option:compose:args %cylWW_2%x0 -composite ^
  -gravity %GRAV% ^
  -crop %cylWW%x%cylHH%+0+0 +repage ^
  -resize "%plWW%x%plHH%^!" ^
  %OUTFILE%


rem First disp args: 0x%cylWW_2%

call echoRestore
