rem From image %1,
rem makes output %2
rem with lowered saturation at coloured fringes.
rem %3 is method:
rem   all    desaturate all coloured pixels
rem   fringe desaturate only coloured pixels that are
rem          within a few pixels of both nearly black
rem          and nearly white
rem default: fringe
rem %4: "regardL" or "ignoreL"
rem %5 percentage values for L*,a*,b* for the colour
rem    Format either "srgb(12.34%,23.45%,34.56%)"
rem    or "12.34,23.45,34.56".
@rem
@rem
@rem
@rem Also uses:
@rem   DP_DBG_PREF if set, prefix for debug files.
@rem     (Default: make no debug files.)
@rem   DP_DBG_EXT if set, extension for debug files.
@rem     (Default: "miff".)
@rem
@rem Reference:
@rem   http://im.snibgo.com/colfring.htm
@rem
@rem Updated:
@rem   10-August-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 dcl

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set METHOD=%3
if "%METHOD%"=="." set METHOD=
if "%METHOD%"=="" set METHOD=fringe

set doL=%4
if "%doL%"=="." set doL=
if "%doL%"=="" set doL=ignoreL

set VALS=%~5
if "%VALS%"=="." set VALS=
if "%VALS%"=="" set VALS=50,50,50

set START=%VALS:~0,4%
if /I "%START%"=="srgb" (
  set COL="%VALS%"
) else (
  call parseCommaList "%VALS%" dcl_argc dcl_argv

  if not !dcl_argc!==3 (
    echo VALS needs 3 comma-separated numbers "%VALS%"
    exit /B 1
  )

  set COL=sRGB(!dcl_argv[0]!%%,!dcl_argv[1]!%%,!dcl_argv[2]!%%)
)

echo COL=[%COL%]

if /I %METHOD%==all (
  set ENDFIRST=mpr:GRAY ^
    +swap ^
    -compose Over -composite ^
    %OUTFILE%
) else if /I %METHOD%==fringe (
  set ENDFIRST=NULL:
) else (
  echo %0: Unknown method [%METHOD%]
  exit /B 1
)

if /I %doL%==regardL (
  set setL=
) else if /I %doL%==ignoreL (
  set setL=-channel R -evaluate set 50%% +channel
) else (
  echo %0: Unknown doL [%doL%]
  exit /B 1
)


set TMPDIR=\temp\
set TMPEXT=miff
set TMPPREF=%TMPDIR%dcl_%~n1

if "%DP_DBG_EXT%"=="" set DP_DBG_EXT=miff

%IMG7%magick ^
  %INFILE% ^
  ( +clone ^
    -colorspace Lab ^
    -set colorspace sRGB ^
    ( +clone ^
      -channel R -separate +channel ^
      +write mpr:GRAY ^
      +write %TMPPREF%_gray.%TMPEXT% ^
      +delete ^
    ) ^
    %setL% ^
    ( +clone ^
      -fill %COL% ^
      -colorize 100 ^
    ) ^
    -compose Difference -composite ^
    -grayscale RMS ^
    -level 10,5%% ^
    +write %TMPPREF%_colmsk.%TMPEXT% ^
  ) ^
  %ENDFIRST%

if ERRORLEVEL 1 exit /B 1

if /I %METHOD%==fringe %IMG7%magick ^
  %INFILE% ^
  ( %TMPPREF%_gray.%TMPEXT% +write mpr:GRAY ) ^
  ( ^
    ( mpr:GRAY ^
      ( -clone 0 ^
        -threshold 15%% -blur 0x3 -fill Black +opaque White ^
        -negate ^
        +write %TMPPREF%_t0.%TMPEXT% ^
      ) ^
      ( -clone 0 ^
        -threshold 70%% -blur 0x3 -fill White +opaque Black ^
        +write %TMPPREF%_t1.%TMPEXT% ^
      ) ^
      -delete 0 ^
      -compose Darken -composite ^
      -blur 0x2 ^
      +write %TMPPREF%_frng.%TMPEXT% ^
    ) ^
    %TMPPREF%_colmsk.%TMPEXT% ^
    -compose Multiply -composite ^
    +write %TMPPREF%_cfmsk.%TMPEXT% ^
  ) ^
  -compose Over -composite ^
  %OUTFILE%

call echoRestore

endlocal & set dclOUTFILE=%OUTFILE%
