rem From %1, a Bayer mosaic image,
rem makes demosaiced image, same size.
rem %2 %3 %4 and %5 are multipliers for R G0 G1 and B channels.
rem %6 is optional filter for resize.
rem %7 is optional output file.
@rem
@rem Also uses:
@rem   dmMIN_HALO  if 1, uses experimental technique for minimising halos.
@rem   dmDEBUG     if 1, creates extra debugging outputs.
@rem
@rem Assumes Bayer filter pattern is:
@rem   R  G0
@rem   G1 B
@rem
@rem Updated:
@rem   26-July-2022 for IM 7.

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

call echoOffSave

call %PICTBAT%setInOut %1 dm

set FILT=%6

if not "%7"=="" if not "%7"=="." set OUTFILE=%7

if "%dmDEBUG%"=="1" (
  set dm_WR_R=-write %BASENAME%_R.png
  set dm_WR_G0=-write %BASENAME%_G0.png
  set dm_WR_G1=-write %BASENAME%_G1.png
  set dm_WR_B=-write %BASENAME%_B.png
) else (
  set dm_WR_R=
  set dm_WR_G0=
  set dm_WR_G1=
  set dm_WR_B=
)


for /F "usebackq" %%L ^
in (`%IMG7%magick identify -format "WW=%%w\nHH=%%h" %INFILE%`) ^
do set /A %%L

set MULT_R=
set MULT_G0=
set MULT_G1=
set MULT_B=

if not "%2"=="" if not "%2"=="." set MULT_R=-evaluate Multiply %2
if not "%3"=="" if not "%3"=="." set MULT_G0=-evaluate Multiply %3
if not "%4"=="" if not "%4"=="." set MULT_G1=-evaluate Multiply %4
if not "%5"=="" if not "%5"=="." set MULT_B=-evaluate Multiply %5

set sFILT=

if not "%FILT%"=="" if not "%FILT%"=="." set sFILT=-filter %FILT%

set RESIZE=-resize "%WW%x%HH%^^^!"

if "%dmMIN_HALO%"=="1" (

  set shpAmount=70

  for /f "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "deblurValue=%%[fx:1-0.0011450938298236*!shpAmount!]" ^
    xc:`) do set %%L

  set FILT=-define filter:blur=!deblurValue! -filter Lanczos -distort Resize "%WW%x%HH%^^^!"

  set gamIn=-gamma 3
  set gamOut=-gamma 0.3333333333333333

  set RESIZE= ^
    ^( -clone 0 !gamIn! !FILT! !gamOut! ^) ^
    ^( -clone 0 !FILT! ^) ^
    -delete 0 ^
    ^( -clone 1 -colorspace gray -auto-level ^) ^
    -compose over -composite
)


%IMG7%magick ^
  %INFILE% ^
  %sFILT% ^
  ( -clone 0 -define sample:offset=25 -sample 50%% %MULT_R% ^
    %RESIZE% %dm_WR_R% ) ^
  ( -clone 0 ^
    ( -clone 0 -define sample:offset=75x25 -sample 50%% %MULT_G0% ^
      %RESIZE% %dm_WR_G0% ) ^
    ( -clone 0 -define sample:offset=25x75 -sample 50%% %MULT_G1% ^
      %RESIZE% %dm_WR_G1% ) ^
    -delete 0 ^
    -evaluate-sequence mean ^
  ) ^
  ( -clone 0 -define sample:offset=75 -sample 50%% %MULT_B% ^
    %RESIZE% %dm_WR_B% ) ^
  -delete 0 ^
  -combine ^
  %COMPR% ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

if "%dmDEBUG%"=="1" (
  set bpText=%~n1_R
  call %PICTBAT%blockPix %BASENAME%_R.png
  set bpText=%~n1_G0
  call %PICTBAT%blockPix %BASENAME%_G0.png
  set bpText=%~n1_G1
  call %PICTBAT%blockPix %BASENAME%_G1.png
  set bpText=%~n1_B
  call %PICTBAT%blockPix %BASENAME%_B.png
  set bpText=%~n1_RGB
  call %PICTBAT%blockPix %OUTFILE%
)


call echoRestore

endlocal & set dmOUTFILE=%OUTFILE%
