#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vsn_defines.h"


// The next function corresponds in style to functions in enhance.c
// It takes one image, and returns a status.
//
static MagickBooleanType EchoImage(const Image *image,
  int count,
  ExceptionInfo *exception)
{
  VirtualPixelMethod
    vpm;

  const char *mattecolor = GetImageArtifact (image, "mattecolor");

  if (!mattecolor) mattecolor = "black";

  printf ("mc=%s\n", mattecolor);


  assert(image != (Image *) NULL);
  assert(image->signature == MAGICK_CORE_SIG);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  printf ("echostuff: Input image [%i] [%s]  depth %i  size %ix%i\n",
          count,
          image->filename, (int)image->depth,
          (int)image->columns, (int)image->rows);

  vpm = GetImageVirtualPixelMethod(image);
  printf ("  Virtual pixel method %i [%s]\n",
   (int)vpm,
   GetImageArtifact(image, "virtual-pixel"));

  printf ("  mattecolor: %s\n", GetImageArtifact (image, "mattecolor"));

  printf ("  fuzz %g\n",       image->fuzz);

//  printf ("  channels %li\n",   image->channels);

  printf ("  alpha is on: %s\n", IS_ALPHA_CH(image) ? "yes" : "no");

  ResetImageArtifactIterator(image);
  const char * p = GetNextImageArtifact(image);
  while (p && *p) {
    printf ("Artifact=[%s]  value=[%s]\n",
      p,
      GetImageArtifact(image, p));
    p = GetNextImageArtifact(image);
  }

  return MagickTrue;
}


ModuleExport size_t echostuffImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  int
    i;

  MagickBooleanType
    status;


  (void) argc;
  (void) argv;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  if (argc) {
    printf ("echostuff: %i arguments:", argc);
    for (i=0; i < argc; i++) {
      printf (" [%s]", argv[i]);
    }
    printf ("\n");
  } else {
    printf ("echostuff: no arguments\n");
  }


  i = 0;
  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    status = EchoImage(image, i, exception);

    if (status == MagickFalse)
      continue;

    i++;
  }

  assert (status == MagickTrue);

  return(MagickImageFilterSignature);
}
