rem For image %1, detects "best" vertical line with black on the left and white on the right.
rem by long Sobel filter.
rem %2 is minumum length. Default 25 percent of shorter dimension.
rem
rem %3 is angle to be found, eg:
rem     0 for vertical with black on left,
rem    90 for horizontal with black on top,
rem   180 for vertical with black on right,
rem   270 for horizontal with black on right
rem %4 is prefix for line environment variable.
@rem
@rem Also uses:
@rem   elWINDOW eg 3x3
@rem   elDILATE eg 11. Default: no dilation
@rem   elOPEN_RECT eg 22. Default: no open rect.
@rem   elSOB_LEN length of Sobel-like edge detector. Typical range 3 to 100.
@rem
@rem See also edgeLinReg.bat
@rem
@rem Updated:
@rem   24-August-2022 for IM v7.
@rem

@rem This assumes "enabledelayedexpansion" is in effect.

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1


set elINFILE=%1

echo %0: elINFILE=%elINFILE%

set elMVG_FILE=%~dpn1_el.mvg

set elMIN_LEN=%2
if "%elMIN_LEN%"=="." set elMIN_LEN=

if "%elMIN_LEN%"=="" for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "elMIN_LEN=%%[fx:int(min(w,h)*0.25)]" ^
  %elINFILE%`) do set %%L

set elANG=%3
if "%elANG%"=="." set elANG=
if "%elANG%"=="" set elANG=0

set elLinePref=%4
if "%elLinePref%"=="." set elLinePref=
if "%elLinePref%"=="" set elLinePref=LINE

:: This should be more intelligent?
if "%elWINDOW%"=="" set elWINDOW=15x15

set elIS_VERT=0
if %elANG%==0 set elIS_VERT=1
if %elANG%==180 set elIS_VERT=1

if "%elOPEN_RECT%"=="" (
  set elOPEN_RECT=%elMIN_LEN%
)

if %elANG%==0 (
  set SOB_PARAMS=1 1
) else if %elANG%==90 (
  set SOB_PARAMS=0 1
) else if %elANG%==180 (
  set SOB_PARAMS=1 0
) else if %elANG%==270 (
  set SOB_PARAMS=0 0
)

if "%elSOB_LEN%"=="" set elSOB_LEN=%elMIN_LEN%
rem if %elSOB_LEN% GTR 100 set elSOB_LEN=100

call %PICTBAT%mLongSobel %elSOB_LEN% %SOB_PARAMS% elSOB.lis

if "%elDILATE%"=="" (
  set sDILATE=
) else if "%elDILATE%"=="0" (
  set sDILATE=
) else (
  if %elMIN_LEN% LSS %elDILATE% set elMIN_LEN=%elDILATE%
  if %elIS_VERT%==0 (
    set sDILATE=-morphology dilate rectangle:%elDILATE%x1
  ) else (
    set sDILATE=-morphology dilate rectangle:1x%elDILATE%
  )
)

if "%elOPEN_RECT%"=="" (
  set sOPEN_RECT=
) else if "%elOPEN_RECT%"=="0" (
  set sOPEN_RECT=
) else (

  if %elIS_VERT%==0 (
    set sOPEN_RECT=-morphology Open Rectangle:%elOPEN_RECT%x1
  ) else (
    set sOPEN_RECT=-morphology Open Rectangle:1x%elOPEN_RECT%
  )
)

if "%sDILATE%"=="" (
  set sOPEN_RECT=
)

echo %0: conv %elWINDOW%+%elMIN_LEN%  sOPEN_RECT=%sOPEN_RECT%  sDILATE=%sDILATE% elSOB_LEN=%elSOB_LEN%

set sOPEN_RECT=

rem type elSOB.lis

%IMG7%magick ^
  %elINFILE% ^
  -precision 19 ^
  -define "convolve:scale=^!" ^
  -morphology convolve @elSOB.lis ^
  -auto-level -level 50%%,80%% ^
  %sDILATE% ^
  el_x0.miff

rem   %sOPEN_RECT% ^
rem +write el_x1.miff ^
rem   %sDILATE% ^
rem +write el_x2.miff ^
rem   -hough-lines %elWINDOW%+%elMIN_LEN% ^
rem   %elMVG_FILE%

:: FIXME: but we do need sOPEN_RECT or sDILATE, for broken lines.

:: Or: from x0.png, made white where it isn't black, find the longest connected-component.

set nLongest=
set elLONGEST_LEN=0
for /F "usebackq skip=1 tokens=1-8 delims=:x+ " %%A in (`%IMG7%magick ^
  el_x0.miff ^
  -threshold 50%% ^
  -colorspace Gray -alpha off ^
  -clamp ^
  +write el_x0b.miff ^
  -define "connected-components:area-threshold=%elMIN_LEN%" ^
  -define "connected-components:verbose=true" ^
  -precision 3 ^
  -connected-components 8 ^
  NULL:`) do (

  rem echo %%A %%B %%C %%D %%E %%F %%G %%H

  set DO_IT=0
  if "%%H"=="gray(255)" set DO_IT=1
  if "%%H"=="gray(100%%)" set DO_IT=1
  if !DO_IT!==1 (
    for /F "usebackq" %%L in (`%IMG7%magick identify ^
      -format "LEN=%%[fx:hypot(%%B,%%C)]\nIsLonger=%%[fx:(hypot(%%B,%%C)>!elLONGEST_LEN!)?1:0]" ^
      xc:`) do set %%L
    if !IsLonger!==1 (
      set elLONGEST_LEN=!LEN!
      set nLongest=%%A
    )
  )
)
echo %0: elLONGEST_LEN=%elLONGEST_LEN% nLongest=%nLongest%

if %elLONGEST_LEN%==0 exit /B 1

%IMG7%magick ^
  el_x0b.miff ^
  -alpha set ^
  -define "connected-components:area-threshold=%elMIN_LEN%" ^
  -define "connected-components:remove=%nLongest%" ^
  -connected-components 8 ^
  +write el_x0c.miff ^
  -channel RGB -evaluate set 100%% +channel ^
  -background Black ^
  -compose Over ^
  -layers Flatten ^
  -negate ^
  el_x0d.miff

if ERRORLEVEL 1 exit /B 1

:: Skeletonize, prune (but _only_ if we have any junctions), find line ends.

:: For performance, trim first.
%IMG7%magick ^
  el_x0d.miff ^
  -trim -bordercolor Black -border 1 ^
  -virtual-pixel Edge ^
  -morphology Thinning:-1 Skeleton -clamp ^
  el_x0e.miff

:: Line may be nothing but stubs, so we can't remove them all.
call %PICTBAT%pruneStubs ^
  el_x0e.miff el_x0f.miff 5

call %PICTBAT%nonJcnLines ^
  el_x0e.miff el_x0g.miff

%IMG7%magick ^
  el_x0f.miff ^
  el_x0g.miff ^
  -compose Lighten -composite ^
  el_x0f.miff

:: Don't shave, because lines may extend to edges.
%IMG7%magick ^
  el_x0f.miff ^
  -background Black -layers Flatten ^
  -morphology HMT LineEnds ^
  -transparent Black ^
  sparse-color:el_endsp1.lis

sed -e 's/ /\n/g' el_endsp1.lis >el_endsp2.lis

type el_endsp2.lis

set II=0

for /F "tokens=1-2 delims=, " %%X in (el_endsp2.lis) do (
  if !II!==0 (
    set %elLinePref%.A.x=%%X
    set %elLinePref%.A.y=%%Y
    set %elLinePref%.B.x=%%X
    set %elLinePref%.B.y=%%Y
  ) else (
    if %elIS_VERT%==0 (
      if !%elLinePref%.A.x! GTR %%X (
        set %elLinePref%.A.x=%%X
        set %elLinePref%.A.y=%%Y
      )
      if !%elLinePref%.B.x! LSS %%X (
        set %elLinePref%.B.x=%%X
        set %elLinePref%.B.y=%%Y
      )
    ) else (
      if !%elLinePref%.A.y! GTR %%Y (
        set %elLinePref%.A.y=%%Y
        set %elLinePref%.A.x=%%X
      )
      if !%elLinePref%.B.y! LSS %%Y (
        set %elLinePref%.B.y=%%Y
        set %elLinePref%.B.x=%%X
      )
    )
  )
  set /A II+=1  
)

if %II% LSS 2 (
  echo %0: %II% ends found on line.
  exit /B 1
)

:: Now do the shave:
set /A %elLinePref%.A.x-=1
set /A %elLinePref%.A.y-=1
set /A %elLinePref%.B.x-=1
set /A %elLinePref%.B.y-=1

echo %0: done conv

echo elLONGEST_LEN=%elLONGEST_LEN%
set %elLinePref%.Len=%elLONGEST_LEN%

if %elLONGEST_LEN%==0 (
  echo %0: No lines found in [%elINFILE%]
  exit /B 1
)

exit /B 0
