rem Applies contrast-limited histogram-equalisation
rem (with iterative redistribution) as appropriate for four tiles,
rem   blending between the four quarters.
@rem
@rem Optional parameters:
@rem   %2 is limiting factor SD_FACT, so limit = mean + SD_FACT * standard_devition.
@rem        Default 1.
@rem   %3 is percentage lift for shadows. Default 0, no lift.
@rem   %4 is percentage drop for highlights. Default 0, no drop.
@rem   %5 is output file.
@rem
@rem Can also use:
@rem   eqlqDIFF_LIMIT iteration stops when the count redistribted is less than this percentage.
@rem     Default 1.0.
@rem
@rem Updated:
@rem   27-July-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 eqlq

set P2=%2
if "%P2%"=="" set P2=.
set P3=%3
if "%P3%"=="" set P3=.
set P4=%4
if "%P4%"=="" set P4=.

if not "%5"=="" set OUTFILE=%5

set eqlSUPPRESS_OUT=1
set eqlDIFF_LIMIT=%eqlqDIFF_LIMIT%

%IMG7%magick %INFILE% -crop 2x2@ +adjoin eqlq_tmp.miff
if ERRORLEVEL 1 exit /B 1

call %PICTBAT%eqLimit eqlq_tmp-0.miff %P2% %P3% %P4% eqlq_0
set eqlCLUT0=%eqlCLUT%

call %PICTBAT%eqLimit eqlq_tmp-1.miff %P2% %P3% %P4% eqlq_1
set eqlCLUT1=%eqlCLUT%

call %PICTBAT%eqLimit eqlq_tmp-2.miff %P2% %P3% %P4% eqlq_2
set eqlCLUT2=%eqlCLUT%

call %PICTBAT%eqLimit eqlq_tmp-3.miff %P2% %P3% %P4% eqlq_3
set eqlCLUT3=%eqlCLUT%

rem %IMG7%magick ^
rem   %INFILE% ^
rem   ( +clone %eqlCLUT0% -clut -write eqlq_tmp-0.miff +delete ) ^
rem   ( +clone %eqlCLUT1% -clut -write eqlq_tmp-1.miff +delete ) ^
rem   ( +clone %eqlCLUT2% -clut -write eqlq_tmp-2.miff +delete ) ^
rem   ( +clone %eqlCLUT3% -clut -write eqlq_tmp-3.miff +delete ) ^
rem   NULL:

%IMG7%magick ^
  %INFILE% ^
  ( -clone 0 %eqlCLUT0% -clut ) ^
  ( -clone 0 %eqlCLUT1% -clut ) ^
  ( -clone 0 %eqlCLUT2% -clut ) ^
  ( -clone 0 %eqlCLUT3% -clut ) ^
  ( -clone 0 -sparse-color Bilinear 0,0,#000,%%[fx:w-1],0,#fff ) ^
  -delete 0 ^
  ( -clone 0,1,4 -composite ) ^
  ( -clone 2,3,4 -composite ) ^
  -delete 0-4 ^
  ( -clone 0 -sparse-color Bilinear 0,0,#000,0,%%[fx:h-1],#fff ) ^
  -composite ^
  +depth ^
  %OUTFILE%

rem del eqlq_tmp-0.miff
rem del eqlq_tmp-1.miff
rem del eqlq_tmp-2.miff
rem del eqlq_tmp-3.miff

call echoRestore

@endlocal & set eqlqOUTFILE=%OUTFILE%
