rem Applies contrast-limited histogram-equalisation
rem (with iterative redistribution) as appropriate for NxM tiles,
rem   blending between the tiles.
@rem
@rem Optional parameters:
@rem   %2 is limiting factor SD_FACT, so limit = mean + SD_FACT * standard_devition.
@rem        Default 1.
@rem   %3 is percentage lift for shadows. Default 0, no lift.
@rem   %4 is percentage drop for highlights. Default 0, no drop.
@rem   %5 is number of columns of tiles. Deafult 3.
@rem   %6 is number of rows of tiles. Deafult 3.
@rem   %7 is output file.
@rem
@rem Can also use:
@rem   eqltDEBUG if 1, also creates curve histograms.
@rem   eqltDIFF_LIMIT iteration stops when the count redistribted is less than this percentage.
@rem     Default 1.0.
@rem   eqltLINEAR if 1, composes in linear colourspace.
@rem   eqltCLEANUP if not 1, doesn't remove temporary files
@rem
@rem Updated:
@rem   27-July-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 eqlt


set P2=%2
if "%P2%"=="" set P2=.
set P3=%3
if "%P3%"=="" set P3=.
set P4=%4
if "%P4%"=="" set P4=.

set N_COL=%5
if "%N_COL%"=="." set N_COL=
if "%N_COL%"=="" set N_COL=3

set N_ROW=%6
if "%N_ROW%"=="." set N_ROW=
if "%N_ROW%"=="" set N_ROW=3

if not "%7"=="" set OUTFILE=%7

if "%eqltCLEANUP%"=="" set eqltCLEANUP=1

set eqlSUPPRESS_OUT=1
set eqlDIFF_LIMIT=%eqltDIFF_LIMIT%

set VSN_LIST=%TEMP%\eqltVsn.lis
del %VSN_LIST% 2>nul

set CMD_FILE=%TEMP%\eqltCommand.bat

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -crop "%N_COL%x%N_ROW%@" +repage ^
  -precision 15 ^
  -format "MAX_N=%%p\n" ^
  -write INFO: ^
  +adjoin ^
  eqltTEMP_TILE.miff`) do @set %%L

echo MAX_N=%MAX_N%

set /A COL_X_ROWm1=%N_COL%*%N_ROW%-1

if not %MAX_N%==%COL_X_ROWm1% (
  echo %0: Bug: %MAX_N% neq %COL_X_ROWm1%
  exit /B 1
)

set eqlLofLAB=0

echo %IMG7%magick %INFILE% >%CMD_FILE%
if "%eqlLofLAB%"=="1" (
  echo -colorspace Lab >>%CMD_FILE%

  set COL_SP_IN=-channel R
  set COL_SP_OUT=+channel -colorspace sRGB
) else (
  set COL_SP_IN=
  set COL_SP_OUT=
)

for /L %%i in (0,1,%MAX_N%) do (
  call %PICTBAT%eqLimit eqltTEMP_TILE-%%i.miff %P2% %P3% %P4% eqlt_%%i
  set eqltCLUT%%i=!eqlCLUT!

  echo eqltCLUT%%i=!eqltCLUT%%i!

  set VSN=%TEMP%\eqltTEMP_TILE_%%i.miff
  echo ^( +clone %COL_SP_IN% !eqlCLUT! -clut %COL_SP_OUT% +write !VSN! +delete ^) >>%CMD_FILE%
  echo !VSN! >>%VSN_LIST%
)

cPrefix /i%CMD_FILE% /r"^"
echo NULL: >>%CMD_FILE%

type %CMD_FILE%

call %CMD_FILE%

set btLINEAR=%eqltLINEAR%

call %PICTBAT%blendTile %VSN_LIST% %N_COL% %N_ROW% %OUTFILE%
if ERRORLEVEL 1 exit /B 1

set eqlSUPPRESS_OUT=
set eqlDIFF_LIMIT=

rem FIXME: cleanup
if "%eqltCLEANUP%"=="1" (
  del %VSN_LIST%
  del %CMD_FILE%
  del %TEMP%\eqlt*.BAK
  for /L %%i in (0,1,%MAX_N%) do (
    del eqltTEMP_TILE-%%i.miff
    del %TEMP%\eqlt_%%i_*.miff
    del %TEMP%\eqltTEMP_TILE_%%i.miff
  )
)

call echoRestore

@endlocal & set eqltOUTFILE=%OUTFILE%
