rem From %1 a histogram,
rem makes a cumulative histogram but with specified slopes at shadow and highlight.
rem
rem %2 is percentage along x-axis for shadow slope. [0]
rem %3 is percentage along x-axis for start of highlight slope. [100]
rem %4 is percentage up y-axis for shadow slope. [same as %2]
rem %5 is percentage up y-axis for start of highlight slope. [same as %3]
rem %6 is optional output filename.
@rem
@rem Updated:
@rem   27-July-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 eqsh

set AX=%2
set BX=%3
set AY=%4
set BY=%5

if "%AX%"=="." set AX=
if "%BX%"=="." set BX=
if "%AY%"=="." set AY=
if "%BY%"=="." set BY=

if "%AX%"=="" set AX=0
if "%BX%"=="" set BX=100
if "%AY%"=="" set AY=%AX%
if "%BY%"=="" set BY=%BX%

rem echo %AX% %BX% %AY% %BY%

if not "%6"=="" set OUTFILE=%6

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "Wm1=%%[fx:w-1]\nAXP=%%[fx:int(w*%AX%/100+0.5)]\nBXP=%%[fx:int(w*%BX%/100+0.5)]\nAXP2=%%[fx:w-int(w*%AX%/100+0.5)]\nBXP2=%%[fx:w-int(w*%BX%/100+0.5)]\n" ^
  %INFILE%`) do set %%L

rem echo AXP=%AXP% BXP=%BXP% AXP2=%AXP2% BXP2=%BXP2%

if "%eqshDEBUG%"=="1" (
  set WR_G1=+write %~n1_G1.png
  set WR_G2=+write %~n1_G2.png
) else (
  set WR_G1=
  set WR_G2=
)

if %AX%==0 (
  set BKL=
  set G1=
) else (
  set BKL=-size %AXP%x1 xc:#000 -gravity West -composite

  set G1=^( -size %AXP%x1 gradient:black-gray^(%AY%%%^) ^
    -size %AXP2%x1 xc:#fff  ^
    +append ^
    %WR_G1% ^
  ^) ^
  -evaluate-sequence Min

)

if %BX%==100 (
  set BKR=
  set G2=
) else (
  set BKR=-size %BXP2%x1 xc:#000 -gravity East -composite

  set G2=^( -size %BXP%x1 xc:#000 ^
    -size %BXP2%x1 gradient:gray^(%BY%%%^)-white ^
    +append ^
    %WR_G2% ^
  ^) ^
  -evaluate-sequence Max

)

%IM7DEV%magick ^
  %INFILE% ^
  -set colorspace sRGB ^
  -compose Over ^
  %BKL% ^
  %BKR% ^
  -process 'cumulhisto norm' ^
  +level %AY%%%,%BY%%% ^
  %G1% ^
  %G2% ^
  %OUTFILE%

if "%eqshDEBUG%"=="1" (
  call %PICTBAT%graphLineCol %OUTFILE%
  call %PICTBAT%graphLineCol %~n1_G1.png
  call %PICTBAT%graphLineCol %~n1_G2.png
)

call echoRestore

@endlocal & set eqshOUTFILE=%OUTFILE%
