rem Given %1 is white quadrangle on black background,
rem with each quadrangle corner in corresponding quarter of the image,
rem write the four coordinate-pairs into environment variable prefix %2
rem with a polar distance method.
rem
rem Also creates debugging output %3. Blank or . or NULL: for no output image.
rem %4 is 0 (don't trim first) or 1 (do trim first). Default 1=do trim.
rem %5 is supersampling percentage, eg 300. Defalt 100=no supersampling.
rem %6 is three (quoted) parameters for drawCircs.bat: colour, radius, strokewidth.
@rem
@rem Updated:
@rem   25-August-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave


set INFILE=%1

set ENV_PREF=%2
if "%ENV_PREF%"=="." set ENV_PREF=
if "%ENV_PREF%"=="" set ENV_PREF=f4cENV

set OUTFILE=%3
if "%OUTFILE%"=="." set OUTFILE=
if /I "%OUTFILE%"=="NULL:" set OUTFILE=

set DO_TRIM=%4
if "%DO_TRIM%"=="." set DO_TRIM=
if "%DO_TRIM%"=="" set DO_TRIM=1

set SUPER=%5
if "%SUPER%"=="." set SUPER=
if "%SUPER%"=="" set SUPER=100

set dcPARAMS=%~6
if "%dcPARAMS%"=="." set dcPARAMS=
if "%dcPARAMS%"=="" set dcPARAMS=. . .

if %DO_TRIM%==1 (
  set sTRIM=-trim
) else (
  set sTRIM=
)

if %SUPER%==100 (
  set sSUPERSAMP=
) else (
  set sSUPERSAMP=-resize %SUPER%%%
)

set nFnd=0
for /F "usebackq tokens=1-6 delims=+x, " %%A in (`%IM7DEV%magick ^
  %INFILE% ^
  -precision 19 ^
  -virtual-pixel Black ^
  -format "input: %%P%%O\n" ^
  +write info: ^
  %sTRIM% ^
  -format "trimmed: %%wx%%h%%O\n" ^
  +write info: ^
  +repage ^
  -format "semidiag: %%[fx:hypot(w,h)/2]\n" ^
  +write info: ^
  %sSUPERSAMP% ^
  -distort Depolar -1 ^
  -flip ^
  -format "unrolled: %%wx%%h\n" ^
  +write info: ^
  -crop 4x1@ ^
  -process onewhite ^
  -format "offset: %%s %%wx%%h%%O  %%wx%%h\n" ^
  info: 2^>^&1`) do (
  rem echo %%A %%B %%C %%D %%E %%F
  if "%%A"=="input:" (
    set IN_WW=%%B
    set IN_HH=%%C
    set IN_XX=%%D
    set IN_YY=%%E
  ) else if "%%A"=="trimmed:" (
    set TRM_WW=%%B
    set TRM_HH=%%C
    set TRM_XX=%%D
    set TRM_YY=%%E
  ) else if "%%A"=="semidiag:" (
    set SEMIDIAG=%%B
  ) else if "%%A"=="unrolled:" (
    set UNR_WW=%%B
    set UNR_HH=%%C
  ) else if "%%A"=="offset:" (
    set OFFS[%%B].WW=%%C
    set OFFS[%%B].HH=%%D
    set OFFS[%%B].XX=%%E
    set OFFS[%%B].YY=%%F
  ) else if "%%A"=="onewhite:" (
    set POL[!nFnd!].x=%%B
    set POL[!nFnd!].y=%%C
    set /A nFnd+=1
  )
)

if not "%nFnd%"=="4" exit /B 1

rem set POL
rem set OFFS
rem set TRM_

for /L %%I in (0,1,3) do (
  call :doCalc theta "(!POL[%%I].x!+!OFFS[%%I].XX!)*2*pi/%UNR_WW%"
  call :doCalc rad "(1-!POL[%%I].y!/%UNR_HH%)*%SEMIDIAG%"

  call :doCalc C[%%I].x "%TRM_WW%/2-!rad!*sin(!theta!)+%TRM_XX%"
  call :doCalc C[%%I].y "%TRM_HH%/2-!rad!*cos(!theta!)+%TRM_YY%"

  rem echo theta=!theta! rad=!rad! x=!C[%%I].x! y=!C[%%I].y!
)


if not "%OUTFILE%"=="" call %PICTBAT%drawCircs %INFILE% %OUTFILE% %dcPARAMS% C[0] C[1] C[2] C[3]

set %ENV_PREF%.TL.x=%C[0].x%
set %ENV_PREF%.TL.y=%C[0].y%
set %ENV_PREF%.BL.x=%C[1].x%
set %ENV_PREF%.BL.y=%C[1].y%
set %ENV_PREF%.BR.x=%C[2].x%
set %ENV_PREF%.BR.y=%C[2].y%
set %ENV_PREF%.TR.x=%C[3].x%
set %ENV_PREF%.TR.y=%C[3].y%

set ENV_LIST=%BASENAME%_f4cpdEnv.lis
set %ENV_PREF% >%ENV_LIST%

call echoRestore

@endlocal & for /F %%L in (%ENV_LIST%) do @set %%L

@exit /B 0

::----------------------------------------------------
:: Subroutine

:doCalc
set %1=
for /F "usebackq" %%L in (`%IMG7%magick identify -precision 19 -format "%1=%%[fx:%~2]" xc:`) do set %%L

rem echo %1 is !%1!

if "!%1!"=="" exit /B 1

exit /B 0
