/* Updated:
     3-April-2018 for v7.0.7-28
     5-April-2018 bug: using ip when should be oq
*/

#ifndef FINDSINKS_INC
#define FINDSINKS_INC


static Image * LightenNonSinks (
  Image *image,
  MagickBooleanType verbose,
  ExceptionInfo *exception)
{
  // In blue channel, makes non-sinks QuantumRange.
  // A "sink" is a pixel,
  //   or connected group of pixels with the same value,
  // with all 8 neighbours lighter than it.
  // Thus, every pixel (or connected group) with a darker neighbour
  // is made white.

#if IMV6OR7==6
  //SetPixelCacheVirtualMethod (image, EdgeVirtualPixelMethod);
  SetImageVirtualPixelMethod (image, EdgeVirtualPixelMethod);
#else
  //SetPixelCacheVirtualMethod (image,
  //  EdgeVirtualPixelMethod, exception);
  SetImageVirtualPixelMethod (image, EdgeVirtualPixelMethod, exception);
#endif

  Image * new_img = CloneImage (image, 0, 0, MagickTrue, exception);
  if (!new_img) return NULL;

  CacheView * in_view = AcquireVirtualCacheView (image, exception);
  CacheView * out_view = AcquireAuthenticCacheView (new_img, exception);

  MagickBooleanType okay = MagickTrue;
  const ssize_t dy1 = image->columns+2;
  const ssize_t dy2 = 2 * (image->columns+2);
  ssize_t y;
  const int ip = Inc_ViewPixPtr (image);
  const int oq = Inc_ViewPixPtr (new_img);

  for (y = 0; y < image->rows; y++) {
    if (!okay) continue;

    VIEW_PIX_PTR *q = GetCacheViewAuthenticPixels(
      out_view,0,y,image->columns,1,exception);
    if (!q) {okay = MagickFalse; continue; }

    const VIEW_PIX_PTR *pIn = GetCacheViewVirtualPixels(
      in_view,-1,y-1,image->columns+2,3,exception);
    if (!pIn) {okay = MagickFalse; continue; }

    ssize_t x;
    for (x = 0; x < image->columns; x++) {

      double score = GET_PIXEL_BLUE (new_img, q);

      if (score < QuantumRange) {
        if (
        score > GET_PIXEL_BLUE (image, pIn + (x+0)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (x+1)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (x+2)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (dy1+x+0)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (dy1+x+1)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (dy1+x+2)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (dy2+x+0)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (dy2+x+1)*ip)
     || score > GET_PIXEL_BLUE (image, pIn + (dy2+x+2)*ip)
           )
        {
          SET_PIXEL_BLUE (new_img, QuantumRange, q);

          // If any pixels to the right have the same score,
          // set them to QuantumRange.
          if (x < image->columns-1) {
            ssize_t x2;
            VIEW_PIX_PTR *q2 = q + ip;
            for (x2=x+1; x2 < image->columns; x2++) {
              double score2 = GET_PIXEL_BLUE (new_img, q2);
              if (score2 != score) break;
              SET_PIXEL_BLUE (new_img, QuantumRange, q2);
              q2 += ip;
            }
          }

        }
      }
      // (pIn is _not_ incremented)
      q += oq;
    }  // for x
    if (SyncCacheViewAuthenticPixels(out_view,exception) == MagickFalse)
      okay = MagickFalse;
  }
  if (!okay) return NULL;

  in_view = DestroyCacheView (in_view);


  // To get all the pixels on a terrace, we iterate:
  // every pixel that is next to a white pixel,
  // where that white pixel was the same colour as this pixel,
  // is made white.

  MagickBooleanType anyChanged = MagickTrue;
  int nIter = 0;

  while (anyChanged) {
    anyChanged = MagickFalse;

    CacheView * out_view_rdm = AcquireVirtualCacheView (new_img, exception);
    CacheView * out_view_rdp = AcquireVirtualCacheView (new_img, exception);

    CacheView * in_view0 = AcquireVirtualCacheView (image, exception);
    CacheView * in_view1 = AcquireVirtualCacheView (image, exception);
    CacheView * in_view2 = AcquireVirtualCacheView (image, exception);

    for (y = 0; y < image->rows; y++) {
      if (!okay) continue;

      VIEW_PIX_PTR *q = GetCacheViewAuthenticPixels(
        out_view,0,y,image->columns,1,exception);
      if (!q) {okay = MagickFalse; continue; }

      const VIEW_PIX_PTR *qm1 = GetCacheViewVirtualPixels(
        out_view_rdm,-1,y-1,image->columns+2,1,exception);
      if (!qm1) {okay = MagickFalse; continue; }

      const VIEW_PIX_PTR *qp1 = GetCacheViewVirtualPixels(
        out_view_rdp,-1,y+1,image->columns+2,1,exception);
      if (!qp1) {okay = MagickFalse; continue; }

      qm1 += oq;
      qp1 += oq;

      const VIEW_PIX_PTR *p0 = GetCacheViewVirtualPixels(
        in_view0,-1,y-1,image->columns+2,1,exception);
      if (!p0) {okay = MagickFalse; continue; }

      const VIEW_PIX_PTR *p1 = GetCacheViewVirtualPixels(
        in_view1,-1,y,image->columns+2,1,exception);
      if (!p1) {okay = MagickFalse; continue; }

      const VIEW_PIX_PTR *p2 = GetCacheViewVirtualPixels(
        in_view2,-1,y+1,image->columns+2,1,exception);
      if (!p2) {okay = MagickFalse; continue; }

      p0 += ip;
      p1 += ip;
      p2 += ip;

      ssize_t x;
      for (x = 0; x < image->columns; x++) {
        double score = GET_PIXEL_BLUE (new_img, q);
        if (score < QuantumRange) {

          // FIXME: for now,
          //  just the pixels directly above, left, right and down.
          if (
                (    score == GET_PIXEL_BLUE (image, p0)
                  && GET_PIXEL_BLUE (new_img, qm1)==QuantumRange)
             || (    x > 0
                  && score == GET_PIXEL_BLUE (image, p1-ip)
                  && GET_PIXEL_BLUE (new_img, q-ip)==QuantumRange)
             || (    x < image->columns-1
                  && score == GET_PIXEL_BLUE (image, p1+ip)
                  && GET_PIXEL_BLUE (new_img, q+ip)==QuantumRange)
             || (    score == GET_PIXEL_BLUE (image, p2)
                  && GET_PIXEL_BLUE (new_img, qp1)==QuantumRange)
             )
          {
            SET_PIXEL_BLUE (new_img, QuantumRange, q);
            anyChanged = MagickTrue;
          }
        }
        q += oq;
        qm1 += oq;
        qp1 += oq;
        p0 += ip;
        p1 += ip;
        p2 += ip;
      }
      if (SyncCacheViewAuthenticPixels(out_view,exception) == MagickFalse)
        okay = MagickFalse;
    }
    if (!okay) break;

    in_view2 = DestroyCacheView (in_view2);
    in_view1 = DestroyCacheView (in_view1);
    in_view0 = DestroyCacheView (in_view0);

    out_view_rdp = DestroyCacheView (out_view_rdp);
    out_view_rdm = DestroyCacheView (out_view_rdm);

    nIter++;
  } // while

  if (!okay) return NULL;

  out_view = DestroyCacheView (out_view);

  if (verbose) fprintf (stderr, "LightenNonSinks: nIter %i\n", nIter);
  return new_img;
}


#endif
