rem Animated fractal noise.
rem Given image %9, makes frames with %9 displaced by varying fractal noise.

@rem %1 number of frames [5]
@rem %2 quoted string of four parameters to fractNoise.bat [". . . ."]
@rem   (width and height are determined by %9.)
@rem %3 prefix for frames and output gif [fna2_]
@rem %4 colour variation percent per frame [0]
@rem %5 dx, pixels per frame [0]
@rem %6 dy, pixels per frame [0]
@rem %7 zoom factor per frame eg 0.998 [1]
@rem %8 power factor: 0 = all frequencies change equally. [0]
@rem
@rem %9 image to be displaced
@rem
@rem Also uses:
@rem
@rem   fnA_SUPER_SAMP percentage for supersampling, eg 400. If blank or 100, no supersamplimg.
@rem   fnA_DISPLACE_ARGS
@rem   fnA_SINCOS makes fractal grayscale, and applies sin N to R and cos N to G.
@rem
@rem Updated:
@rem   24-September-2022 for IM v7.
@rem


@rem No compulsory parameters
@rem if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 fa2


set NUM_FR=%1
if "%NUM_FR%"=="." set NUM_FR=
if "%NUM_FR%"=="" set NUM_FR=5

set FN_PARAM=%~2
if "%FN_PARAM%"=="." set FN_PARAM=
if "%FN_PARAM%"=="" set FN_PARAM=. . . .

set PREF=%3
if "%PREF%"=="." set PREF=
if "%PREF%"=="" set PREF=fna_fire_

set fnA_COL=%4
if "%fnA_COL%"=="." set fnA_COL=
if "%fnA_COL%"=="" set fnA_COL=0

set fnA_DX=%5
if "%fnA_DX%"=="." set fnA_DX=
if "%fnA_DX%"=="" set fnA_DX=0

set fnA_DY=%6
if "%fnA_DY%"=="." set fnA_DY=
if "%fnA_DY%"=="" set fnA_DY=0

set ZM=%7
if "%ZM%"=="." set ZM=
if "%ZM%"=="." set ZM=1

set fnA_POW_FAC=%8
if "%fnA_POW_FAC%"=="." set fnA_POW_FAC=
if "%fnA_POW_FAC%"=="" set fnA_POW_FAC=0

set SRC=%9
if "%SRC%"=="" set SRC=fn_fire.png

set WW=
for /F "usebackq" %%L in (`%IMG7%magick identify -format "WW=%%w\nHH=%%h" %SRC%`) do set %%L
if "%WW%"=="" (
  echo %0: can't find [%SRC%]
  exit /B 1
)


if "%fnA_SUPER_SAMP%"=="" set fnA_SUPER_SAMP=100

if "%fnA_SUPER_SAMP%"=="100" (
  set SS_UP=
  set SS_DN=
) else (
  set SS_UP=-resize %fnA_SUPER_SAMP%%%
  set SS_DN=-resize something
)

set SRC_SSUP=%PREF%_ssup.miff

%IMG7%magick ^
  %SRC% ^
  %SS_UP% ^
  %SRC_SSUP%

if "%fnA_DISPLACE_ARGS%"=="" set fnA_DISPLACE_ARGS=10%%%%x50%%%%

if "%fnA_SINCOS%"=="" (
  set sSINCOS=
) else (
  set sSINCOS=-channel R -evaluate sin %fnA_SINCOS% -channel G -evaluate cos %fnA_SINCOS%
)

set fnAUTO_LEVEL_EACH=0
set fnAUTO_LEVEL_END=0

set FRAME_EXT=.miff

set /A LAST_FR=%NUM_FR%-1

del %PREF%_fr??????%FRAME_EXT% 2>nul

set fnA_FRM_NUM=0
call %PICTBAT%fnInit %WW% %HH% %FN_PARAM%
  if ERRORLEVEL 1 (
    echo %0: fnInit failure
    exit /B 1
  )

for /L %%I in (0,1,%LAST_FR%) do (

  set LZ=000000%%I
  set LZ=!LZ:~-6!

  set fnA_FRM_NUM=%%I

  call %PICTBAT%fnOne ^
    %PREF%_fr!LZ!%FRAME_EXT%

  if ERRORLEVEL 1 (
    echo %0: fnOne failure
    exit /B 1
  )

  if not "%afSUPER_SAMP%"=="100" (
    set SS_DN=-resize "!fnWW!x!fnHH!^!"
  )

  %IMG7%magick ^
    %SRC_SSUP% ^
    ^( %PREF%_fr!LZ!%FRAME_EXT% ^
      %SS_UP% ^
      -auto-gamma ^
      -sigmoidal-contrast 10x50%% ^
      %sSINCOS% ^
    ^) ^
    -compose Displace ^
      -set option:compose:args %fnA_DISPLACE_ARGS% ^
      -composite ^
    !SS_DN! ^
    %PREF%_fr!LZ!%FRAME_EXT%

  if ERRORLEVEL 1 (
    echo %0: magick failure for SRC_SSUP [%SRC_SSUP%]
    exit /B 1
  )

  echo %~n0: %fnOUTFILE%
)

call %PICTBAT%frMinMax %PREF%_fr??????%FRAME_EXT%
if ERRORLEVEL 1 exit /B 1

call %PICTBAT%frGamma %PREF%_fr??????%FRAME_EXT% . 1
if ERRORLEVEL 1 exit /B 1

%IMG7%magick ^
  -loop 0 -delay 20 ^
  %PREF%_fr*%FRAME_EXT% ^
  -level %fmmMIN_PC%%%,%fmmMAX_PC%%% ^
  %PREF%.gif


call echoRestore

@endlocal & set faOUTFILE=%OUTFILE%
