rem Fractal noise initialisation: sets environment variables

@rem %1 %2 Width, height in pixels.
@rem %3 minimum block size in pixels. [1]
@rem %4 maximum block size in pixels. [min(w,h)/2]
@rem %5 block factor: Block size is increased by this factor from min to max. >=1 [2]
@rem %6 power factor for amplitude proportional to frequency. [Default 0.0]
@rem   0 for all frequencies have same amplitude;
@rem   > 0 (eg 1, 1.5) for highest frequencies have lowest amplitude.
@rem
@rem Updated:
@rem   24-September-2022 for IM v7
@rem

@rem No compulsory parameters
@rem No setlocal

set fix=99
if not "!fix!"=="99" (
  echo %0: must be run in enabledelayedexpansion
  exit /B 1
)
set fix=


set fnWW=%1
if "%fnWW%"=="." set fnWW=
if "%fnWW%"=="" set fnWW=600
set fnHH=%2
if "%fnHH%"=="." set fnHH=
if "%fnHH%"=="" set fnHH=400

set BLK_MIN=%3
if "%BLK_MIN%"=="." set BLK_MIN=
if "%BLK_MIN%"=="" set BLK_MIN=1
set BLK_MAX=%4
if "%BLK_MAX%"=="." set BLK_MAX=
if "%BLK_MAX%"=="" (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "BLK_MAX=%%[fx:max(%fnWW%,%fnHH%)/2]" ^
    xc:`) do set %%L
)

set BLK_FAC=%5
if "%BLK_FAC%"=="." set BLK_FAC=
if "%BLK_FAC%"=="" set BLK_FAC=2

set POW_FAC=%6
if "%POW_FAC%"=="." set POW_FAC=
if "%POW_FAC%"=="" set POW_FAC=0

echo %0: %fnWW% %fnHH% %BLK_MIN% %BLK_MAX% %BLK_FAC% %POW_FAC%

if "%fnIM%"=="" set fnIM=%IMG7%

if "%fnPREFIX%"=="" set fnPREFIX=fn_

if "%fnNG_EXT%"=="" set fnNG_EXT=.tiff

set pyIM=%fnIM%

set pyPREFIX=%fnPREFIX%
set pyLINEAR=%fnLINEAR%

set pyWEIGHTING=sum

set BLK_LIS=%fnPREFIX%blk.lis

set NUM_OCTAVES=

rem call %PICTBAT%mkpVar %fnWW% %fnHH% %3 %4 %5 %6
call %PICTBAT%mkpVar %fnWW% %fnHH% %BLK_MIN% %BLK_MAX% %BLK_FAC% %POW_FAC%
if ERRORLEVEL 1 (
  echo %0: mkpVar failed
  exit /B 1
)

rem if not exist %BLK_LIS% (
rem   echo %0: Can't find BLK_LIS [%BLK_LIS%]
rem   exit /B 1
rem )
rem 
rem 
rem for /F "tokens=*" %%L in (%BLK_LIS%) do set "%%L"

if "%NUM_OCTAVES%"=="" (
  echo %0: NUM_OCTAVES has no value
  exit /B 1
)

if not "%fnA_FRM_NUM%"=="" call %PICTBAT%mkpAnimVar

if "%fnNOISE_TYPE%"=="" (
  set NSE=+noise Random
) else (
  set NSE=+noise %fnNOISE_TYPE%
)

if "%fnGRAY%"=="1" (
  set "sBASE_COL=gray^^^(50%%%%^^^)"
  set sONE_CHAN=-channel R -separate +channel
) else (
  set "sBASE_COL=rgb^^^(50%%%%,50%%%%,50%%%%^^^)"
  set sONE_CHAN=
)

if "%fnAUTO_LEVEL_EACH%"=="1" (
  set sAUTO_EACH=-auto-level
) else if "%fnAUTO_LEVEL_EACH%"=="2" (
  set sAUTO_EACH=-channel RGB -auto-level +channel
) else (
  set sAUTO_EACH=
)

if "%fnAUTO_LEVEL_END%"=="" set fnAUTO_LEVEL_END=1

if "%fnAUTO_LEVEL_END%"=="1" (
  set sAUTO_END=-auto-level
) else if "%fnAUTO_LEVEL_END%"=="2" (
  set sAUTO_END=-channel RGB -auto-level +channel
) else (
  set sAUTO_END=
)

if "%fnAUTO_GAM_END%"=="" set fnAUTO_GAM_END=1

if "%fnAUTO_GAM_END%"=="1" (
  set sAUTO_END=%sAUTO_END% -auto-gamma
)

if "%fnABS_NSE%"=="1" (
  set sABS=-solarize 50%%%% -negate
) else (
  set sABS=
)

if "%fnA_FRM_NUM%"=="" (
  set RD_PYR=
  set WR_PYR=
  if "%fnA_COLCYCMETH%"=="" set fnA_COLCYCMETH=null
  call %PICTBAT%colCyclStr !fnA_COLCYCMETH! 0
) else (
  if "%fnA_COLCYCMETH%"=="" set fnA_COLCYCMETH=sinarc2o
  if "%fnA_FRM_NUM%"=="0" (
    set RD_PYR=
    set WR_PYR=1
    set ZM_F=1
    set DX_F=0
    set DY_F=0
rem  ) else (
rem Wrong place! fnInit is run _before_ we know an actual frame number.
rem But it may be called with fnA_FRM_NUM=0, to initialise for an animation.
rem    set RD_PYR=1
rem    set WR_PYR=
rem
rem    call xyCoord %fnWW% %fnHH% %fnA_DX% %fnA_DY%
rem
rem    for /F "usebackq" %%L in (`%IMG7%magick identify ^
rem      %PREC% ^
rem      -format "ZM_F=%%[fx:pow(%fnA_ZM%,%fnA_FRM_NUM%)]\nDX_F=%%[fx:!xycX!*%fnA_FRM_NUM%]\nDY_F=%%[fx:!xycY!*%fnA_FRM_NUM%]" ^
rem      xc:`) do set %%L
rem
rem    if "%fnTXT_OUT%"=="1" echo fnA_FRM_NUM=%fnA_FRM_NUM% ZM_F=!ZM_F! DX_F=!DX_F! DY_F=!DY_F!
  )
)

if not "%fnRD_PYR%"=="" set RD_NG=%fnRD_PYR%
if not "%fnWR_PYR%"=="" set WR_PYR=%fnWR_PYR%

if not "%fnTXT_OUT%"=="" set pyTXT_OUT=%fnTXT_OUT%

rem No endlocal
