rem From %1, a list of focus-stack images, makes a composite image.
rem If %2 is given, that will be the output image file.
@rem
@rem Also uses:
@rem   fsMAKE_VP_MASK if 0, don't make virtual-pixel mask.
@rem
@rem
@rem Updated:
@rem   22-August-2022 Upgraded for IM v7.
@rem



@rem fsFILES.Cnt
@rem Structure of fsFILES[n]
@rem   .SrcFile
@rem   .Aligned
@rem   .Angle
@rem   .Scale
@rem   .DX
@rem   .DY
@rem   .CumulAngle
@rem   .CumulScale
@rem   .CumulDX
@rem   .CumulDY
@rem
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

rem @call echoOffSave

call %PICTBAT%setInOut %1 fs


if not exist %INFILE% (
  echo Can't find INFILE [%INFILE%]
  exit /B 1
)

if not "%2"=="" set OUTFILE=%2

set TMPDIR=%TEMP%
set TMPEXT=.miff
set TMPOUT=%TMPDIR%\%INNAME%_fs_out%TMPEXT%
set DUMP_DATA=%TMPDIR%\%INNAME%_fs_data.lis
set VP_MASK=%TMPDIR%\%INNAME%_fs_vpmask.miff


if exist %DUMP_DATA% (
  for /F %%L in (%DUMP_DATA%) do set %%L
)


rem ------------------------------------------
echo Read the list of input photos.

set i=0
for /F "tokens=*" %%F in (%INFILE%) do (
  echo %%F
  set fsFILES[!i!].SrcFile=%%F
  set /A i+=1
)
set /A fsFILES.Cnt=%i%

if %fsFILES.Cnt% LSS 2 (
  echo Need at least 2 entries in INFILE [%INFILE%]
  exit /B 1
)

set /A LastNum=fsFILES.Cnt-2
set /A CntM1=fsFILES.Cnt-1

rem goto masks
goto findScales



if not "%fsMAKE_VP_MASK%"=="0" %IMG7%magick ^
  %fsFILES[0].SrcFile% ^
  -fill White -colorize 100 ^
  %VP_MASK%

if ERRORLEVEL 1 exit /B 1


rem ------------------------------------------
echo Find the angle of each against the next.

for /L %%i in (0,1,%CntM1%) do (
  set fsFILES[%%i].Angle=0
)

if 1==0 goto skipFindAngle

set wrDO_SCORE_SUB=0
set wrDEBUG=

for /L %%i in (0,1,%LastNum%) do (
  echo Find angle of %%i
  set /A j=%%i+1

  call :NextSrcFile !j!

  call %PICTBAT%whatRot ^
    !fsFILES[%%i].SrcFile! ^
    !NextSrcFile!

  if ERRORLEVEL 1 exit /B 1

  set fsFILES[%%i].Angle=!wrANGLE!
)

:skipFindAngle


rem ------------------------------------------
echo Find the scale of each against the next.

:findScales

for /L %%i in (0,1,%CntM1%) do (
  set fsFILES[%%i].Scale=1
)

if 1==1 goto skipFindScale

rem FIXME: We should apply angle

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  -format "PREC=%%[fx:1+1/max(w,h)]" %fsFILES[0].SrcFile%`) ^
do set %%L

echo PREC=%PREC%

set MIN_SCALE=0.9
set MAX_SCALE=1.1

for /L %%i in (0,1,%LastNum%) do (
  echo Find scale of %%i
  set /A j=%%i+1

  call :NextSrcFile !j!

  call %PICTBAT%whatScalePrec ^
    !fsFILES[%%i].SrcFile! ^
    !NextSrcFile! ^
    %MIN_SCALE% %MAX_SCALE% 10 %PREC%

  if ERRORLEVEL 1 exit /B 1

  set fsFILES[%%i].Scale=!wspSCALE!
)

:skipFindScale

rem -------------------------------------
echo Calculate cumulative scales.

set fsFILES[%CntM1%].CumulScale=1

for /L %%i in (%LastNum%,-1,0) do (
  set /A j=%%i+1
  call :NextCumulScale !j!

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 9 ^
    -format "CumulScale=%%[fx:!NextCumulScale!*!fsFILES[%%i].Scale!]" ^
    xc:`) do set %%L

  set fsFILES[%%i].CumulScale=!CumulScale!
)


rem ----------------------------------------
echo Calculate cumulative angles.

set fsFILES[%CntM1%].CumulAngle=0

for /L %%i in (%LastNum%,-1,0) do (
  set /A j=%%i+1

  call :NextCumulAngle !j!

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 9 ^
    -format "CumulAngle=%%[fx:VV=!NextCumulAngle!+!fsFILES[%%i].Angle!;VV<360?VV:VV-360]" ^
    xc:`) do set %%L

  set fsFILES[%%i].CumulAngle=!CumulAngle!
)


rem --------------------------------------
echo Scale and rotate each image.
for /L %%i in (0,1,%LastNum%) do (
  echo Scale and rotate %%i

  set OutScale=%TMPDIR%\%INNAME%_fs_%%i%TMPEXT%

  %IMG7%magick ^
    !fsFILES[%%i].SrcFile! ^
    -virtual-pixel Edge -distort SRT !fsFILES[%%i].CumulScale!,!fsFILES[%%i].CumulAngle! ^
    !OutScale!

  set fsFILES[%%i].Aligned=!OutScale!
)

set fsFILES[%CntM1%].Aligned=!fsFILES[%CntM1%].SrcFile!

rem ----------------------------------------
echo Find translations between scaled files.

for /L %%i in (0,1,%LastNum%) do (
  echo Find translation %%i

  set /A j=%%i+1
  call :NextAligned !j!

  call %PICTBAT%whatTrans ^
    !fsFILES[%%i].Aligned! ^
    !NextAligned!

  set fsFILES[%%i].DX=!wtX!
  set fsFILES[%%i].DY=!wtY!
)

rem ----------------------------------------
echo Calculate cumulative translations.

set fsFILES[%CntM1%].CumulDX=0
set fsFILES[%CntM1%].CumulDY=0

for /L %%i in (%LastNum%,-1,0) do (
  set /A j=%%i+1

  set /A fsFILES[%%i].CumulDX=fsFILES[%%i].DX+fsFILES[!j!].CumulDX
  set /A fsFILES[%%i].CumulDY=fsFILES[%%i].DY+fsFILES[!j!].CumulDY
)


rem ----------------------------------------
echo Scale, rotate and translate images.

rem FIXME: The translate will be an integer number of pixels,
rem so we can save time by not repeating the scale.

for /F "usebackq" %%L ^
in (`%IMG7%magick !fsFILES[0].SrcFile! ^
  -format "x0=%%[fx:int(w/2)]\ny0=%%[fx:int(h/2)]" ^
  info:`) ^
do set %%L

for /L %%i in (0,1,%LastNum%) do (
  echo Scale, rotate and translate %%i

  set /A x1=%x0%+fsFILES[%%i].CumulDX
  set /A y1=%y0%+fsFILES[%%i].CumulDY

  echo fsFILES[%%i].CumulScale=!fsFILES[%%i].CumulScale!

  set sSRT=!x0!,!y0!,!fsFILES[%%i].CumulScale!,!fsFILES[%%i].CumulAngle!,!x1!,!y1!
  echo sSRT=!sSRT!

  %IMG7%magick ^
    !fsFILES[%%i].SrcFile! ^
    -virtual-pixel Edge -distort SRT !sSRT! ^
    !fsFILES[%%i].Aligned!

  if not "%fsMAKE_VP_MASK%"=="0" %IMG7%magick ^
    %VP_MASK% ^
    -virtual-pixel Black -distort SRT !sSRT! ^
    %VP_MASK%
)


rem ----------------------------------------
echo Make and apply masks

:masks

set wpDEBUG=

rem Merge 0 with 1 ...

call %PICTBAT%whichPix ^
  !fsFILES[0].Aligned! ^
  !fsFILES[1].Aligned! ^
  %TMPOUT%

rem ... then merge result with 2, 3, ...CntM1

for /L %%i in (2,1,%CntM1%) do (
  echo Mask %%i

  call %PICTBAT%whichPix ^
    %TMPOUT% ^
    !fsFILES[%%i].Aligned! ^
    %TMPOUT%
)


rem ----------------------------------------
echo Wrap up.

%IMG7%magick %TMPOUT% %OUTFILE%

rem Dump the data to a file.
set fsFILES >%DUMP_DATA%


call echoRestore

endlocal & set fsOUTFILE=%OUTFILE%& set fsVP_MASK=%VP_MASK%

exit /B 0


rem ----------------------------------------
rem Subroutines.

:NextSrcFile
set NextSrcFile=!fsFILES[%1].SrcFile!
exit /B 0

:NextCumulScale
set NextCumulScale=!fsFILES[%1].CumulScale!
exit /B 0

:NextCumulAngle
set NextCumulAngle=!fsFILES[%1].CumulAngle!
exit /B 0

:NextAligned
set NextAligned=!fsFILES[%1].Aligned!
exit /B 0

