rem Given %1 is a fabric,
rem %2 is a white line on black background,
rem distorts the fabric to make the central vertical column follow the line.
rem %3 is optional output file.
rem Writes displacement map to fl_disp_map_abs.miff
@rem
@rem Also uses:
@rem
@rem   flDEBUG_TXT if not blank, deletes this file then echoes debugging text to it.
@rem   flFAB_TRANS valid IM transformation to apply to the fabric.
@rem     Eg -flop or -flip or both.
@rem
@rem Updated:
@rem   18-August-2022 for IM v7.
@rem


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 fl

if not "%3"=="" set OUTFILE=%3

set TMPDIR=\temp\

if "%flDEBUG_TXT%"=="" (
  set flDEBUG_TXT=nul
) else (
  del %flDEBUG_TXT% >nul
)

set ERR=0

set LINE_FILE=%2

set APPROX_LEN_PC=90


set WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nDIAG=%%[fx:int(hypot(w,h)+0.5)]" ^
  %INFILE%`) do set %%L

if "%WW%"=="" (
  echo Can't open INFILE [%INFILE%]
  exit /B 1
)

set OKAY=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "OKAY=%%[fx:w==%WW%&&h==%HH%?1:0]" ^
  %LINE_FILE%`) do set %%L

if "%OKAY%"=="" (
  echo Can't open LINE_FILE [%LINE_FILE%]
  exit /B 1
)

if not "%flDEBUG_TXT%"=="nul" echo WW=%WW% HH=%HH% OKAY=%OKAY% >>%flDEBUG_TXT%

if not "%OKAY%"=="1" (
  echo Sizes don't match [%INFILE%] [%LINE_FILE%]
  exit /B 1
)

for /F "usebackq tokens=2,3 delims=:, " %%X in (`%IM7DEV%magick ^
  %LINE_FILE% ^
  -virtual-pixel Black ^
  -morphology HMT LineEnds ^
  -process onewhite ^
  %TMPDIR%fl_two_ends.miff 2^>^&1`) do (
  set X0=%%X
  set Y0=%%Y
)

if "%X0%"=="none" (
  echo %0: no line ends [%LINE_FILE%]
  set ERR=1
  goto end
)

for /F "usebackq tokens=2,3 delims=:, " %%X in (`%IM7DEV%magick ^
  %TMPDIR%fl_two_ends.miff ^
  -fill Black -draw "point %X0%,%Y0%" ^
  -process onewhite ^
  NULL: 2^>^&1`) do (
  set X1=%%X
  set Y1=%%Y
)

if "%X1%"=="none" (
  echo %0: no second line end [%LINE_FILE%]
  set ERR=1
  goto end
)

if not "%flDEBUG_TXT%"=="nul" echo X0=%X0% Y0=%Y0%  X1=%X1% Y1=%Y1% >>%flDEBUG_TXT%


::===

for /F "usebackq" %%L in (`%IMG7%magick ^
  %LINE_FILE% ^
  -format "APPROX_LEN=%%[fx:int(max(hypot(%X1%-%X0%,%Y1%-%Y0%),mean*w*h+0.5))]" ^
  INFO:`) do set %%L

if not "%flDEBUG_TXT%"=="nul" echo APPROX_LEN=%APPROX_LEN% >>%flDEBUG_TXT%

set /A KNL_SCALE_V=65535*%APPROX_LEN_PC%/%APPROX_LEN%/100

echo %0: Start IterativeDistance   KNL_SCALE_V=%KNL_SCALE_V%
:: Assume next is Q16.
%IMG7%magick ^
  %LINE_FILE% ^
  -negate +write mpr:MASK ^
  -fill White -colorize 100 ^
  -fill Black -draw "point %X0%,%Y0%" ^
  -write-mask mpr:MASK ^
  -morphology ^
    IterativeDistance:%DIAG% ^
    Euclidean:7,%KNL_SCALE_V% ^
  +write-mask ^
  +depth ^
  -define quantum:format=floating-point -depth 32 ^
  %TMPDIR%fl_line_grad0.miff
echo %0: Finished IterativeDistance

if not "%flDEBUG_TXT%"=="nul" echo KNL_SCALE_V=%KNL_SCALE_V% >>%flDEBUG_TXT%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %LINE_FILE% ^
  %TMPDIR%fl_line_grad0.miff ^
  -compose Darken -composite ^
  -format "MAX_01=%%[fx:maxima]\nMAX_GRAY=%%[fx:maxima*QuantumRange]\nLINE_LEN=%%[fx:maxima*QuantumRange/%KNL_SCALE_V%]" ^
  info:`) do set %%L

if not "%flDEBUG_TXT%"=="nul" echo MAX_01=%MAX_01% MAX_GRAY=%MAX_GRAY% LINE_LEN=%LINE_LEN% >>%flDEBUG_TXT%

if "%MAX_01%"=="1" (
  echo %0: Bad first MAX_01 == %MAX_01%
  set ERR=1
  goto end
)

for /F "usebackq" %%L in (`%IMG7%magick ^
  %LINE_FILE% ^
  %TMPDIR%fl_line_grad0.miff ^
  -compose Darken -composite ^
  -format "KNL_SCALE_V=%%[fx:%KNL_SCALE_V%/maxima]" ^
  info:`) do set %%L

if not "%flDEBUG_TXT%"=="nul" echo KNL_SCALE_V=%KNL_SCALE_V% >>%flDEBUG_TXT%

echo %0: Start IterativeDistance2. Why do this twice? Different KNL_SCALE_V=%KNL_SCALE_V%
%IMG7%magick ^
  %LINE_FILE% ^
  -negate +write mpr:MASK ^
  -fill White -colorize 100 ^
  -fill Black -draw "point %X0%,%Y0%" ^
  -write-mask mpr:MASK ^
  -morphology ^
    IterativeDistance:%DIAG% ^
    Euclidean:7,%KNL_SCALE_V% ^
  +write-mask ^
  +depth ^
  -define quantum:format=floating-point -depth 32 ^
  %TMPDIR%fl_line_grad.miff
echo %0: Finished IterativeDistance2

for /F "usebackq" %%L in (`%IMG7%magick ^
  %LINE_FILE% ^
  %TMPDIR%fl_line_grad.miff ^
  -compose Darken -composite ^
  -format "MAX_01=%%[fx:maxima]\nMAX_GRAY=%%[fx:maxima*QuantumRange]\nLINE_LEN=%%[fx:maxima*QuantumRange/%KNL_SCALE_V%]" ^
  info:`) do set %%L

if not "%flDEBUG_TXT%"=="nul" echo MAX_01=%MAX_01% MAX_GRAY=%MAX_GRAY% LINE_LEN=%LINE_LEN% >>%flDEBUG_TXT%

:: "%MAX_01%" should be "1", more or less.
:: MAX_GRAY should be >= QuantumRange - KNL_SCALE_V

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "GRAY_OK=%%[fx:%MAX_GRAY%>QuantumRange-%KNL_SCALE_V%?1:0]"
  xc:`) do set %%L

if not "%GRAY_OK%"=="1" (
  echo %0: Bad second MAX_01 = %MAX_01% MAX_GRAY = %MAX_GRAY%
  set ERR=1
  goto end
)

:===


%IMG7%magick ^
  %TMPDIR%fl_line_grad.miff ^
  -transparent White ^
  %TMPDIR%fl_line_grad_tw.miff


echo %0: Start blurFill.

set bfTHRESH_PC=3
set bfMAX_ITER=
set bfMAX_SIG=

call %PICTBAT%blurFill ^
  %TMPDIR%fl_line_grad_tw.miff 0.6+ %TMPDIR%fl_line_bf.miff

if "%bfBUST%"=="1" (
  echo %0: bfBUST
  set ERR=1
  goto end
)
echo %0: Finished blurFill.


for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format ^
    "KNL_SCALE_H=%%[fx:%KNL_SCALE_V%*%HH%/%WW%]" ^
  xc:`) do set %%L

rem Next should be HDRI? Just Q16 because we use 65536 above.
rem 17-July-2018: change next from %IM16f%convert to %IMDEV%convert
%IMG7%magick ^
  %LINE_FILE% ^
  -negate ^
  -morphology Distance Euclidean:7,%KNL_SCALE_H% ^
  +depth ^
  -define quantum:format=floating-point -depth 32 ^
  %TMPDIR%fl_dperp.miff

if ERRORLEVEL 1 exit /B 1

if not "%flDEBUG_TXT%"=="nul" echo KNL_SCALE_H=%KNL_SCALE_H% >>%flDEBUG_TXT%

set /A Wm1=%WW%-1
set /A Hm1=%HH%-1

call :FindEdge %X0% %Y0%
set XF0=%XF%
set YF0=%YF%
call :FindEdge %X1% %Y1%

if %XF0%==%XF% if %YF0%==%YF% (
  echo %0: ends have same closest edge pixel [%LINE_FILE%]
  set ERR=1
  goto end
)

if not "%flDEBUG_TXT%"=="nul" echo Closest edges: (%XF0%,%YF0%) (%XF%,%YF%) >>%flDEBUG_TXT%

:: Join line-ends to nearest edge,
:: and find first black pixel for flood-fill.
:: (Probably coord 0,0.)

for /F "usebackq tokens=2,3 delims=:, " %%X in (`%IM7DEV%magick ^
  %LINE_FILE% ^
  -fill White ^
  -draw "line %X0%,%Y0% %XF0%,%YF0%" ^
  -draw "line %X1%,%Y1% %XF%,%YF%" ^
  +write %TMPDIR%fl_mask_side.miff ^
  -alpha off ^
  -negate ^
  -process onewhite ^
  NULL: 2^>^&1`) do (
  set Xblk=%%X
  set Yblk=%%Y
)

if "%Xblk%"=="none" (
  echo %0: no black pixels [%LINE_FILE%]
  set ERR=1
  goto end
)

if not "%flDEBUG_TXT%"=="nul" echo Floodfill mask: (%Xblk%,%Yblk%) >>%flDEBUG_TXT%

:: FIXME: collapse into next
::
%IMG7%magick ^
  %TMPDIR%fl_mask_side.miff ^
  -fill White ^
  -draw "color %Xblk%,%Yblk% floodfill" ^
  %TMPDIR%fl_mask_side.miff

set flTWEAK_RED=-auto-level
set flTWEAK_RED=

%IMG7%magick ^
  %TMPDIR%fl_dperp.miff ^
  -evaluate Add 50%% ^
  ( +clone -negate ) ^
  ( %TMPDIR%fl_mask_side.miff -alpha off ) ^
  -compose Over -composite ^
  %flTWEAK_RED% ^
  %TMPDIR%fl_line_bf.miff ^
  ( +clone -evaluate set 0 ) ^
  -set colorspace sRGB ^
-define quantum:format=floating-point -depth 32 ^
+write %TMPDIR%fl_x3.miff ^
+write %TMPDIR%fl_x3.png ^
  -combine ^
+write %TMPDIR%fl_x2.png ^
  +write fl_disp_map_abs.miff ^
  ( +clone ^
    -sparse-color bilinear ^
0,0,#000,^
%Wm1%,0,#f00,^
0,%Hm1%,#0f0,^
%Wm1%,%Hm1%,#ff0 ^
  ) ^
  -compose Mathematics ^
    -define compose:args=0,-0.5,0.5,0.5 ^
    -composite ^
  fl_disp_map_rel.miff

%IMG7%magick ^
  %INFILE% ^
  %flFAB_TRANS% ^
  fl_disp_map_rel.miff ^
  -compose Displace ^
    -set option:compose:args %Wm1%x%Hm1% ^
    -composite ^
  %OUTFILE%

if not "%flDEBUG_TXT%"=="nul" echo OUTFILE=%OUTFILE% >>%flDEBUG_TXT%

:end

call echoRestore

endlocal & set flOUTFILE=%OUTFILE%& set flERR=%ERR%

exit /B %flERR%

:: =========================================
:: Subroutine

:FindEdge
echo fe %1 %2

set /A DW=%1
set /A DN=%2
set /A DE=Wm1-%1
set /A DS=Hm1-%2

set /A Dmin=DW

if %Dmin% GTR %DN% set /A Dmin=DN
if %Dmin% GTR %DE% set /A Dmin=DE
if %Dmin% GTR %DS% set /A Dmin=DS

if %Dmin%==%DW% (
  set /A XF=0
  set /A YF=DN
) else if %Dmin%==%DN% (
  set /A XF=DW
  set /A YF=0
) else if %Dmin%==%DE% (
  set /A XF=Wm1
  set /A YF=DN
) else if %Dmin%==%DS% (
  set /A XF=DW
  set /A YF=Hm1
)

echo XF=%XF% YF=%YF%

exit /B 0
