rem Make fractal noise.

@rem %1 %2 Width, height in pixels.
@rem %3 minimum block size in pixels. [1]
@rem %4 maximum block size in pixels. [min(w,h)/2]
@rem %5 block factor: Block size is increased by this factor from min to max. >=1 [2]
@rem %6 power factor for amplitude proportional to frequency. [Default 0.0]
@rem   0 for all frequencies have same amplitude;
@rem   > 0 (eg 1, 1.5) for highest frequencies have lowest amplitude.
@rem %7 output file. [fn.png]

@rem Returns fnNUM_OCTAVES number of octaves.

@rem Also uses:
@rem
@rem   fnSEED Seed for pseudo-random generator. [Default: no seed]
@rem   fnGRAY if 1, creates grayscale output, otherwise RGB.
@rem   fnNOISE_TYPE Noise type: random, gaussian, uniform, etc.
@rem   fnABS_NSE if 1, takes absolute value of noise.
@rem   fnFILTER Resize filter.
@rem   fnIM location of IM's "magick" to be used.
@rem   fnPREFIX prefix for working files (but not output file). [fn_]
@rem   fnNG_EXT extension for noise grid files. [.miff]
@rem   fnLINEAR if 1, converts noise to linear, the result to non-linear.
@rem   fnAUTO_LEVEL_EACH whether to apply auto-levelling to each octave. [0]
@rem   fnAUTO_LEVEL_END whether to apply auto-levelling at the end. [1]
@rem     0 no auto-levelling,
@rem     1 synchronised auto-levelling,
@rem     2 unsynchronised (more colouful) auto-levelling.
@rem     ? Equalize
@rem     ? Gaussianise
@rem   fnAUTO_GAM_END whether to apply auto-gamma at the end. [1]
@rem   fnRD_PYR 0 or 1, override automatic choice about reading pyramid files instead of creating them.
@rem   fnWR_PYR 0 or 1, override automatic choice about writing pyramid files.
@rem   fnPROC_EACH a process to be applied to each octave.
@rem   fnTXT_OUT if 1, writes some text data.
@rem
@rem   fnA_FRM_NUM frame number, typically an integer. If blank, no animation.
@rem Animation control.
@rem   fnA_COL percentage colour change per frame. 100%==0%. [0]
@rem   fnA_DX horizontal displacement per frame, pixels, positive=pan right (image moves left). [0]
@rem   fnA_DY vertical displacement per frame, pixels, positive=pan down (image moves up). [0]
@rem   fnA_ZM zoom factor per frame, eg 0.998. <1 zooms out, shrinks image. [1]
@rem   fnA_POW_FAC power factor for animation. [0]
@rem   fnA_COLCYCMETH colour cycling method. [null or sinarc2o]
@rem
@rem
@rem First release: 28 September 2015.
@rem
@rem Updated:
@rem   24-September-2022 for IM v7.
@rem

@rem TODO: power defined by clut image file, so we can emphasise middle frequencies.


@setlocal enabledelayedexpansion

@call echoOffSave


set pyGRAPHIC=0
set pyWEIGHTING=sum

set OUTFILE=%7
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=fn.png

call %PICTBAT%fnInit %1 %2 %3 %4 %5 %6
if ERRORLEVEL 1 exit /B 1

call %PICTBAT%fnOne %OUTFILE%
if ERRORLEVEL 1 exit /B 1


call echoRestore

@endlocal & set fnOUTFILE=%OUTFILE%&set fnPYR_FILE=%fnPYR_FILE%& set fnWW=%fnWW%& set fnHH=%fnHH%& set fnNUM_OCTAVES=%NUM_OCTAVES%& set fnPOW_FAC=%POW_FAC%
