rem From input image %1,
rem makes outputs %2 (must contain XX)
rem   _mag _ph and _spec.
rem Also creates environment variable prefix %3 with suffixes
rem   _ww  input image width
rem   _hh  input image height
rem   _sqdim width and height of outputs.
rem   _max maximum value of magnitude component
rem   _scale scale
rem %4 Assumed maximum value of magnitude.
rem %5 Assumed mean value of magnitude.
@rem
@rem Also uses:
@rem   fftPOST_SPEC eg -resize "600x600>"
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 fft

set OUTFILE=%2
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=%BASENAME%_fft_XX%EXT%

set ENV_PREF=%3
if "%ENV_PREF%"=="." set ENV_PREF=
if "%ENV_PREF%"=="" set ENV_PREF=fft

set ASS_MAX=%4
if "%ASS_MAX%"=="." set ASS_MAX=

set ASS_MEAN=%5
if "%ASS_MEAN%"=="." set ASS_MEAN=


set OUT_MAG=%OUTFILE:XX=mag%
set OUT_PH=%OUTFILE:XX=ph%
set OUT_SPEC=%OUTFILE:XX=spec%

if "%OUT_MAG%"=="%OUT_PH%" (
  echo Bad OUTFILE [%OUTFILE%] -- should contain XX
  exit /B 1
)

set WW=
for /F "usebackq" %%L in (`%IM%identify ^
  -format "WW=%%w\nHH=%%h" ^
  %INFILE%`) do set %%L
if "%WW%"=="" exit /B 1


%IMDEV%convert ^
  %INFILE% ^
  -fft ^
  -define quantum:format=floating-point ^
  ( -clone 0 -write %OUT_MAG% +delete ) ^
  -delete 0 ^
  %OUT_PH%

if ERRORLEVEL 1 (
  echo No FFT
  exit /B 1
)

for /F "usebackq" %%L in (`%IMDEV%convert ^
  %OUT_MAG% ^
  -precision 19 ^
  -format "SCALE=%%[fx:exp(log(mean)/log(0.5))]\nWH=%%w\nMEAN=%%[fx:mean]\nMIN=%%[fx:minima]\nMAX=%%[fx:maxima]" ^
  info:`) do set %%L

if not "%ASS_MAX%"=="" set MAX=%ASS_MAX%
if not "%ASS_MEAN%"=="" set MEAN=%ASS_MEAN%

echo %0: SCALE=%SCALE% MIN=%MIN% MEAN=%MEAN% MAX=%MAX%

for /F "usebackq" %%L in (`%IMDEV%identify ^
  -precision 19 ^
  -format "SCALE=%%[fx:exp(log(%MEAN%/%MAX%)/log(0.5))]" ^
  xc:`) do set %%L

echo %0: SCALE=%SCALE% MIN=%MIN% MEAN=%MEAN% MAX=%MAX%

%IMDEV%convert ^
  %OUT_MAG% ^
  -evaluate divide %MAX% ^
  -evaluate log %SCALE% ^
  %fftPOST_SPEC% ^
  %OUT_SPEC%


call echoRestore

endlocal & set %ENV_PREF%_ww=%WW%&^
set %ENV_PREF%_hh=%HH%&^
set %ENV_PREF%_sqdim=%WH%&^
set %ENV_PREF%_max=%MAX%&^
set %ENV_PREF%_scale=%SCALE%&^
set %ENV_PREF%_mean=%MEAN%
