rem From image %1
rem make output %2 the Gaussian slope magnitude.
rem %3 is the Gaussian blur sigma.
rem %4 is post-pocessing (eg "-grayscale Brightness -auto-level").
@rem
@rem Last updated:
@rem   2-August-2020 for IM v7. Assumes magick is HDRI.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 gsm

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set BLR_SIG=%3
if "%BLR_SIG%"=="." set BLR_SIG=
if "%BLR_SIG%"=="" set BLR_SIG=1

set POST_PROC=%~4
if "%POST_PROC%"=="." set POST_PROC=

set TMPDIR=\temp\
set PREF=%TMPDIR%gsm

set TMP_KNL=%PREF%_knl.miff
set KNL_L=%PREF%_knl_l.txt
set KNL_T=%PREF%_knl_t.txt
set KNL_R=%PREF%_knl_r.txt
set KNL_B=%PREF%_knl_b.txt
set KNL_LR=%PREF%_knl_lr.txt
set KNL_TB=%PREF%_knl_tb.txt

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "N_BORD=%%[fx:%BLR_SIG%*10]" ^
  xc:`) do set %%L

set OKAY=
for /F "usebackq" %%L in (`%IMG7%magick ^
  -size 1x1 xc:White ^
  -bordercolor Black -border %N_BORD% ^
  -gaussian-blur 0x%BLR_SIG% ^
  -auto-level ^
  -trim ^
  -crop 2x1@ +repage ^
  -delete 1 ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  -write %TMP_KNL% ^
  -format "OKAY=%%[fx:(w-1)*2+1==h?1:0]\nOFFS=%%[fx:w-1]\n" ^
  info:`) do set %%L

if "%OKAY%"=="" exit /B 1
if "%OKAY%"=="0" exit /B 1

echo %0: N_BORD=%N_BORD% OKAY=%OKAY%  OFFS=%OFFS%


%IM7DEV%magick ^
  %TMP_KNL% ^
  -precision 19 ^
  -process img2knl NULL: >%KNL_L%

%IM7DEV%magick ^
  %TMP_KNL% ^
  -precision 19 ^
  -flop ^
  -process img2knl NULL: >%KNL_R%

%IM7DEV%magick ^
  %TMP_KNL% ^
  -precision 19 ^
  -rotate 90 ^
  -process img2knl NULL: >%KNL_T%

%IM7DEV%magick ^
  %TMP_KNL% ^
  -precision 19 ^
  -rotate 90 ^
  -flip ^
  -process img2knl NULL: >%KNL_B%

rem FIXME: Replace following when we have offsets in img2knl.

rem If chStrs is not available, use "sed".
rem
rem chStrs /p0 /l1000000 /i%KNL_L% /m1 /f":" /t"+%OFFS%+%OFFS%:"
rem chStrs /p0 /l1000000 /i%KNL_R% /m1 /f":" /t"+0+%OFFS%:"
rem chStrs /p0 /l1000000 /i%KNL_T% /m1 /f":" /t"+%OFFS%+%OFFS%:"
rem chStrs /p0 /l1000000 /i%KNL_B% /m1 /f":" /t"+%OFFS%+0:"

sed -e s/:/+%OFFS%+%OFFS%:/ --in-place=BAK %KNL_L%
sed -e s/:/+0+%OFFS%:/ --in-place=BAK %KNL_R%
sed -e s/:/+%OFFS%+%OFFS%:/ --in-place=BAK %KNL_T%
sed -e s/:/+%OFFS%+0:/ --in-place=BAK %KNL_B%

rem FIXME? Or we could start with an entire kernel,
rem   zero the centre column and negate the left side.

%IMG7%magick ^
  %INFILE% ^
  -define compose:clamp=off ^
  -define "convolve:scale=^!" ^
  -define "morphology:showkernel=1" ^
  ( -clone 0 ^
    ( -clone 0 -morphology convolve @%KNL_R% ) ^
    ( -clone 0 -morphology convolve @%KNL_L% ) ^
    -delete 0 ^
    -compose Difference -composite ^
  ) ^
  ( -clone 0 ^
    ( -clone 0 -morphology convolve @%KNL_B% ) ^
    ( -clone 0 -morphology convolve @%KNL_T% ) ^
    -delete 0 ^
    -compose Difference -composite ^
  ) ^
  -delete 0 ^
  -evaluate Pow 2 ^
  -compose Plus -composite ^
  -evaluate Divide 2 ^
  -evaluate Pow 0.5 ^
  %POST_PROC% ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  %OUTFILE%

if ERRORLEVEL 1 (
  echo %0: mprphology failed
  exit /B 1
)

rem -------------------------
rem New method

call %PICTBAT%mSliceGauss %TMP_KNL% %BLR_SIG%

if ERRORLEVEL 1 exit /B 1

%IM7DEV%magick ^
  %TMP_KNL% ^
  -precision 19 ^
  -process img2knl NULL: >%KNL_LR%

%IM7DEV%magick ^
  %TMP_KNL% ^
  -rotate 90 ^
  -precision 19 ^
  -process img2knl NULL: >%KNL_TB%

rem  -define "convolve:scale=^^^"

rem FIXME: Following goes bad, unless we have "-debug all". ??

%IMG7%magick ^
  %INFILE% ^
  -define compose:clamp=off ^
  -define "morphology:showkernel=1" ^
  -define "convolve:scale=^^" ^
  ( -clone 0 -morphology convolve @%KNL_LR% ) ^
  ( -clone 0 -morphology convolve @%KNL_TB% ) ^
  -delete 0 ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
+write xy.miff ^
  -evaluate Pow 2 ^
  -compose Plus -composite ^
  -evaluate Divide 2 ^
  -evaluate Pow 0.5 ^
  %POST_PROC% ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  x.miff


call echoRestore

@endlocal & set gsmOUTFILE=%OUTFILE%
