/* Updated:
     3-April-2018 for v7.0.7-28
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>
#include <math.h>
#include "vsn_defines.h"


// The next function corresponds in style to functions in transform.c
// It takes one image, and returns an image.
//
static Image *CopyGrad2(const Image *image,
  ExceptionInfo *exception)
{
  Image
    *new_image;

  CacheView
    *image_view;

  ssize_t
    y;

  MagickBooleanType
    status;

  assert(image != (Image *) NULL);
  assert(image->signature == MAGICK_CORE_SIG);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  // Make a clone of this image, same size but undefined pixel values:
  //
  new_image=CloneImage(image, image->columns, image->rows, MagickTrue, exception);
  if (new_image == (Image *) NULL)
    return(new_image);

  if (SetNoPalette (new_image, exception) == MagickFalse)
    return (Image *)NULL;

  image_view = AcquireAuthenticCacheView(new_image,exception);

  status = MagickTrue;

#if defined(MAGICKCORE_OPENMP_SUPPORT)
  #pragma omp parallel for schedule(static,4) shared(status) \
    MAGICK_THREADS(new_image,new_image,new_image->rows,1)
#endif
  for (y=0; y < (ssize_t) new_image->rows; y++)
  {
    register VIEW_PIX_PTR
      *q;

    register ssize_t
      x;

    if (status == MagickFalse)
      continue;
    q=GetCacheViewAuthenticPixels(image_view,0,y,new_image->columns,1,exception);
    if (q == (const VIEW_PIX_PTR *) NULL)
    {
      status=MagickFalse;
      continue;
    }
    for (x=0; x < (ssize_t) image->columns; x++)
    {
      SET_PIXEL_RED   (new_image, QuantumRange * (x/(image->columns-1.0)), q);
      SET_PIXEL_GREEN (new_image, QuantumRange * (y/(image->rows-1.0)), q);
      SET_PIXEL_BLUE  (new_image, QuantumRange/2, q);
      q += Inc_ViewPixPtr (new_image);
    }
    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
      status=MagickFalse;
  }
  image_view=DestroyCacheView(image_view);

  return (new_image);
}



ModuleExport size_t grad2Image(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image,
    *new_image;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  (void) argc;
  (void) argv;

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    new_image = CopyGrad2(image,exception);
    if (new_image == (Image *) NULL)
      return(-1);

    ReplaceImageInList(&image,new_image);
    // Replace messes up the images pointer. Make it good:
    *images=GetFirstImageInList(image);
  }

  return(MagickImageFilterSignature);
}
