rem From image %1 with height=1,
rem makes a colour line graph with same width but max height=256.
@rem
@rem Optional:
@rem   %2 is background colour, can be "none" (default Black).
@rem   %3 is grid (default 0 = no grid).
@rem   %4 is whether to show clut below graph (default 1 = do show).
@rem   %5 output file
@rem   %6 limit output width [1000]
@rem   %7 graph height [256]
@rem   %8 post-processing eg "-compose Over -bordercolor #888 -border 5"
@rem
@rem Also uses:
@rem  glcCLUT  a process to apply to the clut image only,
@rem           eg "-set colorspace RGB -colorspace sRGB"
@rem  glcSHOW_STEPS if not blank or 0, add stepped sampler.
@rem
@rem Updated:
@rem   15-May-2016 to use "-layers flatten".
@rem   13-October-2016 Fixed bug in OUTFILE.
@rem   16-February-2021 Added parameter %7.
@rem   21-July-2021 Added "+repage" after "-append".
@rem   8-April-2022 Added glcCLUT.
@rem   14-April-2022 Changed "convert" to "magick".
@rem   21-November-2022 Added glcXscale output.


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal disabledelayedexpansion
@rem Otherwise ! in LINE_IMG causes IM to misread.

@call echoOffSave

rem Can't use setInOut.
rem call %PICTBAT%setInOut %1 glc
set INFILE=%1
set OUTFILE=%~dpn1_glc.png

if not exist %INFILE% (
  %0: can't find INFILE [%INFILE%]
  exit /B 1
)

set BACKCOL=%2
if "%BACKCOL%"=="." set BACKCOL=
if "%BACKCOL%"=="" set BACKCOL=#000

set GRID_SPEC=%3
if "%GRID_SPEC%"=="." set GRID_SPEC=
if "%GRID_SPEC%"=="" set GRID_SPEC=0

set SHOW_CLUT=%4
if "%SHOW_CLUT%"=="." set SHOW_CLUT=
if "%SHOW_CLUT%"=="" set SHOW_CLUT=1

set sOUT=%5
if "%sOUT%"=="." set sOUT=
if "%sOUT%"=="" set sOUT=%OUTFILE%
set OUTFILE=%sOUT%

set LIM_WW=%6
if "%LIM_WW%"=="." set LIM_WW=
if "%LIM_WW%"=="" set LIM_WW=1000

if "%glcSHOW_STEPS%"=="" set glcSHOW_STEPS=0


set WW=
FOR /F "usebackq" %%L IN (`%IMG7%magick identify ^
  -precision 15 ^
  -format "WW=%%w\nHH=%%h\nXscale=%%[fx:%LIM_WW%/w]\n" ^
  %INFILE%`) DO set %%L

if "%WW%"=="" exit /B 1

if %WW% GTR %LIM_WW% (
  set WW=%LIM_WW%
) else (
  set Xscale=1
)

echo %0: Xscale=%Xscale%

set newH=%7
if "%newH%"=="." set newH=
if "%newH%"=="" set newH=256

if %WW% LSS %newH% (
  set newH=%WW%
)

set sPOSTPROC=%~8

if "%sPOSTPROC%"=="." set sPOSTPROC=

echo %0: sPOSTPROC=%sPOSTPROC%

if %GRID_SPEC%==0 (
  set S_GRID=
) else (
  call %PICTBAT%grid %WW% %newH% 10 10 . gray50
  set S_GRID=grid.png -compose Exclusion -composite
)

if not "%glcGRID_IMG%"=="" (
  set S_GRID=%glcGRID_IMG% -compose Exclusion -composite
)

echo %0: S_GRID=%S_GRID%

rem set /A newH4=4*%newH%
set /A newH4=%newH%

if /I %BACKCOL%==none (
  set sBACK=
) else (
  set sBACK=-background %BACKCOL% -compose Over -flatten
)

set sBACK=-background %BACKCOL%

echo %0: sBACK=%sBACK%


set LINE_IMG=^
  -separate +channel -scale "%WW%x%newH4%^!" ^
  -size %WW%x%newH4% gradient: ^
  -compose MinusDst -composite ^
  -fill #fff +opaque #000 ^
  -edge 1 

rem   -morphology edgeout diamond:1   +write o1.png
rem   -canny 0x0.5+10%%+30%% 
rem   -edge 3
rem   -scale "%WW%x%newH%^!" +write o2.png


set /A SHOW_CLUT_HT=%WW%/10
if %SHOW_CLUT_HT% LSS 25 set SHOW_CLUT_HT=25

set STEPS_IMG=%TEMP%\glc_steps.miff

if %glcSHOW_STEPS%==0 (
  set STEPS_IMG=
) else (

  if %SHOW_CLUT%==0 (
    set STEPS_IMG=
  ) else (
    %IMG7%magick ^
      %INFILE% ^
      -crop %glcSHOW_STEPS%x1@ +repage -scale "1x1^!" +append +repage ^
      -define quantum:format=floating-point -depth 32 ^
      %glcCLUT% -scale "%WW%x%SHOW_CLUT_HT%^!" ^
      %STEPS_IMG%
  )
)

if %SHOW_CLUT%==0 (
  set sCLUT=
) else (
  set sCLUT= ^( mpr:INP %glcCLUT% -scale "%WW%x%SHOW_CLUT_HT%^!" ^) %STEPS_IMG% -append +repage
)


echo %0: sCLUT=%sCLUT%

%IMG7%magick ^
  "%INFILE%" -clamp +write mpr:INP ^
  -colorspace sRGB ^
  -scale "%WW%x1^!" ^
  -scale "%WW%x%newH%^!" ^
  ( -clone 0 ^
    ( -clone 0 ^
      -fill #f00 -colorize 100 ^
    ) ^
    ( -clone 0 -channel R %LINE_IMG% ) ^
    -delete 0 ^
    -alpha off ^
    -compose CopyOpacity -composite ^
  ) ^
  ( -clone 0 ^
    ( -clone 0 ^
      -fill #0f0 -colorize 100 ^
    ) ^
    ( -clone 0 -channel G %LINE_IMG% ) ^
    -delete 0 ^
    -alpha off ^
    -compose CopyOpacity -composite ^
  ) ^
  ( -clone 0 ^
    ( -clone 0 ^
-set colorspace sRGB ^
      -fill #00f -colorize 100 ^
    ) ^
    ( -clone 0 -channel B %LINE_IMG% ) ^
    -delete 0 ^
    -alpha off ^
    -compose CopyOpacity -composite ^
  ) ^
  -delete 0 ^
  %sBACK% ^
  -compose Add -layers Flatten ^
  -alpha opaque ^
  %S_GRID% ^
  %sPOSTPROC% ^
  %sCLUT% ^
  %OUTFILE%

if ERRORLEVEL 1 (
  echo %0: magick failed
  exit /B 1
)

call echoRestore

endlocal & set glcOUTFILE=%OUTFILE%& set glcXscale=%Xscale%
