rem From image %1, assumed grayscale, makes a histogram (clut) image.
@rem
@rem Also uses:
@rem   g2hINBITS optional, 8, 16. If not set, uses depth of input.
@rem     8 is much quicker but less precise.
@rem   g2hDO_CUMUL if 1, makes cumulative histogram.
@rem
@rem Updated:
@rem   25-September-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 g2h


rem Output is normalised so maximum count is white.
rem Or cumulative, from 0.0 to 1.0.

if "%g2hDO_CUMUL%"=="" set g2hDO_CUMUL=0

set OUTTXT=%TEMP%\%~n1_%sioCODE%_out.txt
set TXTFILE=%TEMP%\%~n1_%sioCODE%.txt

del %TXTFILE% 2>nul



set INBITS=%g2hINBITS%

if "%INBITS%"=="" (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "INBITS=%%z" %INFILE%`) do set %%L
)

if "%INBITS%"=="" (
  echo %0: Failed to find INBITS for INFILE [%INFILE%]
  exit /B 1
)

set OUTBITS=16

%IMG7%magick %INFILE% -depth %INBITS% -format %%c histogram:info:%TXTFILE%

if %INBITS%==8 (
  set OUT_W=256
) else if %INBITS%==16 (
  set OUT_W=65536
) else (
  echo %0: Bad INBITS=%INBITS%
  exit /B 1
)

set /A OUT_Wm1=%OUT_W%-1

if %OUTBITS%==8 (
  set OUT_MAX=255
) else if %OUTBITS%==16 (
  set OUT_MAX=65535
) else (
  echo %0: Bad OUTBITS=%OUTBITS%
  exit /B 1
)



echo # ImageMagick pixel enumeration: %OUT_W%,1,%OUT_MAX%,srgb>%OUTTXT%

set MAX_CNT=0
set CUMUL_CNT=0

for /F "tokens=1 delims=:(), " %%L in (%TXTFILE%) do (
  if !MAX_CNT! LSS %%L set MAX_CNT=%%L
)

set MAX_CNT=65535

if %g2hDO_CUMUL%==1 (
  for /F "usebackq" %%L in (`%IMG7%magick ^
    %INFILE% -format "MAX_CNT=%%[fx:w*h]" ^
    info:`) do set %%L
)

set NEXT_X=0
set CNTint=0

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 9 ^
  -format "FACT=%%[fx:%OUT_MAX%/%MAX_CNT%]" ^
  info:`) do set %%L

for /F "tokens=1,2 eol=# delims=:(), " %%L in (%TXTFILE%) do (
  set CNT=%%L
  set VAL=%%M
  set /A CUMUL_CNT+=%%L

  if %g2hDO_CUMUL%==1 (

    set CNT=!CUMUL_CNT!

    if !VAL! GTR !NEXT_X! (
      set /A LAST_X=!VAL!-1

      for /L %%x in (!NEXT_X!,1,!LAST_X!) do echo %%x,0: ^(!CNTint!,!CNTint!,!CNTint!^)>>%OUTTXT%
    )
  )

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "CNTint=%%[fx:int(%FACT%*!CNT!+0.5)]" ^
    xc:`) do set %%L

  echo !VAL!,0: ^(!CNTint!,!CNTint!,!CNTint!^)>>%OUTTXT%

  set /A NEXT_X=!VAL!+1
)

if %NEXT_X% LEQ %OUT_Wm1% (
  for /L %%x in (%NEXT_X%,1,%OUT_Wm1%) do echo %%x,0: ^(!CNTint!,!CNTint!,!CNTint!^)>>%OUTTXT%
)

%IMG7%magick -background Black %OUTTXT% %OUTFILE%
if ERRORLEVEL 1 (
  echo %0: error processing %*
)

call echoRestore

endlocal & set g2hOUTFILE=%OUTFILE%
