rem Makes a grid with specified number of intervals.
rem %1,%2 are width and height.
rem %3 and %4 are number of gaps between horizontal and vertical grid lines, >=1 (default 4)
rem %5 is whether to draw diagonal (default 0)
rem %6 is grid colour (default white)
rem %7 is background colour, can be "none" (default black)
rem %8 is colour for boundary
rem %9 is output file (default grid.png)
@rem
@rem Also uses:
@rem   gridSTROKE_WIDTH default 1
@rem
@rem
@rem Updated:
@rem   5-August-2022 for IM v7.
@rem

rem No required arguments.
rem @if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set WW=%1
if "%WW%"=="." set WW=
if "%WW%"=="" set WW=150
set HH=%2
if "%HH%"=="." set HH=
if "%HH%"=="" set HH=100

set nX=%3
if "%nX%"=="." set nX=
if "%nX%"=="" set nX=4
set nY=%4
if "%nY%"=="." set nY=
if "%nY%"=="" set nY=4

set DO_DIAG=%5
if "%DO_DIAG%"=="." set DO_DIAG=
if "%DO_DIAG%"=="" set DO_DIAG=0

set GRID_COL=%6
if "%GRID_COL%"=="." set GRID_COL=
if "%GRID_COL%"=="" set GRID_COL=White

set BACK_COL=%7
if "%BACK_COL%"=="." set BACK_COL=
if "%BACK_COL%"=="" set BACK_COL=Black

set BOUND_COL=%8
if "%BOUND_COL%"=="." set BOUND_COL=

set OUTFILE=%9
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=grid.png

if "%gridSTROKE_WIDTH%"=="" set gridSTROKE_WIDTH=0


FOR /F "usebackq" %%L ^
IN (`%IMG7%magick identify ^
  -format ^
Wm1^=%%[fx:%WW%-1]\n^
Hm1^=%%[fx:%HH%-1]\n^
dX^=%%[fx:%WW%/%nX%]\n^
dY^=%%[fx:%HH%/%nY%]\n^
nXm1^=%%[fx:%nX%-1]\n^
nYm1^=%%[fx:%nY%-1] ^
  xc:`) ^
DO set %%L

set LINESFILE=%TEMP%\gridlines.txt
del %LINESFILE% 2>nul

if %nXm1% GTR 0 for /L %%i in (1, 1, %nXm1%) do (
  FOR /F %%L IN ('%IMG7%magick identify ^
    -format "v=%%[fx:int(%dX%*%%i+0.5)]" xc:') DO @set %%L
  echo line !v!,0 !v!,%Hm1% >>%LINESFILE%
)

if %nYm1% GTR 0 for /L %%i in (1, 1, %nYm1%) do (
  FOR /F %%L IN ('%IMG7%magick identify ^
    -format "v=%%[fx:int(%dY%*%%i+0.5)]" xc:') DO @set %%L
  echo line 0,!v! %Wm1%,!v! >>%LINESFILE%
)

if %DO_DIAG%==1 (
  echo line %Wm1%,0 0,%Hm1% >>%LINESFILE%
  echo line 0,0 %Wm1%,%Hm1% >>%LINESFILE%
)

if not "%BOUND_COL%"=="" (
  echo fill %BOUND_COL% >>%LINESFILE%
  echo line 0,0 %Wm1%,0 >>%LINESFILE%
  echo line 0,0 0,%Hm1% >>%LINESFILE%
  echo line %Wm1%,0 %Wm1%,%Hm1% >>%LINESFILE%
  echo line 0,%Hm1% %Wm1%,%Hm1% >>%LINESFILE%
)

if %gridSTROKE_WIDTH%==1 (
  set sSTROKEW=
) else (
  set sSTROKEW=-strokewidth %gridSTROKE_WIDTH%
)


%IMG7%magick ^
  -size %WW%x%HH% ^
  xc:%BACK_COL% ^
  -fill None ^
  -stroke %GRID_COL% ^
  %sSTROKEW% ^
  -draw "@%LINESFILE%" ^
  %OUTFILE%

call echoRestore
