rem %1 input image
rem %2 guide image
rem %3 output
rem %4 filter method: auto, simple, grayCol, colCol [auto]
rem %5 method for "mean": one of blur, gauss, stat, stat2, integ, bilateral [integ]
rem %6 window radius XxY [9x9]
rem %7 epsilon: number or "auto" [0.01]
rem %8 scale factor eg 4 for quarter-size linear [1]
rem %9 power factor for AK [1]
@rem
@rem Assumes IM "magick" has HDRI.
@rem
@rem Reference: http://im.snibgo.com/gdefilt.htm
@rem
@rem Also uses:
@rem   gfPRE_INP
@rem   gfPRE_GDE
@rem   gfPOST
@rem   gfPROC_AK eg "-clamp"
@rem
@rem Updated:
@rem   1-January-2023 %7 (epsilon) "auto" option.
@rem

:: FIXME: When output is grayscale, ensure it has only one channel.

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 gf

set SCR_FILE=gfc.im

del %SCR_FILE% 2>nul

set INFILE=%1
if "%INFILE%"=="." set INFILE=
if "%INFILE%"=="" set INFILE=toes.png

set GDEFILE=%2
if "%GDEFILE%"=="." set GDEFILE=
if "%GDEFILE%"=="" (
  set GDEFILE=mpr:IN0
  set fGDEFILE=%INFILE%
) else (
  set fGDEFILE=%GDEFILE%
)

set OUTFILE=%3
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=x.png

set FiltMeth=%4
if "%FiltMeth%"=="." set FiltMeth=
if "%FiltMeth%"=="" set FiltMeth=auto

set MeanMeth=%5
if "%MeanMeth%"=="." set MeanMeth=
if "%MeanMeth%"=="" set MeanMeth=integ

for /F "usebackq" %%L in (`%IMG7%magick ^
  -ping %INFILE% ^
  -format "WW=%%w\nHH=%%h" ^
  info:`) do set %%L

set WINRADS=%6
if "%WINRADS%"=="." set WINRADS=
if "%WINRADS%"=="" set WINRADS=9x9

set EPS=%7
if "%EPS%"=="." set EPS=
if "%EPS%"=="" set EPS=0.01

if /I %EPS%==auto (
  echo %0: AutoEps

  @rem : Crude, should do this in main command.

  for /F "usebackq" %%L in (`%IMG7%magick ^
    %fGDEFILE% ^
    -format "nEPS=%%[fx:standard_deviation*standard_deviation/2]\n" ^
    info:`) do set %%L
) else (
  set nEPS=%EPS%
)

set AdjEps=-evaluate Multiply 5
set AdjEps=-poly 1,1,%%[fx:0.1*QuantumRange],1
set AdjEps=-evaluate Multiply 1.1
set AdjEps=-poly 0.99,1,%%[fx:0.5*QuantumRange],1

set AdjEps=-evaluate Add %%[fx:%nEPS%*QuantumRange]

echo %0: AdjEps=%AdjEps%

set SCAL=%8
if "%SCAL%"=="." set SCAL=
if "%SCAL%"=="" set SCAL=1

set POW=%9
if "%POW%"=="." set POW=
if "%POW%"=="" set POW=1

if %POW%==1 (
  set sPOW=
) else (
  set sPOW=-evaluate Pow %POW%
)


if /I %FiltMeth%==auto (
  call %PICTBAT%isGray %INFILE% INP_IS_GRAY
  if ERRORLEVEL 1 exit /B 1

  set GDE_IS_GRAY=!INP_IS_GRAY!

  if not %GDEFILE%==%INFILE% if not %GDEFILE%==mpr:IN0 (
    call %PICTBAT%isGray %GDEFILE% GDE_IS_GRAY
    if ERRORLEVEL 1 exit /B 1
  )

  echo %0: isGray: !INP_IS_GRAY! !GDE_IS_GRAY!

  if !INP_IS_GRAY!==1 (
    if !GDE_IS_GRAY!==1 (
      set FiltMeth=simple
    ) else (
      set FiltMeth=grayCol
    )
  ) else (
    if !GDE_IS_GRAY!==1 (
      rem Or grayCol?
      set FiltMeth=colCol
    ) else (
      set FiltMeth=colCol
    )
  )
)

if /I %FiltMeth%==simple (
  echo %0: FiltMeth is %FiltMeth%
) else if /I %FiltMeth%==grayCol (
  echo %0: FiltMeth is %FiltMeth%
) else if /I %FiltMeth%==colCol (
  echo %0: FiltMeth is %FiltMeth%
) else (
  echo %0: FiltMeth is %FiltMeth%
)

if %SCAL%==1 (
  set ScaleDn=
  set ScaleUp=
  set NewWW=%WW%
  set NewHH=%HH%
) else (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "NewWW=%%[fx:floor(%WW%/%SCAL%+0.5)]\nNewHH=%%[fx:floor(%HH%/%SCAL%+0.5)]\n"
    xc:`) do set %%L

  rem Note: these need trailing bangs. We do this later.
  set ScaleDn=!NewWW!x!NewHH!
  set ScaleUp=%WW%x%HH%
)
:: FIXME: maybe different ! for direct command?

call parseXxY2i %WW% %HH% Rad %WINRADS%
if ERRORLEVEL 1 exit /B 1

set /A Rad_X/=%SCAL%
set /A Rad_Y/=%SCAL%

set /A DIAM_X=%Rad_X%*2+1
set /A DIAM_Y=%Rad_Y%*2+1
set WINDIMS=%DIAM_X%x%DIAM_Y%

set IMI=%IMG7%

if /I %MeanMeth%==blur (
  set MEAN=-blur 0x%Rad_X%
) else if /I %MeanMeth%==blur2 (
  set MEAN=-morphology Convolve Blur:0x%Rad_X% -morphology Convolve Blur:0x%Rad_Y%,90
) else if /I %MeanMeth%==gauss (
  set MEAN=-gaussian-blur 0x%Rad_X%
) else if /I %MeanMeth%==stat (
  set MEAN=-statistic Mean %WINDIMS%
) else if /I %MeanMeth%==stat2 (
  set MEAN=-statistic Mean %DIAM_X%x1 -statistic Mean 1x%DIAM_Y%
) else if /I %MeanMeth%==integ (
  set MEAN=-process 'integim' -process 'deintegim window %WINDIMS%'
  set IMI=%IM7DEV%
) else if /I %MeanMeth%==bilateral (
  set MEAN=-bilateral-blur %Rad_X%
) else (
  echo %0: bad mean method %MeanMeth%
  exit /B 1
)

echo %0: MEAN = %MEAN%

setlocal disabledelayedexpansion

if not %SCAL%==1 (
  set ScaleDn=-resize "%ScaleDn%!"
  set ScaleUp=-resize "%ScaleUp%!"
)

echo %0: ScaleDn = %ScaleDn%
echo %0: ScaleUp = %ScaleUp%

set PROC_PRE=^
  -define compose:clamp=off +depth ^
  %INFILE%  +write mpr:IN0 %ScaleDn% %gfPRE_INP% +write mpr:INP +delete ^
  %GDEFILE% +write mpr:GDE0 %ScaleDn% %gfPRE_GDE% +write mpr:GDE %MEAN% +write mpr:MN_GDE +delete ^
  mpr:GDE -channel RGB -separate -write mpr:GDEch -delete 0--1 ^
  mpr:MN_GDE -channel RGB -separate -write mpr:MN_GDEch -delete 0--1 ^
 ^
  ( mpr:GDEch[0] mpr:GDEch[0] -compose Multiply -composite %MEAN% ) ^
  ( mpr:MN_GDEch[0] mpr:MN_GDEch[0] -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    %AdjEps% ^
    -write mpr:var_rr +delete ^
  ( mpr:GDEch[0] mpr:GDEch[1] -compose Multiply -composite %MEAN% ) ^
  ( mpr:MN_GDEch[0] mpr:MN_GDEch[1] -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:var_rg +delete ^
  ( mpr:GDEch[0] mpr:GDEch[2] -compose Multiply -composite %MEAN% ) ^
  ( mpr:MN_GDEch[0] mpr:MN_GDEch[2] -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:var_rb +delete ^
  ( mpr:GDEch[1] mpr:GDEch[1] -compose Multiply -composite %MEAN% ) ^
  ( mpr:MN_GDEch[1] mpr:MN_GDEch[1] -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    %AdjEps% ^
    -write mpr:var_gg +delete ^
  ( mpr:GDEch[1] mpr:GDEch[2] -compose Multiply -composite %MEAN% ) ^
  ( mpr:MN_GDEch[1] mpr:MN_GDEch[2] -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:var_gb +delete ^
  ( mpr:GDEch[2] mpr:GDEch[2] -compose Multiply -composite %MEAN% ) ^
  ( mpr:MN_GDEch[2] mpr:MN_GDEch[2] -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    %AdjEps% ^
    -write mpr:var_bb +delete ^
 ^
  ( mpr:var_gg mpr:var_bb -compose Multiply -composite ) ^
  ( mpr:var_gb mpr:var_gb -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:invrr +delete ^
  ( mpr:var_gb mpr:var_rb -compose Multiply -composite ) ^
  ( mpr:var_rg mpr:var_bb -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:invrg +delete ^
  ( mpr:var_rg mpr:var_gb -compose Multiply -composite ) ^
  ( mpr:var_gg mpr:var_rb -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:invrb +delete ^
  ( mpr:var_rr mpr:var_bb -compose Multiply -composite ) ^
  ( mpr:var_rb mpr:var_rb -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:invgg +delete ^
  ( mpr:var_rb mpr:var_rg -compose Multiply -composite ) ^
  ( mpr:var_rr mpr:var_gb -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:invgb +delete ^
  ( mpr:var_rr mpr:var_gg -compose Multiply -composite ) ^
  ( mpr:var_rg mpr:var_rg -compose Multiply -composite ) ^
    -compose MinusSrc -composite ^
    -write mpr:invbb +delete ^
 ^
  ( mpr:invrr mpr:var_rr -compose Multiply -composite ) ^
  ( mpr:invrg mpr:var_rg -compose Multiply -composite ) ^
  ( mpr:invrb mpr:var_rb -compose Multiply -composite ) ^
  -evaluate-sequence Add ^
  -write mpr:covDet +delete ^
 ^
  ( mpr:invrr mpr:covDet -compose DivideSrc -composite +write mpr:invrr +delete ) ^
  ( mpr:invrg mpr:covDet -compose DivideSrc -composite +write mpr:invrg +delete ) ^
  ( mpr:invrb mpr:covDet -compose DivideSrc -composite +write mpr:invrb +delete ) ^
  ( mpr:invgg mpr:covDet -compose DivideSrc -composite +write mpr:invgg +delete ) ^
  ( mpr:invgb mpr:covDet -compose DivideSrc -composite +write mpr:invgb +delete ) ^
  ( mpr:invbb mpr:covDet -compose DivideSrc -composite +write mpr:invbb +delete )

set PROC_ONE=^
-define quantum:format=floating-point -depth 32 ^
  mpr:MN_GDE mpr:MN_INP ^
  -compose Multiply -composite ^
  ( mpr:GDE mpr:INP_CH -compose Multiply -composite %MEAN% ) ^
  -compose MinusDst -composite ^
  -channel RGB -separate -write mpr:COVch -delete 0--1 ^
 ^
  ( mpr:invrr mpr:COVch[0] -compose Multiply -composite ) ^
  ( mpr:invrg mpr:COVch[1] -compose Multiply -composite ) ^
  ( mpr:invrb mpr:COVch[2] -compose Multiply -composite ) ^
  -evaluate-sequence Add ^
  -write mpr:AK_R +delete ^
  ( mpr:invrg mpr:COVch[0] -compose Multiply -composite ) ^
  ( mpr:invgg mpr:COVch[1] -compose Multiply -composite ) ^
  ( mpr:invgb mpr:COVch[2] -compose Multiply -composite ) ^
  -evaluate-sequence Add ^
  -write mpr:AK_G +delete ^
  ( mpr:invrb mpr:COVch[0] -compose Multiply -composite ) ^
  ( mpr:invgb mpr:COVch[1] -compose Multiply -composite ) ^
  ( mpr:invbb mpr:COVch[2] -compose Multiply -composite ) ^
  -evaluate-sequence Add ^
  -write mpr:AK_B +delete ^
  mpr:AK_R mpr:AK_G mpr:AK_B -combine -set colorspace sRGB ^
  %sPOW% ^
  %gfPROC_AK% -write mpr:AK ^
  mpr:MN_GDE -compose Multiply -composite -channel RGB -separate -evaluate-sequence Add ^
 ^
  mpr:MN_INP ^
  -compose MinusDst -composite ^
  -write mpr:BK +delete ^
  mpr:AK %MEAN% %ScaleUp% ^
  mpr:GDE0 ^
  -compose Multiply -composite ^
  -channel RGB -separate ^
  ( mpr:BK %MEAN% %ScaleUp% ) ^
  -evaluate-sequence Add

:: Computed goto. Yuck, but it avoids escape-parenthesis problems.
::
if /I %FiltMeth%==simple (
  echo %0: FiltMeth is %FiltMeth%
  goto simple
) else if /I %FiltMeth%==grayCol (
  echo %0: FiltMeth is %FiltMeth%
  goto grayCol
) else if /I %FiltMeth%==colCol (
  echo %0: FiltMeth is %FiltMeth%
  goto colCol
) else (
  echo %0: Bad FiltMeth [%FiltMeth%]
  exit /B 1
)

:simple

:: FIXME: maybe use "-set colorspace sRGB" in either or both.
:: Below, "-set colorspace sRGB" is needed only where GDE is grayscale and INP is RGB.

%IMI%magick ^
  -define compose:clamp=off ^
  ( %INFILE%   +write mpr:IN0 %ScaleDn% %gfPRE_INP% +write mpr:INP %MEAN% +write mpr:MN_INP ) ^
  ( %GDEFILE%  +write mpr:GDE0 %ScaleDn% %gfPRE_GDE% +write mpr:GDE %MEAN% +write mpr:MN_GDE ) ^
  +depth ^
  -compose Multiply -composite ^
  ( mpr:INP mpr:GDE -compose Multiply -composite %MEAN% ) ^
  -compose MinusDst -composite ^
  -write mpr:COV +delete ^
  mpr:GDE -set colorspace sRGB -evaluate Pow 2 %MEAN% ^
  ( mpr:MN_GDE -evaluate Pow 2 ) ^
  -compose MinusSrc -composite ^
  %AdjEps% ^
  mpr:COV ^
  -compose DivideDst -composite ^
  %sPOW% ^
  +write mpr:AK ^
  mpr:MN_GDE ^
  -compose Multiply -composite ^
  mpr:MN_INP ^
  -compose MinusDst -composite ^
  +write mpr:BK ^
  +delete ^
  mpr:AK %MEAN% %ScaleUp% ^
  mpr:GDE0 ^
  -compose Multiply -composite ^
  ( mpr:BK %MEAN% %ScaleUp% ) ^
  -compose Plus -composite ^
  %gfPOST% ^
-define quantum:format=floating-point -depth 32 ^
  %OUTFILE%

if ERRORLEVEL 1 (
  echo %0: simple error
  exit /B 1
)

goto end


:grayCol

rem echo -verbose >%SCR_FILE%
if not "%PROC_PRE%"=="" echo %PROC_PRE% >%SCR_FILE%

echo mpr:INP +write mpr:INP_CH %MEAN% +write mpr:MN_INP +delete >>%SCR_FILE%
echo %PROC_ONE% >>%SCR_FILE%
echo %gfPOST% -write %OUTFILE% -exit >>%SCR_FILE%

%IMI%magick -script %SCR_FILE%
if ERRORLEVEL 1 (
  echo %0: grayCol error
  exit /B 1
)

goto end

:colCol

rem set gfPRE_GDE=-colorspace Gray -colorspace sRGB
rem set gfPRE_GDE=

rem set gfPRE_INP=
rem set gfPRE_INP=-colorspace Gray -colorspace sRGB
rem set gfPRE_INP=-negate
rem set gfPRE_INP=-colors 4
rem set gfPRE_INP=-colorspace Gray -auto-level -auto-gamma -threshold 50%%

rem -verbose +write info: 
rem -define quantum:format=floating-point -depth 32 
rem +write akc.miff 

rem -verbose +write info: 
rem +write bkc.miff 

rem echo -verbose >%SCR_FILE%
if not "%PROC_PRE%"=="" echo %PROC_PRE% >%SCR_FILE%

echo mpr:INP -channel R -separate +channel +write mpr:INP_CH %MEAN% +write mpr:MN_INP +delete >>%SCR_FILE%
echo %PROC_ONE% >>%SCR_FILE%
echo -write mpr:OUT_R +delete >>%SCR_FILE%

echo mpr:INP -channel G -separate +channel +write mpr:INP_CH %MEAN% +write mpr:MN_INP +delete >>%SCR_FILE%
echo %PROC_ONE% >>%SCR_FILE%
echo -write mpr:OUT_G +delete >>%SCR_FILE%

echo mpr:INP -channel B -separate +channel +write mpr:INP_CH %MEAN% +write mpr:MN_INP +delete >>%SCR_FILE%
echo %PROC_ONE% >>%SCR_FILE%
echo -write mpr:OUT_B +delete >>%SCR_FILE%

echo mpr:OUT_R mpr:OUT_G mpr:OUT_B -combine >>%SCR_FILE%
echo %gfPOST% -write %OUTFILE% -exit >>%SCR_FILE%

%IMI%magick -script %SCR_FILE%
if ERRORLEVEL 1 (
  echo %0: colCol error
  exit /B 1
)

:end

call echoRestore

:: Note: yes, endlocal twice.
@endlocal

@endlocal & set gfFiltMeth=%FiltMeth%
