rem From image %1, returns list of ordinates to guillotine into pieces.
@rem
@rem %2 is output text file, one line per cut.
@rem %3 is input background colour [default White] between pieces.
@rem %4 is fuzz percentage from declared background colour (default 0).
@rem %5 is permitted percentage of non-background pixels at cut (default 0).
@rem %6 is 0 (guillotine vertically) or 1 (guillotine horizontally).
@rem
@rem Also uses:
@rem   guilMIN_GAP if set, ignores gaps smaller than this.
@rem   guilWIDEST_GAP if 1, ignores gaps smaller than the widest gap.
@rem
@rem
@rem Also sets:
@rem   guilNUM_CUTS the number of possible cuts, hence lines in %2. Possibly zero.
@rem
@rem Future: This could have feature to ignore narrow strips of background,
@rem   and narrow strips of non-background.
@rem
@rem
@rem Updated:
@rem   6-August-2022 for IM v7
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 guil

for /F %%F in ("%OUTFILE%") do set OUTFILE=%%~dpnF.lis

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set TMPTXT1=%BASENAME%_guil1.lis
set TMPTXT2=%BASENAME%_guil2.lis
set TEMP_IMG=%BASENAME%_guil.png

set BACK_COL=%3
if "%BACK_COL%"=="." set BACK_COL=
if "%BACK_COL%"=="" set BACK_COL=White

set FUZZ=%4
if "%FUZZ%"=="." set FUZZ=
if "%FUZZ%"=="" set FUZZ=0

set NON_BACK=%5
if "%NON_BACK%"=="." set NON_BACK=
if "%NON_BACK%"=="" set NON_BACK=0

set DIRN=%6
if "%DIRN%"=="." set DIRN=
if "%DIRN%"=="" set DIRN=0

if %DIRN%==0 (
  set ROT_IN=
  set ROT_OUT=
) else (
  set ROT_IN=-rotate -90
  set ROT_OUT=-rotate 90
)

if %FUZZ%==0 (
  set sFUZZ=
) else (
  set sFUZZ=-fuzz %FUZZ%%%
)

:: Make pixels with FUZZ of background colour black; all others white.
:: Then make single row, with black where less than NON_BACK non-background colours black.

%IMG7%magick ^
  %INFILE% ^
  -strip ^
  %ROT_IN% ^
  %sFUZZ% ^
  -alpha set ^
  -transparent %BACK_COL% ^
  -channel RGB -evaluate set 100%% +channel ^
  -background Black -layers Flatten ^
  -scale "x1^!" ^
  -fuzz %NON_BACK%%% ^
  -fill Black -opaque Black ^
  -fill White +opaque Black ^
  -alpha off ^
  %TEMP_IMG%

:: Possibly refine this.

if not "%guilMIN_GAP%"=="" (
  set /A MG=%guilMIN_GAP%/2

  %IMG7%magick ^
    %TEMP_IMG% ^
    -negate ^
    -morphology Distance:-1 Chebyshev:1,1 ^
    -black-threshold !MG! ^
    -fill White +opaque Black ^
    -alpha off ^
    -negate ^
    %TEMP_IMG%
)

if "%guilWIDEST_GAP%"=="1" %IMG7%magick ^
  %TEMP_IMG% ^
  -negate ^
  -morphology Distance:-1 Chebyshev:1,1 ^
  -auto-level ^
  -fill Black +opaque White ^
  -alpha off ^
  -negate ^
  %TEMP_IMG%

:: trim black

for /F "usebackq tokens=3 delims=x+" %%C in (`%IMG7%magick ^
  %TEMP_IMG% ^
  -quiet ^
  -bordercolor Black -border 1 ^
  -format "%%@" ^
  info:`) do (
  set /A OFFSET=%%C-1
)

echo %~n0: OFFSET=%OFFSET%

:: "-clamp" is for possible bug https://github.com/ImageMagick/ImageMagick/issues/5433
:: Clamping doesn't solve the problem.
:: Was: -morphology Thicken:-1 Skeleton 
:: Now: -alpha off -negate -morphology thinning:-1 "3x1>:1,-,0" -negate
::
%IMG7%magick ^
  %TEMP_IMG% ^
  -quiet ^
  -crop 0x1+%OFFSET%+0 +repage ^
  -alpha off -negate ^
  -morphology thinning:-1 "3x1>:1,-,0" -clamp ^
  -negate ^
  -transparent White ^
  sparse-color:%TMPTXT1%

sed -e 's/ /\n/g' %TMPTXT1% >%TMPTXT2%

type %TMPTXT2%

set nCUTS=0
(
  for /F "tokens=1 delims=, " %%X in (%TMPTXT2%) do (
    set /A V=%%X+%OFFSET%
    echo !V!
    set /A nCUTS+=1
  )
) >%OUTFILE%

echo %~n0: nCUTS=%nCUTS%

type %OUTFILE%

call echoRestore

endlocal & set guilOUTFILE=%OUTFILE%& set guilNUM_CUTS=%nCUTS%
