rem %1 text file of list of input raws

@setlocal enabledelayedexpansion

set INRAWS=%1

set PREF=%~n1

set TMPDIR=\temp\

set MINMAX_LIS=%TMPDIR%%PREF%_minmax.lis
set LIGHTVAL_LIS=%TMPDIR%%PREF%_lv.lis
set LIGHTVAL_SRT=%TMPDIR%%PREF%_lv_srt.lis
set TMPTIFF=%TMPDIR%%PREF%_tmp.tiff

del %LIGHTVAL_LIS%
del %MINMAX_LIS%

set CNT=0

for /F %%F in (%INRAWS%) do (
  echo %%F

  if not exist %%F exit /B 1

  set RAWNAME=%%~nF

  exiftool -T -LightValue -FileName %%F >>%LIGHTVAL_LIS%

  set %PREF%.RawFile[!RAWNAME!]=%%F

  set /A CNT+=1
)
if ERRORLEVEL 1 exit /B 1

set %PREF%.count=%CNT%

sort -n <%LIGHTVAL_LIS% >%LIGHTVAL_SRT%

set %PREF%.lightestLV=

set CNT=0

rem FIXME: use consistent white balance, not camera setting.

echo %0: pass 1:

for /F "usebackq tokens=1,2" %%A in (%LIGHTVAL_SRT%) do (
  set RAWNAME=%%~nB

  set %PREF%.ndx[!CNT!]=!RAWNAME!

  set %PREF%.LV[!RAWNAME!]=%%A

  if "!%PREF%.lightestLV!"=="" set %PREF%.lightestLV=%%A
  set %PREF%.darkestLV=%%A

  call :GetRawFileName !RAWNAME!
  %DCRAW% -v -D -W -w -g 1 1 -6 -T -O %TMPTIFF% !RawFileName!

  for /F "usebackq" %%L in (`%IMG7%magick ^
    %TMPTIFF% ^
    -statistic Maximum 3x3 ^
    -white-threshold 16382 ^
    -fill Black +opaque White ^
    -format "SAT_PC=%%[fx:mean*100]\n" +write info: ^
    -negate ^
    %TMPDIR%!RAWNAME!_msk.miff`) do set %%L

  echo %0: !RAWNAME! SAT_PC=!SAT_PC!

  set %PREF%.sat_pc[!RAWNAME!]=!SAT_PC!

  set /A CNT+=1
)

echo %0: pass 2:

set SMFILES=
for /F "usebackq tokens=1,2" %%A in (%LIGHTVAL_SRT%) do (

  set RAWNAME=%%~nB

  call :GetRawFileName !RAWNAME!
  %DCRAW% -v -W -w -4 -o 5 -T -O %TMPTIFF% !RawFileName!

  set MIFFILE=%TMPDIR%!RAWNAME!.miff

  call :GetLV !RAWNAME!

  rem FIXME: don't give any transparency to darkest image

  set SMFILE=%TMPDIR%!RAWNAME!_sm.miff

  echo %0: SMFILE=!SMFILE!

  for /F "usebackq" %%L in (`%IMG7%magick ^
    !TMPTIFF! ^
    -define "compose:clamp=off" ^
    -set colorspace XYZ ^
    -profile %ICCPROF%\sRGB-elle-V4-g10.icc ^
    -clamp ^
    -set gamma 1.0 ^
    -format "MIN=%%[fx:minima]\nMN=%%[fx:mean]\nMAX=%%[fx:maxima]\n" +write info: ^
    -evaluate Multiply "%%[fx:pow(2,!LV!-!%PREF%.lightestLV!)]" ^
    -format "MIN2=%%[fx:minima]\nMN2=%%[fx:mean]\nMAX2=%%[fx:maxima]\n" +write info: ^
    %TMPDIR%!RAWNAME!_msk.miff ^
    -alpha off ^
    -compose CopyOpacity -composite ^
    -define "quantum:format=floating-point" -depth 32 ^
    +write !MIFFILE! ^
    -resize 500x500 -write !SMFILE! ^
    NULL:`) do set %%L

  set %PREF%.min[!RAWNAME!]=!MIN!
  set %PREF%.mn[!RAWNAME!]=!MN!
  set %PREF%.max[!RAWNAME!]=!MAX!
  set %PREF%.min2[!RAWNAME!]=!MIN2!
  set %PREF%.mn2[!RAWNAME!]=!MN2!
  set %PREF%.max2[!RAWNAME!]=!MAX2!

  set SMFILES=!SMFILES! !SMFILE!
)

echo SMFILES=%SMFILES%

%IMG7%magick ^
  %SMFILES% ^
  +append ^
  +write %TMPDIR%%PREF%_sm.png ^
  -channel RGB,sync ^
  -auto-level ^
  +channel ^
  %TMPDIR%%PREF%_sm1.png

%IMG7%magick ^
  %SMFILES% ^
  -reverse ^
  -define compose:clamp=off ^
  -compose Over -layers flatten ^
  +write %TMPDIR%%PREF%_sm2.miff ^
  +write %TMPDIR%%PREF%_sm2.png ^
  -alpha off -auto-level ^
  +write %TMPDIR%%PREF%_sm3.miff ^
  +write %TMPDIR%%PREF%_sm3.png ^
  NULL:

%IMG7%magick ^
  %SMFILES% ^
  -reverse ^
  -define compose:clamp=off ^
  -compose Plus -layers flatten ^
  -alpha off ^
  -evaluate Divide !%PREF%.count! ^
  +write %TMPDIR%%PREF%_sm4.miff ^
  +write %TMPDIR%%PREF%_sm4.png ^
  -alpha off -auto-level ^
  +write %TMPDIR%%PREF%_sm5.miff ^
  +write %TMPDIR%%PREF%_sm5.png ^
  NULL:

rem   -evaluate Multiply %%[fx:pow(2,!%PREF%.lightestLV!-!%PREF%.darkestLV!)]

set %PREF%

set %PREF% >%PREF%_env.lis

exit /B 0

rem ------------------------------------------------------
rem Subroutines

:GetRawFileName
set RawFileName=!%PREF%.RawFile[%1]!
exit /B 0

:GetLV
set LV=!%PREF%.LV[%1]!
exit /B 0
