#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <windows.h>
#include <winuser.h>

#include "vsn_defines.h"

#if IMV6OR7==6
#include <wand/MagickWand.h>
#elif IMV6OR7==7
#include <MagickWand/MagickWand.h>
#else
#error IMV6OR7 defined but not valid
#endif


typedef struct {
  int version;
  int verbose;
  char * infile;
  char * outfile;
  MagickBooleanType doMult;
  double multR;
  double multG;
  double multB;
} ProgOptT;




static void testWin (void)
{
  printf ("testWin\n");
  HWND hw;
  printf ("GetDoubleClickTime=%i\n", GetDoubleClickTime ());
  //InitializeFlatSB (hw);
}


static void version (void)
{
  printf ("hellowld v1.0 (c) 2015 Alan Gibson 22-Dec-2015\n");
  printf ("%s\n", MagickGetCopyright());
  printf ("%s\n", MagickGetVersion(NULL));
  exit (0);
}

static void usage (void)
{
  printf ("Writes some text information.\n\n");

  printf ("Usage: hellowld {options}\n");
  printf ("  -h  --help      write usage data and exit\n");
  printf ("  -v  --verbose   write text information about the processing\n");
  printf ("  -i  --infile    filename(s) for input\n");
  printf ("  -o  --outfile   filename for output\n");
  printf ("  -m  --multiply  one or three comma-separated numbers, to multiply each RGB channel\n");
  printf ("      --version   write version data and exit\n");
  exit (0);
}


static void GetOptions (int argc, char **argv, ProgOptT * po)
{
  struct option long_options[] = {
    /* Second field is one of: no_argument, required_argument, optional_argument. */
    /* Third field is 0 or pointer to int. */

    /* These options set a flag. */
    {"version", no_argument,       &po->version, 1},
    {"verbose", no_argument,       &po->verbose, 1},
    {"brief",   no_argument,       &po->verbose, 0},
    /* These options don't set a flag.
       We distinguish them by their indices. */
    {"help",    no_argument,       0, 'h'},
    {"add",     no_argument,       0, 'a'},
    {"append",  no_argument,       0, 'b'},
    {"create",  required_argument, 0, 'c'},
    {"delete",  required_argument, 0, 'd'},
    {"file",    required_argument, 0, 'f'},
    {"infile",  required_argument, 0, 'i'},
    {"outfile", required_argument, 0, 'o'},
    {"multiply",required_argument, 0, 'm'},
    {0, 0, 0, 0}
  };

  char * short_options = "hvabc:d:f:i:o:m:";

  MagickBooleanType
    okay = MagickTrue;

  po->version = po->verbose = 0;
  po->infile = po->outfile = NULL;
  po->doMult = MagickFalse;
  po->multR = po->multG = po->multB = 0;

  int c;

  while (1)
  {
    int option_index = 0;

    c = getopt_long (argc, argv, short_options,
                     long_options, &option_index);

    /* Detect the end of the options. */
    if (c == -1) break;

//    printf ("char [%c]\n", c);

    switch (c)
    {
      case 0:
//        printf ("long option with option_index `%i'\n", option_index);
        /* If this option set a flag, do nothing else now. */
        if (long_options[option_index].flag != 0)
          break;
//        printf ("option %s", long_options[option_index].name);
//        if (optarg)
//          printf (" with arg %s", optarg);
//        printf ("\n");
        break;

      case 'h':
        puts ("option -h\n");
        usage ();
        break;

      case 'v':
        puts ("option -v\n");
        po->verbose = 1;
        break;

      case 'a':
        puts ("option -a\n");
        break;

      case 'b':
        puts ("option -b\n");
        break;

      case 'c':
        printf ("option -c with value `%s'\n", optarg);
        break;

      case 'd':
        printf ("option -d with value `%s'\n", optarg);
        break;

      case 'f':
        printf ("option -f with value `%s'\n", optarg);
        break;

      case 'i':
        printf ("option -i with value `%s'\n", optarg);
        po->infile = optarg; // FIXME: is this safe?
        break;

      case 'o':
        printf ("option -o with value `%s'\n", optarg);
        po->outfile = optarg;
        break;

      case 'm': {
        po->doMult = MagickTrue;
        printf ("option -m with value `%s'\n", optarg);
        int n = sscanf (optarg, "%lg,%lg,%lg", &po->multR, &po->multG, &po->multB);
        if (n!=1 && n!=3) {
          printf ("--multiply needs 1 or 3 numbers.\n");
          okay = MagickFalse;
        }
        if (n==1) po->multB = po->multG = po->multR;
        break;
      }

      case '?':
        /* getopt_long already printed an error message. */
        break;

      default:
        abort ();
    }
  }

  if (po->version) {
    version ();
  }

  /* Print any remaining command line arguments (not options). */
  if (optind < argc)
  {
    printf ("non-option ARGV-elements: ");
    while (optind < argc)
      printf ("%s ", argv[optind++]);
    putchar ('\n');
    okay = MagickFalse;
  }

  if (!okay) exit (-1);
}


static void WrOptions (ProgOptT *po)
{
  fprintf (stderr, "Options:\n");
  if (po->verbose) fprintf (stderr, "  --verbose\n");
  if (po->infile)  fprintf (stderr, "  --infile  %s\n", po->infile);
  if (po->outfile) fprintf (stderr, "  --outfile %s\n", po->outfile);
  if (po->doMult)  fprintf (stderr, "  --multiply %lg,%lg,%lg\n", po->multR, po->multG, po->multB);
}

#define ThrowWandException(wand) \
{ \
  char \
    *description; \
 \
  ExceptionType \
    severity; \
 \
  description=MagickGetException(wand,&severity); \
  (void) fprintf(stderr,"%s %s %lu %s\n",GetMagickModule(),description); \
  description=(char *) MagickRelinquishMemory(description); \
  exit(-1); \
}


static MagickBooleanType ReadInputs (ProgOptT *po, MagickWand *magick_wand)
// po->infile is comma-separated list of filenames.
// Each filename may be quoted with single-quote or double-quote characters.
{
  MagickBooleanType
    status = MagickTrue;

  const char
    sq = '\'',
    dq = '"',
    cSep = ',';

  char * p = po->infile;

  char quote='\0';

//  printf ("p:[%s]\n", p);

  char * pEnd;

  while (*p) {
    while (*p==' ') p++;
    if (*p==sq || *p==dq) {
      quote = *p;
      p++;
      while (*p==' ') p++;
      pEnd = strchr (p, quote);
      if (pEnd) {
        printf ("pEnd:[%s]\n", pEnd);
        *pEnd = '\0';
        pEnd++;
      }
    } else {
      pEnd = p;
    }

    char * pSep = strchr (pEnd, cSep);
    if (pSep) {
//      printf ("pSep:[%s]\n", pSep);
      *pSep = '\0';
    }
    if (po->verbose) fprintf (stderr, "reading:[%s]\n", p);

    status=MagickReadImage (magick_wand, p);
    if (status == MagickFalse) {
      ThrowWandException (magick_wand);
      return status;
    }

    p = pSep ? pSep + 1 : strchr (p, '\0');
  }

  return status;
}


static MagickBooleanType Multiply (
  WandView *pixel_view,
  const ssize_t y,const int id,void *context)
{
  RectangleInfo
    extent;

  PIX_INFO
    pixel;

  PixelWand
    **pixels;

  register long
    x;

  ProgOptT *po = context;

  printf ("%li ", y);

  extent=GetWandViewExtent(pixel_view);
  pixels=GetWandViewPixels(pixel_view);
  for (x=0; x < (long) (extent.width); x++)
  {
    PixelGetMagickColor(pixels[x],&pixel);
    pixel.red   *= po->multR;
    pixel.green *= po->multG;
    pixel.blue  *= po->multB;
#if IMV6OR7==6
    PixelSetMagickColor(pixels[x],&pixel);
#else
    PixelSetPixelColor(pixels[x],&pixel);
#endif
  }
  return(MagickTrue);
}


static void DoOneImage (ProgOptT *po, MagickWand *magick_wand)
{
  MagickBooleanType
    status;

  WandView
    *wand_view;

  if (po->verbose) fprintf (stderr, "%lix%li\n",
    MagickGetImageWidth (magick_wand),
    MagickGetImageHeight (magick_wand)
  );

  if (po->doMult)  {
    // Update the image in-place.

    wand_view=NewWandView(magick_wand);
    if (wand_view == (WandView *) NULL)
      ThrowWandException(magick_wand);

    status=UpdateWandViewIterator(wand_view,Multiply,(void *) po);
    if (status == MagickFalse)
      ThrowWandException(magick_wand);

    wand_view=DestroyWandView(wand_view);
  }

}


int main (int argc, char **argv)
{
  MagickBooleanType
    status;

  MagickWand
    *magick_wand;

  ProgOptT
    ProgOpt;

  GetOptions (argc, argv, &ProgOpt);

  if (ProgOpt.verbose) {
    WrOptions (&ProgOpt);
  }

  if (ProgOpt.infile) {
    MagickWandGenesis ();
    magick_wand=NewMagickWand ();  

    status = ReadInputs (&ProgOpt, magick_wand);

    if (status == MagickFalse)
      ThrowWandException (magick_wand);

    MagickSetImageArtifact (magick_wand, "verbose", "true");
    MagickSetImageProperty (magick_wand, "verbose", "true");

    /*
      Do something with or to each image.
    */
    MagickResetIterator (magick_wand);
    while (MagickNextImage (magick_wand) != MagickFalse) {
      DoOneImage (&ProgOpt, magick_wand);
    }

    /*
      Write the image(s) then destroy it/them.
    */
    if (ProgOpt.outfile) {
      if (ProgOpt.verbose) fprintf (stderr, "Writing [%s]\n", ProgOpt.outfile);
      status=MagickWriteImages (magick_wand, ProgOpt.outfile, MagickTrue);
      if (status == MagickFalse)
        ThrowWandException (magick_wand);
    }

    magick_wand=DestroyMagickWand (magick_wand);
    MagickWandTerminus ();
  } else {
    if (ProgOpt.verbose) fprintf (stderr, "Nothing to do.");
  }

  testWin ();

  return(0);
}
