rem From non-cumulised histogram %1,
rem find clipping.
@rem
@rem Returns hcCLIP_LO, hcCLIP_HI.
@rem Compares first/last value to window of values.
@rem If this=0 or this < max(window), returns 0
@rem Else if mean(window)=0, returns 99999
@rem Else returns this/mean(window)
@rem
@rem Can also use:
@rem   hcWINDOW_PC window width, as percent of histogram width. Default 10.
@rem
@rem Examines ONLY first and last buckets.
@rem Histogram might be pre-processed, removing empty buckets at start and end.
@rem
@rem Also problem if either window has MAX=1, or much above average of entire histogram.
@rem
@rem Updated:
@rem   17-September-2022
@rem   25-September-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave

call %PICTBAT%setInOut %1 hc

if "%hcWINDOW_PC%"=="" set hcWINDOW_PC=10

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -format "MEAN_ALL=%%[fx:mean]\nMAX_ALL=%%[fx:maxima]\n" ^
  -write info: ^
  ^( +clone ^
    -crop %hcWINDOW_PC%%%x1+0+0 +repage ^
    -format "END_LO=%%[fx:p{0,0}.intensity]\n" ^
    -write info: ^
    -crop +1+0 +repage ^
    -format "MEAN_LO=%%[fx:mean]\nMAX_LO=%%[fx:maxima]\n" ^
    -write info: ^
    +delete ^) ^
  ^( +clone -flop ^
    -crop %hcWINDOW_PC%%%x1+0+0 +repage ^
    -format "END_HI=%%[fx:p{0,0}.intensity]\n" ^
    -write info: ^
    -crop +1+0 +repage ^
    -format "MEAN_HI=%%[fx:mean]\nMAX_HI=%%[fx:maxima]\n" ^
    -write info: ^
    +delete ^) ^
  NULL:`) do set %%L

if %END_LO%==0 (
  set CLIP_LO=0
) else if %MEAN_LO%==0 (
  set CLIP_LO=99999
) else (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "CLIP_LO=%%[fx:%END_LO%<=%MAX_LO%?0:%END_LO%/%MEAN_LO%]" ^
    xc:`) do set %%L
)

if %END_HI%==0 (
  set CLIP_HI=0
) else if %MEAN_HI%==0 (
  set CLIP_HI=99999
) else (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "CLIP_HI=%%[fx:%END_HI%<=%MAX_HI%?0:%END_HI%/%MEAN_HI%]" ^
    xc:`) do set %%L
)

if "%MAX_LO%"=="1" (
  set CLIP_LO_WIN=1
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "CLIP_LO_WIN=%%[fx:%MAX_LO%/%MEAN_LO%]" ^
    xc:`) do set %%L
) else (
  set CLIP_LO_WIN=0
)

if "%MAX_HI%"=="1" (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "CLIP_HI_WIN=%%[fx:%MAX_HI%/%MEAN_HI%]" ^
    xc:`) do set %%L
) else (
  set CLIP_HI_WIN=0
)

echo CLIP_LO=%CLIP_LO%  CLIP_HI=%CLIP_HI% CLIP_LO_WIN=%CLIP_LO_WIN%  CLIP_HI_WIN=%CLIP_HI_WIN%

call echoRestore

@endlocal & set hcCLIP_LO=%CLIP_LO%& set hcCLIP_HI=%CLIP_HI%
