rem From image %1,
rem finds histogram peaks (and troughs).
@rem
@rem   hpkOUT_PREFIX. Default "hpk".
@rem   hpkDEBUG if 1, creates debugging files
@rem   hpkCAPNUMBUCKETS
@rem   hpkENSURE_ENDS if 1, also process end segments
@rem   hpkHIST_BLR string for blurring the histogram, eg -blur 0x2
@rem   hpkWIN_PROP window width as proportion of histogram width. Default 0.1
@rem   hpkPK_THRESHPC peaks less than this threshold (percent) are ignored. Default 0.
@rem   [hpkTR_THRESHPC troughs more than this threshold (percent) are ignored. Default 100.]
@rem   hpkLIMIT_NUM_PKS if not zero, limits the number of peaks.
@rem
@rem Updated:
@rem   12-August-2022 for IM v7
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 hpk

if not "%2"=="" set OUTFILE=%2

if "%hpkOUT_PREFIX%"=="" set hpkOUT_PREFIX=%sioCODE%

if "%HPK_THRESH%"=="" set HPK_THRESH=0.1

if "%hpkWIN_PROP%"=="" set hpkWIN_PROP=0.1

if "%hpkPK_THRESHPC%"=="" set hpkPK_THRESHPC=0

if "%hpkLIMIT_NUM_PKS%"=="" set hpkLIMIT_NUM_PKS=0


call StopWatch

set /A NUM_BUCK=4000
set /A NUM_BUCKm1=%NUM_BUCK%-1

if "%hpkCAPNUMBUCKETS%"=="" (
  set CAP_BUCK=
) else (
  set CAP_BUCK=capnumbuckets %hpkCAPNUMBUCKETS%
)

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WIN_WIDTH=%%[fx:%NUM_BUCK%*%hpkWIN_PROP%]" ^
  xc:`) do set %%L

if "%hpkDEBUG%"=="1" (
  set WR_hmax=+write %hpkOUT_PREFIX%_hmax.miff
  set WR_hmn=+write %hpkOUT_PREFIX%_hmn.miff
  set WR_hm=+write %hpkOUT_PREFIX%_hm.miff
  set WR_hd=+write %hpkOUT_PREFIX%_hd.miff
) else (
  set WR_hmax=
  set WR_hmn=
  set WR_hm=
  set WR_hd=
)

%IM7DEV%magick ^
  %INFILE% ^
  -process 'mkhisto %CAP_BUCK% norm' ^
  %hpkOUT_PREFIX%_h0.miff

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "NUM_BUCK=%%[fx:w]\nNUM_BUCKm1=%%[fx:w-1]\nWIN_WIDTH=%%[fx:int(w*%hpkWIN_PROP%+0.5)]" ^
  %hpkOUT_PREFIX%_h0.miff`) do set %%L


set DECLIP_LO=-fill #000 -draw "point 0,0"
set DECLIP_HI=-fill #000 -draw "point %NUM_BUCKm1%,0"

:: temp: save as integer and read that, to convert to integer,
:: to avoid bug where "-statistic" is wrong for non-integers.
:: Bother, no, that doesn't help.

%IM7DEV%magick ^
  %hpkOUT_PREFIX%_h0.miff ^
  -alpha off ^
  -define quantum:format=floating-point ^
  -depth 32 ^
  %DECLIP_LO% ^
  %DECLIP_HI% ^
  %hpkHIST_BLR% ^
  -auto-level ^
  +write %hpkOUT_PREFIX%_h0.miff ^
  -define quantum:format=integer ^
  +write %hpkOUT_PREFIX%_h0.tiff ^
  -define quantum:format=floating-point ^
  +delete ^
  %hpkOUT_PREFIX%_h0.tiff ^
  -alpha off ^
  ( -clone 0 ^
    ( -clone 0 -statistic maximum %WIN_WIDTH%x1 %WR_hmax% ) ^
    ( -clone 0 -statistic mean %WIN_WIDTH%x1 %WR_hmn% ) ^
    ( -clone 2 -evaluate Add 2 ) ^
    ( -clone 2 -evaluate Multiply 1.1 ) ^
    -delete 2 ^
    -delete 0 ^
    -evaluate-sequence Max ^
    -evaluate Subtract 1 ^
    %WR_hm% ^
  ) ^
+write y.miff ^
  -compose MinusSrc -composite ^
  -clamp ^
  -fill #fff +opaque #000 ^
  -alpha off ^
+write x.miff ^
  -process allwhite ^
  %WR_hd% ^
  NULL: 2>%hpkOUT_PREFIX%_hd.lis

if ERRORLEVEL 1 exit /B 1

rem After -compose MinusSrc -composite, values are 1, 0 or <0.

rem This gives us the X-coord of the peaks.
rem We can retrieve the histogram values at these locations,
rem and eliminate "small" peaks.
rem
rem Then we can loop through the gaps,
rem and process that portion of the histogram to find the lowest points,
rem which are the troughs.

rem We have segments, that can be numbered with "-connected-components".

call StopWatch

if "%hpkDEBUG%"=="1" (
  call %PICTBAT%graphLineCol %hpkOUT_PREFIX%_h0.miff . . 0 %hpkOUT_PREFIX%_h0_glc.png
  call %PICTBAT%graphLineCol %hpkOUT_PREFIX%_hd.miff . . 0 %hpkOUT_PREFIX%_hd_glc.png
  call %PICTBAT%graphLineCol %hpkOUT_PREFIX%_hm.miff . . 0 %hpkOUT_PREFIX%_hm_glc.png
  call %PICTBAT%graphLineCol %hpkOUT_PREFIX%_hmn.miff . . 0 %hpkOUT_PREFIX%_hmn_glc.png
  call %PICTBAT%graphLineCol %hpkOUT_PREFIX%_hmax.miff . . 0 %hpkOUT_PREFIX%_hmax_glc.png

  %IMG7%magick ^
    %hpkOUT_PREFIX%_h0_glc.png ^
    %hpkOUT_PREFIX%_hd_glc.png ^
    %hpkOUT_PREFIX%_hm_glc.png ^
    %hpkOUT_PREFIX%_hmn_glc.png ^
    -evaluate-sequence Max ^
    %hpkOUT_PREFIX%_h3_glc.png

  %IMG7%magick ^
    ^( %hpkOUT_PREFIX%_h0_glc.png ^) ^
    ^( %hpkOUT_PREFIX%_hd_glc.png +level-colors black,lime ^) ^
    ^( %hpkOUT_PREFIX%_hm_glc.png -evaluate Multiply 0.5 ^) ^
    ^( %hpkOUT_PREFIX%_hmn_glc.png -evaluate Multiply 0.5 ^) ^
    -channel RGB -evaluate-sequence Max +channel ^
    %hpkOUT_PREFIX%_h3_glc.png
)

rem %IMG7%magick %hpkOUT_PREFIX%_hd.miff +transparent #fff sparse-color:

rem type %hpkOUT_PREFIX%_hd.lis




rem Write a list of peaks, optionally adding the two ends.

set PKS_F=%hpkOUT_PREFIX%_pks.lis
del %PKS_F% 2>nul

for /F "skip=1 tokens=1 delims=, " %%x in (%hpkOUT_PREFIX%_hd.lis) do (
  for /F "usebackq" %%C in (`%IMG7%magick ^
    %hpkOUT_PREFIX%_h0.miff ^
    -precision 9 ^
    -intensity Brightness ^
    -format "COUNT_PC=%%[fx:100*p{%%x,0}.intensity]" ^
    info:`) do set %%C

    echo %%x,!COUNT_PC! >>%PKS_F%
)

if not exist %PKS_F% (
  echo %PKS_F% [%PKS_F%] does not exit
  exit /B 1
)

if "%hpkENSURE_ENDS%"=="1" (
  for /F "usebackq" %%C in (`%IMG7%magick ^
    %hpkOUT_PREFIX%_h0.miff ^
    -precision 9 ^
    -intensity Brightness ^
    -format "COUNT_PC=%%[fx:100*p{0,0}.intensity]" ^
    info:`) do set %%C

  echo 0,!COUNT_PC! >>%PKS_F%

  for /F "usebackq" %%C in (`%IMG7%magick ^
    %hpkOUT_PREFIX%_h0.miff ^
    -precision 9 ^
    -intensity Brightness ^
    -format "COUNT_PC=%%[fx:100*p{%NUM_BUCKm1%,0}.intensity]" ^
    info:`) do set %%C

  echo %NUM_BUCKm1%,!COUNT_PC! >>%PKS_F%

  rem We might already have ends, so sort, removing duplicates.
  cSort /p0 /i%PKS_F% /o%PKS_F% /k0 /u
)

cPrefix  /p0 /i%PKS_F% /tBucket,CountPC


rem Remove low COUNT_PC.

cSelect  /p0 /i%PKS_F% /h /kCountPC /q6 /w%hpkPK_THRESHPC%



rem Build a peak-to-peak list, to identify and remove peaks that make small gaps.
rem
rem We find the smallest gap.
rem If it is less than WIN_WIDTH, we remove one peak from PKS_F and iterate.

set P2P_F=%hpkOUT_PREFIX%_p2p.lis

:loopGap
cNextRow /p0 /i%PKS_F% /o%P2P_F% /f"," /k0-1 /h
cHead    /p0 /i%P2P_F% /t1 /x
cExtend  /p0 /i%P2P_F% /kGap /v=Bucket_0-Bucket=
cSort    /p0 /i%P2P_F% /h /kGap
cHead    /p0 /i%P2P_F% /h2
cExtend  /p0 /i%P2P_F% /kSmaller /v=iif(CountPC^<CountPC_0,Bucket,Bucket_0)=

rem type %P2P_F%

set DONE_ONE=0
for /F "skip=1 tokens=1-6 delims=, " %%A in (%P2P_F%) do (
  if %%E LEQ %WIN_WIDTH% (
    cSelect /p0 /i%PKS_F% /h /kBucket /w%%F /x
    set DONE_ONE=1
  )
)

if %DONE_ONE%==1 goto loopGap


rem Limit the number of peaks.

if "%hpkLIMIT_NUM_PKS%" GTR "0" (
  set /A LIM=%hpkLIMIT_NUM_PKS%+1
  cSort /i%PKS_F% /h /kCountPC /r /o- | cHead /p0 /i- /o- /h%LIM% | cSort /i- /o%PKS_F% /h /kBucket
)


rem Re-build the peak-to-peak list.

cProject /p0 /i%PKS_F% /o%P2P_F% /h /kBucket
cNextRow /p0 /i%P2P_F% /f"," /k0 /h
cHead    /p0 /i%P2P_F% /t1 /x


rem Find the troughs.

set TS_F=%hpkOUT_PREFIX%_troughs.lis
echo Bucket,CountPC >%TS_F%

for /F "skip=1 tokens=1-2 delims=, " %%A in (%P2P_F%) do (
  set /A WW=%%B-%%A+1

  set /A WW_2=WW/2
  set /A WW_2+=WW-2*WW_2

  for /F "usebackq tokens=1-2 delims=:, " %%C in (`%IMG7%magick ^
    %hpkOUT_PREFIX%_h0.miff ^
    -crop !WW!x1+%%A+0 +repage ^
    -crop 2x1@ +repage ^
    ^( -clone 0 -rotate -90 ^) ^
    ^( -clone 1 -rotate 90 ^) ^
    -delete 0-1 ^
    -negate ^
    -background #000 ^
    +append +repage ^
    -process onelightest ^
    NULL: 2^>^&1`) do (

    if %%C==0 (
      set /A WX=%%A+!WW_2!-%%D-1
    ) else (
      set /A WX=%%A+!WW_2!+%%D
    )
  )

  for /F "usebackq" %%C in (`%IMG7%magick ^
    %hpkOUT_PREFIX%_h0.miff ^
    -precision 9 ^
    -intensity Brightness ^
    -format "COUNT_PC=%%[fx:100*p{!WX!,0}.intensity]" ^
    info:`) do set %%C

  echo WW=!WW! WW_2=!WW_2! WX=!WX!
  echo !WX!,!COUNT_PC! >>%TS_F%
)

rem Add "BucketPC" columns, as percentages.
cExtend  /i%TS_F% /kBucketPC /v=100*Bucket/%NUM_BUCKm1%=
cProject /i%TS_F% /h /s /kBucket,BucketPC,CountPC
cExtend  /i%PKS_F% /kBucketPC /v=100*Bucket/%NUM_BUCKm1%=
cProject /i%PKS_F% /s /h /kBucket,BucketPC,CountPC

set T2T_F=%hpkOUT_PREFIX%_t2t.lis
cNextRow /p0 /i%TS_F% /o%T2T_F% /f"," /k0-2 /h
cHead    /p0 /i%T2T_F% /t1 /x


if "%hpkDEBUG%"=="1" (
  set DRAW_PKS_F=%TEMP%\dpks.lis
  set DRAW_TS_F=%TEMP%\dts.lis

  cProject /p0 /i%PKS_F% /o!DRAW_PKS_F! /h /kBucket
  cHead    /p0 /i!DRAW_PKS_F! /h1 /x
  cPrefix  /p0 /i!DRAW_PKS_F! /l"point " /r",0"

  cProject /p0 /i%TS_F% /o!DRAW_TS_F! /h /kBucket
  cHead    /p0 /i!DRAW_TS_F! /h1 /x
  cPrefix  /p0 /i!DRAW_TS_F! /l"point " /r",0"

  %IMG7%magick ^
    -size %NUM_BUCK%x1 ^
    xc:#000 ^
    -fill Green -draw @!DRAW_PKS_F! ^
    -fill Red -draw @!DRAW_TS_F! ^
    %hpkOUT_PREFIX%_pktr.png

  call %PICTBAT%graphLineCol %hpkOUT_PREFIX%_pktr.png

  %IMG7%magick ^
    %hpkOUT_PREFIX%_h0_glc.png ^
    %hpkOUT_PREFIX%_pktr_glc.png ^
    -evaluate-sequence Max ^
    %hpkOUT_PREFIX%_h4_glc.png
)


call StopWatch


call echoRestore

@endlocal & set hpkOUTFILE=%OUTFILE%
