rem Given %1 is wildcard file (eg mydir\*.png),
rem writes filename and 21 or 14 or 7 perceptual hash values
rem (first column only)
rem to %2, output CSV file.
rem %3 is colorspace for hashing. (Should have 3 channels.)
rem %4 is which channel to output.
rem   Any of 0, 1, 2. Eg 012 or 12 or 01. Default 012.
@rem
@rem Updated:
@rem   3-September-2022 for IM v7.
@rem

setlocal enabledelayedexpansion

set IMAGES=%1

if "%IMAGES%"=="" (
  echo %0: No images.
  exit /B 1
)

set OUTFILE=%2
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=ip42i.csv

set COLSP=%3
if "%COLSP%"=="." set COLSP=
if "%COLSP%"=="" set COLSP=sRGB

set OUTCH=%4
if "%OUTCH%"=="." set OUTCH=
if "%OUTCH%"=="" set OUTCH=012

set OUT_0=0
set OUT_1=0
set OUT_2=0

set TMP=%OUTCH:0=X%
if not %TMP%==%OUTCH% set OUT_0=1
set TMP=%OUTCH:1=X%
if not %TMP%==%OUTCH% set OUT_1=1
set TMP=%OUTCH:2=X%
if not %TMP%==%OUTCH% set OUT_2=1

if %OUT_0%==0 if %OUT_1%==0 if %OUT_2%==0 (
  echo %0: No channels to output.
  exit /B 1
)

echo %0: %IMAGES% %OUTFILE% %COLSP% %OUT_0% %OUT_1% %OUT_2%

if /I %COLSP%==sRGB (
  set sCOLSP=
) else (
  set sCOLSP=-colorspace %COLSP% -set colorspace sRGB
)


del %OUTFILE% 2>nul

echo off

echo %0: Identify perceptual hashes of files %1 colorspace %COLSP% ...

set nFILES=0
(
  for %%F in (%1) do (
    echo %%F
    set nC=0
    set PH[0][1]B=
    for /F "usebackq tokens=1-3 delims=, " %%A in (`%IMG7%magick ^
      %%F ^
      -resize "600x600>" ^
      %sCOLSP% ^
      -verbose -define identify:moments ^
      info:`) do (
      rem echo %%A %%B %%C
      if "%%A"=="PH1:" (
        set PH[!nC!][1]B=%%B
        set PH[!nC!][1]C=%%C
      ) else if "%%A"=="PH2:" (
        set PH[!nC!][2]B=%%B
        set PH[!nC!][2]C=%%C
      ) else if "%%A"=="PH3:" (
        set PH[!nC!][3]B=%%B
        set PH[!nC!][3]C=%%C
      ) else if "%%A"=="PH4:" (
        set PH[!nC!][4]B=%%B
        set PH[!nC!][4]C=%%C
      ) else if "%%A"=="PH5:" (
        set PH[!nC!][5]B=%%B
        set PH[!nC!][5]C=%%C
      ) else if "%%A"=="PH6:" (
        set PH[!nC!][6]B=%%B
        set PH[!nC!][6]C=%%C
      ) else if "%%A"=="PH7:" (
        set PH[!nC!][7]B=%%B
        set PH[!nC!][7]C=%%C
        set /A nC+=1
      )
    )

    if "!PH[0][1]B!"=="" exit /B 1

    set sVALS=

    if %OUT_0%==1 set sVALS=!sVALS!^
!PH[0][1]B!,!PH[0][2]B!,!PH[0][3]B!,!PH[0][4]B!,!PH[0][5]B!,!PH[0][6]B!,!PH[0][7]B!,

    if %OUT_1%==1 set sVALS=!sVALS!^
!PH[1][1]B!,!PH[1][2]B!,!PH[1][3]B!,!PH[1][4]B!,!PH[1][5]B!,!PH[1][6]B!,!PH[1][7]B!,

    if %OUT_2%==1 set sVALS=!sVALS!^
!PH[2][1]B!,!PH[2][2]B!,!PH[2][3]B!,!PH[2][4]B!,!PH[2][5]B!,!PH[2][6]B!,!PH[2][7]B!,

    set sVALS=!sVALS:~0,-1!

    echo %%F,!sVALS! >>%OUTFILE%

    set /A nFILES+=1
  )
)

echo nFILES=%nFILES%

endlocal
