rem Approximate Gaussian blur by Stacked Integral Images (SII).
rem Reference integim.htm#apprxconv
rem Following is based on gaussIIeff.pdf

rem %1 input image
rem %2 output image
rem %3 sigma [default: 10]
rem %4 K (3, 4 or 5) [default: 4]

setlocal enabledelayedexpansion

echo off

set INFILE=%1
set OUTFILE=%2

set sigma=%3
if "%sigma%"=="." set sigma=
if "%sigma%"=="" set sigma=10

set sig0=(100/PI)

set K=%4
if "%K%"=="." set K=
if "%K%"=="" set K=4

if %K%==3 (
  set PIs=23,46,76
  set WIs=0.9495,0.5502,0.1618,0
) else if %K%==4 (
  set PIs=19,37,56,82
  set WIs=0.9649,0.6700,0.3376,0.0976,0
) else if %K%==5 (
  set PIs=16,30,44,61,85
  set WIs=0.9738,0.7596,0.5031,0.2534,0.0739,0
) else (
  echo %0: invalid K [%K%]
  exit /B 1
)

set I=1
for %%N in (%PIs%) do (
  set Pi[!I!]=%%N
  set /A I+=1
)

set I=1
for %%N in (%WIs%) do (
  set Wi[!I!]=%%N
  set /A I+=1
)

rem When sigma==sig0,
rem P[i] = Pi[i] roughly,
rem but W[i] = Wi[i]/2 roughly. This seems wrong.
rem Why are weights adjusted at all? To make the area under the curve sum to one?
rem But the 1D case works by adding 3 (or 4 or 5) weighted averages, so the weights should sum to one.

for /L %%I in (1,1,%K%) do (

  echo     -format "P[%%I]=%%[fx:floor(%sigma%/%sig0%*!Pi[%%I]!)]"

  for /F "usebackq" %%L in (`%IMG7%magick xc: ^
    -precision 16 ^
    -format "P[%%I]=%%[fx:floor(%sigma%/%sig0%*!Pi[%%I]!)]" ^
    info:`) do set %%L

  call :vplus1 %%I

  for /F "usebackq" %%L in (`%IMG7%magick xc: ^
    -precision 16 ^
    -format "W[%%I]=%%[fx:!Pi[%%I]!*!Wi[%%I]! / (2*!P[%%I]!+1)]" ^
    info:`) do set %%L

  for /F "usebackq" %%L in (`%IMG7%magick xc: ^
    -precision 16 ^
    -format "Wadj[%%I]=%%[fx:!Pi[%%I]!*(!Wi[%%I]!-!Wp1!) / (2*!P[%%I]!+1)]" ^
    info:`) do set %%L

  for /F "usebackq" %%L in (`%IMG7%magick xc: ^
    -precision 16 ^
    -format "W[%%I]=%%[fx:!Wi[%%I]!-!Wp1!]" ^
    info:`) do set %%L

)

set Pi[
set P[

set Wi[
set W[
set Wadj[

if %K%==3 (
  set PROCS1=
  set PROCS2=
  set sPOLY=-poly "%W[1]%,1 %W[2]%,1 %W[3]%,1"

) else if %K%==4 (
  set PROCS1=^( -clone 0 -process 'deintegim window %P[4]%x1' ^)
  set PROCS2=^( -clone 0 -process 'deintegim window 1x%P[4]%' ^)
  set sPOLY=-poly "%W[1]%,1 %W[2]%,1 %W[3]%,1 %W[4]%,1"
) else (
  set PROCS1=^( -clone 0 -process 'deintegim window %P[4]%x1' ^) ^
             ^( -clone 0 -process 'deintegim window %P[5]%x1' ^)
  set PROCS2=^( -clone 0 -process 'deintegim window 1x%P[4]%' ^) ^
             ^( -clone 0 -process 'deintegim window 1x%P[5]%' ^)
  set sPOLY=-poly "%W[1]%,1 %W[2]%,1 %W[3]%,1 %W[4]%,1 %W[5]%,1"
)

:: Next assumes k=4
:: Maybe we should add 8 transformations. This might explain the factor of 2 in the weights.

goto skip1

%IM7DEV%magick ^
  %INFILE% ^
  -process 'integim' ^
  ( -clone 0 -process 'deintegim window %P[1]%x1' -evaluate Multiply %W[1]% ) ^
  ( -clone 0 -process 'deintegim window %P[2]%x1' -evaluate Multiply %W[2]% ) ^
  ( -clone 0 -process 'deintegim window %P[3]%x1' -evaluate Multiply %W[3]% ) ^
  ( -clone 0 -process 'deintegim window %P[4]%x1' -evaluate Multiply %W[4]% ) ^
  -delete 0 ^
  -evaluate-sequence Add ^
+write i.png ^
  -process 'integim' ^
  ( -clone 0 -process 'deintegim window 1x%P[1]%' -evaluate Multiply %W[1]% ) ^
  ( -clone 0 -process 'deintegim window 1x%P[2]%' -evaluate Multiply %W[2]% ) ^
  ( -clone 0 -process 'deintegim window 1x%P[3]%' -evaluate Multiply %W[3]% ) ^
  ( -clone 0 -process 'deintegim window 1x%P[4]%' -evaluate Multiply %W[4]% ) ^
  -delete 0 ^
  +write d.png ^
  -evaluate-sequence Add ^
-auto-level ^
  %OUTFILE%

:skip1

:: Previous, but using "-poly".
%IM7DEV%magick ^
  %INFILE% ^
  -process 'integim' ^
  ( -clone 0 -process 'deintegim window %P[1]%x1' ) ^
  ( -clone 0 -process 'deintegim window %P[2]%x1' ) ^
  ( -clone 0 -process 'deintegim window %P[3]%x1' ) ^
  %PROCS1% ^
  -delete 0 ^
  %sPOLY% ^
  -process 'integim' ^
  ( -clone 0 -process 'deintegim window 1x%P[1]%' ) ^
  ( -clone 0 -process 'deintegim window 1x%P[2]%' ) ^
  ( -clone 0 -process 'deintegim window 1x%P[3]%' ) ^
  %PROCS2% ^
  -delete 0 ^
  %sPOLY% ^
-auto-level ^
  %OUTFILE%


goto skip2
:: Next looks blocky, horrible.
::
%IM7DEV%magick ^
  %INFILE% ^
  -process 'integim' ^
  ( -clone 0 -process 'deintegim window %P[1]%x1' -evaluate Multiply %Wadj[1]% ) ^
  ( -clone 0 -process 'deintegim window %P[2]%x1' -evaluate Multiply %Wadj[2]% ) ^
  ( -clone 0 -process 'deintegim window %P[3]%x1' -evaluate Multiply %Wadj[3]% ) ^
  ( -clone 0 -process 'deintegim window %P[4]%x1' -evaluate Multiply %Wadj[4]% ) ^
  ( -clone 0 -process 'deintegim window 1x%P[1]%' -evaluate Multiply %Wadj[1]% ) ^
  ( -clone 0 -process 'deintegim window 1x%P[2]%' -evaluate Multiply %Wadj[2]% ) ^
  ( -clone 0 -process 'deintegim window 1x%P[3]%' -evaluate Multiply %Wadj[3]% ) ^
  ( -clone 0 -process 'deintegim window 1x%P[4]%' -evaluate Multiply %Wadj[4]% ) ^
  -delete 0 ^
  +write d.png ^
  -evaluate-sequence Add ^
  %OUTFILE%
:skip2

endlocal

exit /B 0

::----------------------
:: Subroutines

:vplus1
set /A vp1=%1+1
set Wp1=!Wi[%vp1%]!