rem Replicate a Gaussian blur exactly by integral images,
rem by "fully stacked integral images".
rem
rem %1 input image
rem %2 output image
rem %3 sigma
@rem
@rem Beware: for large values of sigma, this eats memory.
@rem A version of this could be written that adds weighted images as they are made,
@rem instead of keeping each in memory until the "-poly".
@rem

@rem TODO: make environment variables and file.


setlocal enabledelayedexpansion

set TMPDIR=\temp

set fDeinteg1=%TMPDIR%\iige1.txt
set fDeinteg2=%TMPDIR%\iige2.txt
set fPoly=%TMPDIR%\iipoly.txt
set fScr=%TMPDIR%\iige.scr

del %fDeinteg1% 2>nul
del %fDeinteg2% 2>nul
del %fPoly% 2>nul
del %fScr% 2>nul

set INFILE=%1
set OUTFILE=%2

set sigma=%3
if "%sigma%"=="." set sigma=
if "%sigma%"=="" set sigma=10


echo off

echo %0: Calculating kernel

set CNT=0

for /F "usebackq tokens=1,*" %%A in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -define "morphology:showKernel=1" ^
  -morphology Convolve:0 Blur:0x%sigma% ^
  info: 2^>^&1 ^|grep "0:" ^|head -n 1 ^| tr "  " \r\n `) do (
  if not "%%A"=="0:" (
    set KNL_V[!CNT!]=%%A
    set /A CNT+=1
  )
)

rem %CNT% should be an odd number

set KNL_V[


::------

echo %0: CNT=%CNT% Writing subfiles

set KNL_V[!CNT!]=0

set /A nLast=%CNT%-1
set /A nMid=%CNT%/2

echo %0: nMid=%nMid% nLast=%nLast%

rem set KNL_V

for /L %%N in (%nMid%,1,%nLast%) do (
  set /A "Diam=(%%N-%nMid)*2+1"
  rem echo %%N !KNL_V[%%N]!

  call :knlNp1 %%N

  for /F "usebackq" %%L in (`%IMG7%magick ^
    xc: ^
    -precision 15 ^
    -format "Weight=%%[fx:!KNL_V[%%N]!-!vp1!]" ^
    info:`) do set %%L

  echo !Weight!,1,>>%fPoly%
  if !Diam! GTR 1 (
    echo ^( -clone 0 -process 'deintegim window !Diam!x1 adjedges' ^) >>%fDeinteg1%
    echo ^( -clone 0 -process 'deintegim window 1x!Diam! adjedges' ^) >>%fDeinteg2%
  )
)

echo %0: Writing %fScr%
(
  echo -process integim
  echo mpr:SRC
  type %fDeinteg1%
  echo -delete 0
  echo -poly @"%fPoly%"
  echo -process integim
  echo mpr:SRC
  type %fDeinteg2%
  echo -delete 0
  echo -poly @"%fPoly%"
  echo -write %OUTFILE%
  echo -exit
) >%fScr%

echo %0: Executing IM

%IM7DEV%magick %INFILE% +write mpr:SRC -script %fScr%

exit /B 0

::-----------------------------------------------------
:: Subroutine

:knlNp1
set /A np1=%1+1
set vp1=!KNL_V[%np1%]!

exit /B 0
