rem What multiple different deintegrations of an integral image is closest to a 1D Gaussian blur?
rem Stacked Integral Images (SII) by the mid-edge method.
rem This is quite slow.

rem %1 Sigma [default 10]
rem %2 K, number of blocks [default 3]
rem %3 Prefix for output variables [default iigk_]

rem We can't "setlocal".
rem @setlocal enabledelayedexpansion

@call echoOffSave

set sigma=%1
if "%sigma%"=="." set sigma=
if "%sigma%"=="" set sigma=10

set K=%2
if "%K%"=="." set K=
if "%K%"=="" set K=3

set Prefix=%3
if "%Prefix%"=="." set Prefix=
if "%Prefix%"=="" set Prefix=iigk_

echo %0: Calculating kernel

set CNT=0

for /F "usebackq tokens=1,*" %%A in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -define "morphology:showKernel=1" ^
  -morphology Convolve:0 Blur:0x%sigma% ^
  info: 2^>^&1 ^|grep "0:" ^|head -n 1 ^| tr "  " \r\n `) do (
  if not "%%A"=="0:" (
    set KNL_V[!CNT!]=%%A
    set /A CNT+=1
  )
)

rem %CNT% should be an odd number

rem set KNL_V[


echo %0: CNT=%CNT%

set KNL_V[!CNT!]=0

set /A nLast=%CNT%-1
set /A nMid=%CNT%/2

echo %0: nMid=%nMid% nLast=%nLast%

set /A Km1=%K%-1

set Vtop=!KNL_V[%nMid%]!

set ReqWt=%Vtop%/%K%

if %K% gtr 1 set ReqWt=%ReqWt% + 0.25*%ReqWt%/%Km1%

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -format "ReqWt=%%[fx:%ReqWt%]" ^
  info:`) do set %%L

echo %0: ReqWt=%ReqWt%

set Area=0

set startJ=%nMid%

for /L %%I in (0,1,%Km1%) do (
  if %%I==%Km1% (
    set ThisWt=!VTop!
    set Vbottom=0
  ) else (
    set ThisWt=%ReqWt%
    for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "ThisWt=%%[fx:%ReqWt%]\nVbottom=%%[fx:!Vtop!-^(%ReqWt%^)]\n" ^
      info:`) do set %%L
  )

  for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "Vmid=%%[fx:^(!Vtop!+!Vbottom!^)/2]" ^
      info:`) do set %%L

  @rem Find the value closest to Vmid. This is not fast.

  set MinDiff=999
  set JatMin=0
  set Done=0

  for /L %%J in (!startJ!,1,%nLast%) do (
    if !Done!==0 (
      for /F "usebackq" %%L in (`%IMG7%magick ^
        xc: ^
        -precision 15 ^
        -format "Diff=%%[fx:abs^(!Vmid!-!KNL_V[%%J]!^)]" ^
        info:`) do set %%L
      for /F "usebackq" %%L in (`%IMG7%magick ^
        xc: ^
        -precision 15 ^
        -format "Better=%%[fx:!Diff!<!MinDiff!?1:0]" ^
        info:`) do set %%L
      if !Better!==1 (
        set MinDiff=!Diff!
        set JatMin=%%J
      ) else (
        set Done=1
      )
      rem echo J=%%J  Diff=!Diff! Better=!Better! MinDiff=!MinDiff!
    )
  )

  echo %%I Vtop=!Vtop! Vbottom=!VBottom! Vmid=!Vmid! JatMin=!JatMin!

  set startJ=!JatMin!

  set Vtop=!Vbottom!

  set /A DatMin=^(!JatMin!-%nMid%^)*2+1

  for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "Area=%%[fx:!Area!+!DatMin!*!ThisWt!]" ^
      info:`) do set %%L

  set %Prefix%Diam[%%I]=!DatMin!
  set %Prefix%Weight[%%I]=!ThisWt!

  echo %0: DatMin=!DatMin! ThisWt=!ThisWt!
)

echo Area=%Area%

@rem Quantisation means the area is unlikely to be exactly 1.0.
@rem Adjust all the weights.

echo %0: Adjusting weights for area.

for /L %%I in (0,1,%Km1%) do (
  for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "%Prefix%Weight[%%I]=%%[fx:!%Prefix%Weight[%%I]!/%Area%]" ^
      info:`) do set %%L
)

set %Prefix%K=%K%
set %Prefix%Sigma=%Sigma%

call echoRestore

@endlocal

