rem Like iiGaussCalc.bat, but finds 50% of area, processing by images.

rem What multiple different deintegrations of an integral image is closest to a 1D Gaussian blur?

rem %1 Sigma [default 10]
rem %2 K, number of blocks [default 3]
rem %3 Prefix for output variables [default iigk_]
rem %4 Limit area error (0 to 1) [default 0.0001]

@rem This works from the lowest (widest) slice upwards.


@setlocal enabledelayedexpansion

@call echoOffSave


set sigma=%1
if "%sigma%"=="." set sigma=
if "%sigma%"=="" set sigma=10

set K=%2
if "%K%"=="." set K=
if "%K%"=="" set K=3

set Prefix=%3
if "%Prefix%"=="." set Prefix=
if "%Prefix%"=="" set Prefix=iigk_

set LimAreaErr=%4
if "%LimAreaErr%"=="." set LimAreaErr=
if "%LimAreaErr%"=="" set LimAreaErr=0.0001

set TMPDIR=\temp\
set HalfGauss=%TMPDIR%%Prefix%_iigkc2b.miff

rem echo %0: Calculating kernel

@rem Vtop etc are on a scale of 0.0 to 1.0.

set /A Km1=%K%-1

set Vtop=

for /F "usebackq" %%L in (`%IMG7%magick ^
  -size %%[fx:%sigma%*10]x1 xc:Black ^
  -size 1x1 xc:White ^
  ^( -clone 0 ^) ^
  +append +repage ^
  -colorspace Gray ^
  -define "quantum:format=floating-point" -depth 32 ^
  -morphology Convolve Blur:0x%sigma% ^
  -alpha off ^
  -precision 15 ^
  -bordercolor Black -border 1 ^
  -trim +repage ^
  -format "WholeArea=%%[fx:mean*w]\n" +write info: ^
  -gravity East -crop %%[fx:^(w+1^)/2]x1+0+0 +repage ^
  +write %HalfGauss% ^
  -format "radP1=%%w\nMaxV=%%[fx:p[0,0]]\nHalfArea=%%[fx:mean*w]" ^
  info:`) do set %%L

if "%MaxV%"=="" (
  echo %0: MaxV not set
  exit /B 1
)
set Vtop=%MaxV%

echo %0: radP1=%radP1% Vtop=%Vtop% HalfArea=%HalfArea% WholeArea=%WholeArea%

set ReqWt=%Vtop%/%K%

if %K%==1 (
  set ReqWtBot=%Vtop%
  set ReqWtOth=%Vtop%
) else (
  set ReqWtBot=%Vtop%/%K%/1.5
  set "ReqWtOth=(%Vtop%-!ReqWtBot!)/%Km1%"
)

rem if %K% gtr 1 set ReqWt=%ReqWt% + 0.25*%ReqWt%/%Km1%

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -format "ReqWt=%%[fx:%ReqWt%]\n" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -format "ReqWtOth=%%[fx:%ReqWtOth%]\n" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -format "ReqWtBot=%%[fx:%ReqWtBot%]\n" ^
  info:`) do set %%L

echo %0: Vtop=%Vtop% ReqWt=%ReqWt% ReqWtOth=%ReqWtOth% ReqWtBot=%ReqWtBot%

set Area=0

for /L %%I in (%Km1%,-1,0) do (
  if %%I==%Km1% (
    set Vtop=%ReqWtBot%
    set ThisWt=!VTop!
    set Vbottom=0
  ) else if %%I==0 (
    for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "Vtop=%%[fx:%MaxV%]\n" -write info: ^
      -format "ThisWt=%%[fx:%MaxV%-!Vbottom!]\n" ^
      info:`) do set %%L
  ) else (
    set ThisWt=%ReqWtOth%
    for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "Vtop=%%[fx:!Vbottom!+%ReqWtOth%]\n" ^
      info:`) do set %%L
  )

  echo %%I Vtop=!Vtop! Vbottom=!Vbottom! ThisWt=!ThisWt!

  @rem Ignoring values above Vtop and below Vbottom, just for the RHS, where is the area half of total?.

  set Xoffs=

  set GoodEnuf=0
  set nIter=0

  for /L %%J in (0,1,99) do if !GoodEnuf!==0 (

    if %%I==0 (
      for /F "usebackq" %%L in (`%IMG7%magick ^
        xc: ^
        -precision 15 ^
        -format "ctop=%%[fx:!Vtop!>%MaxV%?!Vtop!:%MaxV%]" ^
        info:`) do set %%L

    ) else (
      set ctop=!Vtop!
    )

    for /F "usebackq" %%L in (`%IMG7%magick ^
      %HalfGauss% ^
      -precision 15 ^
      -alpha off ^
      -fx "u>!ctop!?%%[fx:!ctop!-!Vbottom!]:u-!Vbottom!" ^
      -clamp ^
      -format "CurveSemiArea=%%[fx:mean*w]\n" +write info: ^
      -format "Xoffs=%%[fx:floor(mean*w/!ThisWt!-0.5)]\n" +write info: ^
      -format "BlockSemiArea=%%[fx:!ThisWt!*floor(mean*w/!ThisWt!-0.5)+!ThisWt!]\n" +write info: ^
      NULL:`) do set %%L

rem       -evaluate Subtract %%[fx:!Vbottom!*QuantumRange] -clamp

    if "!Xoffs!"=="" (
      echo %0: Xoffs not set. I=%%I
      exit /B 1
    )
    rem echo %0: %%I Vtop=!Vtop! Vbottom=!VBottom! CurveSemiArea=!CurveSemiArea! BlockSemiArea=!BlockSemiArea! ThisWt=!ThisWt! Xoffs=!Xoffs!

    rem The Xoffs must be an integer, so the area will have a quantisation error.
    rem Adjust this weight to compensate.
    rem We start from bottom slice.

    for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -format "GoodEnuf=%%[fx:abs(!CurveSemiArea!-!BlockSemiArea!)/!CurveSemiArea!>%LimAreaErr%?0:1]\n" ^
      info:`) do set %%L

    set /A nIter+=1
    set DontTrim=0

    if !GoodEnuf!==0 (

      for /F "usebackq" %%L in (`%IMG7%magick ^
        xc: ^
        -precision 15 ^
        -format "ThisWt=%%[fx:!ThisWt!*!CurveSemiArea!/!BlockSemiArea!]\n" ^
        info:`) do set %%L

      for /F "usebackq" %%L in (`%IMG7%magick ^
        xc: ^
        -precision 15 ^
        -format "Vtop=%%[fx:!Vbottom!+!ThisWt!]\n" +write info: ^
        info:`) do set %%L

rem      if %%I==0 (
rem        for /F "usebackq" %%L in (`%IMG7%magick ^
rem          xc: ^
rem          -precision 15 ^
rem          -format "DontTrim=%%[fx:!tmpVtop!<%MaxV%?1:0]\n" +write info: ^
rem          info:`) do set %%L
rem      )
rem
rem      if !DontTrim!==1 (
rem        set GoodEnuf=1
rem      ) else (
rem        set ThisWt=!tmpThisWt!
rem        set Vtop=!tmpVtop!
rem      )
rem
rem      set ThisWt=!tmpThisWt!
rem      set Vtop=!tmpVtop!

      rem echo %0: new ThisWt=!ThisWt! Vtop=!Vtop! GoodEnuf=!GoodEnuf!

      rem The new weight and Vtop means the slice covers a different part of the curve,
      rem so go round again.
    )
  )
  rem End J loop

  set Vbottom=!Vtop!

  set /A DatMin=Xoffs*2+1

  for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "Area%%I=%%[fx:!DatMin!*!ThisWt!]" ^
      info:`) do set %%L

  echo %0: DatMin=!DatMin! ThisWt=!ThisWt! CurveSemiArea=!CurveSemiArea! BlockSemiArea=!BlockSemiArea! Area%%I=!Area%%I! GoodEnuf=!GoodEnuf! nIter=!nIter!

  for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "Area=%%[fx:!Area!+!DatMin!*!ThisWt!]" ^
      info:`) do set %%L

  set %Prefix%Diam[%%I]=!DatMin!
  set %Prefix%Weight[%%I]=!ThisWt!

  rem echo %0: DatMin=!DatMin! ThisWt=!ThisWt!
)

if ERRORLEVEL 1 exit /B 1

echo %0 Area=%Area%

set %Prefix%K=%K%
set %Prefix%Sigma=%sigma%

@rem Quantisation means the total area is unlikely to be exactly 1.0.
@rem Adjust all the weights.

goto skipAdj

set %Prefix%

echo %0: Adjusting weights for area.

for /L %%I in (0,1,%Km1%) do (
  for /F "usebackq" %%L in (`%IMG7%magick ^
      xc: ^
      -precision 15 ^
      -format "%Prefix%Weight[%%I]=%%[fx:!%Prefix%Weight[%%I]!/%Area%]" ^
      info:`) do set %%L
)

set %Prefix%

:skipAdj

set %Prefix% > %Prefix%_vars.lis

call echoRestore

@endlocal & set Prefix=%Prefix%

@for /F %%L in (%Prefix%_vars.lis) do @set %%L
