rem Find the best radius, hence diameter, of a block for a slice of a Gaussian kernel,
rem as measured by RMSE.

rem %1 sigma
rem %2 Vtop
rem %3 Vbottom

@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

echo %0: [%1] [%2] [%3]

set sigma=%1
if "%sigma%"=="." set sigma=
if "%sigma%"=="" set sigma=10

set Vtop=%2
set Vbottom=%3

set TMPDIR=\temp\
set HalfGauss=%TMPDIR%%Prefix%_iigkc2b.miff
set CurveSlice=%TMPDIR%%Prefix%_iicrvsl.miff


for /F "usebackq" %%L in (`%IMG7%magick ^
  -size %%[fx:%sigma%*10]x1 xc:Black ^
  -size 1x1 xc:White ^
  ^( -clone 0 ^) ^
  +append +repage ^
  -colorspace Gray ^
  -define "quantum:format=floating-point" -depth 32 ^
  -morphology Convolve Blur:0x%sigma% ^
  -alpha off ^
  -precision 15 ^
  -bordercolor Black -border 1 ^
  -trim +repage ^
  -format "WholeArea=%%[fx:mean*w]\n" +write info: ^
  -gravity East -crop %%[fx:^(w+1^)/2]x1+0+0 +repage ^
  +write %HalfGauss% ^
  -format "radP1=%%w\nMaxV=%%[fx:p[0,0]]\nHalfArea=%%[fx:mean*w]" ^
  info:`) do set %%L

if "%MaxV%"=="" (
  echo %0: MaxV not set
  exit /B 1
)

echo %0: radP1=%radP1% MaxV=%MaxV% Vtop=%Vtop% HalfArea=%HalfArea% WholeArea=%WholeArea%

set ReqWt=%Vtop%

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -format "ThisWt=%%[fx:%Vtop%-!Vbottom!]\n" ^
  info:`) do set %%L


echo ThisWt=%ThisWt%

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -format "ctop=%%[fx:!Vtop!>%MaxV%?!Vtop!:%MaxV%]" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick ^
  %HalfGauss% ^
  -precision 15 ^
  -alpha off ^
  -fx "u>!Vtop!?%%[fx:!Vtop!-!Vbottom!]:u-!Vbottom!" ^
  -clamp ^
  -define "quantum:format=floating-point" -depth 32 ^
  +write %CurveSlice% ^
  -format "CurveSemiArea=%%[fx:mean*w]\n" +write info: ^
  -format "Xoffs=%%[fx:floor(mean*w/!ThisWt!-0.5)]\n" +write info: ^
  -format "BlockSemiArea=%%[fx:!ThisWt!*floor(mean*w/!ThisWt!-0.5)+!ThisWt!]\n" +write info: ^
  NULL:`) do set %%L


echo %0: ctop=!ctop! Vtop=!Vtop! Vbottom=!VBottom! CurveSemiArea=!CurveSemiArea! BlockSemiArea=!BlockSemiArea! ThisWt=!ThisWt! Xoffs=!Xoffs!


rem Some pixels from the left of the curve and block are equal,
rem so the boundary cannot occur there.

for /F "usebackq" %%L in (`%IMG7%magick ^
  %CurveSlice% ^
  -auto-level ^
  -fill Black +opaque White ^
  -format "StartRad=%%[fx:mean*w]\n" +write info: ^
  NULL:`) do set %%L

echo %0: StartRad=%StartRad% radP1=%radP1%

set BestComp=2.0
set Xoffs=0
set GoodEnuf=0

for /L %%I in (%StartRad%,1,%radP1%) do if !GoodEnuf!==0 (

  for /F "usebackq" %%L in (`%IMG7%magick ^
    -size %%Ix1 xc:White ^
    -evaluate Multiply %ThisWt% ^
    -background Black -extent %radP1% ^
    %CurveSlice% ^
    -metric RMSE ^
    -precision 15 ^
    -format %%[distortion] ^
    -compare ^
    info:`) do set comp=%%L

  for /F "usebackq" %%L in (`%IMG7%magick ^
    xc: ^
    -format "IsBetter=%%[fx:!BestComp!>!comp!?1:0]" ^
    info:`) do set %%L

  if !IsBetter!==1 (
    set BestComp=!comp!
    set Xoffs=%%I
  ) else (
    set GoodEnuf=1
  )
  rem echo %%I !comp! IsBetter=!IsBetter! GoodEnuf=!GoodEnuf!
)

set /A RatBest=%Xoffs%

set /A DatBest=%RatBest%*2+1

echo BestComp=%BestComp% RatBest=%RatBest% DatBest=%DatBest%

rem In next "+1" because we need to include the central pixel.

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -alpha off ^
  -format "BlockSemiArea=%%[fx:%ThisWt%*(%RatBest%+1)]\n" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -alpha off ^
  -format "AreaErr=%%[fx:abs(%CurveSemiArea%-%BlockSemiArea%)/%CurveSemiArea%]\n" ^
  info:`) do set %%L

echo CurveSemiArea=%CurveSemiArea% BlockSemiArea=%BlockSemiArea% AreaErr=%AreaErr% DatBest=%DatBest%

call echoRestore

@endlocal & set DatBest=%DatBest%

