rem From %1, file of multiple same-sized images,
rem returns %2 with same number of same-size images,
rem white where this image is the maximum of all images; otherwise black.
@rem
@rem Also uses:
@rem
@rem   imSTART_LEVEL
@rem
@rem Also returns variables imMinOfMean etc
@rem
@rem Updated:
@rem   5-September-2022 for IM v7.
@rem

set PREF=%~n2_im_

goto skipInMem

:: If fuzz > 0, we get overlaps.

for /F "usebackq" %%L in (`%IMG7%magick ^
  %1 ^
  ^( -clone 0--1 ^
    -evaluate-sequence Max ^
  ^) ^
  null: ^
  +swap ^
  -compose Difference -layers composite ^
  -fill White ^
  -fuzz 0.0%% ^
  +opaque Black ^
  -fill Black -opaque Black ^
  -negate ^
  +write %2 ^
  -format "mean.%%s=%%[fx:mean]\n" +write info: ^
  -evaluate-sequence Mean ^
  -format "imMinOfMean=%%[fx:minima]\nimMeanOfMean=%%[fx:mean]\nimMaxOfMean=%%[fx:maxima]\n" +write info: ^
  -auto-level ^
  -depth 8 -define quantum:format^=integer ^
  %3`) do set %%L

:: Different strategy to conserve memory:
:: To find max, read just two files at a time.
:: Loop through inputs to find where each matches the max.

:skipInMem

if "%imSTART_LEVEL%"=="" set imSTART_LEVEL=0

set isCNT=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "isCNT=%%n\n" ^
  %1`) do set %%L

if "%isCNT%"=="" exit /B 1

if %imSTART_LEVEL% GEQ %isCNT% set imSTART_LEVEL=0

echo %~n0: imSTART_LEVEL=%imSTART_LEVEL%

set /A Nm1=%isCNT%-1

:: Special-case if Nm1==0, return white image

if %Nm1%==0 (
  %IMG7%magick ^
    %1 ^
    -fill White -colorize 100 ^
    +write %2 ^
    -fill Black -colorize 100 ^
    %3

  goto :end
)

:: Find the maximum.

set TMP_MAX=%PREF%TmpMax.miff

%IMG7%magick ^
  %1[%imSTART_LEVEL%] ^
  -depth 64 ^
  -define quantum:format=floating-point ^
  %TMP_MAX%

set /A SLp1=%imSTART_LEVEL%+1

for /L %%N in (%SLp1%,1,%Nm1%) do (
  echo %~n0: %%N %TMP_MAX%

  %IMG7%magick ^
    %TMP_MAX% ^
    %1[%%N] ^
    -evaluate-sequence Max ^
    -depth 64 ^
    -define quantum:format=floating-point ^
    %TMP_MAX%
)

set TMP_BLACK=%PREF%tmp_blk.tiff

if not %imSTART_LEVEL%==0 (
  %IMG7%magick ^
    %TMP_MAX% ^
    -fill Black -colorize 100 ^
    %TMP_BLACK%
)


:: Without a fuzz factor, some pixels will not have a maximum at any frequency.

set FILE_LIST=
for /L %%N in (0,1,%Nm1%) do (
  set TMP_LAYER=%PREF%tmp_%%N.tiff

  echo %~n0: %%N !TMP_LAYER!

  if %%N GEQ %imSTART_LEVEL% (
    %IMG7%magick ^
    %TMP_MAX% ^
    %1[%%N] ^
    -compose Difference -composite ^
    -fuzz 0.000001%% ^
    -fill White +opaque Black ^
    -fill Black -opaque Black ^
    -threshold 0 ^
    -negate ^
    -depth 8 ^
    -define quantum:format=integer ^
    !TMP_LAYER!

    set FILE_LIST=!FILE_LIST! !TMP_LAYER!
  ) else (
    set FILE_LIST=!FILE_LIST! %TMP_BLACK%
  )

)

tiffcp %FILE_LIST% %2

del %FILE_LIST%
del %TMP_MAX%

:end
