rem Given %1 is file of multiple same-size images, each pixel white or black,
rem makes four outputs, all named after %2,
rem with names suffixed iwMin, iwMin_al, iwMax, iwMax_al.
@rem
@rem Updated:
@rem   15 May 2016 use %IML% for @script.
@rem   5-September-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion


set iwPREF=%~n2
set iwEXT=%~x2

if "%iwPREF%"=="" set iwPREF=iw_
if "%iwEXT%"=="" set iwEXT=.png

set iwNm1=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "iwNm1=%%[fx:n-1]\n" ^
  %1`) do set %%L

if "%iwNm1%"=="" exit /B 1

echo %~n0: %1

set OUT_MIN=%iwPREF%iwMin%iwEXT%
set OUT_MAX=%iwPREF%iwMax%iwEXT%

set OUT_MIN_AL=%iwPREF%iwMin_al%iwEXT%
set OUT_MAX_AL=%iwPREF%iwMax_al%iwEXT%

goto skipInMem



set SCR_FILE=iw_scr.scr

(
  echo %1
  echo -auto-level

  rem echo -format "bef min=%%[fx:minima*QuantumRange] max=%%[fx:maxima*QuantumRange]\n" +write info:

  echo ( -clone 0-%iwNm1%
  for /L %%N in (0,1,%iwNm1%) do @(
    set /A Np1=%%N+1
    echo ^( -clone %%N -evaluate Min %%N ^)
  )
  echo -delete 0-%iwNm1%
  rem echo -format "F1a min=%%[fx:minima*QuantumRange] max=%%[fx:maxima*QuantumRange]\n" +write info:
  echo -evaluate-sequence Max
  rem echo -format "F1 min=%%[fx:minima*QuantumRange] max=%%[fx:maxima*QuantumRange]\n" +write info:
  echo +write %iwPREF%iwMax%iwEXT%
  echo -auto-level
  echo +write %iwPREF%iwMax_al%iwEXT%
  echo +delete
  echo ^)

  echo ( -clone 0-%iwNm1%
  echo -negate
  for /L %%N in (0,1,%iwNm1%) do @(
    set /A Np1=%%N+1
    echo ^( -clone %%N -evaluate Max %%N ^)
  )
  echo -delete 0-%iwNm1%
  rem echo -format "F2a min=%%[fx:minima*QuantumRange] max=%%[fx:maxima*QuantumRange]\n" +write info:
  echo -evaluate-sequence Min
  rem echo -format "F2 min=%%[fx:minima*QuantumRange] max=%%[fx:maxima*QuantumRange]\n" +write info:
  echo +write %iwPREF%iwMin%iwEXT%
  echo -auto-level
  echo +write %iwPREF%iwMin_al%iwEXT%
  echo +delete
  echo ^)
  echo NULL:
)>%SCR_FILE%


%IML%magick ^
  +depth ^
  -precision 20 ^
  -script %SCR_FILE% ^

:: ------------------------------------------------
:: Different strategy to conserve memory:

:skipInMem

set MAX_LIST=
set MIN_LIST=

for /L %%N in (0,1,%Nm1%) do (
  set TMP_MAX=%iwPREF%iw_tmp_max_%%N.miff
  set TMP_MIN=%iwPREF%iw_tmp_min_%%N.miff

  set /A Np1=%%N+1

  echo %~n0: %%N !TMP_MAX! !TMP_MIN! !Np1!

  %IMG7%magick ^
    %1[%%N] ^
    +depth ^
    ^( +clone ^
       -evaluate Min !Np1! ^
       +write !TMP_MAX! ^
       +delete ^
    ^) ^
    -negate ^
    -evaluate Max !Np1! ^
    +depth ^
    !TMP_MIN!

  %IMG7%magick identify -precision 20 ^
    -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" !TMP_MAX!

  %IMG7%magick identify -precision 20 ^
    -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" !TMP_MIN!

  set MAX_LIST=!MAX_LIST! !TMP_MAX!
  set MIN_LIST=!MIN_LIST! !TMP_MIN!
)

%IMG7%magick %MAX_LIST% -compose Add -layers flatten -format %%c histogram:info:%iwPREF%x1.lis
%IMG7%magick %MIN_LIST% -compose Add -layers flatten -format %%c histogram:info:%iwPREF%x2.lis

%IMG7%magick %iwPREF%iw_tmp_max_0.miff +depth %OUT_MAX%
%IMG7%magick %iwPREF%iw_tmp_min_0.miff +depth %OUT_MIN%

del %iwPREF%iw_tmp_max_0.miff
del %iwPREF%iw_tmp_min_0.miff

for /L %%N in (1,1,%Nm1%) do (
  set TMP_MAX=%iwPREF%iw_tmp_max_%%N.miff
  set TMP_MIN=%iwPREF%iw_tmp_min_%%N.miff

  echo %~n0: %%N !TMP_MAX! !TMP_MIN!

  %IMG7%magick identify -precision 20 ^
    -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" !TMP_MAX!
  %IMG7%magick identify -precision 20 ^
    -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" !TMP_MIN!

  %IMG7%magick  ^
    %OUT_MAX% ^
    !TMP_MAX! ^
    -evaluate-sequence Max ^
    +depth ^
    %OUT_MAX%

  %IMG7%magick  ^
    %OUT_MIN% ^
    !TMP_MIN! ^
    -evaluate-sequence Min ^
    +depth ^
    %OUT_MIN%

  %IMG7%magick identify -precision 20 ^
    -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" !OUT_MAX!
  %IMG7%magick identify -precision 20 ^
    -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" !OUT_MIN!

  del !TMP_MAX! !TMP_MIN!
)

%IMG7%magick identify ^
  -precision 20 ^
  -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" ^
  %OUT_MAX%

%IMG7%magick identify ^
  -precision 20 ^
  -format "%%f %%[fx:minima*QuantumRange] %%[fx:maxima*QuantumRange]\n" ^
  %OUT_MIN%

%IMG7%magick %OUT_MAX% -auto-level -negate +depth %OUT_MAX_AL%
%IMG7%magick %OUT_MIN% -auto-level -negate +depth %OUT_MIN_AL%


@endlocal
