rem Finds a maximal rectangle of white pixels.
rem Ie provides crop of all non-white pixels.

@setlocal
@call echoOffSave

rem Given image %1, coord (%2,%3) that should be white,
rem returns variable INNER_TRIM	format WxH+X+Y
rem that specifies a rectangle containing coord that is entirely white.

rem If (%2,%3) isn't white, returns with W==0.

rem Optionally call with:
rem   itWHITE_EDGE=1 if image may have white extending to an edge
rem   itDEBUG_FILE=filename to create an illustrative output file.

rem Not guaranteed to be largest possible white rectangle.
rem But is guaranteed to be bounded by non-white on all 4 sides.

rem Algorithm examines pixels in a square spiral pattern starting at (%2,%3).
rem When it hits a non-white pixel on one side,
rem it stops examining pixels on that side but continues radiating on the other sides.
rem When it has hit non-white on all four sides, the job is finished.
@rem
@rem Updated:
@rem   23-August-2022 Upgraded for IM v7.
@rem

call %PICTBAT%setInOut %1 it

@set TMP_IMG=%TEMP%\%~n1_it.miff

set IT_SEED_X=%2
set IT_SEED_Y=%3

if "%3"=="" (
  FOR /F %%L IN ('%IMG7%magick identify -format "IT_SEED_X=%%[fx:int(w/2)]\nIT_SEED_Y=%%[fx:int(h/2)]" %INFILE%') DO set %%L
)

set IT_STOP_L=0
set IT_STOP_T=0
set IT_STOP_R=0
set IT_STOP_B=0

set /A IT_L=%IT_SEED_X%
set /A IT_T=%IT_SEED_Y%
set /A IT_R=%IT_SEED_X%-1
set /A IT_B=%IT_SEED_Y%

set /A IT_H=1
set /A IT_W=0

if "%itWHITE_EDGE%"=="1" (
  %IMG7%magick ^
    %INFILE% ^
    -bordercolor #000 -border 1 ^
    %TMP_IMG%

  if ERRORLEVEL 1 exit /B 1

  set IT_SRC=%TMP_IMG%

  set /A IT_L+=1
  set /A IT_T+=1
  set /A IT_R+=1
  set /A IT_B+=1

  rem No need to change seed.
) else (
  %IMG7%magick %INFILE% %TMP_IMG%

  if ERRORLEVEL 1 exit /B 1
)

set IT_SRC=%INFILE%
set IT_SRC=%TMP_IMG%

for /F "usebackq" %%L ^
in (`%IMG7%magick identify -format "WW=%%w\nHH=%%h" %IT_SRC%`) ^
do set %%L

if "%WW%"=="" exit /B 1

call StopWatch
:loop

if %IT_STOP_R%==1 goto skip_R
rem Try right

set /A IT_R+=1
set /A IT_W+=1
set IT_MIN=
if %IT_R% LSS %WW% FOR /F "usebackq" %%L ^
IN (`%IMG7%magick %IT_SRC% -crop 1x%IT_H%+%IT_R%+%IT_T% -format "%%[fx:minima]" info:`) ^
DO set IT_MIN=%%L

if not "%IT_MIN%"=="1" (
  echo Stop right
  set IT_STOP_R=1
  set /A IT_R-=1
  set /A IT_W-=1
)

:skip_R


if %IT_STOP_B%==1 goto skip_B
rem Try down

set /A IT_B+=1
set /A IT_H+=1
set IT_MIN=
if %IT_B% LSS %HH% FOR /F "usebackq" %%L ^
IN (`%IMG7%magick %IT_SRC% -crop %IT_W%x1+%IT_L%+%IT_B% -format "%%[fx:minima]" info:`) ^
DO set IT_MIN=%%L

if not "%IT_MIN%"=="1" (
  echo Stop down
  set IT_STOP_B=1
  set /A IT_B-=1
  set /A IT_H-=1
)

:skip_B


if %IT_STOP_L%==1 goto skip_L
rem Try left

set /A IT_L-=1
set /A IT_W+=1
set IT_MIN=
if %IT_L% GEQ 0 FOR /F "usebackq" %%L ^
IN (`%IMG7%magick %IT_SRC% -crop 1x%IT_H%+%IT_L%+%IT_T% -format "%%[fx:minima]" info:`) ^
DO set IT_MIN=%%L

if not "%IT_MIN%"=="1" (
  echo Stop left
  set IT_STOP_L=1
  set /A IT_L+=1
  set /A IT_W-=1
)

:skip_L


if %IT_STOP_T%==1 goto skip_T
rem Try up

set /A IT_T-=1
set /A IT_H+=1
set IT_MIN=
if %IT_T% GEQ 0 FOR /F "usebackq" %%L ^
IN (`%IMG7%magick %IT_SRC% -crop %IT_W%x1+%IT_L%+%IT_T% -format "%%[fx:minima]" info:`) ^
DO set IT_MIN=%%L

if not "%IT_MIN%"=="1" (
  echo Stop up
  set IT_STOP_T=1
  set /A IT_T+=1
  set /A IT_H-=1
)

:skip_T


set /A IT_STOP_SUM=%IT_STOP_R%+%IT_STOP_B%+%IT_STOP_L%+%IT_STOP_T%

if %IT_STOP_SUM% LSS 4 goto loop

if "%itWHITE_EDGE%"=="1" (
  set /A IT_L-=1
  set /A IT_T-=1
  set /A IT_R-=1
  set /A IT_B-=1
)

set INNER_TRIM=%IT_W%x%IT_H%+%IT_L%+%IT_T%

if %IT_W%==0 (
  set INNER_TRIM_RECT=
) else (
  set INNER_TRIM_RECT=rectangle %IT_L% %IT_T% %IT_R% %IT_B%
)

set /A INNER_TRIM_AREA=%IT_W%*%IT_H%

if defined itDEBUG_FILE (
  rem Debug. Show the inner-trim area in green, with red dot at given origin.

  %IMG7%magick ^
    %INFILE% ^
    -draw "fill #0F0 %INNER_TRIM_RECT% fill #F00 point %IT_SEED_X%,%IT_SEED_Y%" ^
    %itDEBUG_FILE%
)

call StopWatch

:end
call echoRestore

@endlocal & ^
set INNER_TRIM=%INNER_TRIM%& ^
set INNER_TRIM_RECT=%INNER_TRIM_RECT%& ^
set INNER_TRIM_AREA=%INNER_TRIM_AREA%

@exit /B 0

:error
echo Error in %0 for %*
call echoRestore

@endlocal & ^
set INNER_TRIM=& ^
set INNER_TRIM_RECT=& ^
set INNER_TRIM_AREA=

exit /B 1
