rem Given %1 and %2 are Nx1 images,
rem makes Hermite interpolated image Nx%3 pixels,
rem named %4.
rem %5 is value of t at image top [0].
rem %6 is value of t at image bottom [1].
rem %7 is Nx1 image of gradient at t==0 [default: black].
rem %8 is Nx1 image of gradient at t==1 [default: black].
rem %9 gradient type: perimage, perpercent or perpixel.
@rem
@rem Also uses:
@rem  ichChopEnd if set, increase size by one, then shave it afterwards.
@rem
@rem Gradient images give expected changes in values per pixel. Can be negative.
@rem All inputs and output can be negative.
@rem Even if all inputs are positive, output may be negative.
@rem
@rem See https://en.wikipedia.org/wiki/Cubic_Hermite_spline
@rem
@rem Assumes magick is HDRI.

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 ich

set IMG1=%INFILE%
set IMG2=%2

set HH=%3
if "%HH%"=="." set HH=
if "%HH%"=="" set HH=10

if not "%4"=="" if not "%4"=="." set OUTFILE=%4

set T_TOP=%5
if "%T_TOP%"=="." set T_TOP=
if "%T_TOP%"=="" set T_TOP=0

set T_BOT=%6
if "%T_BOT%"=="." set T_BOT=
if "%T_BOT%"=="" set T_BOT=100
:: TODO: percentage?

set GRAD1=%7
if "%GRAD1%"=="." set GRAD1=

set GRAD2=%8
if "%GRAD2%"=="." set GRAD2=

set GRAD_TYPE=%9
if "%GRAD_TYPE%"=="." set GRAD_TYPE=
if "%GRAD_TYPE%"=="" set GRAD_TYPE=perpixel

set TMPDIR=\temp

if "%GRAD_TYPE%"=="perimage" (
  set GRAD_MULT=
) else if "%GRAD_TYPE%"=="perpercent" (
  set GRAD_MULT=-evaluate Multiply 100
) else if "%GRAD_TYPE%"=="perpixel" (
  set GRAD_MULT=-evaluate Multiply %%[fx:%HH%]
) else (
  echo %0: Bad GRAD_TYPE [%GRAD_TYPE%]
  exit /B 1
)

if "%ichChopEnd%"=="" (
  set ChopEnd=
) else (
  set ChopEnd=-gravity South -chop x1+0+0 +repage
)

echo %0: ChopEnd: %ChopEnd%

:: Better to do these as variables.
::
if "%GRAD1%"=="" (
  set GRAD1=%TMPDIR%\ich_grad1.miff
  %IMG7%magick %IMG1% -fill Black -colorize 100 -alpha off !GRAD1!
) else (
  set GRAD1=^( %GRAD1% %GRAD_MULT% ^)
)

if "%GRAD2%"=="" (
  set GRAD2=%TMPDIR%\ich_grad2.miff
  %IMG7%magick %IMG2% -fill Black -colorize 100 -alpha off !GRAD2!
) else (
  set GRAD2=^( %GRAD2% %GRAD_MULT% ^)
)

echo %0: GRAD1=%GRAD1%
echo %0: GRAD2=%GRAD2%

if "%T_TOP:~0,1%"=="-" (

  set sFMT=^
POS=%%[fx:-^(%T_TOP%^)]\n^
PCBOT=%%[fx:%T_BOT%-^(%T_TOP%^)]

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "!sFMT!" ^
    %IMG1%`) do set %%L

  set SUB=-evaluate subtract !POS!%%
  set T_TOP=0
) else (
  set SUB=
)

echo %0: top=%T_TOP% bot=%T_BOT% sub=%SUB%

:: h00 = 2*t^3 - 3*t^2 + 1
:: h10 = t^3 -2*t^2 + t
:: h01 = -2*t^3 + 3*t^2
:: h11 = t^3 - t^2

%IMG701010%magick ^
  %IMG1% +write mpr:I1 ^
  -define quantum:format=floating-point ^
  -depth 32 ^
  -set option:MYSIZE %%[fx:w]x%HH% ^
  -set option:HHSIZE 1x%HH% ^
  -define compose:clamp=off ^
  -size "%%[HHSIZE]" ^
  ( gradient:gray(%T_TOP%%%)-gray(%T_BOT%%%) ^
    %SUB% ^
    -set colorspace sRGB ^
    +write mpr:Tp1 ^
    -evaluate Pow 2 +write t2.miff -write mpr:Tp2 +delete ^
    mpr:Tp1 ^
    -evaluate Pow 3 +write t3.miff -write mpr:Tp3 ^
    -fill White -colorize 100 -write mpr:Unity +delete ^
    mpr:Tp3 mpr:Tp2 mpr:Unity -poly 2,1,-3,1,1,1 +write h00.miff -write mpr:h00 +delete ^
    mpr:Tp3 mpr:Tp2 mpr:Tp1 -poly 1,1,-2,1,1,1 +write h10.miff -write mpr:h10 +delete ^
    mpr:Tp3 mpr:Tp2 -poly -2,1,3,1 +write h01.miff -write mpr:h01 +delete ^
    mpr:Tp3 mpr:Tp2 -poly 1,1,-1,1 +write h11.miff -write mpr:h11 +delete ^
  ) ^
  -delete 0--1 ^
  ( mpr:h00 %IMG1%  -alpha off -scale "%%[MYSIZE]^!" -compose Multiply -composite -write f0.miff ) ^
  ( mpr:h10 %GRAD1% -alpha off -scale "%%[MYSIZE]^!" -compose Multiply -composite -write f1.miff ) ^
  ( mpr:h01 %IMG2%  -alpha off -scale "%%[MYSIZE]^!" -compose Multiply -composite -write f2.miff ) ^
  ( mpr:h11 %GRAD2% -alpha off -scale "%%[MYSIZE]^!" -compose Multiply -composite -write f3.miff ) ^
  -evaluate-sequence Add ^
  %ChopEnd% ^
  %OUTFILE%


call echoRestore

@endlocal & set ichOUTFILE=%OUTFILE%
