#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vsn_defines.h"


typedef enum {
  ctPixel,
  ctPercent,
  ctProportion
} CoordTypeT;

typedef struct {
  double inx, iny, x, y;
} intPixT;


static CoordTypeT CoordType (const char * s)
{
  char c = *(s+strlen(s)-1);

  if (c=='%' || c == 'c') return ctPercent;
  if (c=='p') return ctProportion;
  return ctPixel;
}


ModuleExport size_t interppixImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  CacheView
    *image_view;

  int
    i,
    argNdx;

  intPixT
    ip;

  int
    precision;

  CoordTypeT
    ct;

  PIX_INFO
    src_pixel;

  (void) argc;
  (void) argv;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  precision = GetMagickPrecision();

  if (argc < 2 || (argc % 2) != 0) {
    fprintf (stderr, "interppix needs x and y coordinate-pairs\n");
    return -1;
  }

  i = 0;

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    argNdx = 0;
    while (argNdx < argc) {
      ip.inx = atof (argv[argNdx]);
      ip.iny = atof (argv[argNdx+1]);

      ct = CoordType (argv[argNdx]);
      if (ct == ctPercent) {
        ip.x = ip.inx * (image->columns-1) / 100.0;
      } else if (ct == ctProportion) {
        ip.x = ip.inx * (image->columns-1);
      } else {
        ip.x = ip.inx;
      }

      ct = CoordType (argv[argNdx+1]);
      if (ct == ctPercent) {
        ip.y = ip.iny * (image->rows-1) / 100.0;
      } else if (ct == ctProportion) {
        ip.y = ip.iny * (image->rows-1);
      } else {
        ip.y = ip.iny;
      }

      fprintf (stderr, "interppix: %i: ", i);

      fprintf (stderr, "@%.*g,%.*g",
        precision, ip.x, precision, ip.y);

      image_view = AcquireVirtualCacheView(image,exception);

#if IMV6OR7==6
      InterpolateMagickPixelPacket (
        image, image_view,
        image->interpolate,
        ip.x, ip.y, &src_pixel, exception);

      fprintf (stderr, " (%.*lg,%.*lg,%.*lg,%.*lg,%.*lg)",
        precision, (double)src_pixel.red,
        precision, (double)src_pixel.green,
        precision, (double)src_pixel.blue,
        precision, (double)(QuantumRange - src_pixel.opacity),
        precision, (double)src_pixel.index);

      fprintf (stderr, " (%.*lg%%,%.*lg%%,%.*lg%%,%.*lg%%,%.*lg%%)",
        precision, (double)(src_pixel.red * 100.0 / QuantumRange),
        precision, (double)(src_pixel.green * 100.0 / QuantumRange),
        precision, (double)(src_pixel.blue * 100.0 / QuantumRange),
        precision, (double)(100.0 - src_pixel.opacity * 100.0 / QuantumRange),
        precision, (double)(src_pixel.index * 100.0 / QuantumRange));
#else
     InterpolatePixelInfo (
        image, image_view,
        image->interpolate,
        ip.x, ip.y, &src_pixel, exception);

      fprintf (stderr, " (%.*lg,%.*lg,%.*lg,%.*lg,%.*lg)",
        precision, (double)src_pixel.red,
        precision, (double)src_pixel.green,
        precision, (double)src_pixel.blue,
        precision, (double)src_pixel.alpha,
        precision, (double)src_pixel.index);

      fprintf (stderr, " (%.*lg%%,%.*lg%%,%.*lg%%,%.*lg%%,%.*lg%%)",
        precision, (double)(src_pixel.red * 100.0 / QuantumRange),
        precision, (double)(src_pixel.green * 100.0 / QuantumRange),
        precision, (double)(src_pixel.blue * 100.0 / QuantumRange),
        precision, (double)(src_pixel.alpha * 100.0 / QuantumRange),
        precision, (double)(src_pixel.index * 100.0 / QuantumRange));
#endif

      fprintf (stderr, "\n");

      argNdx += 2;

    }

    i++;
  }

  return(MagickImageFilterSignature);
}

