rem Given:
rem   %1 island image (white island on black background)
rem   %2 list of coords adjacent to island
rem   %3 mainland edge image (white line on black background)
@rem Writes:
@rem   %4 output CSV file, list of coords on mainland that correspond to island coords
@rem
@rem Also uses:
@rem   jimDEBUG if 1, creates debug image
@rem   jimMAX_DIST
@rem
@rem Updated:
@rem   24-May-2016
@rem   24-August-2022 for IM v7.
@rem



@if "%4"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 jim


set ISLAND=%1
set INCOORDS=%2
set MNLAND_EDGE=%3
set OUTCSV=%4

set DEBUG_FILE=%BASENAME%_%sioCODE%_dbg%EXT%
set DBG_LINES=%TEMP%\%~n1_%sioCODE%_dbglns.txt

del %OUTCSV% 2>nul
if "%jimDEBUG%"=="1" del %DBG_LINES% 2>nul

set TMPEXT=.miff
set TMP_ISLAND=%TEMP%\%~n1%TMPEXT%
set TMP_MNLAND_EDGE=%TEMP%\%~n3%TMPEXT%

if "%jimMAX_DIST%"=="" set jimMAX_DIST=100

%IMG7%magick %ISLAND% %TMP_ISLAND%

%IMG7%magick %MNLAND_EDGE% %TMP_MNLAND_EDGE%

for /F "tokens=1-2 delims=, " %%X in (%INCOORDS%) do (
  for /F "usebackq tokens=1-3 delims=," %%W in (`%IMG7%magick ^
    %TMP_ISLAND% ^
    ^( +clone -write mpr:MASK +delete ^) ^
    -fill #fff -colorize 100 ^
    -fill #000 -draw "point %%X,%%Y" ^
    -write-mask mpr:MASK ^
    -morphology IterativeDistance:%jimMAX_DIST% Euclidean ^
    +write-mask ^
    %TMP_MNLAND_EDGE% ^
    -compose CopyOpacity -composite ^
    -background #fff -alpha Remove ^
    -channel RGB -auto-level +channel ^
    -format "%%[fx:minima]," -write info: ^
    +transparent #000 ^
  sparse-color:`) do (
    set MIN=%%W
    set blackX=%%X
    set blackY=%%Y
  )

  echo %0: %%X,%%Y  MIN=!MIN! !blackX!,!blackY!

  if !MIN!==0 (
    echo !blackX!,!blackY! >>%OUTCSV%

    if "%jimDEBUG%"=="1" echo line %%X,%%Y !blackX!,!blackY! >>%DBG_LINES%
  )
)

if "%jimDEBUG%"=="1" if exist %DBG_LINES% %IMG7%magick ^
  %ISLAND% ^
  %MNLAND_EDGE% ^
  -compose Lighten -composite ^
  -stroke Red -draw @%DBG_LINES% ^
  %DEBUG_FILE%

@call echoRestore

@endlocal & set jimDEBUG_FILE=%DEBUG_FILE%
