rem Given image %1 a white line on black background,
rem finds coords of the ends,
rem and makes output image %2 that has gradient for the line,
rem and transparent elsewhere.
rem Sets properties "line:*" in output.
@rem
@rem Updated:
@rem   3-September-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 l2g

if not "%2"=="" set OUTFILE=%2


set TMP_2ENDS=l2g_two_ends.miff
set TMP_LINE_GRAD=l2g_grad.png

set APPROX_LEN_PC=90

set ERR=0


set WW=

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nDIAG=%%[fx:int(hypot(w,h)+0.5)]" ^
  %INFILE%`) do set %%L

if "%WW%"=="" (
  echo Can't open [%INFILE%]
  exit /B 1
)


for /F "usebackq tokens=2,3 delims=:, " %%X in (`%IM7DEV%magick ^
  %INFILE% ^
  -virtual-pixel Black ^
  -morphology HMT LineEnds ^
  -process onewhite ^
  %TMP_2ENDS% 2^>^&1`) do (
  set X0=%%X
  set Y0=%%Y
)

if "%X0%"=="none" (
  echo %0: no line ends [%INFILE%]
  set ERR=1
  goto end
)

for /F "usebackq tokens=2,3 delims=:, " %%X in (`%IM7DEV%magick ^
  %TMP_2ENDS% ^
  fill Black -draw "point %X0%,%Y0%" ^
  -process onewhite ^
  NULL: 2^>^&1`) do (
  set X1=%%X
  set Y1=%%Y
)

if "%X1%"=="none" (
  echo %0: no second line end [%INFILE%]
  set ERR=1
  goto end
)

:: FIXME:
:: If X1 < X0, swap the ends.
:: If X1==X0 and Y1 < Y0, swap the ends.

rem echo X0=%X0% Y0=%Y0% X1=%X1% Y1=%Y1%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -format "APPROX_LEN=%%[fx:int(max(hypot(%X1%-%X0%,%Y1%-%Y0%),mean*w*h+0.5))]" ^
  INFO:`) do set %%L

rem echo APPROX_LEN=%APPROX_LEN%

set /A KNL_SCALE_V=65535*%APPROX_LEN_PC%/%APPROX_LEN%/100

rem echo KNL_SCALE_V=%KNL_SCALE_V%

%IMG7%magick ^
  %INFILE% ^
  -negate +write mpr:MASK ^
  -fill White -colorize 100 ^
  -fill Black -draw "point %X0%,%Y0%" ^
  -write-mask mpr:MASK ^
  -morphology ^
    IterativeDistance:%DIAG% ^
    Euclidean:7,%KNL_SCALE_V% ^
  +write-mask ^
  +depth ^
  %TMP_LINE_GRAD%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  %TMP_LINE_GRAD% ^
  -compose Darken -composite ^
  -format "MAX_01=%%[fx:maxima]\nMAX_GRAY=%%[fx:maxima*QuantumRange]\nLINE_LEN=%%[fx:maxima*QuantumRange/%KNL_SCALE_V%]" ^
  info:`) do set %%L

rem echo MAX_01=%MAX_01% MAX_GRAY=%MAX_GRAY% LINE_LEN=%LINE_LEN%

if "%MAX_01%"=="1" (
  echo %0: Bad first MAX_01 == %MAX_01%
  set ERR=1
  goto end
)

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  %TMP_LINE_GRAD% ^
  -compose Darken -composite ^
  -format "KNL_SCALE_V=%%[fx:%KNL_SCALE_V%/maxima)]" ^
  info:`) do set %%L

rem echo KNL_SCALE_V=%KNL_SCALE_V%

%IMG7%magick ^
  %INFILE% ^
  -negate +write mpr:MASK ^
  -fill White -colorize 100 ^
  -fill Black -draw "point %X0%,%Y0%" ^
  -write-mask mpr:MASK ^
  -morphology ^
    IterativeDistance:%DIAG% ^
    Euclidean:7,%KNL_SCALE_V% ^
  +write-mask ^
  +depth ^
  %TMP_LINE_GRAD%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  %TMP_LINE_GRAD% ^
  -compose Darken -composite ^
  -format "MAX_01=%%[fx:maxima]\nMAX_GRAY=%%[fx:maxima*QuantumRange]\nLINE_LEN=%%[fx:maxima*QuantumRange/%KNL_SCALE_V%]" ^
  info:`) do set %%L

rem echo MAX_01=%MAX_01% MAX_GRAY=%MAX_GRAY% LINE_LEN=%LINE_LEN%

:: "%MAX_01%" should be "1", more or less.
:: MAX_GRAY should be >= QuantumRange - KNL_SCALE_V

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "GRAY_OK=%%[fx:%MAX_GRAY%>QuantumRange-%KNL_SCALE_V%?1:0]"
  xc:`) do set %%L

if not "%GRAY_OK%"=="1" (
  echo %0: Bad second MAX_01 = %MAX_01% MAX_GRAY = %MAX_GRAY%
  set ERR=1
  goto end
)

%IMG7%magick ^
  %TMP_LINE_GRAD% ^
  %INFILE% ^
  -compose CopyOpacity -composite ^
  -background gray(50%%) -alpha Background ^
  -auto-level ^
  -alpha Background ^
  -set line:len %LINE_LEN% ^
  -set line:x0 %X0% ^
  -set line:y0 %Y0% ^
  -set line:x1 %X1% ^
  -set line:y1 %Y1% ^
  %OUTFILE%

:end

call echoRestore

endlocal & set l2gOUTFILE=%OUTFILE%& set l2gLINE_LEN=%LINE_LEN%&set l2gERR=%ERR%
