rem Given %1 and %2 are two lines
rem returns coordinates of the intersection (if any) of the lines in %3.
rem All may be floating-point.
rem %3 environment variable for output.
rem %4 Fractional threshold for BAD. [0.0 or above, eg 0.25]
rem   Larger numbers are more tolerant.
rem   If either line needs to be extended by proportional fraction %4 to intersect,
rem   it is marked as bad.
rem   %4 can be zero, meaning all intersections that need extending are marked bad.
rem   If blank or ".", no intersections are marked bad.
@rem
@rem Updated:
@rem   19-July-2022 for IM v7
@rem

@rem This assumes "enabledelayedexpansion" is in effect.

set FRAC_THRESH=%4
if "%FRAC_THRESH%"=="." set FRAC_THRESH=

call :vect AB %1.A %1.B
call :vect CD %2.A %2.B
call :vect CA %2.A %1.A

call :perp Dp AB CD

if %Dp%==0 exit /B 1

call :perp sp CD CA
call :perp tp AB CA

call :doCalc sI %sp%/(%Dp%)
call :doCalc tI %tp%/(%Dp%)

call :doCalc %3.x !%1.A.x!+%sI%*%AB.x%
if ERRORLEVEL 1 exit /B 1

call :doCalc %3.y !%1.A.y!+%sI%*%AB.y%
if ERRORLEVEL 1 exit /B 1

call :doCalc %3.liNeedExtend "(%sI%<0||%sI%>1||%tI%<0||%tI%>1)?1:0"
if ERRORLEVEL 1 exit /B 1

set %3.sI=%sI%
set %3.tI=%tI%

if not "%FRAC_THRESH%"=="" if !%3.liNeedExtend!==1 (
  call :doCalc siBAD "(%sI%<(-%FRAC_THRESH%)||%sI%>(1+%FRAC_THRESH%))?1:0"
  if ERRORLEVEL 1 exit /B 1
  call :doCalc tiBAD "(%tI%<(-%FRAC_THRESH%)||%tI%>(1+%FRAC_THRESH%))?1:0"
  if ERRORLEVEL 1 exit /B 1

  if !siBAD!==1 set %3.BAD=1
  if !tiBAD!==1 set %3.BAD=1
)

echo %0: sI=%sI% tI=%tI% %3.liNeedExtend=!%3.liNeedExtend!

exit /B 0


rem -----------------------------------------------------
rem Subroutines

:vect
call :doCalc %1.x !%3.x!-(!%2.x!)
call :doCalc %1.y !%3.y!-(!%2.y!)

exit /B 0


:perp
call :doCalc %1 !%2.x!*!%3.y!-(!%2.y!*!%3.x!)

exit /B 0


:doCalc
set %1=
for /F "usebackq" %%L in (`%IMG7%magick identify -precision 19 -format "%1=%%[fx:%~2]" xc:`) do set %%L

rem echo %1 is !%1!

if "!%1!"=="" exit /B 1

exit /B 0
