rem Makes an up-down clut.
rem
rem %1 is width of clut [default 10000]
rem %2 is centre of peak (% of width) [50]
rem %3 is width of peak (% of width) [20]
rem %4 is width of up ramp (% of width) [10]
rem %5 is width of down ramp (% of width) [10]
rem %6 is 0 or 1, whether to wrap. If 0, there may be only one ramp (up or down). For hue, use 1.
rem %7 is output. Default mcud.png.
@rem
@rem Can also use:
@rem   mcudTRANS a transformation such as "-sigmoidal-contrast 5,50%".
@rem
@rem Updated:
@rem   10-December-2017 
@rem     Inputs can be floating point.
@rem     Corrected processing of strBLANK.
@rem   11-March-2018
@rem     Instead of one value for both ramps, separate parameters,
@rem     Also check for zero width of each piece.
@rem   30-July-2018
@rem     Added "+repage" to "+append".
@rem   25-September-2022 for IM v7.
@rem

rem @if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 mcud


set WW=%1
if "%WW%"=="." set WW=
if "%WW%"=="" set WW=10000

set PKpc=%2
if "%PKpc%"=="." set PKpc=
if "%PKpc%"=="" set PKpc=50

set PKWpc=%3
if "%PKWpc%"=="." set PKWpc=
if "%PKWpc%"=="" set PKWpc=20

set URWpc=%4
if "%URWpc%"=="." set URWpc=
if "%URWpc%"=="" set URWpc=10

set DRWpc=%5
if "%DRWpc%"=="." set DRWpc=
if "%DRWpc%"=="" set DRWpc=10

set DO_WRAP=%6
if "%DO_WRAP%"=="." set DO_WRAP=
if "%DO_WRAP%"=="" set DO_WRAP=0

set OUTFILE=%7
if "%OUTFILE%"=="" set OUTFILE=mcud.png

set DO_ROLL=%DO_WRAP%

set FMT=^
PK=%%[fx:floor(%PKpc%*%WW%/100+0.5)]\n^
PKW=%%[fx:floor(%PKWpc%*%WW%/100+0.5)]\n^
URW=%%[fx:floor(%URWpc%*%WW%/100+0.5)]\n^
DRW=%%[fx:floor(%DRWpc%*%WW%/100+0.5)]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

rem set FMT=^
rem FLAT_L=%%[fx:%WW%/2-%PKW%/2-%RW%]\n

set FMT=^
FLAT_L=%%[fx:floor(%PK%-%PKW%/2-%URW%+0.5)]\n

if %DO_ROLL%==1 set FMT=^
FLAT_L=%%[fx:floor((%WW%-%PKW%-%URW%-%DRW%)/2+0.5)]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

if %FLAT_L% LSS 0 (
  set /A URW=%URW%+^(%FLAT_L%^)
  set FLAT_L=0
)

set FMT=^
FLAT_R=%%[fx:%WW%-%FLAT_L%-%PKW%-%URW%-%DRW%]\n^
ROLL=%%[fx:(%PKpc%-50)*%WW%/100]\n^
isNegRoll=%%[fx:%PKpc%^<50?1:0]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FMT%" ^
  xc:`) do set %%L

if %FLAT_R% LSS 0 (
  set /A DRW=%DRW%+^(%FLAT_R%^)
  set FLAT_R=0
)

set sROLL=%ROLL%
set absROLL=%ROLL%
if %isNegRoll%==0 (
  set sROLL=+%ROLL%
) else (
  set absROLL=%ROLL:~1%
)

echo DO_ROLL=%DO_ROLL% ROLL=%ROLL% isNegRoll=%isNegRoll% sROLL=%sROLL% absROLL=%absROLL%

rem FIXME: wrong. If no wrap, we want new part to be zero.
set strROLL=
set strBLANK=
if "%PKpc%" NEQ "50" (
  set strROLL=-roll %sROLL%+0
  if "%DO_WRAP%"=="0" (
    if %isNegRoll%==0 (
      set strBLANK=-size %absROLL%x1 xc:#000 -gravity West -compose Over -composite
    ) else (
      set strBLANK=-size %absROLL%x1 xc:#000 -gravity East -compose Over -composite
    )
  )
)

if %DO_ROLL%==0 (
  set strROLL=
  set strBLANK=
)

echo FLAT_L=%FLAT_L% PK=%PK% PKW=%PKW% URW=%URW% DRW=%DRW% FLAT_R=%FLAT_R% absROLL=%absROLL% strROLL=%strROLL%
echo strBLANK=%strBLANK%
echo strROLL=%strROLL%

echo on

set PIECES=
if not %FLAT_L%==0 set PIECES=%PIECES% -size %FLAT_L%x1 xc:#000
if not %URW%==0 set PIECES=%PIECES% -size %URW%x1 gradient:#000-#fff
if not %PKW%==0 set PIECES=%PIECES% -size %PKW%x1 xc:#fff
if not %DRW%==0 set PIECES=%PIECES% -size %DRW%x1 gradient:#fff-#000
if not %FLAT_R%==0 set PIECES=%PIECES% -size %FLAT_R%x1 xc:#000

if "%PIECES%"=="" (
  echo %0: no PIECES
  exit /B 1
)

%IMG7%magick ^
  %PIECES% ^
  +append +repage ^
  %mcudTRANS% ^
  %strROLL% ^
  %strBLANK% ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

%IMG7%magick identify %OUTFILE%

call %PICTBAT%graph1d %OUTFILE%


call echoRestore

endlocal & set mcudOUTFILE=%OUTFILE%
