rem Given image %1
rem and %2 image, size Nx1,
rem makes N masks, each the size of %1, white where %1 pixel is closest to that %2 pixel.
rem Also makes cumulative masks.
@rem
@rem Optional %3: base name for masks. They will be numbered 1 to N.
@rem
@rem Returns mmmNumMasks number of masks created.
@rem
@rem This is the opposite to deMapMasks.bat.
@rem
@rem Updated:
@rem   26-May-2016 for v7.
@rem   7-August-2022 for IM v7.

rem We could have another map file to colour the contours.
rem We could dilate the lines.


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 mmm

set MAP_FILE=%2

set MASK_DD=
set MASK_PREF=
set MASK_EXT=
if not "%3"=="" (
  set MASK_DD=%~dp3
  set MASK_PREF=%~n3
  set MASK_EXT=%~x3
)

if "%MASK_PREF%"=="" set MASK_PREF=%sioCODE%
if "%MASK_EXT%"=="" set MASK_EXT=.png

set TMP_FILE=%TEMP%\%MASK_PREF%_tmp%MASK_EXT%
set SCR_FILE=%TEMP%\%MASK_PREF%_scr.bat
set CUMUL_SCR_FILE=%TEMP%\%MASK_PREF%_cumul_scr.bat

set HH=
set NumMasks=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "HH=%%h\nNumMasks=%%w" ^
  %MAP_FILE%`) do set %%L

if not "%HH%"=="1" (
  echo %0: map file [%MAP_FILE%] not height 1
  exit /B 1
)
if "%NumMasks%"=="" exit /B 1

%IMG7%magick %INFILE% +dither -remap %MAP_FILE% %TMP_FILE%
if ERRORLEVEL 1 (
  echo %0: [%1] [%2] convert failed
  exit /B 1
)

echo %IMG7%magick %TMP_FILE% -fill Black -background White >%SCR_FILE%
echo %IMG7%magick -compose Lighten >%CUMUL_SCR_FILE%

set CNT=1
for /F "usebackq skip=1 tokens=6 delims=,:() " %%C in (`%IMG7%magick ^
  %MAP_FILE% ^
  -colorspace sRGB ^
  txt:`) do (
  echo %%C

  set MASK_FILE=%MASK_DD%%MASK_PREF%!CNT!%MASK_EXT%
  set CUMUL_FILE=%MASK_DD%%MASK_PREF%c_!CNT!%MASK_EXT%

  echo ^( +clone -transparent %%C -colorize 100 -flatten +write !MASK_FILE! +delete ^) >>%SCR_FILE%

  echo !MASK_FILE! >>%CUMUL_SCR_FILE%

  if !CNT! GTR 1 (
    echo -composite +write !CUMUL_FILE! >>%CUMUL_SCR_FILE%
  )

  set /A CNT+=1
)

cPrefix /i%SCR_FILE% /r^^ 
echo NULL: >>%SCR_FILE%
cPrefix /i%CUMUL_SCR_FILE% /r^^ 
echo NULL: >>%CUMUL_SCR_FILE%

type %SCR_FILE%
call %SCR_FILE%

type %CUMUL_SCR_FILE%
call %CUMUL_SCR_FILE%

call echoRestore

endlocal & set mmmOUTFILE=%OUTFILE%& set mmmNumMasks=%NumMasks%
