rem Makes a power-linear-power curve specified from a fulcrum.
rem Calls mPowLinPow.bat

rem %1 output filename
rem %2 output width, pixels [4096]
rem %3 fulcrum F, so linear portion passes through (F,F) eg 0.5 [0.5]
rem   or quoted "Fx,Fy" so linear portion passes through (Fx,Fy) eg "0.4,0.5"
rem %4 gradient G. 0=flat, 1=neutral, >1 increases contrast. [1.5]
rem %5 proportion of curve that is linear. [0.5]
@rem
@rem Updated:
@rem   2-October-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mplp

if not "%1"=="" if not "%1"=="." set OUTFILE=%1

set WW=%2
if "%WW%"=="." set WW=
if "%WW%"=="" set WW=4096

rem set F=%3
if "%F%"=="." set F=
if "%F%"=="" set F=0.5

set Fxy=%3
if [%Fxy%]==[.] set Fxy=
if [%Fxy%]==[,] set Fxy=
if [%Fxy%]==[] set Fxy="0.5,0.5"
call parseCommaList %Fxy% num Fxy
set Fx=%Fxy[0]%
set Fy=%Fxy[1]%
if "%Fy%"=="" set Fy=%Fx%

echo Fx=%Fx% Fy=%Fy%

set G=%4
if "%G%"=="." set G=
if "%G%"=="" set G=1.5

set pLin=%5
if "%pLin%"=="." set pLin=
if "%pLin%"=="" set pLin=0.5

for /F "usebackq" %%L in (`%IMG7%magick identify
  -precision 15 ^
  -format "IsGradSteep=%%[fx:(%G%>1||%G%<-1)?1:0]\n" ^
  xc:`) do set %%L

echo %0: IsGradSteep=%IsGradSteep%

if %IsGradSteep%==0 (
  set "x0=%Fx%-%Fx%*%pLin%"
  set "x1=%Fx%+%pLin%-%Fx%*%pLin%"

  set FMT=^
x0=%%[fx:!x0!]\n^
x1=%%[fx:!x1!]\n^
y0=%%[fx:max^(0,%Fy%-^(%Fx%-^(!x0!^)^)*^(%G%^)^)]\n^
y1=%%[fx:min^(1,%Fy%+^(^(!x1!^)-%Fx%^)*^(%G%^)^)]\n

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 15 ^
    -format "!FMT!" ^
    xc:`) do set %%L

) else (
  set "y0=%Fy%-%Fy%*%pLin%"
  set "y1=%Fy%+%pLin%-%Fy%*%pLin%"

  set FMT=^
x0=%%[fx:max^(0,%Fx%-^(%Fy%-^(!y0!^)^)/^(%G%^)^)]\n^
x1=%%[fx:min^(1,%Fx%+^(^(!y1!^)-%Fy%^)/^(%G%^)^)]\n^
y0=%%[fx:!y0!]\n^
y1=%%[fx:!y1!]\n

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 15 ^
    -format "!FMT!" ^
    xc:`) do set %%L
)

set forceGrad=.
if %x0%==%x1% set forceGrad=%G%

echo %0: Fxy=%Fx%,%Fy% G=%G% (%x0%,%y0%) (%x1%,%y1%) forceGrad=%forceGrad%

call %PICTBAT%mPowLinPow %OUTFILE% %WW% "%x0%,%y0%" "%x1%,%y1%" %IsGradSteep% %forceGrad%

if ERRORLEVEL 1 exit /B 1

rem call %PICTBAT%graph1d %OUTFILE%

call echoRestore

@endlocal
