rem %1 output filename
rem %2 output width, pixels [4096]
rem %3 quoted coords of start of linear portion x0,y0
rem %4 quoted coords of end of linear portion x1,y1
rem %5 s, slope at both ends
rem %6 if not "." or blank, use this as gradient insead of dy/dx.
@rem
@rem   Inflection is at (a,b).
@rem
@rem Also uses:
@rem   mplpPREPROC IM process applied to input
@rem   mplpPOSTPROC IM process applied to output
@rem
@rem Updated:
@rem   15-August-2021 Calculate p and p1 separately, in case points coincide.
@rem   9-September-2021 Treat negative gradient specially.
@rem   2-October-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mplp

if not "%1"=="" if not "%1"=="." set OUTFILE=%1

set WW=%2
if "%WW%"=="." set WW=
if "%WW%"=="" set WW=4096

set x0y0=%3
if [%x0y0%]==[.] set x0y0=
if [%x0y0%]==[,] set x0y0=
if [%x0y0%]==[] set x0y0="0.2,0.1"
call parseCommaList %x0y0% num x0y0
if not "%num%"=="2" (
  echo %0: x0y0 num=%num%
  exit /B 1
)

set x1y1=%4
if [%x1y1%]==[.] set x1y1=
if [%x1y1%]==[,] set x1y1=
if [%x1y1%]==[] set x1y1="0.8,0.9"
call parseCommaList %x1y1% num x1y1
if not "%num%"=="2" (
  echo %0: x1y1 num=%num%
  exit /B 1
)

set s=%5
if "%s%"=="." set s=
if "%s%"=="" set s=1

set forceGrad=%6
if "%forceGrad%"=="." set forceGrad=

set x0=%x0y0[0]%
set y0=%x0y0[1]%

set x1=%x1y1[0]%
set y1=%x1y1[1]%

echo %0: (%x0%,%y0%) (%x1%,%y1%) s=%s%

:: If x0>x1, swap the ends.
::
for /F "usebackq" %%L in (`%IMG7%magick identify
  -format "DoSwap=%%[fx:%x0%>%x1%?1:0]\n" ^
  xc:`) do set %%L

if %DoSwap%==1 (
  set t=%x0%
  set x0=%x1%
  set x1=!t!
  set t=%y0%
  set y0=%y1%
  set y1=!t!
)

:: FIXME: for forceGrad

for /F "usebackq" %%L in (`%IMG7%magick identify
  -format "IsGradNeg=%%[fx:%y0%>%y1%?1:0]\n" ^
  xc:`) do set %%L

if %IsGradNeg%==1 (
  set FMT=^
x0a=%%[fx:1-%x1%]\n^
x1a=%%[fx:1-%x0%]\n^
y0a=%%[fx:%y1%]\n^
y1a=%%[fx:%y0%]\n

  for /F "usebackq" %%L in (`%IMG7%magick identify
    -precision 15 ^
    -format "!FMT!" ^
    xc:`) do set %%L

  set sFLOP=-flop
) else (
  set x0a=%x0%
  set x1a=%x1%
  set y0a=%y0%
  set y1a=%y1%
  set sFLOP=
)

echo %0: (%x0a%,%y0a%) (%x1a%,%y1a%) s=%s%

if "%forceGrad%"=="" (

  set "E=%x1a%-%x0a%==0?1:(%y1a%-%y0a%)/(%x1a%-%x0a%)"

  for /F "usebackq" %%L in (`%IMG7%magick identify
    -precision 15 ^
    -format "E=%%[fx:!E!]\n" ^
    xc:`) do set %%L

  if ERRORLEVEL 1 exit /B 1
) else (
  set E=%forceGrad%
)

:: In p or p1 calcs, div by zero means we don't have that power curve.

for /F "usebackq" %%L in (`%IMG7%magick identify
  -precision 15 ^
  -format "p=%%[fx:%x0a%*%E%/%y0a%]\n" ^
  xc:`) do set %%L

if "%p%"=="" set p=0

for /F "usebackq" %%L in (`%IMG7%magick identify
  -precision 15 ^
  -format "p1=%%[fx:(1-%x1a%)*%E%/(1-%y1a%)]\n" ^
  xc:`) do set %%L

if "%p1%"=="" set p1=0


for /F "usebackq" %%L in (`%IMG7%magick identify
  -precision 15 ^
  -format "a=%%[fx:%y0a%/pow(%x0a%,%p%)]\na1=%%[fx:(1-%y1a%)/pow((1-%x1a%),%p1%)]\n" ^
  xc:`) do set %%L

echo %0: E=%E% p=%p%  a=%a% p1=%p1%  a1=%a1%

%IMG7%magick -size %WW%x1 gradient:Black-White ^
  %mplpPREPROC% ^
  -fx "u<%x0a%?%a%*pow(u,%p%):u<%x1a%?%y0a%+%E%*(u-%x0a%):1-%a1%*pow(1-u,%p1%)" ^
  %mplpPOSTPROC% ^
  %sFLOP% ^
  %OUTFILE%

call %PICTBAT%graph1d %OUTFILE%

call echoRestore

@endlocal
