rem Given %1 an image,
rem %2 a Nx1 map,
rem %3 a proportional improvement RMSE threshold [default 0.01]
rem if we remap the image to the first (i) map colours,
rem what is the smallest i so the improvement of the remapped image
rem   is within the threshold?
rem %4 if given, writes the first (i) colors to this file as ix1 image.
rem
rem Returns:
rem   msCNT the required number of colours.
rem   msERROR is 1 the threshold isn't reached even with all the colours.
@rem
@rem Updated:
@rem   6-August-2022 for IM v7
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 ms

set MAP_FILE=%2

set THRESH=%3
if "%THRESH%"=="." set THRESH=
if "%THRESH%"=="" set THRESH=0.01

set OUTFILE=
if not "%4"=="" if not "%4"=="." set OUTFILE=%4


set HH=
set NumCols=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "HH=%%h\nNumCols=%%w" ^
  %MAP_FILE%`) do set %%L

if not "%HH%"=="1" (
  echo %0: map file [%MAP_FILE%] not height 1
  exit /B 1
)
if "%NumCols%"=="" exit /B 1

set ERROR=1
set CNT=%NumCols%
set PREV_SCORE=1
set DONE=0

for /L %%I in (1,1,%NumCols%) do (
  if !DONE!==0 (
    for /F "usebackq" %%L in (`%IMG7%magick ^
      %MAP_FILE% ^
      -crop %%Ix1+0+0 +repage ^
      +write mpr:MAP +delete ^
      %INFILE% ^
      ^( +clone ^
         +dither -remap mpr:MAP ^
      ^) ^
      -metric RMSE ^
      -format %%[distortion] ^
      -compare ^
      info:`) do set SCORE=%%L

    for /F "usebackq" %%L in (`%IMG7%magick identify ^
      -format "DONE=%%[fx:(!PREV_SCORE!-!SCORE!)/!PREV_SCORE!>%THRESH%?0:1]" ^
      xc:`) do set %%L

    if !DONE!==1 (
      set /A CNT=%%I-1
      set ERROR=0
    )

    echo %%I SCORE=!SCORE! DONE=!DONE!

    set PREV_SCORE=!SCORE!
  )
)

if %CNT%==0 set CNT=1

if not "%OUTFILE%"=="" (
  %IMG7%magick ^
    %MAP_FILE% ^
    -crop %CNT%x1+0+0 +repage ^
    %OUTFILE%
)

echo %0: CNT=%CNT% ERROR=%ERROR%

call echoRestore

endlocal & set msOUTFILE=%OUTFILE%& set msCNT=%CNT%& set msERROR=%ERROR%
