rem Given image %1, creates marked-up version with numbered rectangles.
rem Takes coordinates of rectangle centres from file %2.
rem %3 width of rectangle, default 1
rem %4 height of rectangle, default 1
rem %5 output file
@rem
@rem %2 is CSV file with columns X,Y{,N}
@rem If third column is present, uses this for annotation.
@rem Otherwise numbers seqentially.
@rem
@rem Also uses:
@rem   mrSTROKE  eg "-strokewidth 10" to get thick strokes
@rem   mrPIX_HT  height of font in pixels. If 0, no numbering.
@rem   mrCOL_X   column number in %2 for x-coordinate, counting from 1
@rem   mrCOL_Y   column number in %2 for y-coordinate, counting from 1
@rem   mrNhead   number of lines on header, to skip
@rem
@rem Updated:
@rem   10-December-2017 Stroke has defaults.
@rem   26-August-2018 Fixed bug in skip setting.
@rem   14-August-2020 for IM v7.
@rem


@rem FIXME: Also option for circles? Ellipes?

@if "%4"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mr


if "%mrPIX_HT%"=="" set mrPIX_HT=20

if "%mrCOL_X%"=="" set mrCOL_X=1

if "%mrCOL_Y%"=="" set mrCOL_Y=2

set RW=%3
if "%RW%"=="" set RW=1

set RH=%4
if "%RW%"=="" set RH=1

if not "%5"=="" if not "%5"=="." set OUTFILE=%5

set sSKIP=
if not "%mrNhead%"=="" if not "%mrNhead%"=="0" (
  set sSKIP=skip=%mrNhead%
)

set /A W_2=%RW%/2
set /A H_2=%RH%/2

set nCrop=0
set sRECT=
set sNUMB=

call %PICTBAT%getPointSize %INFILE% %H_2%
set mrTEXT=-pointsize %gpPointsize% -fill #f00 -stroke None

for /F "tokens=%mrCOL_X%,%mrCOL_Y% %sSKIP% delims=, " %%X in (%2) do (
  if not "%%X" GTR "a" (
    set /A dx=%%X-%W_2%
    set /A dy=%%Y-%H_2%

    set /A endX=!dx!+%RW%-1
    set /A endY=!dy!+%RH%-1
    set sRECT=!sRECT! rectangle !dx!,!dy!,!endX!,!endY!
    set ANNOT=%%Z
    if "%ANNOT%"=="" set ANNOT=!nCrop!
    if not "%mrPIX_HT%"=="0" set sNUMB=!sNUMB! text %%X,%%Y '!ANNOT!'

    set /A nCrop+=1
  )
)

if ERRORLEVEL 1 exit /B 1

if "%sRECT%"=="" exit /B 1

if "%mrPIX_HT%"=="0" (
  set sTEXT=
) else (
  set sTEXT=%mrTEXT% -draw "%sNUMB%"
)

%IMG7%magick ^
  %INFILE% ^
  -stroke #f00 -strokewidth 1 -fill None ^
  %mrSTROKE% ^
  -draw "%sRECT%" ^
  %sTEXT% ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

@call echoRestore

@endlocal & set mrOUTFILE=%OUTFILE%
