typedef double mat3x3T[9];

// abcdefghi
// 012345678

static void ZeroMat3x3 (mat3x3T out)
{
  int i;
  for (i=0; i<9; i++) out[i] = 0.0;
}

static void IdentMat3x3 (mat3x3T out)
{
  ZeroMat3x3 (out);
  out[0] = out[4] = out[8] = 1.0;
}

static void CopyMat3x3 (const mat3x3T in, mat3x3T out)
{
  int i;
  for (i=0; i<9; i++) out[i] = in[i];
}

static void TransposeMat3x3self (mat3x3T inout)
{
  double t;

  t = inout[1];
  inout[1] = inout[3];
  inout[3] = t;

  t = inout[2];
  inout[2] = inout[6];
  inout[6] = t;

  t = inout[5];
  inout[5] = inout[7];
  inout[7] = t;
}

static void MultMat3x3 (const mat3x3T inA, const mat3x3T inB, mat3x3T out)
{
  out[0] = inA[0]*inB[0] + inA[1]*inB[3] + inA[2]*inB[6];
  out[1] = inA[0]*inB[1] + inA[1]*inB[4] + inA[2]*inB[7];
  out[2] = inA[0]*inB[2] + inA[1]*inB[5] + inA[2]*inB[8];

  out[3] = inA[3]*inB[0] + inA[4]*inB[3] + inA[5]*inB[6];
  out[4] = inA[3]*inB[1] + inA[4]*inB[4] + inA[5]*inB[7];
  out[5] = inA[3]*inB[2] + inA[4]*inB[5] + inA[5]*inB[8];

  out[6] = inA[6]*inB[0] + inA[7]*inB[3] + inA[8]*inB[6];
  out[7] = inA[6]*inB[1] + inA[7]*inB[4] + inA[8]*inB[7];
  out[8] = inA[6]*inB[2] + inA[7]*inB[5] + inA[8]*inB[8];
}

static MagickBooleanType InvMat3x3 (const mat3x3T in, mat3x3T out)
{
  // Ref: https://en.wikipedia.org/wiki/Invertible_matrix#Inversion_of_3_%C3%97_3_matrices
  out[0] =  (in[4]*in[8] - in[5]*in[7]);
  out[1] = -(in[3]*in[8] - in[5]*in[6]);
  out[2] =  (in[3]*in[7] - in[4]*in[6]);

  out[3] = -(in[1]*in[8] - in[2]*in[7]);
  out[4] =  (in[0]*in[8] - in[2]*in[6]);
  out[5] = -(in[0]*in[7] - in[1]*in[6]);

  out[6] =  (in[1]*in[5] - in[2]*in[4]);
  out[7] = -(in[0]*in[5] - in[2]*in[3]);
  out[8] =  (in[0]*in[4] - in[1]*in[3]);

  double det = in[0] * out[0] + in[1] * out[1] + in[2] * out[2];

  if (fabs(det) < MagickEpsilon) return MagickFalse;

  TransposeMat3x3self (out);

  int i;
  for (i=0; i < 9; i++) {
    out[i] /= det;
  }
  return MagickTrue;
}

static void WrMat3x3 (FILE * fh, int precision, const mat3x3T in, char * title)
{
  fprintf (fh, "%s\n", title);

  fprintf (fh, "  %.*g, %.*g, %.*g\n",
    precision, in[0],
    precision, in[1],
    precision, in[2]);

  fprintf (fh, "  %.*g, %.*g, %.*g\n",
    precision, in[3],
    precision, in[4],
    precision, in[5]);

  fprintf (fh, "  %.*g, %.*g, %.*g\n",
    precision, in[6],
    precision, in[7],
    precision, in[8]);
}
