rem From image %1,
rem writes %2 with maximised local colour contrast
rem   (variation in L, a* and b*).
rem %3 sigma to find the local mean.
rem %4 sigma to find local maxima.
rem %5 blend percentage
rem %6 Lab or L or ab [default Lab]
rem      whether to maximize all L*a*b* channels,
rem      or just L*,
rem      or just a* and b*.
rem %7 Post-process, eg -auto-level
@rem
@rem Assumes magick is HDRI.
@rem
@rem Reference:
@rem   http://im.snibgo.com/maxloccol.htm

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mlc


set INFILE=%~1

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set SIG1=%3
if "%SIG1%"=="." set SIG1=
if "%SIG1%"=="" set SIG1=2

set SIG2=%4
if "%SIG2%"=="." set SIG2=
if "%SIG2%"=="" set SIG2=%SIG1%

set BLENDPC=%5
if "%BLENDPC%"=="." set BLENDPC=
if "%BLENDPC%"=="" set BLENDPC=100

set CHANS=%6
if "%CHANS%"=="." set CHANS=
if "%CHANS%"=="" set CHANS=Lab

set POST_PROC=%7
if "%POST_PROC%"=="." set POST_PROC=

set BASE=%~n1

echo BASE=[%BASE%]

set TMPDIR=\temp\
set GRAY=%TMPDIR%mlcl_%BASE%_gray.miff
set INLAB=%TMPDIR%mlcl_%BASE%_inlab.miff
set BLR=%TMPDIR%mlcl_%BASE%_blr.miff
set MINBLR=%TMPDIR%mlcl_%BASE%_minblr.miff
set AVG=%TMPDIR%mlcl_%BASE%_minblr_avg.miff
set DIV=%TMPDIR%mlcl_%BASE%_minblr_div.miff
set MAXC=%TMPDIR%mlcl_%BASE%_maxc.miff


set "sHDRI=-depth 32 -define compose:clamp^=off -define quantum:format^=floating-point"

%IMG7%magick ^
  %INFILE% ^
  %sHDRI% ^
  -colorspace Lab ^
  -set colorspace sRGB ^
  -channel GB -evaluate Subtract 50%% +channel ^
  +write %INLAB% ^
  ( +clone ^
    -blur 0x%SIG1% ^
    +write %BLR% ^
  ) ^
  -compose MinusSrc -composite ^
  %MINBLR%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %MINBLR% ^
  %sHDRI% ^
  -precision 19 ^
  -evaluate Abs 0 ^
  -format "maxAr=%%[fx:maxima.r]\nmaxAg=%%[fx:maxima.g]\nmaxAb=%%[fx:maxima.b]\n" ^
  +write info: ^
  -blur 0x%SIG2% ^
  +write %AVG% ^
  -format "maxBr=%%[fx:maxima.r]\nmaxBg=%%[fx:maxima.g]\nmaxBb=%%[fx:maxima.b]\n" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 19 ^
  -format "multr=%%[fx:%maxAr%/%maxBr%]\nmultg=%%[fx:%maxAg%/%maxBg%]\nmultb=%%[fx:%maxAb%/%maxBb%]\n"
  xc:`) do set %%L

echo maxAr=%maxAr% maxBr=%maxBr% multr=%multr%
echo maxAg=%maxAg% maxBg=%maxBg% multg=%multg%
echo maxAb=%maxAb% maxBb=%maxBb% multb=%multb%

%IM7DEV%magick ^
  %MINBLR% ^
  %sHDRI% ^
  ( %AVG% ^
    -channel R -evaluate Multiply %multr% ^
    -channel G -evaluate Multiply %multg% ^
    -channel B -evaluate Multiply %multb% ^
    +channel ^
  ) ^
  -compose DivideSrc -composite ^
  -evaluate Divide 2 ^
  -evaluate Add 50%% ^
  -process 'oogbox' ^
  %DIV%

rem Next can't oogbox because a* and b* are -50% to +50%.

%IMG7%magick ^
  %BLR% ^
  %sHDRI% ^
  %DIV% ^
  -compose Mathematics ^
    -define compose:args=0,1.0,1.0,-0.5 ^
  -composite ^
  %MAXC%


if /I %CHANS%==Lab (
  echo %0: channels Lab

  set READ_FILES=^
    %INLAB% ^
    %MAXC%

) else if /I %CHANS%==L (
  echo %0: channel L only

  set READ_FILES=^
    %INLAB% ^
    ^( +clone ^
       -channel GB -separate +channel ^
       ^( %MAXC% -channel R -separate +channel ^) ^
       -insert 0 ^
       -combine ^
    ^)

) else if /I %CHANS%==ab (
  echo %0: channels ab only

  set READ_FILES=^
    %INLAB% ^
    ^( +clone ^
       -channel R -separate +channel ^
       ^( %MAXC% -channel GB -separate +channel ^) ^
       -combine ^
    ^)

) else (
  echo %0: Bad CHANS [%CHANS%]
  exit /B 1
)

echo READ_FILES:
echo %READ_FILES%


%IM7DEV%magick ^
  %READ_FILES% ^
  %sHDRI% ^
  -compose blend -define compose:args=%BLENDPC% -composite ^
  -set colorspace Lab ^
  -channel GB -evaluate Add 50%% +channel ^
  -process 'oogbox' ^
  -colorspace sRGB ^
  -process 'oogbox' ^
  %POST_PROC% ^
  %OUTFILE%


call echoRestore

@endlocal & set mlclOUTFILE=%OUTFILE%& set mlclAVG=%AVG%
