rem From an opaque image, finds maximal rectangle where all pixels are a given foreground colour.
rem
rem %1 Input image.
rem %2 Prefix for output environment variables.
rem %3 Output image, highlighting the found rectangle. Default: no output image.
rem %4 Input foreground colour. Default: Black.
rem %5 Increment for width and height. Default "1,1".
rem %6 Minimum width and height for rectangles. Default "1,1".
rem %7 Maximum width and height for rectangles. Default: input image width and height.
rem %8 IM operations to save the rectangle area map. Default: map is not saved.
@rem    Example operations:
@rem      -write ramap.png
@rem      -auto-level -write ramap.png
@rem      -define quantum:format=floating-point -write ramap.miff
rem %9 Input mask. White where top-left can occur; black where it can't. Default: no mask.
@rem
@rem Reference: http://im.snibgo.com/maxrect.htm
@rem
@rem Also need:
@rem   range of top-left locations
@rem
@rem FIXME: How do we get the map of rectangle sizes?
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mr

set EnvPref=%2
if "%EnvPref%"=="." set EnvPref=
if "%EnvPref%"=="" set EnvPref=mr_

set OUTFILE=%3
if "%OUTFILE%"=="." set OUTFILE=

set FgndCol=%~4
if "%FgndCol%"=="." set FgndCol=
if "%FgndCol%"=="" set FgndCol=Black

set IncWH=%~5
if "%IncWH%"=="." set IncWH=
if "%IncWH%"=="" set IncWH=1,1

set MinWH=%~6
if "%MinWH%"=="." set MinWH=
if "%MinWH%"=="" set MinWH=1,1

set MaxWH=%~7
if "%MaxWH%"=="." set MaxWH=

set SaveRam=%~8
if "%SaveRam%"=="." set SaveRam=

if "%SaveRam%"=="" (
  set sRAM=
) else (
  set sRAM="(" +clone %SaveRam% +delete ")"
)

set Mask=%~9
if "%Mask%"=="." set Mask=

if "%Mask%"=="" (
  set WrMask1=
  set WrMask2=
  set RdMask1=
  set RdMask2=
) else (
  set WrMask1=-write-mask "%Mask%"
  set WrMask2=+write-mask
  set RdMask1=-read-mask "%Mask%"
  set RdMask2=+read-mask
)

set IncH=1
set IncW=1

call parseCommaList "%IncWH%" num_inc vals_inc

if "%num_inc%"=="1" (
  set IncW=%vals_inc[0]%
  set IncH=1
) else if "%num_inc%"=="2" (
  set IncW=%vals_inc[0]%
  set IncH=%vals_inc[1]%
) else (
  echo Bad parameter IncWH: [%IncWH%]
  exit /B 1
)

echo IncW=%IncW% IncH=%IncH%


set MinW=1
set MinH=1

call parseCommaList "%MinWH%" num_inc vals_inc

if "%num_inc%"=="1" (
  set MinW=%vals_inc[0]%
) else if "%num_inc%"=="2" (
  set MinW=%vals_inc[0]%
  set MinH=%vals_inc[1]%
) else (
  echo Bad parameter MinWH: [%MinWH%]
  exit /B 1
)

echo MinW=%MinW% MinH=%MinH%


set MaxW=w
set MaxH=h

call parseCommaList "%MaxWH%" num_inc vals_inc

if "%num_inc%"=="1" (
  set MaxW=%vals_inc[0]%
) else if "%num_inc%"=="2" (
  set MaxW=%vals_inc[0]%
  set MaxH=%vals_inc[1]%
) else if not "%num_inc%"=="0" (
  echo Bad parameter MaxWH: [%MaxWH%]
  exit /B 1
)

echo MaxW=%MaxW% MaxH=%MaxH%

set TmpFile=%TEMP%\mr_tmp.miff

set X0=
set Y0=

set sFX=^
  maxVal = -9e9; ^
  areaMax = maxVal; ^
  okayW = 1; ^
  pZeroR = p[-1,-1]; ^
  thisMaxH = %MaxH%; ^
  for (ww=%MinW%, ww ^<= %MaxW% ^&^& i+ww ^<= w ^&^& okayW, ^
    wmOne = ww-1; ^
    pwwR = p[wmOne,-1]; ^
    okayH = 1; ^
    for (hh=%MinH%, hh ^<= thisMaxH ^&^& j+hh ^<= h ^&^& okayH, ^
      hmOne = hh-1; ^
      sum = floor ((pZeroR-p[-1,hmOne]-pwwR+p[wmOne,hmOne]) * QuantumRange + 0.5); ^
      area = ww * hh; ^
      if (sum == area, ^
        if (maxVal ^< sum, ^
            maxVal = sum; areaMax = area ^
           , ) ^
      , okayH = 0; thisMaxH = hh; if (hh==1, okayW = 0, ) ^
      ); ^
      hh += %IncH% ^
    ); ^
    ww += %IncW% ^
  ); ^
  if (i == 9999, debug(areaMax) , ); ^
  (areaMax ^< 0) ? 0 : areaMax/QuantumRange;

for /F "usebackq skip=1 tokens=4,5 delims=:(), " %%A in (`%IMG7%magick ^
  %INFILE% ^
  -precision 15 ^
  +transparent %FgndCol% ^
  -fill Black -colorize 100 ^
  -background White -layers Flatten ^
  -colorspace Gray ^
  -alpha off ^
  -negate ^
  -evaluate Min 1 ^
  -define "quantum:format=floating-point" -depth 32 ^
  -integral ^
  -write %TmpFile% ^
  -virtual-pixel Black ^
  %WrMask1% ^
  -fx "%sFX%" ^
  %WrMask2% ^
  %sRAM% ^
  %RdMask1% ^
  -define "identify:locate=maximum" ^
  -define "identify:limit=1" ^
  -identify ^
  %RdMask2% ^
  NULL:`) do (
  set X0=%%A
  set Y0=%%B
  echo top-left at  %%A, %%B
)

if "%X0%"=="" exit /B 1
if "%Y0%"=="" exit /B 1

rem Now find the width and height for the largest rectangle with top-left at X0,Y0.

set sFX2=^
  maxVal = -9e9; ^
  areaMax = maxVal; ^
  okayW = 1; ^
  pZeroR = p[%X0%-1,%Y0%-1]; ^
  thisMaxH = h; ^
  wwAtMax = -999; ^
  hhAtMax = -999; ^
  for (ww=%MinW%, ww ^<= w ^&^& %X0%+ww ^<= w ^&^& okayW, ^
    wmOne = ww-1; ^
    pwwR = p[%X0%+wmOne,%Y0%-1]; ^
    okayH = 1; ^
    for (hh=%MinH%, hh ^<= thisMaxH ^&^& %Y0%+hh ^<= h ^&^& okayH, ^
      hmOne = hh-1; ^
      sum = floor ((pZeroR-p[%X0%-1,%Y0%+hmOne]-pwwR+p[%X0%+wmOne,%Y0%+hmOne]) * QuantumRange + 0.5); ^
      area = ww * hh; ^
      if (sum == area, ^
        if (maxVal ^< sum, ^
            maxVal = sum; areaMax = area; wwAtMax = ww; hhAtMax = hh; ^
           , ) ^
      , okayH = 0; thisMaxH = hh; if (hh==1, okayW = 0, ) ^
      ); ^
      hh++ ^
    ); ^
    ww++ ^
  ); ^
  debug(wwAtMax); ^
  debug(hhAtMax); ^
  debug(areaMax); ^
  areaMax;

set wwAtMax=
set hhAtMax=
for /F "usebackq tokens=2,3 delims=:= " %%A in (`%IMG7%magick ^
  %TmpFile% ^
  -virtual-pixel Black ^
  -format "%%[fx:%sFX2%]\n" ^
  info: 2^>^&1`) do (
  if "%%A"=="wwAtMax" set wwAtMax=%%B
  if "%%A"=="hhAtMax" set hhAtMax=%%B
  if "%%A"=="areaMax" set areaMax=%%B
)
if "%wwAtMax%"=="" exit /B 1
if "%hhAtMax%"=="" exit /B 1

set /A X1=%X0%+%wwAtMax%-1
set /A Y1=%Y0%+%hhAtMax%-1

if not "%OUTFILE%"=="" %IMG7%magick ^
  %INFILE% ^
  -fill sRGBA(100%%,100%%,0,0.75) ^
  -draw "rectangle %X0%,%Y0%,%X1%,%Y1%" ^
  %OUTFILE%

call echoRestore

endlocal & set mrOUTFILE=%OUTFILE%& ^
set %EnvPref%_area=%areaMax%& ^
set %EnvPref%_crop=%wwAtMax%x%hhAtMax%+%X0%+%Y0%& ^
set %EnvPref%_H=%hhAtMax%& ^
set %EnvPref%_W=%wwAtMax%& ^
set %EnvPref%_X0=%X0%& ^
set %EnvPref%_X1=%X1%& ^
set %EnvPref%_Y0=%Y0%& ^
set %EnvPref%_Y1=%Y1%

