rem Called from mebcAssemble.bat and mkQuilt.bat, and others.

rem Given %1 is input/output canvas,
rem %2 is image patch to be merged onto canvas, with the required canvas offset,
rem %3 is horizontal overlap in pixels
rem %4 is vertical overlap in pixels
rem merges images possibly with Minimum Error Boundary Cut along top edge or left edge or both.

@rem
@rem Updated:
@rem   14-July-2022 for IM v7.
@rem

@rem FUTURE: accept parameters for offsets, then no need to "identify".
@rem FUTURE: make the blur an environment variable.

@setlocal enabledelayedexpansion

set CANV=%1
set IMG=%2
set overX=%3
set overY=%4

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\noffX=%%X\noffY=%%Y" ^
  %IMG%`) do set /A %%L

echo !IMG! WW=%WW% HH=%HH% offX=%offX% offY=%offY%

set /A xPlusy=%offX%+%offY%

if %xPlusy%==0 (
  echo Cut nothing
  if exist %CANV% (
    %IMG7%magick ^
      %CANV% ^
      %IMG% ^
      -compose Over -layers Merge ^
      %CANV%
  ) else (
    %IMG7%magick ^
      %IMG% ^
      %CANV%
  )

  if ERRORLEVEL 1 exit /B 1

) else if %offX%==0 (
  set /A upY=!offY!-%overY%
  echo Cut top upY=!upY!

  %IM7DEV%magick ^
    ^( %CANV% +write mpr:CANV ^
       +repage ^
       -crop %WW%x%overY%+0+%upY% +repage ^
    ^) ^
    ^( %IMG% +write mpr:IMG ^
       +repage ^
       -crop %WW%x%overY%+0+0 +repage ^
    ^) ^
    -colorspace Gray ^
    -alpha off ^
    -compose Difference -composite ^
    -rotate -90 ^
    -process darkestpath ^
    -morphology dilate:-1 2x1+1+0:1,1 ^
    -alpha off ^
    -blur 0x0.5 ^
    -rotate 90 ^
    -background White -extent %WW%x%HH% ^
    mpr:IMG ^
    +swap ^
    -alpha off ^
    -compose CopyOpacity -composite ^
    mpr:CANV ^
    +swap ^
    -background None ^
    -compose Over -layers Merge ^
    +repage ^
    %CANV%

  if ERRORLEVEL 1 exit /B 1

) else if %offY%==0 (
  set /A leftX=!offX!-%overX%
  echo Cut leftX=!LeftX!

  %IM7DEV%magick ^
    ^( %CANV% +write mpr:CANV ^
       +repage ^
       -crop %overX%x%HH%+%leftX%+0 +repage ^
    ^) ^
    ^( %IMG% +write mpr:IMG ^
       +repage ^
       -crop %overX%x%HH%+0+0 +repage ^
    ^) ^
    -colorspace Gray ^
    -alpha off ^
    -compose Difference -composite ^
    -process darkestpath ^
    -morphology dilate:-1 2x1+1+0:1,1 ^
    -alpha off ^
    -blur 0x0.5 ^
    -background White -extent %WW%x%HH% ^
    mpr:IMG ^
    +swap ^
    -alpha off ^
    -compose CopyOpacity -composite ^
    mpr:CANV ^
    +swap ^
    -background None ^
    -compose Over -layers Merge ^
    +repage ^
    %CANV%

  if ERRORLEVEL 1 exit /B 1

) else (
  set /A upY=!offY!-%overY%
  set /A leftX=!offX!-%overX%
  rem set /A upY=!offY!
  rem set /A leftX=!offX!

  echo Cut left and top leftX=!LeftX! upY=!upY!

  rem Does this correctly cut left and top "torn" edges?

  %IM7DEV%magick ^
    ^( %CANV% +write mpr:CANV ^
       +repage ^
       -crop %WW%x%overY%+0+%upY% +repage ^
    ^) ^
    ^( %IMG% +write mpr:IMG ^
       +repage ^
       -crop %WW%x%overY%+0+0 +repage ^
    ^) ^
    -colorspace Gray ^
    -alpha off ^
    -compose Difference -composite ^
    -rotate -90 ^
    -process darkestpath ^
    -morphology dilate:-1 2x1+1+0:1,1 ^
    -alpha off ^
    -blur 0x0.5 ^
    -rotate 90 ^
    -background White -extent %WW%x%HH% ^
    ^( ^
      ^( mpr:CANV ^
         +repage ^
         -crop %overX%x%HH%+%leftX%+0 +repage ^
      ^) ^
      ^( mpr:IMG ^
         +repage ^
         -crop %overX%x%HH%+0+0 +repage ^
      ^) ^
      -colorspace Gray ^
      -alpha off ^
      -compose Difference -composite ^
      -process darkestpath ^
      -morphology dilate:-1 2x1+1+0:1,1 ^
      -alpha off ^
      -blur 0x0.5 ^
      -background White -extent %WW%x%HH% ^
    ^) ^
    -compose Multiply -composite ^
    mpr:IMG ^
    +swap ^
    -alpha off ^
    -compose CopyOpacity -composite ^
    mpr:CANV ^
    +swap ^
    -background None ^
    -compose Over -layers Merge ^
    +repage ^
    %CANV%

  if ERRORLEVEL 1 exit /B 1

)

endlocal
