rem From %1, a image where pixels on a rim are opaque and others are transparent,
rem write %2, a membrane image.
@rem
@rem Also uses:
@rem   memSUPER_SAMP_PC percentage for supersampling, eg 400.
@rem
@rem Also sets environment vaiables:
@rem   memCX, memCY  coordinates of rim centre.
@rem   memVc  colour at (memCX,memCY).
@rem
@rem Updated:
@rem   13-August-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mem

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set memSUPER_SAMP_PC=




set EXT=.miff
set RIMGRAD=%BASENAME%_mem_rimgrad%EXT%
set RADGRAD=%BASENAME%_mem_radgrad%EXT%
set FANGRAD=%BASENAME%_mem_fangrad%EXT%
set RIMSPD=%BASENAME%_mem_rimspd%EXT%
set FCPIX=%BASENAME%_mem_fcpix%EXT%

set WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nWW_2=%%[fx:w/2]\nHH_2=%%[fx:h/2]\nDIAG_2=%%[fx:hypot(w,h)/2]" ^
  %INFILE%`) do set %%L

if "%WW%"=="" exit /B 1

:: Next two magicks must use same Q-number.

for /F "usebackq" %%L in (`%IM7DEV%magick identify ^
  -precision 19 ^
  -format "DELTA=%%[fx:int(QuantumRange/%DIAG_2%)]" ^
  %INFILE%`) do set %%L

echo %0: WW=%WW% HH=%HH% DIAG_2=%DIAG_2% DELTA=%DELTA% 

set CX=
for /F "usebackq tokens=1-2 delims=," %%X in (`%IM7DEV%magick ^
  %INFILE% ^
  -alpha extract ^
  -bordercolor Black -border 1 ^
  -fill Red -draw "color 0,0 floodfill" ^
  -shave 1x1 ^
  -fill White -opaque Red ^
  -negate ^
  -morphology Distance "Euclidean:4,%DELTA%" ^
  +depth ^
  -alpha off ^
  +write %RIMGRAD% ^
  -process midlightest ^
  NULL: 2^>^&1`) do (
  set CX=%%X
  set CY=%%Y
)

if "%CX%"=="" exit /B 1

echo %0: CX=%CX%  CY=%CY%

%IMG7%magick ^
  -size %WW%x%HH% ^
  -define gradient:center=%CX%,%CY% ^
  -define gradient:radii=%DIAG_2%,%DIAG_2% ^
  radial-gradient: ^
  -alpha off ^
  %RADGRAD%
if ERRORLEVEL 1 exit /B 1

call %PICTBAT%fanComp ^
  %RADGRAD% ^
  %RIMGRAD% ^
  %FANGRAD%
if ERRORLEVEL 1 exit /B 1

echo %0: Made fancomp.

%IMG7%magick ^
  %RADGRAD% ^
  -negate ^
  %INFILE% ^
  -compose copy_opacity -composite ^
  +write mpr:RDIST ^
  %INFILE% ^
  -compose Multiply -composite ^
  mpr:RDIST ^
  -scale "1x1^!" ^
  -alpha off ^
  -compose DivideSrc -composite ^
  %FCPIX%
if ERRORLEVEL 1 exit /B 1

echo %0: Made fcpix.

if "%memSUPER_SAMP_PC%"=="" (
  set sRES=
) else (
  set sRES=-resize %memSUPER_SAMP_PC%%%
)


%IMG7%magick ^
  %INFILE% ^
  %sRES% ^
  -distort depolar -1,0 ^
  ( +clone -alpha extract -threshold 50%% ) ^
  -alpha off ^
  -compose CopyOpacity -composite ^
  %RIMSPD%
if ERRORLEVEL 1 exit /B 1

echo %0: Made rimspd.

call %PICTBAT%shiftFill %RIMSPD% 0x %RIMSPD%
if ERRORLEVEL 1 exit /B 1

echo %0: done shiftFill.

%IMG7%magick ^
  %RIMSPD% ^
  -crop x1+0+0 +repage ^
  %RIMSPD%
if ERRORLEVEL 1 exit /B 1

call %PICTBAT%blurFill ^
  %RIMSPD% . %RIMSPD%
if ERRORLEVEL 1 exit /B 1

%IMG7%magick ^
  %RIMSPD% ^
  -scale "%WW%x%HH%^!" ^
  -distort polar -1,0 ^
  -resize "%WW%x%HH%^!" ^
  %RIMSPD%
if ERRORLEVEL 1 exit /B 1


:: out = rimspd - (fan * rimspd) + (fan * fcpix)

%IMG7%magick ^
  %FANGRAD% +write mpr:FAN ^
  ( %FCPIX% -scale "%WW%x%HH%^!" +write mpr:FC ) ^
  -compose Multiply -composite ^
  +write mpr:FxGC ^
  -delete 0-2 ^
  %RIMSPD% +write mpr:SPD ^
  ( mpr:FAN mpr:SPD -compose Multiply -composite -negate ) ^
  mpr:FxGC ^
  -colorspace sRGB ^
  -evaluate-sequence AddModulus ^
  mpr:FC ^
  mpr:FAN ^
  -compose Over -composite ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

echo %0: Made outfile %OUTFILE%.


call echoRestore

endlocal & set memOUTFILE=%OUTFILE%
