#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vsn_defines.h"

/*
    Find coordinate of the lightest pixel.

    If there is more than one with equal lightness,
    finds the one that is nearest the centroid of those pixels.
*/


// The next function corresponds in style to functions in enhance.c
// It takes one image, and returns a status.
//
static MagickBooleanType midlightest(const Image *image,
  ExceptionInfo *exception)
{
  MagickBooleanType
    status;

  CacheView
    *image_view;

  ssize_t
    y;

  ssize_t
    xLightest,
    yLightest;

  long double
    val,
    valLightest;


  assert(image != (Image *) NULL);
  assert(image->signature == MAGICK_CORE_SIG);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  image_view = AcquireVirtualCacheView(image,exception);

  status = MagickTrue;

  valLightest = 0;
  xLightest = yLightest = 0;
  int nFound = 0;

  int sigX=0, sigY=0;

  for (y=0; y < (ssize_t) image->rows; y++)
  {
    VIEW_PIX_PTR const *p;

    register ssize_t x;

    if (status == MagickFalse) continue;

    p=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
    if (p == (const VIEW_PIX_PTR *) NULL) status=MagickFalse;

    for (x=0; x < (ssize_t) image->columns; x++) {
      val = GetPixelIntensity(image, p);

      if (valLightest < val) {
        valLightest = val;
        sigX = x;
        sigY = y;
        nFound = 1;
      } else if (valLightest == val) {
        sigX += x;
        sigY += y;
        nFound++;
      }
      p += Inc_ViewPixPtr (image);
    }
  }

  if (nFound < 2) {
    xLightest = sigX;
    yLightest = sigY;
  } else {
    float midX = sigX / (float)nFound;
    float midY = sigY / (float)nFound;

    // Find the closest pixel to (midX,midY) with value valLightest.

    float minRad=999999999e9;

    for (y=0; y < (ssize_t) image->rows; y++)
    {
      VIEW_PIX_PTR const *p;

      register ssize_t x;

      if (status == MagickFalse) continue;

      p=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
      if (p == (const VIEW_PIX_PTR *) NULL) status=MagickFalse;

      for (x=0; x < (ssize_t) image->columns; x++) {
        val = GetPixelIntensity(image, p);

        if (valLightest == val) {
          float r = hypot (x-midX, y-midY);
          if (minRad > r) {
            minRad = r;
            xLightest = x;
            yLightest = y;
          }
        }
        p += Inc_ViewPixPtr (image);
      }
    }
  }

  fprintf (stderr, "%lu,%lu\n", xLightest, yLightest);

  return MagickTrue;
}


ModuleExport size_t midlightestImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image;

  MagickBooleanType
    status;

  (void) argc;
  (void) argv;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  for (image=(*images); image != (Image *) NULL; image=GetNextImageInList(image))
  {
    status = midlightest(image, exception);

    if (status == MagickFalse)
      continue;
  }

  assert (status == MagickTrue);

  return(MagickImageFilterSignature);
}
