rem %1 text data file
rem %2 output Nx1 grayscale clut file
rem
rem Data file contains at least 2 lines with CSV fields:
rem   x-coordinate (pixel or c or % or p suffix)
rem   y value
rem   gradient. Optional.

rem Gradient defaults:
rem   zero at each end
rem   otherwise, slope of adjacent two stations

rem x-coords are the start of each segment.
rem Generally, first x-coord should be zero.
rem x-coords should increase.
rem Final x-coord should be width minus 1.

rem Throughout, we assume values and gradients are percentages.


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INFILE=%1

set OUTFILE=%2

set TMPDIR=\temp

if not exist %INFILE% (
  echo %0: INFILE [%INFILE%] does not exist
  exit /B 1
)

set N=0
for /F "eol=# tokens=1,2,3 delims=	, " %%A in (%INFILE%) do (
  echo %%A,%%B,%%C
  if !N!==0 if not %%A==0 (
    set hc_Xcoord[0]=0
    set hc_Yval[0]=0
    set hc_Grad[0]=0
    set N=1
  )
  set hc_Xcoord[!N!]=%%A
  set hc_Yval[!N!]=%%B
  set hc_Grad[!N!]=%%C
  set /A N+=1
)

echo %0: %N% values
if %N% lss 2 (
  echo %0: Only %N% values. Needs at least two.
  exit /B 1
)

set /A nLast=N-1
set /A nLastM1=N-2

rem set hc_

:: Calculate lengths, and any blank gradients.

set FMT=^
hc_Len[0]=%%[fx:!hc_Xcoord[1]!-!hc_Xcoord[0]!+1]\n

for /F "usebackq" %%L in (`%IMG7%magick xc: ^
  -precision 15 ^
  -format "%FMT%" ^
  info:`) do set %%L

:: Default gradient at ends is zero.
:: We might have option for default gradient as straight line from start to next point,
::   and penultimate point to last point.

if "%hc_Grad[0]%"=="" set hc_Grad[0]=0
if "!hc_Grad[%nLast%]!"=="" set hc_Grad[%nLast%]=0

for /L %%I in (1,1,%nLastM1%) do (
  if "!hc_Grad[%%I]!"=="" (
    echo Calc gradient %%I
    call :CalcGrad %%I
  )
  call :CalcLen %%I
)

set hc_

:: Calculate the individual curves.

set FILES=
for /L %%I in (0,1,%nLastM1%) do (
  echo curve %%I
  set FNAME=%TMPDIR%\hc_%%I.miff
  call :DrawCurve %%I !FNAME!
  if ERRORLEVEL 1 exit /B 1
  set FILES=!FILES! !FNAME!
)

:: Append the individual curves into a spline.

echo %0: FILES=%FILES%

%IMG7%magick %FILES% +write info: +append +repage +write info: %OUTFILE%



call echoRestore

@endlocal & set mhsOUTFILE=%OUTFILE%

@exit /B 0

::------------------------------------------------
:: Subroutines

:CalcGrad
set I=%1
set /A Ip1=%1+1
set /A Im1=%1-1

set FMT=^
hc_Grad[%I%]=%%[fx:(!hc_Yval[%Ip1%]!-!hc_Yval[%Im1%]!)/(!hc_Xcoord[%Ip1%]!-!hc_Xcoord[%Im1%]!)]\n

for /F "usebackq" %%L in (`%IMG7%magick xc: ^
  -precision 15 ^
  -format "%FMT%" ^
  info:`) do set %%L

exit /B 0

:CalcLen
set I=%1
set /A Ip1=%1+1

set FMT=^
hc_Len[%I%]=%%[fx:!hc_Xcoord[%Ip1%]!-!hc_Xcoord[%I%]!+1]\n

for /F "usebackq" %%L in (`%IMG7%magick xc: ^
  -precision 15 ^
  -format "%FMT%" ^
  info:`) do set %%L

exit /B 0


:DrawCurve
set I=%1
set /A Ip1=%1+1

set DO_CHOP=
if not %I%==%nLastM1% set DO_CHOP=chop

call %PICTBAT%mkHermiteClut ^
  %2 ^
  !hc_Yval[%I%]! !hc_Yval[%Ip1%]! ^
  !hc_Len[%I%]! ^
  !hc_Grad[%I%]! !hc_Grad[%Ip1%]! ^
  . %DO_CHOP%

if ERRORLEVEL 1 exit /B 1

exit /B 0
