rem From image %1, make Laplacian pyramid %2.
@rem %2 should be capable of containing multiple images, eg type TIFF.
@rem %3 %4 %5 %6 parameters to mkPyrComm
@rem
@rem Also uses:
@rem   mkpDEBUG if 1, creates other image files (full and diff).
@rem   mkpHTM if 1, creates HTM file
@rem   mkpKEEP_ALPHA_RES if 1, keeps alpha in grids at best resolution.
@rem   pyPREFIX prefix for working files.
@rem
@rem Updated:
@rem   5-September-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mklp


if "%pyWEIGHTING%"=="" set pyWEIGHTING=max

call %PICTBAT%mkPyrComm %1 %2 %3 %4 %5 %6
if ERRORLEVEL 1 exit /B 1

set sASSOC=-define tiff:alpha=associated

set KEEP_ALPH=1

for /L %%N in (%LAST_OCT%,-1,0) do (

  rem echo N_BLK_W.%%N=!N_BLK_W.%%N! N_BLK_H.%%N=!N_BLK_H.%%N! AMP.%%N=!AMP.%%N! M1AMP.%%N=!M1AMP.%%N!

rem  if !M1AMP.%%N!==1 (
rem    set sLEVEL=
rem  ) else (
rem    for /F "usebackq" %%L in (`%IMG7%magick identify ^
rem      -format "LO=%%[fx:50*(1-!M1AMP.%%N!)]\nHI=%%[fx:50*(1+!M1AMP.%%N!)]" ^
rem      xc:`) do set %%L
rem
rem    set sLEVEL=+level !LO!%%,!HI!%%
rem  )

  set DO_RES=1
  if !N_BLK_W.%%N!==%WW% if !N_BLK_H.%%N!==%HH% set DO_RES=0

  set WRDIFF=1
  if not %%N==0 if not "%mkpDEBUG%"=="1" set WRDIFF=0

  ( 
    if %%N==%LAST_OCT% if "%mkpKEEP_ALPHA_RES%"=="1" echo ^( +clone -alpha extract +write mpr:ALPH +delete ^)
    if !DO_RES!==1 echo -channel RGB -resize "!N_BLK_W.%%N!x!N_BLK_H.%%N!^!" +channel
    if "%mkpDEBUG%"=="1" echo +write %pyPREFIX%grd_%%N%fnNG_EXT%
    echo +write mpr:GRD%%N
    if !DO_RES!==1 echo -channel RGB -resize "%WW%x%HH%^!" +channel
    if "%mkpDEBUG%"=="1" echo +write %pyPREFIX%full_%%N%fnNG_EXT% !sLEVEL.%%N!
    echo mpr:!PREV_DIFF!
    echo -channel RGB -compose Mathematics -define compose:args=0,1,-1,0.5 -composite +channel
    if "%mkpKEEP_ALPHA_RES%"=="1" echo mpr:ALPH -alpha off -compose CopyOpacity -composite
    set PREV_DIFF=DIFF%%N
    if "!WRDIFF!"=="1" echo +write %pyPREFIX%diff_%%N%fnNG_EXT%
    echo +write mpr:!PREV_DIFF!
  )>>%MK_SCR%

  ( if %%N==%LAST_OCT% if "%mkpKEEP_ALPHA_RES%"=="1" echo %PYR_FILE%[%NUM_OCTAVES%] -alpha extract +write mpr:ALPH +delete
    echo ^( %PYR_FILE%[%%N]
    if !DO_RES!==1 echo -resize "%WW%x%HH%^!"
    if "%mkpKEEP_ALPHA_RES%"=="1" echo mpr:ALPH -alpha off -compose CopyOpacity -composite
    echo !sLEVEL.%%N! ^)

    if !DONE_FIRST!==0 (
      set DONE_FIRST=1
    ) else (
      echo -compose Mathematics -define compose:args=0,1,1,-0.5 -composite
    )
  )>>%RECON_SCR%

  if "%mkpHTM%"=="1" (
    echo ^<tr^>
    echo ^<td^>%%N^</td^>
    echo ^<td^>^<img src="%pyPREFIX%grd_%%N%fnNG_EXT%%EXTEXT%" /^>^</td^>
    echo ^<td^>^<img src="%pyPREFIX%full_%%N%fnNG_EXT%%EXTEXT%" /^>^</td^>
    echo ^<td^>^<img src="%pyPREFIX%full_%%N_h_glc.png" /^>^</td^>
    echo ^<td^>^<img src="%pyPREFIX%diff_%%N%fnNG_EXT%%EXTEXT%" /^>^</td^>
    echo ^</tr^>
  )>>%HTM_TAB%
)

:: List the MPRs, ready to write to pyramid file.
( echo +delete
  for /L %%N in (0,1,%LAST_OCT%) do @(
    @echo mpr:GRD%%N
  )
  echo mpr:DIFF0

  echo -define quantum:format=floating-point
  echo +depth
  echo -depth 32
  echo %sASSOC%
  echo -write %PYR_FILE%
  echo -exit

)>>%MK_SCR%

:: Add final difference, and delinear, to reconstruction.
(
  echo %PYR_FILE%[%NUM_OCTAVES%]
  rem if "%mkpKEEP_ALPHA_RES%"=="1" echo ^( +clone -alpha extract +write mpr:ALPH +delete ^)
  echo -compose Mathematics -define compose:args=0,1,1,-0.5 -composite %sDELIN%
  rem if "%mkpKEEP_ALPHA_RES%"=="1" echo mpr:ALPH -alpha off -compose CopyOpacity -composite
  echo -write %RECON_FILE%
  echo -exit
)>>%RECON_SCR%

if "%mkpHTM%"=="1" echo ^</table^> >>%HTM_TAB%

if "%pyIM%"=="" (
  set pyIM=%IMG7%
)
if "%pyIM%"=="" (
  echo %0: pyIM not set
  exit /B 1
)

%pyIM%magick ^
  -script %MK_SCR%

if "%mkpHTM%"=="1" (
  for /L %%N in (0,1,%LAST_OCT%) do @(
    %IM7DEV%magick ^
      %pyPREFIX%full_%%N%fnNG_EXT% ^
      -process 'mkhisto capnumbuckets 512 norm' ^
      %pyPREFIX%full_%%N_h%fnNG_EXT%

    call %PICTBAT%graphLineCol %pyPREFIX%full_%%N_h%fnNG_EXT% . . 0 %pyPREFIX%full_%%N_h_glc.png
  )
)

call echoRestore

@endlocal & set mklpOUTFILE=%PYR_FILE%& set mklpNUM_OCTAVES=%NUM_OCTAVES%& set mklpWW=%WW%& set mklpHH=%HH%& set mklpRECON_FILE=%RECON_FILE%

