rem From %1 texture image, make quilt.
@rem %2 output file.
@rem %3 width and height of each patch (including overlap) [default one-third]
@rem %4 width and height of overlaps [default 30 percent of patch size]
@rem %5 number of columns and rows of patches [default 4x4]
@rem %6 coordinates within input texture for first patch [default 0,0]
@rem %7 method for choosing patch from texture: "random" or "search". Default: "random".
@rem
@rem %3 to %6 are each of format XxY
@rem   where X and Y are both optional integers each optionally suffixed with one of %cp.
@rem
@rem Future: For %6, take strings "random" or "centre".
@rem
@rem Updated:
@rem   14-July-2022 for IM v7. Added %7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mq

if not "%2"=="" set OUTFILE=%2

set TEMP_IN=mq_%INNAME%_in.miff
set TEMP_OUT=mq_%INNAME%_out.miff
set TEMP_PATCH=mq_%INNAME%_patch.miff
set TEMP_PATCH2=mq_%INNAME%_patch2.miff

set WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h" ^
  %INFILE%`) do set %%L
if "%WW%"=="" exit /B 1


call parseXxY 33.3333c 33.3333c %3
set patchW=%pxxyX%
set patchH=%pxxyY%

call xyCoord %WW% %HH% %patchW% %patchH%
set patchW=%xycXi%
set patchH=%xycYi%

call parseXxY 33.3333c 33.3333c %4
set overX=%pxxyX%
set overY=%pxxyY%

call xyCoord %patchW% %patchH% %overX% %overY%
set overX=%xycXi%
set overY=%xycYi%

call parseXxY 4 4 %5
set patchCols=%pxxyX%
set patchRows=%pxxyY%

call parseXxY 0 0 %6
set firstX=%pxxyX%
set firstY=%pxxyY%

set chText=%7
if "%chText%"=="." set chText=
if "%chText%"=="" set chText=random

if /I %chText%==search (
  set sCHTEXT=s
) else if /I %chText%==random (
  set sCHTEXT=r
) else (
  echo %0: Bad chText [%chText%]
  exit /B 1
)

set /A outW=%patchW%*%patchCols%-%overX%*(%patchCols%-1)
set /A outH=%patchH%*%patchRows%-%overY%*(%patchRows%-1)

echo %0: patchWH=%patchW%x%patchH% overXY=%overX%x%overY% patchCR=%patchCols%x%patchRows% firstXY=%firstX%x%firstY%
echo %0: %OUTFILE% out=%outW%x%outH%

if %outW% LSS 1 (
  echo %0: outW=%outW%
  exit /B 1
)

if %outH% LSS 1 (
  echo %0: outH=%outH%
  exit /B 1
)

%IMG7%magick ^
  %INFILE% ^
  +write %TEMP_IN% ^
  -crop %patchW%x%patchH%+%firstX%+%firstY% +repage ^
  -background None ^
  -extent %outW%x%outH% ^
  %TEMP_OUT%

set /A nColm1=%patchCols%-1
set /A nRowm1=%patchRows%-1

set FIRST=1

set siDELTEMPSRC=1
set siDELTEMPSUB=1

set siCOMP_FLT=
set siCOMP_CROP=

for /L %%R in (0,1,%nRowm1%) do (
  set /A rowY=%%R*^(%patchH%-%overY%^)
  for /L %%C in (0,1,%nColm1%) do (
    if !FIRST!==1 (
      set FIRST=0
    ) else (
      set /A colX=%%C*^(%patchW%-%overX%^)
      rem echo %0: rowY=!rowY! colX=!colX!

      if %sCHTEXT%==s (
        for /F "usebackq tokens=1-2 delims= " %%A in (`%IM7DEV%magick ^
          %TEMP_IN% ^
          ^( %TEMP_OUT% ^
             -crop %patchW%x%patchH%+!colX!+!rowY! +repage ^
          ^)
          -process 'rmsealpha multi_scale stdout' ^
          NULL:`) do (
            if "%%A"=="rmsealphaCrop:" set siCOMP_CROP=%%B
          )

        rem call %PICTBAT%srchImgAlpha %TEMP_IN% %TEMP_PATCH%

      ) else (
        for /F "usebackq" %%L in (`%IMG7%magick xc: ^
         -format "siCOMP_CROP=%patchW%x%patchH%+%%[fx:int((%WW%-%patchW%)*rand())]+%%[fx:int((%HH%-%patchH%)*rand())]" ^
         info:`) do set %%L
      )
      echo %0: siCOMP_FLT=!siCOMP_FLT! siCOMP_CROP=!siCOMP_CROP!

      %IMG7%magick ^
        %TEMP_IN% ^
        -crop !siCOMP_CROP! +repage ^
        -repage +!colX!+!rowY! ^
        %TEMP_PATCH2%

      call %PICTBAT%mebcOne %TEMP_OUT% %TEMP_PATCH2% %overX% %overY%
    )
  )
)

set siDELTEMPSRC=
set siDELTEMPSUB=

%IMG7%magick %TEMP_OUT% %OUTFILE%

call echoRestore

endlocal & set mqOUTFILE=%OUTFILE%
