rem From %1 texture image and %2 reference, make quilt like the reference.

@rem %3 output file.
@rem %4 width and height of each patch (including overlap) [default one-third]
@rem %5 width and height of overlaps [default 30 percent of patch size]
@rem
@rem %4 and %5 are each of format XxY
@rem   where X and Y are both optional numbers each optionally suffixed with one of %cp.
@rem
@rem Also uses:
@rem   mqlGAIN_BIAS if 1, adjusts gain and bias of texture patch to match canvas.
@rem   mqlSNAP_X if set, snaps horizontally to multiple of this (pixels, can be fractional).
@rem   mqlSNAP_Y likewise, vertically
@rem   mqlFRAME_FILE if set, writes one frame per patch.
@rem     XX in filename will be sustituted by a frame number with leading zeros, 6 digits.
@rem     Eg mql_fr_XX.png will create frames named mql_fr_000000.png, mql_fr_000001.png etc.
@rem     First and last frames (all reference, and all texture) will be included.
@rem
@rem Future: For %7, take strings "random" or "centre" or "bottom-right".
@rem Future: Use proxies for texture and reference.


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mql

set REF_IMG=%2

if not "%3"=="" set OUTFILE=%3

set TMP_EXT=.miff
set TEMP_IN=mq_%INNAME%_in%TMP_EXT%
set TEMP_OUT=mq_%INNAME%%TMP_EXT%
set TEMP_PATCH=mq_%INNAME%_patch%TMP_EXT%
set TEMP_PATCH2=mq_%INNAME%_patch2%TMP_EXT%

set TEX_WW=
for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  +write %TEMP_IN% ^
  -format "TEX_WW=%%w\nTEX_HH=%%h" ^
  info:`) do set %%L
if "%TEX_WW%"=="" exit /B 1


call parseXxY 33.3333c 33.3333c %4
set patchW=%pxxyX%
set patchH=%pxxyY%

call xyCoord %TEX_WW% %TEX_HH% %patchW% %patchH%
set patchW=%xycX%
set patchH=%xycY%

set patchWi=%xycXi%
set patchHi=%xycYi%

call parseXxY 33.3333c 33.3333c %5
set overX=%pxxyX%
set overY=%pxxyY%

call xyCoord %patchW% %patchH% %overX% %overY%
set overX=%xycXi%
set overY=%xycYi%

echo %0: patchWH=%patchW%x%patchH%  patchWHi=%patchWi%x%patchHi%  overXY=%overX%,%overY%

set REF_WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "REF_WW=%%w\nREF_HH=%%h\n" ^
  %REF_IMG%`) do set %%L
if "%REF_WW%"=="" exit /B 1

echo %0: REF_WW=%REF_WW% REF_HH=%REF_HH%


set CALC=^
patchCols=%%[fx:ceil((%REF_WW%-%overX%)/(%patchWi%-%overX%))]\n^
patchRows=%%[fx:ceil((%REF_HH%-%overY%)/(%patchHi%-%overY%))]

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%CALC%" ^
  xc:`) do set %%L

set CALC=^
texCols=%%[fx:ceil((%TEX_WW%-%overX%)/(%patchWi%-%overX%))]\n^
texRows=%%[fx:ceil((%TEX_HH%-%overY%)/(%patchHi%-%overY%))]

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%CALC%" ^
  xc:`) do set %%L

echo %0: patchCols=%patchCols% patchRows=%patchRows% texCols=%texCols% texRows=%texRows%

rem set /A outW=%patchWi%*%patchCols%
rem if %patchCols% GTR 1 set /A outW-=%overX%
rem 
rem set /A outH=%patchHi%*%patchRows%
rem if %patchRows% GTR 1 set /A outH-=%overY%

rem set /A outW=%patchWi%*%patchCols%-%overX%*(%patchCols%-1)
rem set /A outH=%patchHi%*%patchRows%-%overY%*(%patchRows%-1)

set CALC=^
overXi=%%[fx:int(%overX%+0.5)]\n^
overYi=%%[fx:int(%overY%+0.5)]\n^
outW=%%[fx:ceil^(%patchWi%*%patchCols%-%overX%*^(%patchCols%-1^)^)]\n^
outH=%%[fx:ceil^(%patchHi%*%patchRows%-%overY%*^(%patchRows%-1^)^)]

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%CALC%" ^
  xc:`) do set %%L

echo outW=%outW% outH=%outH%

echo %0: patchWH=%patchW%x%patchH% overXY=%overX%,%overY% patchRC=%patchCols%x%patchRows%
echo %0: %OUTFILE% outWH=%outW%x%outH%

if %outW% LSS 1 (
  echo outW=%outW%
  exit /B 1
)

if %outH% LSS 1 (
  echo outH=%outH%
  exit /B 1
)

%IMG7%magick ^
  %REF_IMG% ^
  -virtual-pixel Edge ^
  -define distort:viewport=%outW%x%outH% ^
  -distort SRT 1,0 +repage ^
  %TEMP_OUT%

set /A nColm1=%patchCols%-1
set /A nRowm1=%patchRows%-1

rem Last row/column of texture may be incomplete, so never match it.
set /A nTexColm2=%texCols%-2
set /A nTexRowm2=%texRows%-2

set siDELTEMPSRC=1
set siDELTEMPSUB=1


if not "%mqlFRAME_FILE%"=="" (
  call :getFrameName 000000
  echo FRAME_NAME=!FRAME_NAME!

  %IMG7%magick %TEMP_OUT% !FRAME_NAME!
)

if "%mqlGAIN_BIAS%"=="1" (
  set wrTMP_PATCH=+write %TEMP_PATCH%
) else (
  set wrTMP_PATCH=
)

set FRAME_NUM=1


for /L %%R in (0,1,%nRowm1%) do (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "rowY=%%[fx:int(%%R*(%patchHi%-%overY%)+0.5)]" ^
    xc:`) do set %%L

  for /L %%C in (0,1,%nColm1%) do (
    for /F "usebackq" %%L in (`%IMG7%magick identify ^
      -format "colX=%%[fx:int(%%C*(%patchWi%-%overX%)+0.5)]" ^
      xc:`) do set %%L

    echo %0: rowY=!rowY! colX=!colX!

    for /F "usebackq tokens=1-5 delims=x+ " %%A in (`%IM7DEV%magick ^
      %TEMP_IN% ^
      ^( %TEMP_OUT% ^
         -crop %patchWi%x%patchHi%+!colX!+!rowY! +repage ^
         %wrTMP_PATCH% ^
      ^) ^
      -process 'rmsealpha multi_scale stdout' ^
      NULL:`) do (
        if "%%A"=="rmsealphaCrop:" (
          set siSUB_W=%%B
          set siSUB_H=%%C
          set siCOMP_XW=%%D
          set siCOMP_YW=%%E
          set siCOMP_CROP=!siSUB_W!x!siSUB_H!+!siCOMP_XW!+!siCOMP_YW!
        )
      )

    rem call %PICTBAT%srchImgAlpha %TEMP_IN% %TEMP_PATCH%

    echo %0: siCOMP_FLT=!siCOMP_FLT! siCOMP_CROP=!siCOMP_CROP! in bits !siSUB_W!x!siSUB_H!+!siCOMP_XW!+!siCOMP_YW!

    if "!siCOMP_XW!"=="" exit /B 1

    if not !siSUB_W!==%patchWi% echo bad W !siSUB_W!==%patchWi%
    if not !siSUB_H!==%patchHi% echo bad H !siSUB_H!==%patchHi%

    if not "%mqlSNAP_X%"=="" (

      set CALC=^
siCOMP_XW=%%[fx:int^(int^(!siCOMP_XW!/^(%mqlSNAP_X%^)+0.5^)*^(%mqlSNAP_X%^)+0.5^)]

      set CALC=^
siCOMP_XW=%%[fx:NN=int^(!siCOMP_XW!/^(%mqlSNAP_X%^)+0.5^);NN=NN^>%nTexColm2%?%nTexColm2%:NN;int^(NN*^(%mqlSNAP_X%^)+0.5^)]

      for /F "usebackq" %%L in (`%IMG7%magick identify ^
        -format "!CALC!" ^
        xc:`) do set %%L
    )

    if not "%mqlSNAP_Y%"=="" (
      set CALC=^
siCOMP_YW=%%[fx:int^(int^(!siCOMP_YW!/^(%mqlSNAP_Y%^)+0.5^)*^(%mqlSNAP_Y%^)+0.5^)]

      set CALC=^
siCOMP_YW=%%[fx:NN=int^(!siCOMP_YW!/^(%mqlSNAP_Y%^)+0.5^);NN=NN^>%nTexRowm2%?%nTexRowm2%:NN;int^(NN*^(%mqlSNAP_Y%^)+0.5^)]

      for /F "usebackq" %%L in (`%IMG7%magick identify ^
        -format "!CALC!" ^
        xc:`) do set %%L
    )

    set sCROP=!siSUB_W!x!siSUB_H!+!siCOMP_XW!+!siCOMP_YW!

    echo sCROP=!sCROP!

    %IMG7%magick ^
      %TEMP_IN% ^
      -crop !sCROP! +repage ^
      -repage +!colX!+!rowY! ^
      %TEMP_PATCH2%

    if "%mqlGAIN_BIAS%"=="1" call %PICTBAT%imgGainBias %TEMP_PATCH2% %TEMP_PATCH% %TEMP_PATCH2%

    call %PICTBAT%mebcOne %TEMP_OUT% %TEMP_PATCH2% %overXi% %overYi%

    if not "%mqlFRAME_FILE%"=="" (
      set LZ=000000!FRAME_NUM!
      set LZ=!LZ:~-6!

      call :getFrameName !LZ!
      echo FRAME_NAME=!FRAME_NAME!

      %IMG7%magick %TEMP_OUT% !FRAME_NAME!

      set /A FRAME_NUM+=1
    )

    set siDELTEMPSRC=0
  )
)

set siDELTEMPSRC=
set siDELTEMPSUB=

%IMG7%magick ^
  %TEMP_OUT% ^
  -crop %REF_WW%x%REF_HH%+0+0 +repage ^
  %OUTFILE%

echo %0: Written %OUTFILE%

call echoRestore

@endlocal & set mqlOUTFILE=%OUTFILE%

@goto :eof

rem ----------------- Subroutines -------------------------

:getFrameName
set FRAME_NAME=!mqlFRAME_FILE:XX=%1!
