rem Ref angdist.htm

rem %1,%2 Input masks, white shapes on black background.
rem %3 output file
rem %4 centre: CXxCY, or furthest or centroid or boundingbox.
rem      Assumes this is in white of mask, and can "see" entire shape boundary.

set INFILE1=C:\prose\pictures\rad_mask1.png
set INFILE2=C:\prose\pictures\rad_mask2.png


set sCent=%4
if "%sCent%"=="" set sCent=.

call %PICTBAT%shpCent %INFILE1% %sCent% mtm

if ERRORLEVEL 1 (
  echo %0: Error in shpCent. sCent=%sCent%
  exit /B 1
)

set CX=%mtm_CX%
set CY=%mtm_CY%

rem Scale to 100 high for demonstration only.

set EPS=0.001


rem Find slopes

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE1% ^
  -set option:WW %%w ^
  -set option:HH %%h ^
  %INFILE2% ^
  -virtual-pixel Black ^
  -distort DePolar "-1,0,%CX%,%CY%" +repage ^
  -clamp ^
  -set option:mtmW %%w ^
  -set option:mtmH %%H ^
  -format "mtmW=%%w\nmtmH=%%h\n" +write info: ^
+write mtmb.png ^
  +adjoin +write mtmb.miff ^
  -scale "%%[WW]x1^!" ^
  -blur 0x1 ^
  +adjoin +write mtmc.miff ^
  ^( -clone 0 -fx "DIFF=p[1,0]-u; DIFF<-%EPS%?0:DIFF>%EPS%?1:0.5" ^) ^
  -scale "%%[WW]x100^!" ^
  -define "quantum:format=floating-point" -depth 32 ^
  +adjoin +write mtm.miff ^
  ^( -clone 0 -fx "DIFF=p[1,0]-u; DIFF<-%EPS%?0:DIFF>%EPS%?1:0.5" ^) ^
  ^( -clone 1 -fx "DIFF=p[1,0]-u; DIFF<-%EPS%?0:DIFF>%EPS%?1:0.5" ^) ^
  -delete 0-1 ^
  mtm2.png`) do set %%L

echo mtmW=%mtmW% mtmH=%mtmH%


rem Find largest peak in 1.

set LP=
for /F "usebackq skip=1 tokens=1-5 delims=:(), " %%A in (`%IMG7%magick ^
  mtmc-0.miff ^
  -define "identify:locate=maximum" ^
  -define "identify:limit=1" ^
  info:`) do if /I "%%A"=="Gray" set LP=%%D

if "%LP%"=="" (
  echo %0: LP not set
  exit /B 1
)

echo LP=%LP%

rem Roll both images
%IMG7%magick ^
  mtmc-0.miff mtmc-1.miff ^
  -roll -%LP%+0 ^
  +adjoin ^
  +write mtmc2.miff ^
  -auto-level ^
  ( -clone 0 -fx "DIFF=p[1,0]-u; DIFF<-%EPS%?0:DIFF>%EPS%?1:0.5" ) ^
  ( -clone 1 -fx "DIFF=p[1,0]-u; DIFF<-%EPS%?0:DIFF>%EPS%?1:0.5" ) ^
  -delete 0-1 ^
  mtmc3.miff


rem How about cumulative histograms? Shown as graphs, subtract one from the other horizontally,
rem gives the required relative displacement of theta??
rem If we needed the vertical distance, this would be easy to calculate. But that represents the difference in rho at each theta.

rem Before doing next, we should put matching peaks at first column of both images.

rem maybe auto-level after write mtmc2.miff? Better without.

rem Maybe   -process 'invclut v' after cumulhisto?


%IM7DEV%magick ^
  mtmc-0.miff mtmc-1.miff ^
  -roll -%LP%+0 ^
+adjoin +write mtmc2.miff ^
-auto-level ^
-set colorspace sRGB ^
  -process 'cumulhisto norm v' ^
  -fx "u-v+i/(w-1)" ^
  -scale "%mtmW%x%mtmH%^!" ^
  ( -size 1x%mtmH% gradient:Black-White ^
    -scale "%mtmW%x%mtmH%^!" ^
  ) ^
  ( -size %mtmW%x%mtmH% xc:Black ^
  ) ^
  -combine ^
  +adjoin ^
  mtmc4.png

%IM7DEV%magick ^
  mtmb-1.png ^
  -roll -%LP%+0 ^
+adjoin +write mtmc3.miff ^
-set colorspace sRGB ^
  mtmc4.png ^
  -compose Distort -set option:compose:args 100x100%% -composite ^
  mtmc5.miff


rem We can regard the two unrolled images that are one pixel high as histograms.
rem We want a clut that transforms one histogram to the other.
rem See customim.htm#matchhist

goto skipCode

magick ^
  image_A ^
  ( -clone 0 ^
    -process 'mkhisto cumul norm' ^
  ) ^
  ( image_B ^
    -process 'mkhisto cumul norm' ^
    -process 'mkhisto cumul norm' ^
  ) ^
  ( -clone 1-2 -clut ) ^
  -delete 1-2 ^
  -clut ^
  out_image

:skipCode
rem This is equivalent to taking image_A and clutting it with its own cumulative histogram (which equalises it),
rem and clutting that with the inverted cumulative histogram of image_B.
rem The result is a clut for transforming image_A into image_B.
rem Okay, so we can get a clut, but what image should we apply that to? What corresponds to image_A?

rem We can't auto-level next.

%IM7DEV%magick ^
  mtmc-0.miff mtmc-1.miff ^
  -roll -%LP%+0 ^
  ( -clone 0 ^
    -process 'cumulhisto norm v' ^
  ) ^
  ( -clone 1 ^
    -process 'cumulhisto norm v' ^
    -process 'mkhisto cumul norm' ^
  ) ^
  -delete 0-1 ^
  -clut ^
  -scale "%mtmW%x%mtmH%^!" ^
  ( -size 1x%mtmH% gradient:Black-White ^
    -scale "%mtmW%x%mtmH%^!" ^
  ) ^
  ( -size %mtmW%x%mtmH% xc:Black ^
  ) ^
  -combine ^
  mtmc6.miff

%IM7DEV%magick ^
  mtmb-1.png ^
  -roll -%LP%+0 ^
+adjoin +write mtmc3.miff ^
-set colorspace sRGB ^
  mtmc6.miff ^
  -compose Distort -set option:compose:args 100x100%% -composite ^
  +write mtmc7.miff ^
  -distort Polar "-1,0,%CX%,%CY%" +repage ^
  mtmc8.miff

rem Roll unrolled infiles for debugging
%IMG7%magick ^
  %INFILE1% %INFILE2% ^
  -distort DePolar "-1,0,%CX%,%CY%" +repage ^
  -roll -%LP%+0 ^
  +adjoin mtmc9.miff

