rem Makes many variable assignments, for making pyramids.
rem Optionally also creates file of variable assignments.

@rem %1 width in pixels. [600]
@rem %2 height in pixels. [400]
@rem %3 minimum block size in pixels. [1]
@rem %4 maximum block size in pixels. [min(w,h)/2]
@rem %5 block factor: Block size is increased by this factor from min to max. >=1 [2]
@rem %6 power factor for amplitude proportional to frequency. [Default 0.0]
@rem   0 for all frequencies have same amplitude;
@rem   > 0 (eg 1, 1.5) for highest frequencies have lowest amplitude.

@rem returns fnNUM_OCTAVES number of octaves.

@rem Also uses:
@rem
@rem   pyIM location of ImageMagick programs. [%IMG7%]
@rem   pyPREFIX prefix for working files (but not output file). [py_]
@rem   pyGRAPHIC if 1, normalises amplitude so sum or maximum is 0.5.
@rem     [Otherwise normalises so sum or maximum is 1.0.]
@rem   pyWEIGHTING scaling for weights:
@rem     sum: scale so sum is 1.0 or 0.5 (default)
@rem     max: scale so maximum is 1.0 or 0.5
@rem   pyLINEAR if 1, operates in linear colorspace
@rem   pyWR_VAR if 1, writes {variable}={value} lines to %BLK_LIS%
@rem   fnNG_EXT extension for noise grid files. [.miff]
@rem   pyTXT_OUT if 1, writes some text data.
@rem   sioCODE of the caller (eg mklp, mkgp)
@rem
@rem First release: 3 October 2015.
@rem Updated:
@rem   5-September-2022 for IM v7.
@rem
@rem

@rem TODO: power defined by clut image file, so we can emphasise middle frequencies.

@rem No compulsory parameters
@rem if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

rem @setlocal enabledelayedexpansion

rem call %PICTBAT%setInOut %1 mkpv


set WW=%1
if "%WW%"=="." set WW=
if "%WW%"=="" set WW=600

set HH=%2
if "%HH%"=="." set HH=
if "%HH%"=="" set HH=400

set BLK_MIN=%3
if "%BLK_MIN%"=="." set BLK_MIN=
if "%BLK_MIN%"=="" set BLK_MIN=1

set BLK_MAX=%4
if "%BLK_MAX%"=="." set BLK_MAX=
if "%BLK_MAX%"=="" (
  set BLK_MAX=%HH%
  if !BLK_MAX! GTR %WW% set BLK_MAX=%WW%
  set /A BLK_MAX=!BLK_MAX!/2
)

set BLK_FAC=%5
if "%BLK_FAC%"=="." set BLK_FAC=
if "%BLK_FAC%"=="" set BLK_FAC=2

set POW_FAC=%6
if "%POW_FAC%"=="." set POW_FAC=
if "%POW_FAC%"=="" set POW_FAC=0


if "%pyPREFIX%"=="" set pyPREFIX=py_
if "%fnNG_EXT%"=="" set fnNG_EXT=.miff

if "%BLK_LIS%"=="" set BLK_LIS=%pyPREFIX%blk.lis
del %BLK_LIS% 2>nul

if "%pyIM%"=="" set pyIM=%IMG7%

if "%pyWEIGHTING%"=="" set pyWEIGHTING=sum

if not "%pyWEIGHTING%"=="sum" if not "%pyWEIGHTING%"=="max" (
  echo %0: pyWEIGHTING [%pyWEIGHTING%] should be sum or max.
  exit /B 1
)

if "%pyLINEAR%"=="1" (
  set sLIN=-set colorspace sRGB -colorspace RGB
  set sDELIN=-set colorspace RGB -colorspace sRGB
) else (
  set sLIN=
  set sDELIN=
)

set PREC=-precision 15

set CONST_FAC=%fnCONST_FAC%
if "%CONST_FAC%"=="" set CONST_FAC=1

:: ------------------ Pre-loop ------------------
if "%pyMAX_NUM_OCT%"=="" (
  set /A LOOP_MAX=99
) else (
  set /A LOOP_MAX=%pyMAX_NUM_OCT%-1
)

set BLK_SZ=%BLK_MIN%
set NUM_OCTAVES=0
set MAX_AMP=0
set SIG_AMP=0
set BUST_BLK_WH=0

:preloop

set BLK_SZ.%NUM_OCTAVES%=%BLK_SZ%

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  %PREC% ^
  -format "FR.%NUM_OCTAVES%=%%[fx:sqrt(%BLK_SZ%*%BLK_SZ%/%WW%/%HH%)]" ^
  xc:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  %PREC% ^
  -format "N_BLK_W.%NUM_OCTAVES%=%%[fx:int^(%WW%/!BLK_SZ.%NUM_OCTAVES%!+0.5^)]\nN_BLK_H.%NUM_OCTAVES%=%%[fx:int^(%HH%/!BLK_SZ.%NUM_OCTAVES%!+0.5^)]" ^
  xc:`) do set %%L

if "!N_BLK_W.%NUM_OCTAVES%!"=="0" set N_BLK_W=1
if "!N_BLK_H.%NUM_OCTAVES%!"=="0" set N_BLK_H=1

if not "%pyMIN_BLK_WH%"=="" (
  if /I !N_BLK_W.%NUM_OCTAVES%! LSS %pyMIN_BLK_WH% set BUST_BLK_WH=1
  if /I !N_BLK_H.%NUM_OCTAVES%! LSS %pyMIN_BLK_WH% set BUST_BLK_WH=1
)

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  %PREC% ^
  -format "AMP.%NUM_OCTAVES%=%%[fx:pow(!FR.%NUM_OCTAVES%!,%POW_FAC%)]\nBLK_SZ=%%[fx:%BLK_SZ%*%BLK_FAC%]" ^
  xc:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  %PREC% ^
  -format "SIG_AMP=%%[fx:%SIG_AMP%+!AMP.%NUM_OCTAVES%!]\nFIN=%%[fx:%BLK_SZ%>=%BLK_MAX%?1:%NUM_OCTAVES%>=%LOOP_MAX%?1:!BUST_BLK_WH!]\nMAX_AMP=%%[fx:%MAX_AMP%>!AMP.%NUM_OCTAVES%!?%MAX_AMP%:!AMP.%NUM_OCTAVES%!]" ^
  xc:`) do set %%L

if "%pyTXT_OUT%"=="1" echo BLK_SZ.%NUM_OCTAVES%=!BLK_SZ.%NUM_OCTAVES%! FR.%NUM_OCTAVES%=!FR.%NUM_OCTAVES%!  AMP.%NUM_OCTAVES%=!AMP.%NUM_OCTAVES%!

set /A NUM_OCTAVES+=1

if %FIN%==0 goto preloop
:: ------------------ End Pre-loop ------------------

if "%BUST_BLK_WH%"=="1" set /A NUM_OCTAVES-=1

if %NUM_OCTAVES%==0 (
  echo %0: No octaves created
  exit /B 1
)

rem if %NUM_OCTAVES% GEQ %LOOP_MAX% (
rem   echo %0: Large number of octaves
rem   exit /B 1
rem )

set MAX_FR=%FR%

if "%pyTXT_OUT%"=="1" echo NUM_OCTAVES=%NUM_OCTAVES%  MAX_FR=%MAX_FR%  SIG_AMP=%SIG_AMP% MAX_AMP=%MAX_AMP%

if "%pyGRAPHIC%"=="1" for /F "usebackq" %%L in (`%IMG7%magick identify ^
  %PREC% ^
  -format "MAX_AMP=%%[fx:%MAX_AMP%*2]\nSIG_AMP=%%[fx:%SIG_AMP%*2]" ^
  xc:`) do set %%L

if "%pyWR_VAR%"=="1" (
  echo %0: Writing to var file [%BLK_LIS%]
)

if "%pyWR_VAR%"=="1" (
  echo pyWEIGHTING=%pyWEIGHTING%
  echo pyGRAPHIC=%pyGRAPHIC%
  echo pyLINEAR=%pyLINEAR%
  echo NUM_OCTAVES=%NUM_OCTAVES%
  echo %sioCODE%RECON_FILE=%RECON_FILE%
  echo IMG_WW=%WW%
  echo IMG_HH=%HH%
)>>%BLK_LIS%

if "%pyWR_VAR%"=="1" type %BLK_LIS%

set IMG_WW=%WW%
set IMG_HH=%HH%


:: ------------------ Main loop ------------------
set /A Nm1=%NUM_OCTAVES%-1

set ERROR=0

if "%SIG_AMP%"=="0" (
  echo %0: Error: SIG_AMP is zero.
  exit /B 1
)

for /L %%N in (0,1,%Nm1%) do (

  set OCT_NUM=%%N

rem  for /F "usebackq" %%L in (`%IMG7%magick identify ^
rem    %PREC% ^
rem    -format "N_BLK_W.%%N=%%[fx:int^(%WW%/!BLK_SZ.%%N!+0.5^)]\nN_BLK_H.%%N=%%[fx:int^(%HH%/!BLK_SZ.%%N!+0.5^)]" ^
rem    xc:`) do set %%L

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    %PREC% ^
    -format "S1AMP.%%N=%%[fx:%CONST_FAC%/%SIG_AMP%*!AMP.%%N!]\nM1AMP.%%N=%%[fx:%CONST_FAC%/%MAX_AMP%*!AMP.%%N!]" ^
    xc:`) do set %%L

  if "%pyTXT_OUT%"=="1" echo BLK_SZ.%%N=!BLK_SZ.%%N!  N_BLK_W.%%N=!N_BLK_W.%%N! N_BLK_H.%%N=!N_BLK_H.%%N!  FR.%%N=!FR.%%N!  S1AMP.%%N=!S1AMP.%%N! M1AMP.%%N=!M1AMP.%%N!

  if "%pyWEIGHTING%"=="sum" (
    set AMP.%%N=!S1AMP.%%N!
  ) else (
    set AMP.%%N=!M1AMP.%%N!
  )

  if !AMP.%%N!==1 (
    set sLEVEL.%%N=
  ) else (
    for /F "usebackq" %%L in (`%IMG7%magick identify ^
      %PREC% ^
      -format "LO=%%[fx:50*^(1-!AMP.%%N!^)]\nHI=%%[fx:50*^(1+!AMP.%%N!^)]" ^
      xc:`) do set %%L

    set sLEVEL.%%N=+level !LO!%%,!HI!%%
  )

  if "%pyWR_VAR%"=="1" (
    echo BLK_SZ.!OCT_NUM!=!BLK_SZ.%%N!
    echo N_BLK_W.!OCT_NUM!=!N_BLK_W.%%N!
    echo N_BLK_H.!OCT_NUM!=!N_BLK_H.%%N!
    echo FR.!OCT_NUM!=!FR.%%N!
    echo S1AMP.!OCT_NUM!=!S1AMP.%%N!
    echo M1AMP.!OCT_NUM!=!M1AMP.%%N!
    echo AMP.!OCT_NUM!=!AMP.%%N!
    echo sLEVEL.!OCT_NUM!=!sLEVEL.%%N!
  )>>%BLK_LIS%

rem  for /F "usebackq" %%L in (`%IMG7%magick identify ^
rem    %PREC% ^
rem    -format "BLK_SZ=%%[fx:!BLK_SZ!*%BLK_FAC%]" ^
rem    xc:`) do set %%L
)

rem @endlocal & set fnWW=%WW%& set fnHH=%HH%& set fnNUM_OCTAVES=%NUM_OCTAVES%& set fnMAX_FR=%MAX_FR%& set fnPOW_FAC=%POW_FAC%
