rem Boosts or cuts amplitudes of grids in a Laplacian pyramid.
rem %1 is the name of the multi-image pyramid file.
rem %2 is "boost" or "cut"
rem %3 is the -sigmoidal-contrast contrast boost or cut to be applied.
rem %4 is the name of the htm file to be generated.
@rem
@rem  Assumes pyPREFIX is correct,
@rem  so we can read the block list file (which must exist),
@rem  and run the reconstruction script.
@rem
@rem Updated:
@rem   15 May 2016 use %IML% for @script.
@rem   5-September-2022 for IM v7.
@rem


@if "%4"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mbc

set PYR_FILE=%1
set BOOST_CUT=%2
set CONT=%3
set HTM_FILE=%4
set HTM_NAME=%~n4
set SAVED_PYR=%~n1_mbpsav%~x1

if /I "%BOOST_CUT%"=="boost" (
  set SIG_SIGN=-
) else if /I "%BOOST_CUT%"=="cut" (
  set SIG_SIGN=+
) else (
  echo %0: BOOST_CUT [%BOOST_CUT%] is neither boost nor cut
  exit /B 1
)

echo SAVED_PYR=%SAVED_PYR%

if not exist %PYR_FILE% (
  echo %0: Can't find PYR_FILE [%PYR_FILE%]
  exit /B 1
)

set RECON_SCR=%pyPREFIX%mklp_recon.scr
set BLK_LIS=%pyPREFIX%blk.lis

if not exist %BLK_LIS% (
  echo %0: Can't find BLK_LIS [%BLK_LIS%]
  exit /B 1
)

set NUM_OCTAVES=

for /F "tokens=*" %%L in (%BLK_LIS%) do set %%L

if "%NUM_OCTAVES%"=="" (
  echo %0: in %BLK_LIS%, NUM_OCTAVES has no value
  exit /B 1
)

if not exist %RECON_SCR% (
  echo %0: Can't find RECON_SCR [%RECON_SCR%]
  exit /B 1
)

if "%mklpRECON_FILE%"=="" (
  echo %0: mklpRECON_FILE is blank.
  exit /B 1
)

copy /Y %PYR_FILE% %SAVED_PYR%

( echo ^<p^>%~n0: %BOOST_CUT% %CONT%^</p^>
) >%HTM_FILE%

:: Don't process final difference.

for /L %%N in (0,1,%NUM_OCTAVES%) do (

  set sOUT=%pyPREFIX%mbc_%HTM_NAME%_%%N.png

  %IMG7%magick ^
    %PYR_FILE% ^
    ^( -clone %%N ^
       %SIG_SIGN%sigmoidal-contrast %CONT%,50%% ^
    ^) ^
    -swap -1,%%N ^
    +delete ^
    %PYR_FILE%

  %IMG7%magick ^
    -script %RECON_SCR%

  %IMG7%magick %mklpRECON_FILE% !sOUT!

  ( echo ^<img src="!sOUT!jpg" /^>
  ) >>%HTM_FILE%

)

copy /Y %SAVED_PYR% %PYR_FILE%

type %HTM_FILE%

call echoRestore

@endlocal & set mklpOUTFILE=%PYR_FILE%
