rem Applies an IM process to grids in a Laplacian pyramid.
rem %1 is the name of the multi-image pyramid file.
rem %2 is the IM process to be applied to each grid.
rem %3 is the name of the htm file to be generated.
@rem
@rem  Assumes pyPREFIX is correct,
@rem  so we can read the block list file,
@rem  and run the reconstruction script.
@rem
@rem Updated:
@rem   5-September-2022 for IM v7.
@rem


@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 mpg

set PYR_FILE=%1
set IM_PROC=%~2
set HTM_FILE=%3
set HTM_NAME=%~n3
set SAVED_PYR=%~n1_mbpsav%~x1

echo %~n0: IM_PROC="%IM_PROC%"

if not exist %PYR_FILE% (
  echo %0: Can't find PYR_FILE [%PYR_FILE%]
  exit /B 1
)

set RECON_SCR=%pyPREFIX%mklp_recon.scr
set BLK_LIS=%pyPREFIX%blk.lis

if not exist %BLK_LIS% (
  echo %0: Can't find BLK_LIS [%BLK_LIS%]
  exit /B 1
)

set NUM_OCTAVES=

for /F "tokens=*" %%L in (%BLK_LIS%) do set %%L

if "%NUM_OCTAVES%"=="" (
  echo %0: in %BLK_LIS%, NUM_OCTAVES has no value
  exit /B 1
)

if not exist %RECON_SCR% (
  echo %0: Can't find RECON_SCR [%RECON_SCR%]
  exit /B 1
)

if "%mklpRECON_FILE%"=="" (
  echo %0: mklpRECON_FILE is blank.
  exit /B 1
)
copy /Y %PYR_FILE% %SAVED_PYR%

( echo ^<p^>%~n0: "%IM_PROC%"^</p^>
) >%HTM_FILE%

:: Don't process final difference.

for /L %%N in (0,1,%NUM_OCTAVES%) do (

  set sOUT=%pyPREFIX%mbc_%HTM_NAME%_%%N.png

  %IMG7%magick ^
    %PYR_FILE% ^
    ^( -clone %%N ^
       %IM_PROC% ^
    ^) ^
    -swap -1,%%N ^
    +delete ^
    %PYR_FILE%

  %IMG7%magick ^
    -script %RECON_SCR%

  %IMG7%magick %mklpRECON_FILE% !sOUT!

  ( echo ^<img src="!sOUT!jpg" /^>
  ) >>%HTM_FILE%

)

copy /Y %SAVED_PYR% %PYR_FILE%

call echoRestore

@endlocal & set mklpOUTFILE=%PYR_FILE%
