rem From image %1,
rem   list of x,y coordinates in text file %2,
rem   creates crops width %3 height %4 in files *_crop_n.*
rem Also creates text file _crops.lis. Name is returned as ncLIST_FILE.
@rem
@rem Also uses:
@rem   ncDEBUG         if 1, creates a marked-up copy of image.
@rem   ncDEBUG_STROKE  eg to get thick strokes
@rem   ncCROPDIR       directory to place the crops, default \temp
@rem   ncBASE          default name is based on %1, %3 and %4, in directory ncCROPDIR
@rem   ncREPAGE        if 0, does not "+repage" each crop.
@rem   ncEXT
@rem
@rem Returns:
@rem   ncLIST_FILE name of created file with list of names of crop files
@rem
@rem See also v7 version nCrop7.bat.
@rem
@rem Updated:
@rem   13-April-2018 added ncREPAGE
@rem   14-August-2020 for IM v7.
@rem

@rem FIXME: we could also use (optional?) standard deviation of each crop.
@rem Or some other measure, such as difference from a blur.

@if "%4"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

echo %0: %1 %2 %3 %4

call %PICTBAT%setInOut %1 nl


if "%ncEXT%"=="" set ncEXT=%EXT%

if "%ncDEBUG%"=="1" set DEBUG_FILE=%BASENAME%_nc_dbg%ncEXT%

if "%ncDEBUG_STROKE%"=="" set ncDEBUG_STROKE=-stroke #f00 -fill None

if "%ncCROPDIR%"=="" set ncCROPDIR=\temp

if "%ncBASE%"=="" set ncBASE=%ncCROPDIR%\%~n1_%3_%4_crop

if "%ncAtBASE%"=="" set ncAtBASE=%~n1_%3_%4_crop

if "%ncREPAGE%"=="0" (
  set sREPAGE=
) else (
  set sREPAGE=+repage
)

set CROP_FILE=%ncAtBASE%_crops.scr
del %CROP_FILE% 2>nul

set RECT_FILE=%ncBASE%_rects.txt
del %RECT_FILE% 2>nul

set LIST_FILE=%ncBASE%s.lis
del %LIST_FILE% 2>nul

echo %0: text files: %CROP_FILE% %RECT_FILE% %LIST_FILE%

set nCrop=0
set sRECT=

set /A W_2=%3/2
set /A H_2=%4/2

for /F "tokens=1,2 delims=, " %%X in (%2) do (
  set /A dx=%%X-%W_2%
  set /A dy=%%Y-%H_2%
  set IMG_FILE=%ncAtBASE%_!nCrop!%ncEXT%
  echo ^( +clone -crop %3x%4+!dx!+!dy! %sREPAGE% -write !IMG_FILE! +delete ^) >> %CROP_FILE%
  echo !IMG_FILE! >>%LIST_FILE%
  set /A nCrop+=1

  if "%ncDEBUG%"=="1" (
    set /A endX=!dx!+%3-1
    set /A endY=!dy!+%4-1
    set sRECT=!sRECT! rectangle !dx!,!dy!,!endX!,!endY!
    echo rectangle !dx!,!dy!,!endX!,!endY! >>%RECT_FILE%
    rem FIXME: also annotate the rectangles
  )
)

echo -exit >>%CROP_FILE%

if %nCrop%==0 (
  %0: coordinates text file [%2] seems to be empty.
  exit /B 1
)

%IMG7%magick ^
  %INFILE% ^
  -script %CROP_FILE%

if ERRORLEVEL 1 (
  echo %0: script CROP_FILE [%CROP_FILE%] failed
  exit /B 1
)

if "%ncDEBUG%"=="1" (
  %IMG7%magick ^
    %INFILE% ^
    %ncDEBUG_STROKE% ^
    -draw @%RECT_FILE% ^
    %DEBUG_FILE%

  if ERRORLEVEL 1 exit /B 1
)

@call echoRestore

@endlocal & set ncLIST_FILE=%LIST_FILE%& set ncDEBUG_FILE=%DEBUG_FILE%
