rem Given image %1 is black mainland around white "sea",
rem finds floating-point coordinates (ncX,ncY) of point on the mainland
rem    nearest to given coordinate (%2,%3).
rem If there are no white pixels, exits with ncCST_CX=-1.
rem If the given coord is in the white "sea", the nearest point will be on the black "coast".
@rem
@rem   %2, %3 coords of central point  [default center of %1]
@rem        Each coord may be suffixed with "%", for percentage of width or height.
@rem        Default: centre of image.
@rem   %4 proportion towards coast point (0 to 100) [100]
@rem   %5 output debugging image [default NULL:, no output]
@rem
@rem Also uses:
@rem   ncPREFIX prefix for temporary files
@rem   ncSEA_COL sea color: black, white or auto. Default white.
@rem     auto will choose automatically, which takes time.
@rem   nc?? automatic choice for negate
@rem   nc?? if 1, assumes %1 is already supersampled and unrolled.
@rem
@rem Returns data about nearest point:
@rem   ncCST_X  x coord of found point, or -1 if none.
@rem   ncCST_Y  y coord of found point.
@rem
@rem Updated:
@rem   24-August-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 nc2


set CENT_X=%2
set CENT_Y=%3

if "%CENT_X%"=="." set CENT_X=
if "%CENT_Y%"=="." set CENT_Y=

if "%CENT_X%"=="" set CENT_X=50%%
if "%CENT_Y%"=="" set CENT_Y=50%%

set PROP=%4
if "%PROP%"=="." set PROP=
if "%PROP%"=="" set PROP=100

set OUTFILE=%5
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=NULL:

if "%ncSEA_COL%"=="" set ncSEA_COL=white

if "%ncSEA_COL%"=="white" (
  set sNEG=-negate
) else if "%ncSEA_COL%"=="black" (
  set sNEG=
) else if "%ncSEA_COL%"=="auto" (
  echo %0: Error: 'auto' not available.
  exit /B 1
) else (
  echo %0: bad ncSEA_COL "%ncSEA_COL%"
  exit /B 1
)

set X=
set CX=

for /F "usebackq tokens=1-3 delims=,= " %%W in (`%IM7DEV%magick ^
  %INFILE% ^
  %sNEG% ^
  -virtual-pixel Black ^
  -threshold 50%% ^
  -process 'nearestwhite cx %CENT_X% cy %CENT_Y% v' ^
  NULL: 2^>^&1`) do (
  set MAGIC=%%W
  if "!MAGIC!"=="nearestwhite.cx" (
    set CX=%%X
  ) else if "!MAGIC!"=="nearestwhite.cy" (
    set CY=%%X
  ) else  if "!MAGIC!"=="nearestwhite:" (
    set X=%%X
    set Y=%%Y
  )
)

if "%X%"=="" exit /B 1
if "%CX%"=="" exit /B 1

if "%X%"=="none" (
  set X=-1
  goto end
)

echo %0: CX=%CX% CY=%CY% X=%X% Y=%Y%

if /I not "%OUTFILE%"=="NULL:" (

  if "%sNEG%"=="" (
    set DBG_COL=yellow
  ) else (
    set DBG_COL=red
  )

  %IMG7%magick ^
    %INFILE% ^
    -fill None -stroke !DBG_COL! ^
    -draw "line   %CX%,%CY% %X%,%Y%" ^
    -draw "circle %CX%,%CY% %X%,%Y%" ^
    -alpha off ^
    %OUTFILE%
)

:end

call echoRestore

@endlocal & set ncOUTFILE=%OUTFILE%& set ncCST_CX=%CX%& set ncCST_CY=%CY%& set ncCST_X=%X%& set ncCST_Y=%Y%
