rem %1 is an image for which a frame will be made,
rem %2 to %5 are input left, top, right and bottom elements for the frame.
rem %6 is format for outputs.
rem    Must contain XX, which will be replaced by l, t, r, b etc.
rem %7 is tiling method, one of:
rem    exact    fractional elements are used to exactly fit the image
rem    floor    may crop the image [default]
rem    round    may crop the image, and may create gaps
rem    ceiling  may create gaps
rem %8 is resizing method, one of:
rem    none     no resizing
rem    image    enlarges image exactly (ignoring aspect ratio)
rem    aspect   enlarges image, keeping aspect ratio
rem    frame    shrinks frame to fit image
rem %9 shadow OxS+X+Y eg 100x4+5+5 default no shadow
@rem
@rem The left, right and bottom pieces will be rotated.

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave

call %PICTBAT%setInOut %1 pf

set LFILE=%2
set TFILE=%3
set RFILE=%4
set BFILE=%5

if "%TFILE%"=="." set TFILE=
if "%RFILE%"=="." set RFILE=
if "%BFILE%"=="." set BFILE=

if "%TFILE%"=="" set TFILE=%LFILE%
if "%RFILE%"=="" set RFILE=%LFILE%
if "%BFILE%"=="" set BFILE=%TFILE%

set OUTFILE=%6
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=pf_XX.png

set OUT_L=%OUTFILE:XX=l%
set OUT_T=%OUTFILE:XX=t%
set OUT_R=%OUTFILE:XX=r%
set OUT_B=%OUTFILE:XX=b%
set OUT_FRM=%OUTFILE:XX=frm%
set OUT_FRMD=%OUTFILE:XX=frmd%
set IMG_RES=%OUTFILE:XX=imgr%

if %OUT_L%==%OUT_T% (
  echo %0: Outfile must include XX.
  exit /B 1
)

set MTHD=%7
if "%MTHD%"=="." set MTHD=
if "%MTHD%"=="" set MTHD=floor

set RESMTHD=%8
if "%RESMTHD%"=="." set RESMTHD=
if "%RESMTHD%"=="" set RESMTHD=aspect

set SHAD=%9
if "%SHAD%"=="" set SHAD=

if "%SHAD%"=="" (
  set sSHAD=
) else (
  set sSHAD=^( +clone -background Black -shadow %SHAD% ^) +swap -background None -layers Merge -layers Flatten
)



set FUNC=floor
if /I %MTHD%==exact (
  set EXTRA=0
  set FUNC=
) else if /I %MTHD%==floor (
  set EXTRA=0
) else if /I %MTHD%==round (
  set EXTRA=0.5
) else if /I %MTHD%==ceiling (
  set EXTRA=0.99999
) else (
  echo %0: Bad MTHD [%MTHD%]
  exit /B 1
)

for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  -format "ImgW=%%w\nImgH=%%h\n" ^
  info:`) do set %%L

:: In following, assume left and right will be rotated.
::
call :getdims %LFILE% LH LW
if ERRORLEVEL 1 exit /B 1

call :getdims %TFILE% TW TH
if ERRORLEVEL 1 exit /B 1

call :getdims %RFILE% RH RW
if ERRORLEVEL 1 exit /B 1

call :getdims %BFILE% BW BH
if ERRORLEVEL 1 exit /B 1

set sFMT=^
nT=%%[fx:%FUNC%((%ImgW%+%LW%+%RW%)/%TW%+(%Extra%))]\n^
nB=%%[fx:%FUNC%((%ImgW%+%LW%+%RW%)/%BW%+(%Extra%))]\n^
nL=%%[fx:%FUNC%((%ImgH%+%TH%+%BH%)/%LH%+(%Extra%))]\n^
nR=%%[fx:%FUNC%((%ImgH%+%TH%+%BH%)/%RH%+(%Extra%))]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%sFMT%" ^
  xc:`) do set %%L

echo %0: n:%nL% %nT% %nR% %nB%

set sFMT=^
FrmdW=%%[fx:floor(%TW%*%nT%+0.5)]\n^
FrmdH=%%[fx:floor(%LH%*%nL%+0.5)]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%sFMT%" ^
  xc:`) do set %%L

set /A WindW=%FrmdW%-%LW%-%RW%
set /A WindH=%FrmdH%-%TH%-%BH%

echo %0: WindW=%WindW% WindH=%WindH%

set sRESIMG=
set sRESFRM=
set ImgRW=%ImgW%
set ImgRH=%ImgH%
set FrmdRW=%FrmdW%
set FrmdRH=%FrmdH%
if /I %RESMTHD%==none (
  rem No resizing.
  echo No res
) else if /I %RESMTHD%==image (
  set sRESIMG=-resize "%WindW%x%WindH%^^^!"
) else if /I %RESMTHD%==aspect (
  set sRESIMG=-resize "%WindW%x%WindH%^^"
) else if /I %RESMTHD%==frame (

  set sFMT=^
FrmdRW=%%[fx:floor^(%FrmdW%*%ImgW%/%WindW%^)]\n^
FrmdRH=%%[fx:floor^(%FrmdH%*%ImgH%/%WindH%^)]\n

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "!sFMT!" ^
    xc:`) do set %%L

  echo %0: FrmdRW=!FrmdRW! FrmdRH=!FrmdRH!

  set sRESFRM=-resize "!FrmdRW!x!FrmdRH!^^^!"
) else (
  echo %0: Bad RESMTHD [%RESMTHD%]
  exit /B 1
)

echo %0: sRESIMG %sRESIMG%
echo %0: sRESFRM %sRESFRM%

%IMG7%magick ^
  -size %FrmdW%x%TH% xc:None ^
  -tile %TFILE% -draw "color 0,0 reset" ^
  -write %OUT_T% ^
  +delete ^
  %LFILE% -rotate -90 -write mpr:TILE +delete ^
  -size %LW%x%FrmdH% xc:None ^
  -tile mpr:TILE -draw "color 0,0 reset" ^
  -write %OUT_L% ^
  +delete ^
  %RFILE% -rotate 90 -write mpr:TILE +delete ^
  -size %RW%x%FrmdH% xc:None ^
  -tile mpr:TILE -draw "color 0,0 reset" ^
  -write %OUT_R% ^
  +delete ^
  %BFILE% -rotate 180 -write mpr:TILE +delete ^
  -size %FrmdW%x%BH% xc:None ^
  -tile mpr:TILE -draw "color 0,0 reset" ^
  %OUT_B%

call %PICTBAT%mitre4 ^
  %OUT_L% %OUT_T% %OUT_R% %OUT_B% ^
  %OUT_FRM%

set needResize=0

if %WindW% GTR %ImgW% set needResize=1
if %WindH% GTR %ImgH% set needResize=1

set INFILE2=%INFILE%
if %needResize%==1 (
  echo %0: need resize.

  if not "%sRESIMG%" EQU "" (
    for /F "usebackq" %%L in (`%IMG7%magick ^
      %INFILE% ^
      %sRESIMG% ^
      +write %IMG_RES% ^
      -format "ImgRW=%%w\nImgRH=%%h\n" ^
      info:`) do set %%L
   set INFILE2=%IMG_RES%
  )

  if not "%sRESFRM%" EQU "" (
    for /F "usebackq" %%L in (`%IMG7%magick ^
      %OUT_FRM% ^
      %sRESFRM% ^
      +write %OUT_FRM% ^
      -format "WindW=%ImgW%\nWindH=%ImgH%\nLW=%%[fx:%LW%*w/%FrmdW%]\nTH=%%[fx:%TH%*h/%FrmdH%]\nFrmdW=%%w\nFrmdH=%%h\n" ^
      info:`) do set %%L
  )
)

%IMG7%magick ^
  %OUT_FRM% ^
  %sSHAD% ^
  %INFILE2% ^
  -geometry +%%[fx:%LW%+(%WindW%-%ImgRW%)/2]+%%[fx:%TH%+(%WindH%-%ImgRH%)/2] ^
  -compose DstOver -composite ^
  %OUT_FRMD%


call echoRestore

@endlocal & set pf_ImgW=%ImgW%& set pf_ImgH=%ImgH%& set pf_FrmdW=%FrmdW%& set pf_FrmdH=%FrmdH%& set pf_WindW=%WindW%& set pf_WindH=%WindH%

@exit /B 0


::------------------------------------------
:: Subroutines

:getdims

set %2=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%2=%%w\n%3=%%H\n" ^
  %1`) do set %%L

if "!%2!"=="" exit /B 1

exit /B 0
