rem rem Given %1 is wildcard file (eg mydir\*.png),
rem writes filename and 21 (or 14) perceptual hash values
rem (first column only)
rem to CSV files for various colorspaces.

set IMAGES=%1

if "%IMAGES%"=="" (
  echo %0: No images.
  exit /B 1
)


call %PICTBAT%setPhEnv

if "%PH_CSVDIR%"=="" exit /B 1

set SCORE_PAIRS=scorePairsNPD.exe


rem --------------------------------------------------------------
rem
rem Build the CSV files (idPh21_*.csv) for single colorspaces.

rem for %%C in (%COLSPACES%) do (
rem   call %PICTBAT%idPh21imgs %IMAGES% %PH_CSVDIR%\idPh21_%%C.csv %%C
rem 
rem   if ERRORLEVEL 1 exit /B 1
rem )

set LIST1=ph1colsp.lis

del %LIST1%

for %%C in (%COLSPACES%) do (
  %SCORE_PAIRS% %PH_CSVDIR%\idPh21_%%C.csv | cgrep /p0 /i- /snErr /f"\r,%%C\n" /O%LIST1%

  rem %SCORE_PAIRS% %PH_CSVDIR%\idPh21_%%C.csv | cgrep /p0 /i- /n[ /s"[NonLo[**[" /f"%%C \s\r\n" /O%LIST2%
)

chStrs /i%LIST1% /s%PICTBAT%\ph1colsp.dat
chStrs /i%LIST1% /f\q
csort  /i%LIST1% /o%LIST1% /k0-2

type %LIST1%

rem --------------------------------------------------------------
rem
rem Build the CSV files (idPh42_*_*.csv) for two colorspaces.

set LIST2=ph1colsp2.lis
del %LIST2%

rem Note: cJoin here makes an output from two inputs, appending the numeric columns.
rem (It also sorts the inputs.)

for %%C in (%COLSPACES%) do (
  set INF0=%PH_CSVDIR%\idPh21_%%C.csv
  for %%D in (%COLSPACES%) do (
    if /I %%C LSS %%D (
      set INF1=%PH_CSVDIR%\idPh21_%%D.csv
      set OUTCSV=%PH_CSVDIR%\idPh42_%%C_%%D.csv
      cJoin /p0 /i!INF0!,!INF1! /k0 /K0 /n /o!OUTCSV!

      %SCORE_PAIRS% !OUTCSV! | cgrep /p0 /i- /snErr /f"\r,%%C,%%D\n" /O%LIST2%

    )
  )
)

chStrs /i%LIST2% /s%PICTBAT%\ph1colsp.dat
chStrs /i%LIST2% /f\q
cSort  /i%LIST2% /o%LIST2% /k0-3



rem --------------------------------------------------------------
rem
rem Build the CSV files (idPh63_*_*_*.csv) for three colorspaces.

set LIST3=ph1colsp3.lis
del %LIST3%

set TMPCSV=ph1_tmp.csv

echo off
for %%C in (%COLSP_FOR3%) do (
  set INF0=%PH_CSVDIR%\idPh21_%%C.csv
  for %%D in (%COLSP_FOR3%) do (
    if /I %%C LSS %%D (
      set INF1=%PH_CSVDIR%\idPh21_%%D.csv
      for %%E in (%COLSP_FOR3%) do (
        if /I %%D LSS %%E (
          set INF2=%PH_CSVDIR%\idPh21_%%E.csv
          echo %%C %%D %%E

          set OUTCSV=%PH_CSVDIR%\idPh63_%%C_%%D_%%E.csv
          cJoin /p0 /i!INF0!,!INF1! /k0 /K0 /n /o%TMPCSV%
          cJoin /p0 /i%TMPCSV%,!INF2! /k0 /K0 /n /o!OUTCSV!

          %SCORE_PAIRS% !OUTCSV! | cgrep /p0 /i- /snErr /f"\r,%%C,%%D,%%E\n" /O%LIST3%
        )
      )
    )
  )
)
echo on

del %TMPCSV%

chStrs /i%LIST3% /s%PICTBAT%\ph1colsp.dat
chStrs /i%LIST3% /f\q
cSort  /i%LIST3% /o%LIST3% /k0-4

rem --------------------------------------------------------------
rem
rem Build the CSV files (idPh84_*_*_*_*.csv) for four colorspaces.

set LIST4=ph1colsp4.lis
del %LIST4%

set TMPCSV=ph1_tmp.csv

echo off
for %%C in (%COLSP_FOR3%) do (
  set INF0=%PH_CSVDIR%\idPh21_%%C.csv
  for %%D in (%COLSP_FOR3%) do (
    if /I %%C LSS %%D (
      set INF1=%PH_CSVDIR%\idPh21_%%D.csv
      for %%E in (%COLSP_FOR3%) do (
        if /I %%D LSS %%E (
          set INF2=%PH_CSVDIR%\idPh21_%%E.csv

          for %%F in (%COLSP_FOR3%) do (
            if /I %%E LSS %%F (
              set INF3=%PH_CSVDIR%\idPh21_%%F.csv
              echo %%C %%D %%E %%F

              set OUTCSV=%PH_CSVDIR%\idPh84_%%C_%%D_%%E_%%F.csv
              cJoin /p0 /i!INF0!,!INF1! /k0 /K0 /n /o%OUTCSV%
              cJoin /p0 /i%OUTCSV%,!INF2! /k0 /K0 /n /o!TMPCSV!
              cJoin /p0 /i%TMPCSV%,!INF3! /k0 /K0 /n /o!OUTCSV!

              %SCORE_PAIRS% !OUTCSV! | cgrep /p0 /i- /snErr /f"\r,%%C,%%D,%%E,%%F\n" /O%LIST4%
            )
          )
        )
      )
    )
  )
)
echo on

del %TMPCSV%

chStrs /i%LIST4% /s%PICTBAT%\ph1colsp.dat
chStrs /i%LIST4% /f\q
cSort  /i%LIST4% /o%LIST4% /k0-5



rem --------------------------------------------------------------
rem
rem Build the CSV files (idPh_multi.csv) for many colorspaces.

set LISTMULTI=ph1colspm.lis
del %LISTMULTI%

set TMPCSV=ph1_tmp.csv

set OUTCSV=%PH_CSVDIR%\idPh_multi.csv

copy /Y %PH_CSVDIR%\idPh21_OHTA.csv %OUTCSV%

for %%C in (%COLSP_FOR3%) do (
  if not %%C==OHTA (
    set INF0=%PH_CSVDIR%\idPh21_%%C.csv
    cJoin /p0 /i%OUTCSV%,!INF0! /k0 /K0 /n /o%TMPCSV%
    copy /Y %TMPCSV% %OUTCSV%
  )
)

%SCORE_PAIRS% !OUTCSV! | cgrep /p0 /i- /snErr /f"\r,%COLSP_FOR3%\n" /O%LISTMULTI%


del %TMPCSV%

chStrs /i%LISTMULTI% /s%PICTBAT%\ph1colsp.dat
chStrs /i%LISTMULTI% /f\q
cSort  /i%LISTMULTI% /o%LISTMULTI% /k0-2

exit /B 0
