rem From images %1 and %2,
rem creates a displacement map, name %3 [mandatory].
@rem
@rem Also uses:
@rem   pmSEED eg "-seed 1234". If not set, PART phase will give different results each time.
@rem   pmVERIFY set to v or whatever pixmatch flags are desired.
@rem   pmMIN_DIM_INITIAL Don't make coarsest level have smaller dimension than this.
@rem   pmDIM_FACT size factor between levels.
@rem   pmMPI_FINAL mpi setting for final PART phase.
@rem   pmMPI_FACT multiplier for mpi seting for previous phases.
@rem
@rem This creates a script, and runs it with the IM v6 "@" mechanism.
@rem It could be modified for v7 by moving the output file from the script to the command.
@rem
@rem Updated:
@rem   22-August-2022 Upgraded for IM v7.
@rem


@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 pm


set REF=%INFILE%
set SRC=%2

if not "%3"=="" set OUTFILE=%3

if "%pmLSR%"=="" set pmLSR=1
if "%pmST%"=="" set pmST=0.01

rem Currently, the following must be integers.
if "%pmMIN_DIM_INITIAL%"=="" set pmMIN_DIM_INITIAL=50
if "%pmDIM_FACT%"=="" set pmDIM_FACT=2
if "%pmMPI_FINAL%"=="" set pmMPI_FINAL=10
if "%pmMPI_FACT%"=="" set pmMPI_FACT=3


set SCR_FILE=%BASENAME%_%sioCODE%.scr

set REF_WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "REF_WW=%%w\nREF_HH=%%h\n" ^
  %REF%`) do set %%L
if "%REF_WW%"=="" exit /B 1

set SRC_WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "SRC_WW=%%w\nSRC_HH=%%h\n" ^
  %SRC%`) do set %%L
if "%SRC_WW%"=="" exit /B 1

set MIN_WW=%REF_WW%
set MIN_HH=%REF_HH%
if %MIN_WW% GTR %SRC_WW% set MIN_WW=%SRC_WW%
if %MIN_HH% GTR %SRC_HH% set MIN_HH=%SRC_HH%

set MIN_DIM=%MIN_WW%
if %MIN_DIM% GTR %MIN_HH% set MIN_DIM=%MIN_HH%

echo MIN_WW=%MIN_WW% MIN_HH=%MIN_HH% MIN_DIM=%MIN_DIM%

set REF_WW_0=%REF_WW%
set REF_HH_0=%REF_HH%
set SRC_WW_0=%SRC_WW%
set SRC_HH_0=%SRC_HH%
set CNT=0
set Cm1=0
set nMPI_0=%pmMPI_FINAL%

:loop

  set /A CNT+=1
  set /A REF_WW_%CNT%=!REF_WW_%Cm1%!/%pmDIM_FACT%
  set /A REF_HH_%CNT%=!REF_HH_%Cm1%!/%pmDIM_FACT%
  set /A SRC_WW_%CNT%=!SRC_WW_%Cm1%!/%pmDIM_FACT%
  set /A SRC_HH_%CNT%=!SRC_HH_%Cm1%!/%pmDIM_FACT%
  set /A nMPI_%CNT%=!nMPI_%Cm1%!*%pmMPI_FACT%
  set /A Cm1+=1
  set /A MIN_DIM/=%pmDIM_FACT%

if %MIN_DIM% GEQ %pmMIN_DIM_INITIAL% goto loop

set /A Cm1=%CNT%-1

echo CNT=%CNT% Cm1=%Cm1%

(
  echo ^( %REF% +write mpr:REF ^)
  echo ^( %SRC% +write mpr:SRC ^)
  echo -filter cubic %pmSEED%

  for /L %%I in (%Cm1%,-1,0) do (
    set N=2
    if %%I==%Cm1% set N=1

    echo ^( -filter cubic

    if %%I==0 (
      echo ^( mpr:REF ^)
      echo ^( mpr:SRC ^)
    ) else (
      echo ^( mpr:REF -resize !REF_WW_%%I!x!REF_HH_%%I!^^! ^)
      echo ^( mpr:SRC -resize !SRC_WW_%%I!x!SRC_HH_%%I!^^! ^)
    )

    set /A Ip1=%%I+1

    if %%I LSS %Cm1% echo ^( mpr:MAP!Ip1! -resize !SRC_WW_%%I!x!SRC_HH_%%I!^^! ^)

    if %%I==%Cm1% (
      set pmOPT=search none part off sweep on %pmVERIFY%
    ) else if %%I==0 (
      set pmOPT=lsr %pmLSR% st %pmST% search entire part on mpi !nMPI_%%I! %pmVERIFY%
    ) else (
      set pmOPT=lsr %pmLSR% st %pmST% search none part on mpi !nMPI_%%I! %pmVERIFY%
    )

    echo -process 'pixmatch !pmOPT!'
    echo +write mpr:MAP%%I
    echo ^)

    if %%I LSS %Cm1% echo -delete 2
  )

  echo -delete 0-1
  echo %OUTFILE%
) >%SCR_FILE%

set SRC_
set REF_
set nMPI_

echo SCR_FILE=%SCR_FILE%
type %SCR_FILE%

%IM7DEV%magick -filter cubic %pmSEED% -script %SCR_FILE%

call echoRestore

endlocal & set pmOUTFILE=%OUTFILE%
