rem Transforms from plane to cylinder.
rem %1 is image.
rem %2 is tilt angle, degrees. Default=0=no tilt. Positive is with camera above cylinder, looking down.
@rem
@rem Also uses:
@rem   pcStartAng. Default -90.
@rem   pcEndAng. Default 90.
@rem   pcDebug. Default 0.
@rem   pcBackCol. Default None.
@rem
@rem Updated:
@rem   5-August-2022 for IM v7.
@rem


@setlocal enabledelayedexpansion

@call echoOffSave

@call %PICTBAT%setInOut %1 p2c


set tiltAng=%2
if "%tiltAng%"=="" set tiltAng=0

call %PICTBAT%planeCylSetup
if ERRORLEVEL 1 exit /B 1

if %pcForceRecalc%==1 (
  del %fP2C_HORIZ%
  del %fP2C_VERT%
  del %fP2C_COMB%
)

if "%pcDebug%"=="1" echo farZAng=%farZAng%

rem plWW = width
rem plWW_2 = half width
rem cylWWxX = w*2/pi = required width of cylinder, assuming we go all the way from -90 to +90.
FOR /F "usebackq" %%L ^
IN (`%CALC% "plWW=%%w\nplWW_2=%%[fx:int(w/2+0.5)]\nplHH=%%h\ncylWWxX=%%[fx:int(w*2/pi+0.5)]" %INFILE%`) ^
DO set %%L

echo %0: plWW=%plWW%

if %ZeroAng%==1 (
  set cylRad=0
  set cylWW=%plWW%
) else (
  FOR /F "usebackq" %%L ^
IN (`%CALC% "cylRad=%%[fx:%plWW%*180/((%pcEndAng%)-(%pcStartAng%))/pi]" xc:`) ^
DO set %%L

echo %0: cylRad=%cylRad%

  FOR /F "usebackq" %%L ^
IN (`%CALC% "cylWW=%%[fx:!cylRad!*(sin(-(%pcStartAng%*pi/180))+sin(%pcEndAng%*pi/180))]\nfarZy=%%[fx:!cylRad!*abs(sin(%farZAng%*pi/180))*%sinT%]" xc:`) ^
DO set %%L
)

echo %0: cylRad=%cylRad%  cylWWxX=%cylWWxX%  cylWW=%cylWW%  farZy=%farZy%

rem cylHH = plHH * cosT
rem tiltFact = -sinT/2
FOR /F "usebackq" %%L ^
IN (`%CALC% "cylWW_2=%%[fx:int(%cylWW%/2+0.5)]\ncylHH=%%[fx:int(%plHH%*%cosT%+0.5)]\ntiltFact=%%[fx:-%sinT%/2]" xc:`) ^
DO set %%L

@rem extendV = plHH.cosT + plWW.2/pi.abs(sinT)/2
@rem FOR /F "usebackq" %%L ^
@rem IN (`%CALC% "extendV=%%[fx:int(%plHH%*%cosT%+%plWW%*2/pi*abs(%sinT%)/2+0.5)]" xc:`) ^
@rem DO set %%L

rem extendV = plHH.cosT + cylRad.abs(sinT)/2  But not /2??
FOR /F "usebackq" %%L ^
IN (`%CALC% "extraV=%%[fx:int(%cylWW%/2*abs(%sinT%)+0.5)]" xc:`) ^
DO set %%L

FOR /F "usebackq" %%L ^
IN (`%CALC% "extendV=%%[fx:int(%plHH%*%cosT%+%cylWW%/2*abs(%sinT%)+0.5)]\nIsSouth=%%[fx:%tiltAng%<0?1:0]" xc:`) ^
DO set %%L

if "%pcDebug%"=="1" echo extendV=%extendV%

if %IsSouth%==1 (
  set GRAV=South
) else (
  set GRAV=North
)

if %tiltAng%==0 (
  set EXTEND=
) else (
  set EXTEND=-gravity %GRAV% -background %pcBackCol% -extent %cylWW%x%extendV%
)

if "%pcDebug%"=="1" echo %0: plWW=%plWW% plHH=%plHH% cylWW=%cylWW% cylHH=%cylHH% plWW_2=%WW_2% cylWW_2=%cylWW_2% extendV=%extendV% tiltFact=%tiltFact%

@rem %IMG7%magick ^
@rem   -size 1x%cylWW% gradient:gray(%pc180end%%%)-gray(%pc180start%%%) -rotate 90 ^
@rem   ( +clone -function arcsin 1,0.5,1,0.5 ) ^
@rem   -evaluate Divide 2 ^
@rem   -compose ModulusSubtract -composite ^
@rem   -evaluate AddModulus 50%% ^
@rem   cp_asin.png


if exist %fP2C_COMB% goto skipComb

call %PICTBAT%p2cHoriz.bat


%IMG7%magick ^
  -size 1x%cylWW% ^
  gradient:gray(%pc180end%%%)-gray(%pc180start%%%) -rotate 90 ^
  -function Polynomial -4,4,0 -evaluate Pow 0.5 ^
  -function Polynomial %tiltFact%,0.5 ^
  %fP2C_VERT%

rem Combine the displacement maps
%IMG7%magick ^
  %fP2C_HORIZ% ^
  %fP2C_VERT% ^
  -size %cylWW%x1 xc:gray50 ^
  -combine -set colorspace sRGB ^
  %fP2C_COMB%

if "%pcDebug%"=="1" (
  %IMG7%magick %fP2C_HORIZ% cp_asin.png
  %IMG7%magick %fP2C_VERT% cp_v.png
  %IMG7%magick %fP2C_COMB% cp_map_p2c.png
  call %PICTBAT%graph1d cp_asin.png
  call %PICTBAT%graph1d cp_v.png
  call %PICTBAT%graphLineCol cp_map_p2c.png Black 0 0
)

:skipComb

if %pcBackCol%==None (
  set VIRT_PIX=-virtual-pixel None
) else (
  set VIRT_PIX=-background %pcBackCol% -virtual-pixel background
)

:: In v6, this had "-interpolate bicubic" before "-compose".
%IMG7%magick ^
  %INFILE% ^
  -resize "%cylWW%x%cylHH%^!" ^
  %EXTEND% ^
  ( %fP2C_COMB% -scale "%cylWW%x%extendV%^!" ) ^
  %VIRT_PIX% ^
  -compose displace -set option:compose:args %cylWW_2%x%cylWW_2% -composite ^
  %OUTFILE%

call echoRestore

endlocal & set p2cExtraV=%extraV%& set pcOUTFILE=%OUTFILE%