rem Seamless boundary by Poisson pasting.
rem %1 is input background (without holes)
rem %2 is input foreground, same size (relevant only when no guidance)
rem %3 is input mask, same size
rem   mask is white where we want adjusted foreground, or black for background
rem %4 is output.
rem %5 0 or 1: whether to negate the mask.
rem %6 optional guidance image
rem %7 radius of final unsharp. 0=no unsharp. Default 0.
@rem
@rem Also uses:
@rem   ppTGT_SCORE target RMSE or PAE score. [default 0.001]
@rem   ppNUM_STEPS number of iteration steps between tests. [default 100]
@rem   ppPOST processing to apply to transition area at end, eg "-unsharp 0x0.5+1+0".
@rem
@rem   rfRLX_FUNC is a relaxation function. [default: average of 4 neighbours]
@rem   rfCOMP_METRIC default RMSE. PAE is more sensitive.
@rem
@rem Updated:
@rem   24-July-2022 for IM v7.
@rem

@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 pp

set IN_A=%INFILE%
set IN_B=%2
set MASK=%3

if not "%4"=="" if not "%4"=="." set OUTFILE=%4

set NEG_MASK=%5
if "%NEG_MASK%"=="." set NEG_MASK=
if "%NEG_MASK%"=="" set NEG_MASK=0

if not %NEG_MASK%==0 if not %NEG_MASK%==1 exit /B 1

if %NEG_MASK%==0 (
  set NEG1=-negate
  set NEG2=
) else (
  set NEG1=
  set NEG2=-negate
)

set GUID=%6
if "%GUID%"=="" set GUID=.
set HAS_GUID=1
if "%GUID%"=="." set HAS_GUID=0

set UNSHP=%7
if "%UNSHP%"=="." set UNSHP=
if "%UNSHP%"=="" set UNSHP=0

if "%ppPOST%"=="" (
  set sUNSHP=
) else (
  set sUNSHP=^^^( +clone %ppPOST% ^^^) %MASK% -compose Over -composite
)

if "%ppTGT_SCORE%"=="" set ppTGT_SCORE=0.001
if "%ppNUM_STEPS%"=="" set ppNUM_STEPS=100

set TMPDIR=\temp\
set TMP_APPRX=%TMPDIR%pp_tmp_apprx.miff
set TMP1=%TMPDIR%pp_tmp1.miff
set TMP2=%TMPDIR%pp_tmp2.miff

:: Provide a first approximation: the background over the foreground.
:: However, this seems to make performance worse.
:: %IMG7%magick ^
::   %IN_B% ^
::   %IN_A% ^
::   -compose Over -composite ^
::   %TMP_APPRX%
set TMP_APPRX=.

if %HAS_GUID%==1 (
  %IMG7%magick ^
    %IN_A% -colorspace sRGB ^
    ^( %MASK% -alpha off %NEG1% ^) ^
    -alpha off -compose CopyOpacity -composite ^
    +depth ^
    %TMP1%

  if ERRORLEVEL 1 exit /B 1
) else (
  %IMG7%magick ^
    -define compose:clamp=off ^
    %IN_A% -colorspace sRGB ^
    %IN_B% ^
    -compose Mathematics ^
      -define "compose:args=0,-0.5,0.5,0.5" -composite ^
    ^( %MASK% -alpha off %NEG1% ^) ^
    -alpha off -compose CopyOpacity -composite ^
    +depth ^
    %TMP1%

  if ERRORLEVEL 1 exit /B 1
)

call %PICTBAT%relaxFillMS ^
  %TMP1% %TMP_APPRX% %TMP2% ^
  %ppTGT_SCORE% %ppNUM_STEPS% ^
  . %GUID%

if ERRORLEVEL 1 exit /B 1

if %HAS_GUID%==1 (
  %IMG7%magick ^
    %TMP2% ^
    %sUNSHP% ^
    +depth ^
    %OUTFILE%

  if ERRORLEVEL 1 exit /B 1
) else (
  %IMG7%magick ^
    -define compose:clamp=off ^
    %IN_B% ^
    %TMP2% ^
    -compose Mathematics ^
      -define "compose:args=0,2,1,-1" -composite ^
    +depth ^
    ^( %MASK% -alpha off %NEG2% ^) ^
    -alpha off ^
    -compose CopyOpacity -composite ^
    %IN_A% ^
    +swap ^
    -compose Over -composite ^
    %sUNSHP% ^
    +depth ^
    %OUTFILE%

  if ERRORLEVEL 1 exit /B 1
)

call echoRestore

@endlocal & set ppOUTFILE=%OUTFILE%& set rfmsITER=%rfmsITER%
