rem Seamless boundary by Poisson pasting, with inner Dirichlet boundary.
rem %1 is input background (without holes)
rem %2 is input foreground, same size
rem %3 is input mask, same size
rem   mask is white where we want adjusted foreground, or black for background
rem %4 inner mask, white where we want exact foreground
rem %5 is output.
rem %6 0 or 1: whether to negate the mask.
rem %7 optional guidance image
@rem
@rem Also uses:
@rem   ppTGT_SCORE target RMSE or PAE score. [default 0.001]
@rem   ppNUM_STEPS number of iteration steps between tests. [default 100]
@rem
@rem   rfRLX_FUNC is a relaxation function. [default average of 4 neighbours]
@rem   rfCOMP_METRIC default RMSE. PAE is more sensitive.
@rem
@rem Updated:
@rem   24-July-2022 for IM v7.
@rem

@if "%4"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 ppi

set IN_A=%INFILE%
set IN_B=%2
set MASK=%3
set MASK_IDB=%4

if "%MASK_IDB%"=="." set MASK_IDB=

if not "%5"=="" if not "%5"=="." set OUTFILE=%5

set NEG_MASK=%6
if "%NEG_MASK%"=="." set NEG_MASK=
if "%NEG_MASK%"=="" set NEG_MASK=0

set GUID=%7
if "%GUID%"=="" set GUID=.
set HAS_GUID=1
if "%GUID%"=="." set HAS_GUID=0

if %NEG_MASK%==0 (
  set NEG1=-negate
  set NEG2=
) else (
  set NEG1=
  set NEG2=-negate
)

if "%MASK_IDB%"=="" (
  call %PICTBAT%poissonPaste ^
    %IN_A% %IN_B% %MASK%
    %OUTFILE% %NEG_MASK% %GUID%

  goto finished
)

if "%ppTGT_SCORE%"=="" set ppTGT_SCORE=0.001
if "%ppNUM_STEPS%"=="" set ppNUM_STEPS=100

set TMPDIR=\temp\
set TMP1=%TMPDIR%ppi_tmp1.miff
set TMP2=%TMPDIR%ppi_tmp2.miff

set TMP_MASK=%TMPDIR%ppi_msk.png

%IMG7%magick ^
  %MASK% ^
  ( %MASK_IDB% -alpha off -negate ) ^
  -alpha off ^
  -compose Darken -composite ^
  %TMP_MASK%

if %HAS_GUID%==1 (
  %IMG7%magick ^
    -define compose:clamp=off ^
    -compose CopyOpacity ^
    ^( %IN_A% -colorspace sRGB ^
       ^( %MASK%     -alpha off %NEG1% ^) ^
       -alpha off -composite ^
    ^) ^
    ^( %IN_B% ^
       ^( %MASK_IDB% -alpha off %NEG2% ^) ^
       -alpha off -composite ^
    ^) ^
    -compose Over -composite ^
    %TMP1%

  if ERRORLEVEL 1 exit /B 1
) else (
  %IMG7%magick ^
    -define compose:clamp=off ^
    %IN_A% -colorspace sRGB ^
    %IN_B% ^
    -compose Mathematics ^
      -define "compose:args=0,-0.5,0.5,0.5" -composite ^
    ^( +clone -fill gray^(50%%^) -colorize 100 ^) ^
    ^( %MASK_IDB% ^) ^
    -compose Over -composite ^
    +depth ^
    ^( %TMP_MASK% -alpha off %NEG1% ^) ^
    -alpha off -compose CopyOpacity -composite ^
    %TMP1%

  if ERRORLEVEL 1 exit /B 1
)

call %PICTBAT%relaxFillMS ^
  %TMP1% . %TMP2% ^
  %ppTGT_SCORE% %ppNUM_STEPS% ^
  . %GUID%

if ERRORLEVEL 1 exit /B 1

if %HAS_GUID%==1 (
  %IMG7%magick ^
    %TMP2% ^
    %OUTFILE%

  if ERRORLEVEL 1 exit /B 1
) else (
  %IMG7%magick ^
    %IN_B% ^
    %TMP2% ^
    -compose Mathematics ^
      -define "compose:args=0,2,1,-1" -composite ^
    +depth ^
    ^( %MASK% -alpha off %NEG2% ^) ^
    -alpha off ^
    -compose CopyOpacity -composite ^
    %IN_A% ^
    -colorspace sRGB ^
    +swap ^
    -compose Over -composite ^
    +depth ^
    %OUTFILE%

  if ERRORLEVEL 1 exit /B 1
)

:finished
call echoRestore

@endlocal & set ppiOUTFILE=%OUTFILE%
	