rem From %1, image of skeletonized white lines on black background, fully opaque,
rem makes output %2 version with short stubs removed.
rem %3 threshold for stub removal, integer number of pixels, or "all".
rem   Stubs larger than this will not be removed.
@rem
@rem Updated:
@rem   15-Feb-2017 added limit so exclusion string doesn't break line length limit.
@rem   6-August-2022 for IM v7
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 ps

echo %0: %1 %2 %3

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set MAX_SIZE=%3
if "%MAX_SIZE%"=="." set MAX_SIZE=
if "%MAX_SIZE%"=="" set MAX_SIZE=10

set DO_ALL=0
if /I "%MAX_SIZE%" EQU "all" set DO_ALL=1

set EXT=.miff
set ENDS=%BASENAME%_ends_ps%EXT%
set SEGS=%BASENAME%_segs_ps%EXT%
set ENDSEGS=%BASENAME%_endsegs_ps%EXT%
set SEGSLIST=%BASENAME%_graysegs_ps.lis
set ENDSLIST=%BASENAME%_ends_ps.lis
set FLOODLIST=%BASENAME%_flood_ps.lis

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: -format "QNUM=%%q" info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "CANVAS=%%P%%O" %INFILE%`) do set %%L

%IMG7%magick ^
  %INFILE% ^
-fill White +opaque Black ^
  +repage ^
  ( -clone 0 ^
    -morphology HMT LineEnds ^
    -alpha off ^
    +write %ENDS% ^
    +delete ^
  ) ^
  -define morphology:compose=Darken ^
  -morphology Thinning "LineJunctions;LineJunctions:3>;LineJunctions:5" -clamp ^
  -alpha off ^
  %SEGS%

if ERRORLEVEL 1 exit /B 1

set /A N_DILATE=2*%MAX_SIZE%

set nPruned=0

if %DO_ALL%==1 (

  (
    for /F "usebackq tokens=1-2 delims=, " %%X in (`%IMG7%magick ^
      %ENDS% ^
      -transparent Black ^
      sparse-color:- ^| sed -e 's/ /\n/g' `) do @(
      @echo color %%X,%%Y floodfill
      @set /A nPruned+=1
    )
  ) >%FLOODLIST%

  type %FLOODLIST%

rem       -morphology thicken "3>:-,0,-,1,-,1,0,1,0" 
rem       -morphology thicken "3>:-,0,0,1,-,0,0,1,-" 

  %IMG7%magick ^
    %INFILE% ^
    -alpha off ^
    ^( +clone ^
       -define "morphology:compose=Darken" ^
       -morphology Thinning "LineJunctions;LineJunctions:3>;LineJunctions:5" -clamp ^
      -alpha off ^
       ^( +clone ^
          -fill Black -draw @%FLOODLIST% ^
       ^) ^
       -compose Difference -composite ^
       -alpha off ^
       -negate ^
    ^) ^
    -compose Darken -composite ^
    ^( +clone ^
       -morphology Hit-and-Miss "3x3:0,0,0,0,1,0,0,0,0" ^
       -alpha off ^
       -negate ^
    ^) ^
    -compose Darken -composite ^
    -compose Over ^
    -alpha off ^
    %OUTFILE%

  goto end
)

%IMG7%magick ^
  %SEGS% -negate +write mpr:MASK +delete ^
  %ENDS% ^
  -write-mask mpr:MASK ^
  -morphology Dilate:%N_DILATE% Square ^
  +write-mask ^
  -alpha off ^
  +write %ENDSEGS% ^
  -define "connected-components:verbose=true" ^
  -connected-components 8 ^
  NULL: >%SEGSLIST%

if ERRORLEVEL 1 exit /B 1

:: Building the string: 1000 is okay; 2200 breaks line length.

set HAS_ANY=0
set CH_COLS=
set CH_EXCL=
set nFnd=0
set nExcl=0
for /F "skip=1 tokens=1-5 delims=: " %%A in (%SEGSLIST%) do (
  set HAS_ANY=1
  set /A nFnd+=1

  set ID=%%A
  set BND_BOX=%%B
  set CENTROID=%%C
  set AREA=%%D
  set COL=%%E

  if !AREA! GTR %MAX_SIZE% if !nExcl! LSS 1000 (
    set CH_EXCL=!CH_EXCL!,!ID!
    set /A nExcl+=1
  )
)

echo %0: SEGSLIST=%SEGSLIST% nFnd=%nFnd% nExcl=%nExcl%

if %HAS_ANY%==0 (
  echo %0: ** No components?? **
  exit /B 1
)

:: remove leading comma
set CH_EXCL=%CH_EXCL:~1%
rem echo CH_EXCL=%CH_EXCL%

:: FIXME? not necessarily an error.
if "%CH_EXCL%"=="" exit /B 1

%IMG7%magick ^
  %ENDSEGS% ^
  -define "connected-components:remove=%CH_EXCL%" ^
  -define "connected-components:mean-color=true" ^
  -connected-components 8 ^
  -background Black -layers Flatten ^
  -fill White +opaque Black ^
  -alpha off ^
  -negate ^
  %INFILE% ^
  +repage ^
  -compose Darken -composite ^
  ( +clone ^
    -morphology Hit-and-Miss "3x3:0,0,0,0,1,0,0,0,0" ^
    -alpha off ^
    -negate ^
  ) ^
  -compose Darken -composite ^
  -compose Over ^
  -repage %CANVAS% ^
  -alpha off ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

set /A nPruned=%nFnd%-%nExcl%


:end

echo %0 end: outfile=%OUTFILE%

call echoRestore

endlocal & set psOUTFILE=%OUTFILE%& set psNPruned=%nPruned%
