rem Speeds of writing outputs with quality 00 to 99.
rem Also measures size of output files, in bytes.
rem For TIFF and MIFF, tests just 00, 01, 10, 11, 20, 21 etc.
rem For JPG format, also measures difference from input image.
rem Writes output in HTML table format.

@rem %1 is input image file.
@rem %2 is prefix for output data files.
@rem %3 is output sufix, eg png, tiff, jpg.
@rem %4 is number of times to execute command [default 1].
@rem %5 is output summary list.

@rem Updated:
@rem   26-September-2016  Bug in comparison fixed.


set INFILE=%1

set OUT_PREFIX=%2
if "%OUT_PREFIX%"=="" set OUT_PREFIX=q9s

set EXT=%3
if "%EXT%"=="" set EXT=tiff

set N_TIMES=%4
if "%N_TIMES%"=="" set N_TIMES=1

set SUM_LIST=
if not "%5"=="" set SUM_LIST=%5

set DO_DIFF=0
if /i %EXT%==jpg set DO_DIFF=1
if /i %EXT%==jpeg set DO_DIFF=1

set OUTFILE=%OUT_PREFIX%_q9s.%EXT%
set TIME_FILE=%OUT_PREFIX%_q9s.lis
del %TIME_FILE% 2>nul
set TMP_LST=%OUT_PREFIX%_q9s_c.csv
del %TMP_LST% 2>nul

set COMP=
if /i %EXT%==miff set COMP=-compress Zip
if /i %EXT%==tiff set COMP=-compress Zip
if /i %EXT%==tif set COMP=-compress Zip

if "%COMP%"=="" (
  set MAX_J=9
) else (
  set MAX_J=1
)

set BASE_CMD=%IMG7%magick %INFILE% -quiet %COMP%

for /F "usebackq" %%B in (`timecc ^
  /n%N_TIMES% /f"\t" ^
  /f"\t" /c"%BASE_CMD% -quality 0 NULL:"`) do set BASE_TIME=%%B

echo BASE_TIME=%BASE_TIME%

set DIFF=
del %OUT_PREFIX%.csv 2>nul

if %MAX_J%==9 (
  cEcho /O%OUT_PREFIX%.csv /f\r\n /s"-quality,0,1,2,3,4,5,6,7,8,9"
) else (
  cEcho /O%OUT_PREFIX%.csv /f\r\n /s"-quality,0,1..9"
)

for /L %%i in (0,1,9) do (
  cEcho /O%OUT_PREFIX%.csv /f\r /s%%i0,
  for /L %%j in (0,1,%MAX_J%) do (

    for /F "usebackq" %%T in (`timecc ^
      /n%N_TIMES% /f"\t" ^
      /o- /b%BASE_TIME% ^
      /c"%BASE_CMD% -quality %%i%%j %OUTFILE%"`) do set TIME=%%T

    for /F "usebackq" %%B in (`stat --printf=%%s %OUTFILE%`) do set BYTES=%%B

    for /F "usebackq" %%C in (`%IMG7%magick identify ^
      -format "%%[fx:int(!BYTES!*100/w/h/6*100+0.5)/100]" ^
      %OUTFILE%`) do set COMPR=%%C

    set nDIFF=0
    set sDIFF=

    if %DO_DIFF%==1 (
      for /F "usebackq tokens=2 delims=() " %%D in (`%IMG7%magick compare ^
        -quiet -metric RMSE %INFILE% %OUTFILE% NULL: 2^>^&1`) do set nDIFF=%%D

      set sDIFF=\^(br /\^)!nDIFF!
    )

    echo TIME=!TIME! BYTES=!BYTES! COMPR=!COMPR! nDIFF=!nDIFF!

    echo !TIME! >>%TIME_FILE%

    if %%j==%MAX_J% (
      cEcho /O%OUT_PREFIX%.csv /X /f\r /s"!TIME!s\^(br /\^)!COMPR!%%!sDIFF!\n"
    ) else (
      cEcho /O%OUT_PREFIX%.csv /X /f\r /s"!TIME!s\^(br /\^)!COMPR!%%!sDIFF!,"
    )

    cEcho /O%TMP_LST% /s"%EXT%,%COMP% -quality %%i%%j,!TIME!,!COMPR!,!nDIFF!"
  )
)

echo BASE_TIME=%BASE_TIME%

call csv2tab %OUT_PREFIX%

del %OUTFILE% 2>nul

type %TIME_FILE%

for /F "usebackq" %%T in (`cSort ^
  /p0 /k0 /i%TIME_FILE% /o- ^| cHead /p0 /i- /h1 /o-`) do set MIN_TIME=%%T

echo MIN_TIME=%MIN_TIME%

chStrs /i%OUT_PREFIX%.htm /o%OUT_PREFIX%.htm /f\(td\)%MIN_TIME% /t"\(td class=\"good\"\)%MIN_TIME%"

cSort /i%TMP_LST% /o- /k2 | cHead /i- /h1 /O%SUM_LIST%
