@rem Given image %1
@rem and same-size mask %2, white shape on black background,
@rem distorts image into white shape with polar displacement.
@rem
@rem Optional arguments:
@rem   %3 output filename [*_r2sp.*]
@rem   %4, %5 coords of central point  [default center of %1]
@rem        Each coord may be suffixed with "%", for percentage of width or height.
@rem        Default: centre of image.
@rem   %6 reverse transformation (shape -> rectangle)
@rem        f=forwards, r=reverse [f]
@rem   %7 proportion towards mask displacement (0 to 100) [100]
@rem   %8 mask-down (make transparent where mask is black)
@rem        0=no, 1=yes, [0]
@rem
@rem Also uses:
@rem   r2spSUP_SAMP Factor for supersampling [default 1].
@rem   r2spBLUR_MAP Operation to blur map, eg "-blur 0x5" [No blur]
@rem   r2spPREFIX prefix for temporary files
@rem   [[ r2spDISP_MAP filename for output absolute displacement map [no file] ]]
@rem
@rem Updated:
@rem   22-August-2022 Upgraded for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 r2sp

set MASK_FILE=%2

if "%r2spDISP_MAP%"=="" set r2spDISP_MAP=%sioCODE%_mnld.png

if "%r2spIM%"=="" set r2spIM=%IMG7%

set OUTF=%3
if "%OUTF%"=="." set OUTF=
if "%OUTF%"=="" set OUTF=%OUTFILE%
set OUTFILE=%OUTF%

set PREFIX=%r2spPREFIX%
if "%PREFIX%"=="" set PREFIX=r2sp

set CENT_X=%4
set CENT_Y=%5

if "%CENT_X%"=="." set CENT_X=
if "%CENT_Y%"=="." set CENT_Y=

set BLANK_CENT=0
if "%CENT_X%"=="" if "%CENT_Y%"=="" set BLANK_CENT=1

if "%CENT_X%"=="" set CENT_X=50%%
if "%CENT_Y%"=="" set CENT_Y=50%%

set X_SUFFIX=%CENT_X:~-1%
if "%X_SUFFIX%"=="%%" (
  set nX=%CENT_X:~0,-1%
  set X_MULT=w/100
) else (
  set nX=%CENT_X%
  set X_MULT=1
)

set Y_SUFFIX=%CENT_Y:~-1%
if "%Y_SUFFIX%"=="%%" (
  set nY=%CENT_Y:~0,-1%
  set Y_MULT=h/100
) else (
  set nY=%CENT_Y%
  set Y_MULT=1
)

set FOR_REV=%6
if "%FOR_REV%"=="." set FOR_REV=
if "%FOR_REV%"=="" set FOR_REV=f
if not "%FOR_REV%"=="f" if not "%FOR_REV%"=="r" (
  echo FOR_REV is "%FOR_REV%" but must be one of "f" or "r".
  exit /B 1
)


set PROP=%7
if "%PROP%"=="." set PROP=
if "%PROP%"=="" set PROP=100

set MASK_DN=%8
if "%MASK_DN%"=="." set MASK_DN=
if "%MASK_DN%"=="" set MASK_DN=0

set SUP_SAMP=%r2spSUP_SAMP%
if "%SUP_SAMP%"=="." set SUP_SAMP=
if "%SUP_SAMP%"=="" set SUP_SAMP=1

for /F "usebackq" %%L in (`%r2spIM%magick identify ^
  -format "WW=%%w\nHH=%%h\nW_SS=%%[fx:w*%SUP_SAMP%]\nH_SS=%%[fx:h*%SUP_SAMP%]\nnX=%%[fx:%nX%*%SUP_SAMP%*%X_MULT%]\nnY=%%[fx:%nY%*%SUP_SAMP%*%Y_MULT%]"
  %INFILE%`) do set %%L

echo WW=%WW%  HH=%HH%  W_SS=%W_SS%  H_SS=%H_SS%  nX=%nX%  nY=%nY%

if %BLANK_CENT%==1 (
  set sUNROLL=-distort depolar "-1,0"
  set sROLL=-distort polar "-1,0"
) else (
  set sUNROLL=-distort depolar "-1,0,%nX%,%nY%"
  set sROLL=-distort polar "-1,0,%nX%,%nY%"
)

if %SUP_SAMP%==1 (
  set sSUP_UP=
  set sSUP_DN=
) else (
  for /F "usebackq tokens=*" %%L in (`%r2spIM%magick identify ^
    -format "sSUP_UP=-resize %%[fx:100*%SUP_SAMP%]%%%%"
    xc:`) do set %%L

  set sSUP_UP=-resize "%W_SS%x%H_SS%^^^!"
  set sSUP_DN=-resize "%WW%x%HH%^^^!"
)

echo sSUP_UP=%sSUP_UP%  sSUP_DN=%sSUP_DN%

set GRAD_RED=0,0,Black,%%[fx:w-1],0,White

if %FOR_REV%==f (
  set INV_NUM=1
) else (
  set INV_NUM=0
)

set ID_ABS_MAP=0,0,#000,^
%%[fx:w-1],0,#f00,^
0,%%[fx:h-1],#0f0,^
%%[fx:w-1],%%[fx:h-1],#ff0

:: Vertical Displacement Mask/Background (Inverse)
rem FIXME: we no longer need _vdm or _vdmi
%IM7DEV%magick ^
  %MASK_FILE% ^
  %sSUP_UP% ^
  +depth ^
  +write %PREFIX%_mask_sup.miff ^
  ( +clone ^
    -sparse-color Bilinear "%ID_ABS_MAP%" ^
    +write %PREFIX%_iam.miff ^
    +delete ^
  ) ^
  -background Black ^
  -virtual-pixel Background ^
  ( +clone ^
    %sUNROLL% ^
    +write %PREFIX%_mnld_unr.miff ^
    -rotate -90 ^
    -process 'cumulhisto norm' ^
    ( +clone ^
      -rotate 90 ^
      +write %PREFIX%_vdm.miff ^
      +delete ^
    ) ^
    -process 'invclut' ^
    -rotate 90 ^
    +write %PREFIX%_vdmi.miff ^
    +delete ^
  ) ^
  ( +clone ^
    -fill White -colorize 100 ^
    %sUNROLL% ^
    -rotate -90 ^
    -process 'cumulhisto norm' ^
    ( +clone ^
      -rotate 90 ^
      +write %PREFIX%_vdb.miff ^
      +delete ^
    ) ^
    -process 'invclut' ^
    -rotate 90 ^
    +write %PREFIX%_vdbi.miff ^
    +delete ^
  ) ^
  NULL:

if ERRORLEVEL 1 exit /B 1

%r2spIM%magick ^
  %PREFIX%_iam.miff ^
  ( +clone ^
    %PREFIX%_vdm.miff ^
    -compose CopyGreen -composite ^
    +write %PREFIX%_vdm.miff ^
    +delete ^
  ) ^
  ( +clone ^
    %PREFIX%_vdmi.miff ^
    -compose CopyGreen -composite ^
    +write %PREFIX%_vdmi.miff ^
    +delete ^
  ) ^
  ( +clone ^
    %PREFIX%_vdb.miff ^
    -compose CopyGreen -composite ^
    +write %PREFIX%_vdb.miff ^
    +delete ^
  ) ^
  ( +clone ^
    %PREFIX%_vdbi.miff ^
    -compose CopyGreen -composite ^
    +write %PREFIX%_vdbi.miff ^
    +delete ^
  ) ^
  NULL:

if ERRORLEVEL 1 exit /B 1

set DIST_MAP=-compose Distort -set option:compose:args 100%%x100%% -composite


for /F "usebackq" %%L in (`%r2spIM%magick identify ^
  -format "PROP_OK=%%[fx:int(%H_SS%*%PROP%/100)>0?1:0]"
  xc:`) do set %%L

set PROP_OK=%PROP_OK%

:: Proportional

rem _mo_pr isn't needed?

rem If PROP_OK==0, _mnld_pr.miff and both masks should be white.

if %PROP_OK%==0 (

  %r2spIM%magick ^
    -size %W_SS%x%H_SS% xc:White ^
    +write %PREFIX%_mo_pr.miff ^
    +write %PREFIX%_mnld_pr.miff ^
    +write %PREFIX%_mask_sup.miff ^
    +delete ^
    -size %WW%x%HH% xc:White ^
    %PREFIX%_mask.miff

  %r2spIM%magick ^
    %INFILE% ^
    %OUTFILE%

  rem FIXME: Do we also need identity maps?

  goto finished
)

%r2spIM%magick ^
  %PREFIX%_mnld_unr.miff ^
  %PREFIX%_vdbi.miff ^
  %DIST_MAP% ^
  ^( -clone 0 -fill White -colorize 100 ^) ^
  ^( -clone 0 -resize 100%%x%PROP%%% ^) ^
  -delete 0 ^
  -gravity South -compose Over -composite ^
  +write %PREFIX%_mo_pr.miff ^
  %PREFIX%_vdb.miff ^
  %DIST_MAP% ^
  +write %PREFIX%_mnld_pr.miff ^
  %sROLL% ^
  +write %PREFIX%_mask_sup.miff ^
  %sSUP_DN% ^
  %PREFIX%_mask.miff

if ERRORLEVEL 1 exit /B 1



%IM7DEV%magick ^
  %PREFIX%_mnld_pr.miff ^
  -rotate -90 ^
  -process 'cumulhisto norm' ^
  -rotate 90 ^
  ( -clone 0 ^
    -sparse-color bilinear "%GRAD_RED%" ) ^
  ( -clone 0 ^
    -fill #000 -colorize 100 ) ^
  -swap 0,1 ^
  -combine ^
  %PREFIX%_vdmp.miff

%IM7DEV%magick ^
  %PREFIX%_vdbi.miff ^
  %PREFIX%_vdmp.miff ^
  %DIST_MAP% ^
  %r2spBLUR_MAP% ^
  +write %PREFIX%_map_fp.miff ^
  ( +clone ^
    -rotate -90 ^
    -process 'invclut' ^
    -rotate 90 ^
  ) ^
  -compose CopyGreen -composite ^
  %PREFIX%_map_rp.miff


if %MASK_DN%==0 (
  set sMASK=
) else (
  set sMASK=%PREFIX%_mask_sup.miff -compose CopyOpacity -composite
)

%r2spIM%magick ^
  %INFILE% ^
  %sSUP_UP% ^
  %sUNROLL% ^
  %PREFIX%_map_%FOR_REV%p.miff ^
  %DIST_MAP% ^
  %sROLL% ^
  %sMASK% ^
  %sSUP_DN% ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1


:finished

call echoRestore

@endlocal & set r2shOUTFILE=%OUTFILE%
