rem Radial distortion of image rectangle to shape.
rem
rem %1 input image to be distorted
rem %2 input mask: white shape on black background. Same size as %1.
rem %3 output file
rem %4 centre: CXxCY, or furthest or centroid or boundingbox.
rem      Assumes this is in white of mask, and can "see" entire shape boundary.
rem %5 supersample percentage [default: 100%, so no resampling]
rem %6 virtual-pixel setting [default None]
rem %7 percentage effect. Negative for reverse direction. [default 100]
@rem
@rem Also uses:
@rem   r2srColSpIn
@rem   r2srColSpOut

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 r2sr

set MASK_FILE=%2

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set sCent=%4
if "%sCent%"=="" set sCent=.

call %PICTBAT%shpCent %MASK_FILE% %sCent% r2sr

if ERRORLEVEL 1 (
  echo %0: Error in shpCent. sCent=%sCent%
  exit /B 1
)

set RESAMP_PC=%5
if "%RESAMP_PC%"=="." set RESAMP_PC=
if "%RESAMP_PC%"=="" set RESAMP_PC=100

set VirtPix=%6
if "%VirtPix%"=="." set VirtPix=
if "%VirtPix%"=="" set VirtPix=None

rem FIXME: if VirtPixel is not set explicitly,
rem   perhaps we should use a more intelligent default.

set PC_Effect=%7
if "%PC_Effect%"=="." set PC_Effect=
if "%PC_Effect%"=="" set PC_Effect=100

set firstCh=%PC_Effect:~0,1%

if "%firstCh%"=="-" (
  set nEffect=%PC_Effect:~1%
  set DIVWHICH=DivideSrc
) else (
  set nEffect=%PC_Effect%
  set DIVWHICH=DivideDst
)

:: For "No displacement" we multiply by 1.0. So we blend between White and the divisor.

if "%nEffect%"=="100" (
  set sEffect=
) else (
  set sEffect= ^( +clone -fill White -colorize 100 ^) ^
    +swap -compose Blend -define compose:args=%nEffect% -composite +define compose:args
)

if %RESAMP_PC%==100 (
  set RESAMP_START=
  set RESAMP_END=
  set CX=%r2sr_CX%
  set CY=%r2sr_CY%
) else (
  set RESAMP_START=-resize %RESAMP_PC%%%
  set RESAMP_END=-resize "%%[InW]x%%[InH]^^^!"

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "CX=%%[fx:%r2sr_CX% * %RESAMP_PC% / 100]\nCY=%%[fx:%r2sr_CY% * %RESAMP_PC% / 100]\n"
    xc:`) do set %%L
)

%IMG7%magick ^
  %MASK_FILE% ^
  -set option:InW %%w ^
  -set option:InH %%h ^
  %RESAMP_START% ^
  -set option:WW %%w ^
  -set option:HH %%h ^
  -define compose:clamp=off ^
  -background None -virtual-pixel Background ^
  -distort DePolar -1,0,%CX%,%CY% +repage ^
  +write mpr:MSK +delete ^
  ( mpr:MSK ^
    -fill White -colorize 100 ^
    -background Black -layers Flatten ^
    -scale "%%[WW]x1^!" ^
    +write mpr:FACT ^
    +delete ^
  ) ^
  ( mpr:MSK ^
    -background Black -layers Flatten ^
    -scale "%%[WW]x1^!" ^
    mpr:FACT ^
    -compose %DIVWHICH% -composite ^
    %sEffect% ^
    -scale "%%[WW]x%%[HH]^!" ^
    -size %%[WW]x%%[HH] ^
    gradient:Black-White ^
    -compose Multiply -composite ^
    -colorspace sRGB ^
    -channel R -fx "i/(w-1)" ^
    +channel ^
    +write mpr:MAP ^
    +delete ^
  ) ^
  ( %INFILE% ^
    %r2srColSpIn% ^
    %RESAMP_START% ^
  ) ^
  -virtual-pixel %VirtPix% ^
  -distort DePolar -1,0,%CX%,%CY% +repage ^
  mpr:MAP ^
  -compose Distort -composite ^
  -distort Polar -1,0,%CX%,%CY% +repage ^
  %RESAMP_END% ^
  %r2srColSpOut% ^
  -define quantum:format=floating-point -depth 32 ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

call echoRestore

endlocal & set r2sr2OUTFILE=%OUTFILE%& set r2srCX=%CX%& set r2srCY=%CY%
