rem Rectangular boundary from dark paths.

@rem %1,%2 input images
@rem %3 output image (%2, with cut margins, composited over %1). Can be either input file.
@rem %4 %5 %6 %7 left, top, right and bottom margins
@rem   Margins are numbers optionally suffixed with c or % (percent) or p (proportion).
@rem   Margin of zero means no ragged cut at that margin.
@rem   Default margins: 30%.
@rem
@rem Also uses:
@rem
@rem   rdpMASK_FILE if not blank, writes mask to this file.
@rem   rdpBLR_SIG sigma for blur of the mask. 0=no blur. [1]
@rem
@rem Updated:
@rem   15-July-2022 for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 rdp

set SRC1=%1
set SRC2=%2

if not "%3"=="" set OUTFILE=%3

if "%rdpBLR_SIG%"=="" set rdpBLR_SIG=1

if "%rdpBLR_SIG%"=="0" (
  set sBLUR=
) else (
  set sBLUR=-blur 0x%rdpBLR_SIG%
)

if "%rdpMASK_FILE%"=="" (
  set sWR_MASK=
) else (
  set sWR_MASK=+write %rdpMASK_FILE%
)

set mL=%4
set mT=%5
set mR=%6
set mB=%7

if "%mL%"=="." set mL=
if "%mT%"=="." set mT=
if "%mR%"=="." set mR=
if "%mB%"=="." set mB=

if "%mL%"=="" set mL=30c
if "%mT%"=="" set mT=30c
if "%mR%"=="" set mR=30c
if "%mB%"=="" set mB=30c

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h" ^
  %SRC1%`) do set %%L

call xyCoord %WW% %HH% %mL% %mT%
set mLi=%xycXi%
set mTi=%xycYi%

call xyCoord %WW% %HH% %mR% %mB%
set mRi=%xycXi%
set mBi=%xycYi%

set /A Rx=%WW%-%mRi%
set /A By=%HH%-%mBi%

echo %0: mLi=%mLi% mTi=%mTi% mRi=%mRi% mBi=%mBi%

set HAS_ANY=0

if %mLi%==0 (
  set MK_L=

  set ASS_L=
) else (
  set HAS_ANY=1

  set MK_L=^
    ^( -clone 0-1 ^
    ^( -clone 0 -crop %mLi%x+0+0 +repage ^) ^
    ^( -clone 1 -crop %mLi%x+0+0 +repage ^) ^
    -delete 0-1 ^
    -compose Difference -composite ^
    -process darkestpath ^
    -flop ^
    -morphology dilate:-1 2x1+1+0:1,1 ^
    -flop ^
    +write mpr:Lm ^
    +delete ^
  ^)

  set ASS_L=mpr:Lm
)

if %mTi%==0 (
  set MK_T=

  set ASS_T=
) else (
  set HAS_ANY=1

  set MK_T=^
    ^( -clone 0-1 ^
    ^( -clone 0 -crop x%mTi%+0+0 +repage ^) ^
    ^( -clone 1 -crop x%mTi%+0+0 +repage ^) ^
    -delete 0-1 ^
    -compose Difference -composite ^
    -rotate -90 ^
    -process darkestpath ^
    -flop ^
    -morphology dilate:-1 2x1+1+0:1,1 ^
    -flop ^
    +write mpr:Tm ^
    +delete ^
  ^)

  set ASS_T=^( mpr:Tm -rotate 90 ^)
)

if %mRi%==0 (
  set MK_R=

  set ASS_R=
) else (
  set HAS_ANY=1

  set MK_R=^
    ^( -clone 0-1 ^
    -gravity East ^
    ^( -clone 0 -crop %mRi%x+0+0 +repage ^) ^
    ^( -clone 1 -crop %mRi%x+0+0 +repage ^) ^
    -delete 0-1 ^
    -compose Difference -composite ^
    -process darkestpath ^
    -morphology dilate:-1 2x1+1+0:1,1 ^
    +write mpr:Rm ^
    +delete ^
  ^)

  set ASS_R=^( mpr:Rm -repage +%Rx%+0 ^)
)

if %mBi%==0 (
  set MK_B=

  set ASS_B=
) else (
  set HAS_ANY=1

  set MK_B=^
    ^( -clone 0-1 ^
    -gravity South ^
    ^( -clone 0 -crop x%mBi%+0+0 +repage ^) ^
    ^( -clone 1 -crop x%mBi%+0+0 +repage ^) ^
    -delete 0-1 ^
    -compose Difference -composite ^
    -rotate -90 ^
    -process darkestpath ^
    -morphology dilate:-1 2x1+1+0:1,1 ^
    +write mpr:Bm ^
    +delete ^
  ^)

  set ASS_B=^( mpr:Bm -rotate 90 -repage +0+%By% ^)
)

if %HAS_ANY%==0 goto :do_none

  %IM7DEV%magick ^
    %SRC1% ^
    %SRC2% ^
    -set colorspace sRGB ^
    +repage ^
    ^( -clone 0-1 ^
       -colorspace Gray ^
       %MK_L% ^
       %MK_T% ^
       %MK_R% ^
       %MK_B% ^
-size %WW%x%HH% xc:White ^
       -delete 0-1 ^
       -background White ^
       -compose Multiply ^
       %ASS_L% %ASS_T% %ASS_R% %ASS_B% ^
       -layers mosaic ^
       %sBLUR% ^
       -alpha off ^
       %sWR_MASK% ^
    ^) ^
    -compose Over -composite ^
    %OUTFILE%

goto :end

:do_none
  %IMG7%magick ^
    %SRC2% ^
    %OUTFILE%

:end

if ERRORLEVEL 1 exit /B 1

call echoRestore

endlocal & set rdpOUTFILE=%OUTFILE%
