rem For image %1, from a GoPro, makes rectilinear version.
@rem %2 is whether to crop within virtual pixels. Default = 0 = don't crop
@rem %3 is whether to overlay virtual pixels (and what colour). Default = 0 = no overlay; none=make them transparent, otherwise colour.
@rem %4 is whether to leave at supersampled size. (Supersamples by rgpSUP_SAMP.) Default = 0 = reduce size down.
@rem %5 is output file.
@rem
@rem Updated:
@rem   28-October-2017 Added parameter 5 and more error-checking.
@rem   21-August-2022 Upgraded for IM v7.
@rem



@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

call echoOffSave

call %PICTBAT%setInOut %1 rgp


set MASK_DIR=%PICTBAT%rectGoPro\
set DISP_DIR=%PICTBAT%rectGoPro\
set WORK_EXT=mpc

if not exist %MASK_DIR% md %MASK_DIR%
if not exist %DISP_DIR% md %DISP_DIR%

set DO_CROP=%2
if "%DO_CROP%"=="." set DO_CROP=
if "%DO_CROP%"=="" set DO_CROP=0

set BORD_COL=%3
if "%BORD_COL%"=="." set BORD_COL=
if "%BORD_COL%"=="" set BORD_COL=0

set SUP_SAMP=%4
if "%SUP_SAMP%"=="." set SUP_SAMP=
if "%SUP_SAMP%"=="" set SUP_SAMP=0

if not "%5"=="" if not "%5"=="." set OUTFILE=%5

if "%rgpSUP_SAMP%"=="" set rgpSUP_SAMP=2

set /A SUP_SAMPpc = %rgpSUP_SAMP%*100


FOR /F "usebackq" %%L ^
IN (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nIS_PORT=%%[fx:w<h?1:0]\nSUP_SAMPinvPc=%%[fx:100/%rgpSUP_SAMP%]\nSUP_SAMPinv=%%[fx:1/%rgpSUP_SAMP%]" ^
  %INFILE%`) ^
DO set %%L

if %IS_PORT%==0 (
  set ROT_FOR=
  set ROT_REV=
) else (
  set ROT_FOR=-rotate 90
  set ROT_REV=-rotate -90
  set t=%WW%
  set WW=%HH%
  set HH=!t!
)

rem We need to find good values of RAT for other image sizes. (3 still and 9 video sizes.)

if not %WW%==4000 (
  echo Dimension needs to be 4000x3000 or 3000x4000.
rem  exit /B 1
)

if not %HH%==3000 (
  echo Dimension needs to be 4000x3000 or 3000x4000.
rem  exit /B 1
)

if "%rgpRAT%"=="" (
  set RAT=3.1
) else (
  set RAT=%rgpRAT%
)


set sRAT=%RAT:.=-%

set SUFFIX=%WW%_%HH%_%sRAT%

set /A WWss=%rgpSUP_SAMP%*%WW%
set /A HHss=%rgpSUP_SAMP%*%HH%
set /A Hm1=%HH%-1
set /A Wm1=%WW%-1
set /A Wssm1=%WWss%-1
set /A Hssm1=%HHss%-1


rem Make displacement maps, if we need them and don't have them.

set sBIG_MAP=%DISP_DIR%rgp_disp_map_%SUFFIX%_%rgpSUP_SAMP%.%WORK_EXT%
set sSML_MAP=%DISP_DIR%rgp_disp_map_%SUFFIX%.%WORK_EXT%

set MAKE_BIG_MAP=0
set MAKE_SML_MAP=0

if not exist %sSML_MAP% if %SUP_SAMP%==0 set MAKE_SML_MAP=1

if not exist %sBIG_MAP% (
  if %SUP_SAMP%==1 set MAKE_BIG_MAP=1
  if %MAKE_SML_MAP%==1 set MAKE_BIG_MAP=1
)

if %MAKE_BIG_MAP%==1 %IMG7%magick ^
  ( -size %HHss%x%WWss% gradient: -rotate 90 ) ^
  ( -size %WWss%x%HHss% gradient: -flip ) ^
  -size %WWss%x%HHss% xc:Black ^
  -combine ^
  -set colorspace sRGB ^
  ( +clone ^
    -virtual-pixel Edge ^
    -distort Depolar -1 ^
    ( ^
      -size 1x%HHss% ^
      gradient:black-white ^
      ( +clone ^
        -fx "atan(%RAT%*u)/atan(%RAT%)" ^
      ) ^
      -compose Mathematics -define compose:args=0,0.5,-0.5,0.5 -composite ^
      -scale "%WWss%x%HHss%^!" ^
    ) ^
    -compose Displace ^
      -set option:compose:args 0x%Hssm1% ^
      -composite ^
    -write drx0.jpg ^
    -background Pink -virtual-pixel Background ^
    -distort Polar -1 ^
  ) ^
  -compose Mathematics -define compose:args=0,0.5,-0.5,0.5 -composite ^
  +depth ^
  %sBIG_MAP%

if ERRORLEVEL 1 exit /B 1

if %MAKE_SML_MAP%==1 %IMG7%magick ^
  %sBIG_MAP% ^
  -scale %SUP_SAMPinvPc%%% ^
  +depth ^
  %sSML_MAP%

if ERRORLEVEL 1 exit /B 1


rem Make borders to compose over virtual pixels, if we need them and don't have them.

set sBIG_BORD=%MASK_DIR%rgp_bord_%SUFFIX%_%rgpSUP_SAMP%.%WORK_EXT%
set sSML_BORD=%MASK_DIR%rgp_bord_%SUFFIX%.%WORK_EXT%

set MAKE_BIG_BORD=0
set MAKE_SML_BORD=0

if not exist %sSML_BORD% if %SUP_SAMP%==0 set MAKE_SML_BORD=1

rem FIXME: large border
rem FIXME: polmap_rel.png

if %MAKE_SML_BORD%==1 (
%IMG7%magick ^
  -size %WW%x%HH% ^
  xc:#fff ^
  -virtual-pixel Black ^
  -define distort:scale=%rgpSUP_SAMP% ^
  -distort Depolar -1 ^
  ^( ^
    -size 1x%HHss% ^
    gradient:black-white ^
    ^( +clone ^
      -fx "atan(%RAT%*u)/atan(%RAT%)" ^
    ^) ^
    -compose Mathematics -define compose:args=0,0.5,-0.5,0.5 -composite ^
    -scale "%WWss%x%HHss%^!" ^
  ^) ^
  -compose Displace ^
    -set option:compose:args 0x%Hssm1% ^
    -composite ^
  ^( +clone ^
    -define distort:scale=1 ^
    -distort Polar -1 ^
    -negate -alpha Copy -evaluate Set 0 ^
    -write %sBIG_BORD% ^
    +delete ^
  ^) ^
  -define distort:scale=%SUP_SAMPinv% ^
  -distort Polar -1 ^
  -negate -alpha Copy -evaluate Set 0 ^
  +depth ^
  %sSML_BORD%

if ERRORLEVEL 1 exit /B 1

)

if %SUP_SAMP%==0 (
  set fBORD=%sSML_BORD%
  set CW=%WW%
  set CH=%HH%
) else (
  set fBORD=%sBIG_BORD%
  set CW=%WWss%
  set CH=%HHss%
)

rem Set crop.

if %DO_CROP%==1 (
  for /F "usebackq" %%L ^
in (`%IMG7%magick identify -format "DIAG=%%[fx:sqrt(%CW%*%CW%+%CH%*%CH%)]" xc:`) ^
do set %%L

  for /F "usebackq" %%L ^
in (`%IMG7%magick identify -format "C_WI=%%[fx:int(!DIAG!*tan(%CW%/!DIAG!*atan(%RAT%))/%RAT%)]\nC_HT=%%[fx:int(!DIAG!*tan(%CH%/!DIAG!*atan(%RAT%))/%RAT%)]" xc:`) ^
do set %%L

  set sCROP=-gravity Center -crop !C_WI!x!C_HT!+0+0 +repage
) else (
  set sCROP=
)

rem Set overlay. For non-black, quicker to pre-process.

if /I %BORD_COL%==0 (
  set sBORD=
) else if /I %BORD_COL%==none (
  set sBORD="(" %fBORD% -channel A -negate ")" -compose CopyOpacity -composite
) else if /I %BORD_COL%==black (
  set sBORD=%fBORD% -compose Over -composite
) else (
  set sBORD="(" %fBORD% -alpha off -fill %BORD_COL% -opaque #000 -alpha on ")" -compose Over -composite
)

if %SUP_SAMP%==1 (
  %IMG7%magick ^
    %INFILE% ^
    %ROT_FOR% ^
    -resize %SUP_SAMPpc%%% ^
    %sBIG_MAP% ^
    -virtual-pixel Black ^
    -compose Displace ^
      -set option:compose:args %Wssm1%x%Hssm1% ^
      -composite ^
    %sBORD% ^
    %sCROP% ^
    %ROT_REV% ^
    %COMPR% ^
    +depth ^
    %OUTFILE%
) else (
  %IMG7%magick ^
    %INFILE% ^
    %ROT_FOR% ^
    %sSML_MAP% ^
    -virtual-pixel Black ^
    -compose Displace ^
      -set option:compose:args %Wm1%x%Hm1% ^
      -composite ^
    %sBORD% ^
    %sCROP% ^
    %ROT_REV% ^
    %COMPR% ^
    +depth ^
    %OUTFILE%
)

if ERRORLEVEL 1 exit /B 1

%IMG7%magick identify %OUTFILE%

call echoRestore
