rem Given %1 is image, probably some pixels have transparency,
rem %2 is input opaque image, first approximation to solution,
rem writes relaxed output to %3. (Can be same as %2.)
rem %3 is output file.
rem %4 is target RMSE or PAE score. [default 0.001]
rem %5 is number of iteration steps between tests. [default 100]
rem %6 is optional mask, black where pixels are to be changed.
rem   (eg from -alpha extract)
@rem
@rem Also uses:
@rem   rfRLX_CONV is a relaxation convolution. [default average of 4 neighbours]
@rem   rfCOMP_METRIC default RMSE. PAE is more sensitive.
@rem   IMRF the location of IM to be used. Default: %IM%.
@rem
@rem  An equivalent compiled program wouldn't need to save images
@rem    between iterations, so would be faster.
@rem
@rem
@rem Updated:
@rem   12-July-2022 Upgraded to IM v7. Assumes magick is HDRI.

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

echo %0: %1 %2 %3 %4 %5 %6

@if not "%7"=="" (
  echo %0: Can't have 7 arguments
  exit /B 1
)

rem set IMRF=%IM%
rem if "%IMRF%"=="" set IMRF=%IM%

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 rf

set APPROX=%2
if "%APPROX%"=="." set APPROX=

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set LIM_DIST=%4
if "%LIM_DIST%"=="." set LIM_DIST=
if "%LIM_DIST%"=="" set LIM_DIST=0.001

set NUM_STEPS=%5
if "%NUM_STEPS%"=="." set NUM_STEPS=
if "%NUM_STEPS%"=="" set NUM_STEPS=100

set MASK=%6
if "%MASK%"=="." set MASK=

echo %0: INFILE=%INFILE% APPROX=%APPROX% OUTFILE=%OUTFILE%

set rfTMP_IN=rf_tmpin.miff

%IMG7%magick %INFILE% -set colorspace sRGB %rfTMP_IN%
if ERRORLEVEL 1 exit /B 1

if "%MASK%"=="" (
  set MASK=rf_mask_tmp.miff
  %IMG7%magick %rfTMP_IN% -alpha extract -threshold 50%% !MASK!
)

if "%MASK%"=="" (
  set sMSK1=
  set sMSK2=
) else (
  set sMSK1=-write-mask %MASK%
  set sMSK2=+write-mask
)


set TMP_OUT=rf_tmp_out.miff

if "%rfCOMP_METRIC%"=="" set rfCOMP_METRIC=RMSE

if [%rfRLX_CONV%]==[] set rfRLX_CONV="3x3:0,0.25,0,0.25,0,0.25,0,0.25,0"

:: Other possibilities:
::   set rfRLX_CONV="3x3:0.05,0.2,0.05,0.2,0,0.2,0.05,0.2,0.05"
:: Or use -blur 1x10000"


if "%APPROX%"=="" (
  echo %0: meanFill
  call %PICTBAT%meanFill %INFILE% %TMP_OUT%
)

if not "%APPROX%"=="" %IMG7%magick ^
  %APPROX% %INFILE% -compose Over -composite -strip +depth %TMP_OUT%

if ERRORLEVEL 1 exit /B 1

set sSTEPS=-morphology convolve:%NUM_STEPS% %rfRLX_CONV% mpr:SRC -composite

%IMG7%magick identify %rfTMP_IN%
if ERRORLEVEL 1 exit /B 1
if not "%APPROX%"=="" %IMG7%magick identify %APPROX%
%IMG7%magick identify %MASK%
if ERRORLEVEL 1 exit /B 1
%IMG7%magick identify %TMP_OUT%
if ERRORLEVEL 1 exit /B 1

set DONE=0
set nITER=0
set PREV_DIST=99999999
set DIST=

for /L %%I in (0,1,99) do if !DONE!==0 (
  for /F "usebackq" %%L in (`%IMG7%magick ^
    -precision 14 ^
    +depth ^
    %rfTMP_IN% +write mpr:SRC +delete ^
    +depth ^
    %TMP_OUT% ^
    ^( +clone ^
       -compose Over ^
       %sMSK1% ^
       -morphology convolve:%NUM_STEPS% %rfRLX_CONV% mpr:SRC -composite ^
       %sMSK2% ^
      -depth 32 ^
      -define "quantum:format=floating-point" ^
       +write %TMP_OUT% ^
    ^) ^
    -metric %rfCOMP_METRIC% -format "DIST=%%[distortion]\n" -compare ^
    info:`) do set %%L

  if "!DIST!"=="" (
    echo %0: nITER=!nITER! DIST is blank
    exit /B 1
  )

  if "!DIST!"=="nan" (
    echo %0: nITER=!nITER! DIST is nan
    exit /B 1
  )

  echo %0: %rfCOMP_METRIC% DIST=!DIST!

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "DONE=%%[fx:!DIST!<=%LIM_DIST%||!PREV_DIST!<!DIST!?1:0]"
    xc:`) do set %%L

  set PREV_DIST=!DIST!
  set /A nITER+=1
)

echo %0: DONE=%DONE%

:: Note: RMSE should strictly improve at each pass. PAE might not.

%IMG7%magick %TMP_OUT% -strip %OUTFILE%

if %nITER%==0 (
  echo %0: nITER is zero
  exit /B 1
)

set /A nITER*=%NUM_STEPS%

echo %0: nITER=%nITER% DIST=%DIST%

call echoRestore

@endlocal & set rfOUTFILE=%OUTFILE%& set rfITER=%nITER%& set rfDIST=%DIST%
