rem Given %1 is image with pixels either opaque or transparent.
rem %2 is input opaque image, first approximation to solution,
rem   (used only for first, coarsest, pass)
rem writes relaxed output to %3. (Mandatory, can be same as %2.)
rem %3 is output file.
rem %4 is target RMSE or PAE score. [default 0.001].
rem %5 is number of iteration steps between tests. [default 100]
rem %6 is optional mask, black where pixels are to be changed.
rem   (eg from -alpha extract)
rem %7 optional guidance image
@rem
@rem Also uses:
@rem   rfRLX_FUNC is a relaxation function. [default average of 4 neighbours]
@rem   rfCOMP_METRIC default RMSE. PAE is more sensitive.
@rem
@rem  An equivalent compiled program wouldn't need to save images
@rem    between iterations, so would be faster.
@rem
@rem
@rem Updated:
@rem   25-June-2017 for guidance 0% means no guidance.
@rem   12-July-2022 Upgraded to IM v7. Assumes magick is HDRI.
@rem   29-August-2023 Resizing of guidance image clamps it. Fixed by adding/subtracting 50%.
@rem

@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 rfms

set IMOUTFMT=-depth 32 -define "quantum:format=floating-point"

echo %0: IMOUTFMT = %IMOUTFMT%

set APPROX=%2
if "%APPROX%"=="" set APPROX=.

set IMG_RLX=%2
if "%IMG_RLX%"=="" set IMG_RLX=.

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set TGT_SCORE=%4
if "%TGT_SCORE%"=="." set TGT_SCORE=
if "%TGT_SCORE%"=="" set TGT_SCORE=0.001

set NSTEPS=%5
if "%NSTEPS%"=="" set NSTEPS=.

set MASK=%6
if "%MASK%"=="." set MASK=

set GUIDANCE=%7
if "%GUIDANCE%"=="." set GUIDANCE=

if "%MASK%"=="" (
  set MASK=rfms_mask_tmp.miff
  %IMG7%magick %INFILE% -alpha extract %IMOUTFMT% !MASK!
)

set MASK_SM=rfms_mask_sm_tmp.miff

set GUID_SM=.
if not "%GUIDANCE%"=="" set GUID_SM=rfms_guid_sm_tmp.miff

echo %0: MASK=%MASK% GUIDANCE=%GUIDANCE%

set WW=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h" ^
  %INFILE%`) do set %%L
if "%WW%"=="" exit /B 1

set TMP_IMG_HOLES=rfms_tih.miff
set TMP_IMG_RLXD=rfms_rlxd.miff

if not "%APPROX%"=="." (
  %IMG7%magick %APPROX% -resize "%W%x%H%^!" %IMOUTFMT% %TMP_IMG_RLXD%

  set IMG_RLX=%TMP_IMG_RLXD%
)

set nITER=0

call :frms %WW% %HH%
if ERRORLEVEL 1 (
  echo %0: call to frms failed
  exit /B 1
)

echo %0: Final %IMG7%magick %TMP_IMG_RLXD% %IMOUTFMT% %OUTFILE%
%IMG7%magick %TMP_IMG_RLXD% %IMOUTFMT% %OUTFILE%
if ERRORLEVEL 1 exit /B 1

rem echo %0: nITER=%nITER%

call echoRestore

@endlocal & set rfmsOUTFILE=%OUTFILE%& set rfmsITER=%nITER%

@exit /B 0

::----------------------------------
:: Subroutine (recursive)

:frms
echo %0: %1 %2
setlocal
set W=%1
set H=%2
set /A W_2=%1/2
set /A H_2=%2/2

set minDim=%W_2%
if %minDim% GTR %H_2% set minDim=%H_2%

rem echo W=%W% H=%H%

if %minDim% GTR 50 call :frms %W_2% %H_2%
if ERRORLEVEL 1 exit /B 1

echo %0: W=%W% H=%H%

set sRESIZE=-resize "%W%x%H%^^^!"
if %W%==%WW% if %H%==%HH% set sRESIZE=

echo %0: sRESIZE=%sRESIZE%

%IMG7%magick ^
  %INFILE% ^
  -strip ^
  %sRESIZE% ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  %TMP_IMG_HOLES%

if ERRORLEVEL 1 exit /B 1

%IMG7%magick ^
  %MASK% ^
  %sRESIZE% ^
  -threshold 50%% ^
  %IMOUTFMT% ^
  %MASK_SM%

@rem Note: resizing clamps. Yuck!
@rem Assume guidance range is [-1,+1].
@rem
if not "%GUIDANCE%"=="" %IMG7%magick ^
  %GUIDANCE% ^
+write guid0.miff ^
  -evaluate Divide 2 ^
  -evaluate Add 50%% ^
  %sRESIZE% ^
  -evaluate Subtract 50%% ^
  -evaluate MUltiply 2 ^
  %IMOUTFMT% ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
+write guid.miff ^
  %GUID_SM%

if ERRORLEVEL 1 exit /B 1

rem %IMG7%magick %MASK_SM% rfm_dbg_%W%x%H%.png

if not "%IMG_RLX%"=="." (
:: Scale previous result up.
  %IMG7%magick ^
    %IMG_RLX% ^
    -scale "%W%x%H%^!" ^
    %TMP_IMG_HOLES% ^
    -compose Over -composite ^
    -depth 32 ^
    -define "quantum:format=floating-point" ^
    %IMG_RLX%

  if ERRORLEVEL 1 exit /B 1

  echo %0: Scale previous result up.
  %IMG7%magick identify %IMG_RLX%
)

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "smTGT_SCORE=%%[fx:%TGT_SCORE%*%WW%/%W%*%HH%/%H%]" ^
  %INFILE%`) do set %%L

echo %0: TGT_SCORE=%TGT_SCORE% smTGT_SCORE=%smTGT_SCORE%

if "%GUIDANCE%"=="" (
  call %PICTBAT%relaxFill ^
    %TMP_IMG_HOLES% ^
    %IMG_RLX% ^
    %TMP_IMG_RLXD% ^
    %smTGT_SCORE% ^
    %NSTEPS% ^
    %MASK_SM%
) else (
  call %PICTBAT%relaxFillScr ^
    %TMP_IMG_HOLES% ^
    %IMG_RLX% ^
    %TMP_IMG_RLXD% ^
    %smTGT_SCORE% ^
    %NSTEPS% ^
    %MASK_SM% ^
    %GUID_SM%
)

if ERRORLEVEL 1 exit /B 1

endlocal& set /A nITER=%nITER%+%rfITER%& set rfDIST=%rfDIST%

echo %0: nITER=%nITER% rfDIST=%rfDIST%

set IMG_RLX=%TMP_IMG_RLXD%

exit /B 0
