rem Given %1 is image, probably some pixels have transparency,
rem %2 is input opaque image, first approximation to solution,
rem writes relaxed output to %3. (Mandatory, can be same as %2.)
rem %3 output file.
rem %4 is target RMSE or PAE score. [default 0.001]
rem %5 is number of iteration steps between tests. [default 100]
rem %6 is optional mask, black where pixels are to be changed.
rem   (eg from -alpha extract)
rem %7 optional guidance image
@rem
@rem Also uses:
@rem   rfRLX_FUNC is a relaxation function. [default average of 4 neighbours]
@rem   rfCOMP_METRIC default RMSE. PAE is more sensitive.
@rem
@rem Exactly like relaxFill.bat, but writes a script for IM to run.
@rem
@rem  An equivalent compiled program wouldn't need to save images
@rem    between iterations, so would be faster.
@rem
@rem  Originally, guidance value 50% meant "zero guidance".
@rem  Now: 0% means "zero guidance".
@rem
@rem was:
@rem   set sGUIDE=mpr:GUIDE -compose Mathematics -define compose:args=0,-0.25,1,0.125 -composite -compose Over
@rem changed 0.125 to 0.
@rem
@rem Assumes magick is HDRI.
@rem
@rem Updated:
@rem   22-July-2022 Upgraded for IM v7.
@rem   28-August-2023 For guidance, replace mathematics with multiply by 0.25 before writing to mpr, then use simpler "-compose MinusSrc".
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

echo %0: %1 %2 %3 %4 %5 %6 %7

@call echoOffSave

call %PICTBAT%setInOut %1 rfs

set TMP_SCR=rfs.scr

set APPROX=%2
if "%APPROX%"=="." set APPROX=

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set LIM_DIST=%4
if "%LIM_DIST%"=="." set LIM_DIST=
if "%LIM_DIST%"=="" set LIM_DIST=0.001

set NUM_STEPS=%5
if "%NUM_STEPS%"=="." set NUM_STEPS=
if "%NUM_STEPS%"=="" set NUM_STEPS=100

set MASK=%6
if "%MASK%"=="." set MASK=

set GUIDANCE=%7
if "%GUIDANCE%"=="." set GUIDANCE=

if "%GUIDANCE%"=="" (
  set sSAVE_GUIDE=
  set sGUIDE=
) else (

  rem This was:
  rem   set sSAVE_GUIDE=%GUIDANCE% +write mpr:GUIDE +delete
  rem   set sGUIDE=mpr:GUIDE -compose Mathematics -define compose:args=0,-0.25,1,0 -composite -compose Over

  set sSAVE_GUIDE=%GUIDANCE% -alpha off -evaluate Multiply 0.25 +write mpr:GUIDE +delete
  set sGUIDE=mpr:GUIDE -compose MinusSrc -composite -compose Over -alpha off
)

set rfsTMP_IN=rf_tmpin.miff

%IMG7%magick %INFILE% -set colorspace sRGB %rfsTMP_IN%

if "%MASK%"=="" (
  set MASK=rfs_mask_tmp.miff
  %IMG7%magick %rfsTMP_IN% -alpha extract -alpha off -threshold 50%% !MASK!
)

if "%MASK%"=="" (
  set sMSK1=
  set sMSK2=
) else (
  set sMSK1=-mask %MASK%
  set sMSK2=+mask
)

if "%rfCOMP_METRIC%"=="" set rfCOMP_METRIC=RMSE

if "%rfRLX_CONV%"=="" set rfRLX_CONV="3x3:0,0.25,0,0.25,0,0.25,0,0.25,0"

:: Other possibilities:
::   set rfRLX_CONV="3x3:0.05,0.2,0.05,0.2,0,0.2,0.05,0.2,0.05"
:: Or use -blur 1x10000"


if "%APPROX%"=="" call %PICTBAT%meanFill %rfsTMP_IN% %OUTFILE%

if not "%APPROX%"=="" if not "%APPROX%"=="%OUTFILE%" %IMG7%magick ^
  %APPROX% %rfsTMP_IN% -compose Over -composite +depth %OUTFILE%

%IMG7%magick identify %rfsTMP_IN%
if not "%APPROX%"=="" %IMG7%magick identify %APPROX%
%IMG7%magick identify %OUTFILE%
if not "%MASK%"=="" %IMG7%magick identify %MASK%
if not "%GUIDANCE%"=="" %IMG7%magick identify %GUIDANCE%

:: FIXME: In following, also use mask, after testing.
(
  echo "%rfsTMP_IN%" +write mpr:SRC +delete %sSAVE_GUIDE%
  echo "%OUTFILE%"
  echo +depth
  echo ^( +clone -compose Over -channel RGB

  for /L %%I in (1,1,%NUM_STEPS%) do (
    echo -morphology convolve %rfRLX_CONV% -alpha off %sGUIDE% mpr:SRC -composite
  )

  echo   -depth 32 -define quantum:format^=floating-point
  echo   +write %OUTFILE%
  echo ^)
  echo +channel
  echo -metric %rfCOMP_METRIC% -format "DIST=%%[distortion]\n" -compare -write info:

) >%TMP_SCR%

if "%IM_VSN_6or7%"=="" call %PICTBAT%isIm6or7

set DONE=0
set nITER=0
set PREV_DIST=99999999
set DIST=

if "%IM_VSN_6or7%"=="7" (
  echo -exit >>%TMP_SCR%
)

for /L %%I in (0,1,99) do if !DONE!==0 (

  if "%IM_VSN_6or7%"=="6" (
    for /F "usebackq" %%L in (`%IMG7%magick ^
      -precision 14 -define compose:clamp^=off +depth @%TMP_SCR% ^
      NULL:`) do set %%L
  ) else (
    for /F "usebackq" %%L in (`%IMG7%magick ^
      -precision 14 -define compose:clamp^=off +depth ^
      -script %TMP_SCR%`) do set %%L
  )

  if "!DIST!"=="" exit /B 1

  echo %0: %rfCOMP_METRIC% DIST=!DIST!

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "DONE=%%[fx:!DIST!<=%LIM_DIST%||!PREV_DIST!<!DIST!?1:0]"
    xc:`) do set %%L

  set PREV_DIST=!DIST!
  set /A nITER+=1
)

:: Note: RMSE should strictly improve at each pass. PAE might not.

if %nITER%==0 exit /B 1

set /A nITER*=%NUM_STEPS%

echo %0: nITER=%nITER% DIST=%DIST%

call echoRestore

@endlocal & set rfsOUTFILE=%OUTFILE%& set rfITER=%nITER%& set rfDIST=%DIST%
