#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "vsn_defines.h"


// The next function corresponds in style to functions in transform.c
// It takes one image, and returns an image.
//
static Image *CopyGreen(const Image *image,
  ExceptionInfo *exception)
{
  Image
    *new_image;

  PIX_INFO
    mpp;

  assert(image != (Image *) NULL);
  assert(image->signature == MAGICK_CORE_SIG);
  if (image->debug != MagickFalse)
    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);

  // Make a clone of this image, same size but undefined pixel values:
  //
  new_image=CloneImage(image, image->columns, image->rows, MagickTrue, exception);
  if (new_image == (Image *) NULL)
    return(new_image);

  if (SetNoPalette (new_image, exception) == MagickFalse)
    return (Image *)NULL;

#if IMV6OR7==6
  // Make a "colour", which will be black.
  GetMagickPixelPacket (new_image, &mpp);

  // We could set it to any colour:
  (void)QueryMagickColor ("green", &mpp, exception);

  // Set all the pixels to this colour.
  (void)SetImageColor(new_image, &mpp);
#else
  GetPixelInfo (new_image, &mpp);

  QueryColorCompliance ("green", AllCompliance, &mpp, exception);

  (void)SetImageColor(new_image, &mpp, exception);
#endif

  return (new_image);
}



ModuleExport size_t replacelastImage(Image **images,const int argc,
  const char **argv,ExceptionInfo *exception)
{
  Image
    *image,
    *new_image;

  assert(images != (Image **) NULL);
  assert(*images != (Image *) NULL);
  assert((*images)->signature == MAGICK_CORE_SIG);

  (void) argc;
  (void) argv;

  // Point to the last image in the list
  //
  image=GetLastImageInList(*images);
  if (image == (Image *) NULL) {
    fprintf (stderr, "replacelast: no images in list\n");
    return (-1);
  }

  new_image = CopyGreen(image,exception);
  if (new_image == (Image *) NULL)
    return(-1);

  ReplaceImageInList(&image,new_image);

  // (Replacing the last image will mess up the images pointer,
  //  if this was the only image.)
  *images=GetFirstImageInList(new_image);

  return(MagickImageFilterSignature);
}
