rem Resampling with halo minimization.
rem   %1 is input image.
rem   %2 is resize, eg 50% or 800x800, etc.
@rem
@rem Optional:
@rem   %3 is d=downsample, u = upsample, a = automatic
@rem   %4 is sharpen amount (0=none, 50=default)
@rem   %5 is deconvolve radius (0=no deconvolution, default)
@rem   %6 is deconvolve sharpness (default=50)
@rem   %7 output filename

@rem See http://www.luminous-landscape.com/forum/index.php?topic=91754.msg763245#msg763245
@rem and http://im.snibgo.com/resamphm.htm

@rem Updated:
@rem   27-October-2017 Added "+depth".
@rem   28-June-2001 For v7. Assumes "magick" is HDRI. Added save and restore alpha.


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 rhm


set mSize=%2
if "%mSize%"=="." set mSize=
if "%mSize%"=="" set mSize=100%

set method=%3
if "%method%"=="." set method=
if "%method%"=="" set method=a

if /I "%method%" NEQ "a" if /I "%method%" NEQ "d" if /I "%method%" NEQ "u" (
  echo Method must be A, D or U.
  exit /B 1
)

set shpAmount=%4
if "%shpAmount%"=="." set shpAmount=
if "%shpAmount%"=="" set shpAmount=50

set deconRad=%5
if "%deconRad%"=="." set deconRad=
if "%deconRad%"=="" set deconRad=0

set deconShp=%6
if "%deconShp%"=="." set deconShp=
if "%deconShp%"=="" set deconShp=50

if not "%7"=="" if not "%7"=="." set OUTFILE=%7


::echo mSize=%mSize% method=%method%  shpAmount=%shpAmount%


@rem If working in AdobeRGB1998, replace the next two lines with:
@rem
@rem   set colspIn=-evaluate Pow 2.19921875
@rem   set colspOut=-evaluate Pow 0.4547069271758437
@rem
@rem For images with embedded profiles, use:
@rem
@rem   set colspIn=-write IMprofile.icc -profile sRGB.icc -set colorspace sRGB -colorspace RGB
@rem   set colspOut=-set colorspace RGB -colorspace sRGB -profile IMprofile.icc

set colspIn=-colorspace RGB
set colspOut=-set colorspace RGB -colorspace sRGB


set nGam=3

for /f "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 16 ^
  -format "invGam=%%[fx:1/%nGam%]" ^
  xc:`) do set %%L

set gamIn=-gamma %nGam%
set gamOut=-gamma %invGam%

if /I "%method%"=="a" (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -ping ^
    -format "W_OLD=%%w\nH_OLD=%%h" ^
    %INFILE%`) do set %%L

  for /F "usebackq" %%L in (`%IMG7%magick ^
    -size %W_OLD%x%H_OLD% ^
    xc: ^
    -scale %2 ^
    -format "W_NEW=%%w\nH_NEW=%%h" ^
    info:`) do set %%L

  set method=u
  if !W_NEW! LSS !W_OLD! if !H_NEW! LSS !H_OLD! set method=d

  rem echo From auto, method=!method!
)

if /I "%method%"=="d" (

  if %shpAmount%==0 (
    set REAL_WORK=-filter Quadratic -distort Resize %mSize%
  ) else (

    set PREPROC=-define filter:c=0.1601886205085204 -filter Cubic -distort Resize %mSize%

    set FILT=-define convolve:scale=%shpAmount%%%^^,100 -morphology Convolve DoG:3,0,0.4981063336734057

    set REAL_WORK= ^
      !PREPROC! ^
      ^( -clone 0 %gamIn% !FILT! %gamOut% ^) ^
      ^( -clone 0 !FILT! ^) ^
      -delete 0 ^
      ^( -clone 1 -colorspace gray -auto-level ^) ^
      -compose over -composite
  )
) else (
  for /f "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "deblurValue=%%[fx:1-0.0011450938298236*%shpAmount%]" ^
    xc:`) do set %%L

  set FILT=-define filter:blur=!deblurValue! -filter Lanczos -distort Resize %mSize%

  set REAL_WORK= ^
    ^( -clone 0 %gamIn% !FILT! %gamOut% ^) ^
    ^( -clone 0 !FILT! ^) ^
    -delete 0 ^
    ^( -clone 1 -colorspace gray -auto-level ^) ^
    -compose over -composite

)

rem echo REAL_WORK=%REAL_WORK%

set DECONV=

if not "%deconRad%"=="0" (
  for /f "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "blurSigma=%%[fx:max(0.4,0.3902*%deconRad%)]" ^
    xc:`) do set %%L

  set DECONFILT=-define convolve:scale=%deconShp%%%^^,100 -morphology Convolve DoG:0,0,!blurSigma!

  set DECONV= ^
    ^( -clone 0 %gamIn% !DECONFILT! %gamOut% ^) ^
    ^( -clone 0 !DECONFILT! ^) ^
    -delete 0 ^
    ^( -clone 1 -colorspace gray -auto-level ^) ^
    -compose over -composite
)

set SAVEALPHA=( +clone -alpha extract -resize %mSize% -write mpr:ALP +delete ) -alpha off

set RESTOREALPHA=mpr:ALP -compose CopyOpacity -composite

%IMG7%magick ^
  %INFILE% ^
  %SAVEALPHA% ^
  %colspIn% ^
  %REAL_WORK% ^
  %DECONV% ^
  %colspOut% ^
  %RESTOREALPHA% ^
  +depth ^
  %OUTFILE%

rem  ( -clone 0 %gamIn% %FILT% %gamOut% )
rem  ( -clone 0 %FILT% )


call echoRestore

@endlocal & set rhmOUTFILE=%OUTFILE%
