rem For image %1, find the angle a, -90 <= a <= +90, that minimises the trimmed width.
rem %2 is 0 (default, minimise width) or 1 (minimise height)
rem Assumes pixel at (0,0) is background colour.
@rem
@rem Updated:
@rem   11-August-2022 for IM v7.
@rem   19-October-2022 changed result to average of 4 angles, instead of ANG1.
@rem     Added test: if abs(W0-W3)==1, that is good enough.
@rem     Removed redundant calcs of Tn and Tx.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INFILE=%1
set TEMPFILE=%TEMP%\%~n1_rmw.miff

if "%2"=="1" (
  set TOKENS=tokens=2
) else (
  set TOKENS=
)

set RESZ=-resize 500x500
set FILT=-filter Box
set PREC=-precision 4

%IMG7%magick %INFILE% -background White -layers flatten %RESZ% %TEMPFILE%

for /F "usebackq" %%C ^
in (`%IMG7%magick %TEMPFILE% -format "%%[pixel:p{0,0}]" info:`) ^
do set ONE_PIXEL=%%C

%IMG7%magick %TEMPFILE% -bordercolor %ONE_PIXEL% -border 1 %TEMPFILE%

set ANG0=-90
set ANG3=90

for /F "usebackq" %%L ^
in (`%IMG7%magick ^
  %TEMPFILE% ^
  -background "%ONE_PIXEL%" ^
  ^( +clone %FILT% -rotate %ANG0% -format "T0=%%@\n" -write info: +delete ^) ^
  ^( +clone %FILT% -rotate %ANG3% -format "T3=%%@\n" -write info: +delete ^) ^
  NULL:`) ^
do set %%L

for /F "%TOKENS% delims=x+" %%L in ("%T0%") do set W0=%%L
for /F "%TOKENS% delims=x+" %%L in ("%T3%") do set W3=%%L

set FINISHED=0

:loop

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  %PREC% ^
  -format "ANG1=%%[fx:%ANG0%+(%ANG3%-(%ANG0%))/3]\nANG2=%%[fx:%ANG0%+2*(%ANG3%-(%ANG0%))/3]" ^
  xc:`) ^
do set %%L

for /F "usebackq" %%L ^
in (`%IMG7%magick ^
  %TEMPFILE% ^
  -background "%ONE_PIXEL%" ^
  ^( +clone %FILT% -rotate %ANG1% -format "T1=%%@\n" -write info: +delete ^) ^
  ^( +clone %FILT% -rotate %ANG2% -format "T2=%%@\n" -write info: +delete ^) ^
  NULL:`) ^
do set %%L

for /F "%TOKENS% delims=x+" %%L in ("%T1%") do set W1=%%L
for /F "%TOKENS% delims=x+" %%L in ("%T2%") do set W2=%%L

echo Ang:   %ANG0% %ANG1% %ANG2% %ANG3%
echo Width: %W0% %W1% %W2% %W3%

if %ANG0%==%ANG1% (
  set FINISHED=1
  goto finished
)

if %ANG2%==%ANG3% (
  set FINISHED=1
  goto finished
)

if %W0%==%W1% if %W0%==%W2% if %W0%==%W3% (
  set FINISHED=1
  goto finished
)

set FINISHED=1

if %W1% LEQ %W0% if %W1% LEQ %W2% if %W1% LEQ %W3% (
  set ANG3=%ANG2%
  set W3=%W2%
  set FINISHED=0
)

if %FINISHED%==1 if %W2% LEQ %W0% if %W2% LEQ %W1% if %W2% LEQ %W3% (
  set ANG0=%ANG1%
  set W0=%W1%
  set FINISHED=0
)

if %FINISHED%==1 if %W3% LEQ %W0% if %W3% LEQ %W1% if %W3% LEQ %W2% (
  set ANG0=%ANG2%
  set W0=%W2%
  set FINISHED=0
)

set /A dW=(%W0%-%W3%)*(%W0%-%W3%)
if %dW%==1 (
  rem set FINISHED=1
  rem goto finished
)

if %FINISHED%==0 goto loop

:finished

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  -format "ANG=%%[fx:(%ANG0%+%ANG1%+%ANG2%+%ANG3%)/4]" ^
  xc:`) ^
do set %%L


call echoRestore

endlocal & set rmwANG=%ANG1%
