rem Makes a mask, white around selected coordinates, otherwise black.
rem %1 output name.
rem %2 dimensions, WxH. [default 600x400]
rem %3 quoted list of coordinate quadlets. Use "c" for percent.
rem   Each quadlet is:
rem     x, y    coordinate [each defaults to 50c]
rem     rx, ry  radii of effect [each defaults to 5c]
rem   Each quad becomes a radial gradient.
@rem
@rem Updated:
@rem   15-August-2022 for IM v7.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 do


set OUTFILE=%1
if "%OUTFILE%"=="." set OUTFILE=
if "%OUTFILE%"=="" set OUTFILE=sc_out.png

set DIMS=%~2
if "%DIMS%"=="." set DIMS=
if "%DIMS%"=="" set DIMS=600x400

set COORDLIST=%~3
if "%COORDLIST%"=="." set COORDLIST=
if "%COORDLIST%"=="" set COORDLIST=50c,60c,.,.,11,12,4,.

call parseXxY2 600 400 dim %DIMS%

rem set dim


:: Parse COORDLIST into gx, gy, grx, gry
::   and use these for radial gradients.

call parseCommaList "%COORDLIST%" nCoordList CoordList

rem echo nCoordList=%nCoordList%
rem set CoordList

set /A remCoord=%nCoordList%%%4

if not %remCoord%==0 (
  echo %~0: COORDLIST should have multiple of 4 entries.
  exit /B 1
)

set /A numCoord=%nCoordList%/4

%IMG7%magick ^
  -size %dim_X%x%dim_Y% ^
  xc:Black ^
  %OUTFILE%

for /L %%i in (1,1,%numCoord%) do (
  call :getQuad %%i

  if "!coord.x!"=="." set coord.x=
  if "!coord.x!"=="" set coord.x=50c

  if "!coord.y!"=="." set coord.y=
  if "!coord.y!"=="" set coord.y=50c

  if "!coord.rx!"=="." set coord.rx=
  if "!coord.rx!"=="" set coord.rx=5c

  call parseNum !coord.x! %dim_X% gx
  call parseNum !coord.y! %dim_Y% gy
  call parseNum !coord.rx! %dim_X% grx
  call parseNum !coord.ry! %dim_Y% gry

  %IMG7%magick ^
    -size %dim_X%x%dim_Y% ^
    -define gradient:center=!gx!,!gy! ^
    -define gradient:radii=!grx!,!gry! ^
    radial-gradient: ^
    %OUTFILE% ^
    -compose Lighten -composite ^
    %OUTFILE%
)

rem set coord

call echoRestore

@endlocal scOUTFILE=%OUTFILE%

@exit /B 0

::----------------------------------
:: Subroutines

:getQuad

set /A n0=(%1-1)*4
set /A n1=(%1-1)*4+1
set /A n2=(%1-1)*4+2
set /A n3=(%1-1)*4+3

set coord.x=!CoordList[%n0%]!
set coord.y=!CoordList[%n1%]!
set coord.rx=!CoordList[%n2%]!
set coord.ry=!CoordList[%n3%]!

exit /B 0
