@rem Given image %1
@rem and mask %2, white shape on black background,
@rem distorts image into white shape with horizontal or linear displacement.
@rem
@rem Optional arguments:
@rem   %3 direction of displacement
@rem        h=horizontal, v=vertical [h]
@rem   %4 proportion towards displacement (0 to 100) [100]
@rem   %5 reverse transformation (shape -> rectangle) NYI
@rem        f=forwards, r=reverse [f]
@rem   %6 output filename [*_s2sl.*]
@rem   %7 mask-down (make transparent where mask is black)
@rem        y=yes, n=no [y]
@rem   %8 filename for output relative displacement map [no file]
@rem
@rem Updated:
@rem   27-August-2022 for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 s2sl

set MASK_FILE=%2

set DIRN=%3
if "%DIRN%"=="." set DIRN=
if "%DIRN%"=="" set DIRN=h

set PROP_DIST=%4
if "%PROP_DIST%"=="." set PROP_DIST=
if "%PROP_DIST%"=="" set PROP_DIST=100

set FOR_REV=%5
if "%FOR_REV%"=="." set FOR_REV=
if "%FOR_REV%"=="" set FOR_REV=f

set OUTF=%6
if "%OUTF%"=="." set OUTF=
if "%OUTF%"=="" set OUTF=%OUTFILE%
set OUTFILE=%OUTF%

set MASK_DOWN=%7
if "%MASK_DOWN%"=="." set MASK_DOWN=
if "%MASK_DOWN%"=="" set MASK_DOWN=y

set OUT_MAP=%8
if "%OUT_MAP%"=="." set OUT_MAP=
if "%OUT_MAP%"=="" set OUT_MAP=


for /F "usebackq tokens=1-6 delims=x+-," %%A ^
in (`%IMG7%magick ^
  %MASK_FILE% ^
  -bordercolor Black -border 1 ^
  -format "%%@,%%w,%%h" ^
  info:`) ^
do (
  set /A MTW=%%A
  set /A MTH=%%B
  set /A MTX=%%C-1
  set /A MTY=%%D-1
  set /A MWW=%%E-2
  set /A MHH=%%F-2
)
echo Mask: MTW=%MTW% MTH=%MTH% MTX=%MTX% MTY=%MTY% MWW=%MWW% MHH=%MHH%


set sSHRINK=
if /I "%DIRN%"=="h" if not %MTH%==%MHH% (
  echo Shrink height

  set sSHRINK=-resize "%TW%x%TH%^^^!" ^
  -background Blue ^
  -extent %WW%x%HH%-%TX%-%TY%
)

if /I "%DIRN%"=="v" if not %MTW%==%MWW% (
  echo Shrink width

  set sSHRINK=-resize "%TW%x%TH%^^^!" ^
  -background Blue ^
  -extent %WW%x%HH%-%TX%-%TY%
)


if /I "%DIRN%"=="h" (
  set sSPARSE=-sparse-color Bilinear ^
      "0,0 Black %%[fx:w-1],0 White"
  set ROTR=
  set ROTF=

  set sDISP=-set option:compose:args %MWW%x0
)

if /I "%DIRN%"=="v" (
  set sSPARSE=-sparse-color Bilinear ^
      0,0,#000,0,%%[fx:h-1],#fff
  set ROTR=-rotate -90
  set ROTF=-rotate 90

  set sDISP=-set option:compose:args 0x%Hm1%
)

if "%PROP_DIST%"=="100" (
  set sPROP=
  set sMASK=mpr:MASK
) else (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format="%%[fx:100-%PROP_DIST%]" ^
    xc:`) do set sPROP=-fill gray^(50%%^) -colorize %%L

  set sMASK=^^^( +clone -fill #fff -colorize 100 ^
    -virtual-pixel Black mpr:DISP ^
    -compose Displace ^
    %sDISP% -composite -write s.png ^^^)
)

echo sPROP=%sPROP%
echo sDISP=%sDISP%
echo sMASK=%sMASK%

if /I "%FOR_REV%"=="r" (
  set sINVERSE=-process invclut
) else (
  set sINVERSE=
)

if /I "%MASK_DOWN%"=="y" (
  set sMASK_DOWN=%sMASK% -alpha off -compose CopyOpacity -composite
) else (
  set sMASK_DOWN=
)

%IM7DEV%magick ^
  %INFILE% ^
  %sSHRINK% ^
  %MASK_FILE% ^
  +depth ^
  ( -clone 1 ^
    -write mpr:MASK ^
    %ROTR% ^
    -process 'cumulhisto norm' ^
    %sINVERSE% ^
    %ROTF% ^
    ( +clone ^
      %sSPARSE% ^
    ) ^
    -compose Mathematics ^
      -define compose:args=0,-0.5,0.5,0.5 ^
      -composite ^
    %sPROP% ^
    -write mpr:DISP ^
    +write sh2sh_map.png ^
  ) ^
  -delete 1 ^
  -compose Displace ^
  %sDISP% -composite ^
  %sMASK_DOWN% ^
  %OUTFILE%

call echoRestore
