rem From two images %1,%2 and a mask %3 (white boundary margin on black),
rem all the same size,
rem find the point-to-point path of minimum difference within the boundary margin,
rem and make a mask to put either image over the other.
rem Output to %4.
@rem
@rem (See also minDp.bat)
@rem
@rem
@rem Also uses:
@rem
@rem   sdpBLUR_SIG blur sigma. 0=no blur. [Default 1.]
@rem   sdpMASK_FILE if not blank, writes mask to this file
@rem
@rem Updated:
@rem   24-October-2016 Fixed bug for sdpBLUR_SIG.
@rem   15-July-2022 for IM v7.


@if "%3"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 sdp

set SRC1=%1
set SRC2=%2
set BND_MSK=%3

if not "%4"=="" if not "%4"=="." set OUTFILE=%4

if "%sdpBLUR_SIG%"=="" set sdpBLUR_SIG=1

if "%sdpBLUR_SIG%"=="0" (
  set sBLUR=
) else (
  set sBLUR=-blur 0x%sdpBLUR_SIG%
)

if "%sdpMASK_FILE%"=="" (
  set sWR_MASK=
) else (
  set sWR_MASK=+write %sdpMASK_FILE%
)

:: 30-Oct-2016 was set /A sdpY1=%%C+5

%IM7DEV%magick ^
  %BND_MSK% ^
  -fuzz 0.0001%% ^
  -process onewhite ^
  -format "sdpWW=%%w\nsdpHH=%%h" ^
  info:

set A=
for /F "usebackq tokens=1-3 delims=:=, " %%A in (`%IM7DEV%magick ^
  %BND_MSK% ^
  -fuzz 0.0001%% ^
  -process onewhite ^
  -format "sdpWW=%%w\nsdpHH=%%h" ^
  info: 2^>^&1`) do (
  if "%%A"=="onewhite" (
    set A=%%A
    if "%%B"=="none" (
      echo %0: no onewhite in BND_MSK=%BND_MSK%
      exit /B 1
    )
    set /A sdpX=%%B
    set /A sdpY1=%%C
  ) else (
    set %%A=%%B
  )
)
if not "%A%"=="onewhite" exit /B 1

echo %0: sdpWW=%sdpWW% sdpHH=%sdpHH% sdpX=%sdpX% sdpY1=%sdpY1%

if "%sdpY1%"=="" exit /B 1

set /A ColHt=%sdpHH%-%sdpY1%

for /F "usebackq tokens=3 delims=:=, " %%A in (`%IM7DEV%magick ^
  %BND_MSK% ^
  -crop 1x%ColHt%+%sdpX%+%sdpY1% +repage ^
  -negate ^
  -fuzz 0.0001%% ^
  -process onewhite ^
  NULL: 2^>^&1`) do (
    set sdpY2=%%A
)

set /A sdpY2+=%sdpY1%

:: New 30-Oct-16
set /A sdpY1+=(%sdpY2%-%sdpY1%)/2

echo %0: %sdpWW% %sdpHH% %sdpX% %sdpY1% %sdpY2%

set /A sdpWWm1=%sdpWW%-1

rem set /A sdpX=%sdpWW%/2

set /A sdpXm2=%sdpX%-2
set /A sdpXm1=%sdpX%-1
set /A sdpXp1=%sdpX%+1
set /A sdpY1p1=%sdpY1%+1
set /A sdpY1p2=%sdpY1%+2

echo %0: start_at %sdpXm1%,%sdpY1% end_at %sdpXp1%,%sdpY1%

%IM7DEV%magick ^
  ( %SRC1% +write mpr:SRC1 ) ^
  ( %SRC2% +write mpr:SRC2 ) ^
  +depth ^
  -strip ^
  -compose Difference -composite ^
  %sBLUR% ^
  -auto-level ^
  %BND_MSK% ^
  -alpha off ^
  -compose CopyOpacity -composite ^
  -background White ^
  -compose Over -layers flatten ^
  -alpha off ^
  -clamp ^
  -fuzz 0.0001%% -fill gray(200%%) -opaque White -fuzz 0 ^
  -strokewidth 1 ^
  -stroke gray(200%%) ^
  -draw "line %sdpX%,0 %sdpX%,%sdpY2%" ^
  -fuzz 0.0001%% -fill gray(200%%) -opaque White -fuzz 0 ^
  -process 'darkestpntpnt start_at %sdpXm1%,%sdpY1% end_at %sdpXp1%,%sdpY1% t 1.5 v' ^
  -fill White ^
  -draw "point %sdpX%,%sdpY1%" ^
  -draw "color %sdpX%,%sdpY1p1% floodfill" ^
  %sBLUR% ^
  %sWR_MASK% ^
  mpr:SRC2 ^
  mpr:SRC1 ^
  -swap 0,2 ^
  -alpha off ^
  -compose Over -composite ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

call echoRestore

endlocal & set sdpOUTFILE=%OUTFILE%
