rem Given image %1
rem and %2, same size, a narrow white line on black background across the full width,
rem makes version of %1 with columns shifted so line becomes straight,
rem at same average height as it was.
rem %3 is optional treatment of repeated pixels:
rem   0 leave them in the image (so image remains the same size).
rem   1 trim them away (so image will have reduced height).
rem   2 trim, then resize to original size (changing aspect ratio).
rem %4 is optional output file.
@rem
@rem Also uses:
@rem   scBLR_SIG radius of blur to be applied to the map. Default 0 = no blur.
@rem   scFLOOD_WHITE 0=don't flood; 1=do flood. Default 1.
@rem   scIMEXE if set, uses this instead of %IMG7%magick
@rem
@rem Assumes bottom-left pixel in %2 is in "lower" half.
@rem
@rem  Updated:
@rem    20-August-2022 Upgraded for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 sc

set LINE_FILE=%2

set TREAT_REP=%3
if "%TREAT_REP%"=="." set TREAT_REP=
if "%TREAT_REP%"=="" set TREAT_REP=0

if not "%4"=="" set OUTFILE=%4

if "%scBLR_SIG%"=="" set scBLR_SIG=0
if "%scBLR_SIG%"=="0" (
  set sBLUR=
) else (
  set sBLUR=-blur 0x%scBLR_SIG%
)

echo %0: sBLUR=%sBLUR%

if "%scFLOOD_WHITE%"=="" set scFLOOD_WHITE=1

if "%scIMEXE%"=="" set scIMEXE=%IMG7%magick

set TMP1=\temp\%~n1_sc.miff
set TMP2=\temp\%~n1_sc2.miff

set FORM=WW=%%w\n^
HH=%%h\n^
Wm1=%%[fx:w-1]\n^
Hm1=%%[fx:h-1]\n^
W_2=%%[fx:w/2]\n^
H_2=%%[fx:h/2]

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "%FORM%" ^
  %LINE_FILE%`) do set %%L

if "%scFLOOD_WHITE%"=="1" (
  set sFLOOD=-fill Red ^
-draw "color 0,%Hm1% floodfill" ^
-fill Gray^^^(50%%^^^) -opaque White ^
-fill White -opaque Red
) else (
  set sFLOOD=
)

echo %0: sFLOOD=%sFLOOD%

call %PICTBAT%quantFp %TMP1%

set ADD_REL=
for /F "usebackq" %%L in (`%scIMEXE% ^
  %LINE_FILE% ^
  %sFLOOD% ^
  -format "ADD_REL=%%[fx:100*(0.5-mean)]" ^
  +write info: ^
  -scale "%WW%x1^!" ^
  %sBLUR% ^
  +depth ^
  %QUANT_FP% ^
  "%TMP1%"`) do set %%L

if "%ADD_REL%"=="" (
  echo %0: scIMEXE [%scIMEXE%] failed
  exit /B 1
)

echo %0: ADD_REL=%ADD_REL% 

for /F "usebackq" %%L in (`%scIMEXE% ^
  "%TMP1%" ^
  -evaluate Add %ADD_REL%%% ^
  -negate ^
  -format "MIN_DY=%%[fx:minima]\nMAX_DY=%%[fx:maxima]"
  +write info: ^
  -scale "%WW%x%HH%^!" ^
  %QUANT_FP% ^
  "%TMP2%"`) do set %%L

echo %0: MIN_DY=%MIN_DY% MAX_DY=%MAX_DY%

if %TREAT_REP% GTR 0 (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "TRIM_TOP=%%[fx:int((0.5-%MIN_DY%)*%HH%+0.5)]\nTRIM_BOT=%%[fx:int((%MAX_DY%-0.5)*%HH%+0.5)]" ^
    xc:`) do set %%L

  set /A NEW_HT=%HH%-!TRIM_TOP!-!TRIM_BOT!

  set sCROP=-crop %WW%x!NEW_HT!+0+!TRIM_TOP! +repage
) else (
  set sCROP=
)

if %TREAT_REP% EQU 2 (
  set sRESIZE=-resize "%WW%x%HH%^^^!"
) else (
  set sRESIZE=
)

call %PICTBAT%quantFp %OUTFILE%

%scIMEXE% ^
  %INFILE% ^
  "%TMP2%" ^
  -compose Displace ^
    -set option:compose:args 0x%H_2% ^
    -composite ^
  %sCROP% ^
  %sRESIZE% ^
  %QUANT_FP% ^
  %OUTFILE%


call echoRestore

endlocal & set scOUTFILE=%OUTFILE%
