rem %1 input image, white shape on black background.
rem %2 output, replacing white with gradient from black at the centre to white at edge of shape.
rem %3 centre: "furthest" or "centroid" or "boundingbox" or "XxY" where X and Y are numbers each possibly suffixed by %% or c or p.
rem      default is point at furthest distance from shape edge.
rem %4 post processing, eg "-negate".
@rem
@rem Also uses:
@rem   shp2gProcBW process for the unrolled black and white image, eg -blur 0x3 -auto-level
@rem
@rem Updated:
@rem   4-March-2023 changed Euclidean:4 to Euclidean:7
@rem   8-August-2023 added parameter %3
@rem   13-August-2023: removed "-clamp" before "-distort Polar"
@rem   18-August-2023: added %4.
@rem     Changed default from "white at the centre to black at edge of shape" to "black at the centre to white at edge of shape".
@rem     If old default is wanted, use "-negate" in %4
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 shp2g

if not "%2"=="" set OUTFILE=%2

set sCent=%3
if "%sCent%"=="." set sCent=
if "%sCent%"=="" set sCent=furthest

set sPROC=%~4
if "%sPROC%"=="." set sPROC=

set CX=

call %PICTBAT%shpCent %INFILE% %sCent% s2g

goto skipCent

if /I "%sCent%"=="furthest" (

  for /F "usebackq tokens=1,4,5 delims=:, " %%A in (`%IMG7%magick ^
    %INFILE% ^
    -morphology Distance Euclidean:7 ^
    -alpha off -auto-level ^
    -define "identify:locate=maximum" ^
    -define "identify:limit=1" ^
    -identify ^
    NULL:`) do (
    if /I "%%A"=="Gray" (
      set CX=%%B
      set CY=%%C
    )
  )

) else if /I "%sCent%"=="centroid" (

  for /F "usebackq skip=1 tokens=3-6 delims=, " %%A in (`%IMG7%magick ^
    %INFILE% -alpha off -threshold 50% ^
    -define "connected-components:verbose=true" ^
    -connected-components 8 ^
    NULL:`) do (
    if "%%D"=="gray(255)" (
      set CX=%%A
      set CY=%%B
    )
  )

) else if /I "%sCent%"=="boundingbox" (

  for /F "usebackq" %%L in (`%IMG7%magick ^
    %INFILE% -alpha off -threshold 50% ^
    -trim ^
    -format "CX=%%[fx:(w-1)/2+page.x]\nCY=%%[fx:(h-1)/2+page.y]\n" ^
    info:`) do set %%L

) else (
  for /F "usebackq" %%A in (`%IMG7%magick ^
    %INFILE% -format "WW=%%w\nHH=%%h\n" ^
    info:`) do set %%A

  call parseXxY2 !WW! !HH! acw %sCent%

  set CX=!acw_X!
  set CY=!acw_Y!
)

echo %0: CX=%CX% CY=%CY%

if "!CX!"=="" (
  echo %0: sCent=%sCent%  no CX
  exit /B 1
)

:skipCent

@rem 13-August-2023: removed "-clamp" before "-distort Polar"
@rem 19-August-2023: This had "-crop 0x+0+1 +repage" after "-clone 0". That removed the top (white) row before scaling.
@rem   I forget why I did this. Remove it.

%IMG7%magick ^
  %INFILE% ^
  -define compose:clamp=off ^
  -distort DePolar -1,0,%s2g_CX%,%s2g_CY% ^
  -set option:MYSIZE %%wx%%h ^
  %shp2gProcBW% ^
  ( -size %%[MYSIZE] ^
    gradient:Black-White ^
  ) ^
  ( -clone 0 ^
    -scale "x1^!" ^
    -scale "%%[MYSIZE]^!" ^
  ) ^
  -delete 0 ^
  -compose DivideSrc -composite ^
  -distort Polar -1,0,%s2g_CX%,%s2g_CY% ^
  -define quantum:format=floating-point -depth 32 ^
  %sPROC% ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

call echoRestore

@endlocal & set shp2gOUTFILE=%OUTFILE%& set shp2gCX=%s2g_CX%& set shp2gCY=%s2g_CY%


