rem For image %1, finds the +/-sigmoidal-contrast that makes the standard deviation a given goal.
rem %2 is required mid-point for sigmoidal-contrast, as percentage, eg "50".
rem   Or "mean" to use the mean of the image.
@rem
@rem Will use:
@rem   sssGOAL_SD default 0.166667
@rem   sssPRECISION default "-precision 6"
@rem
@rem   sssDIRECTION "incOnly", "decOnly" or "both".
@rem     For "incOnly", has goal of only increasing SD. If current SD >= goal, does nothing (and returns sssALREADY_CLOSE=1).
@rem     For "decOnly", has goal of only decreasing SD. If current SD <= goal, does nothing (and returns sssALREADY_CLOSE=1).
@rem     For "both", has goal of increasing or decreasing SD as required.
@rem
@rem Returns:
@rem    sssFALSE_RESULT = 0 (okay) or 1 (algorithm failed).
@rem    sssALREADY_CLOSE = 0 or 1
@rem    sssOPTION = "+sigmoidal-contrast %sssCONTRAST%,%sssMID_PT%" or
@rem                "-sigmoidal-contrast %sssCONTRAST%,%sssMID_PT%" or
@rem                "" (when already close)
@rem    sssCONTRAST, sssMID_PT
@rem
@rem When decreasing contrast, the algorithm can fail when the current SD is high;
@rem the image is mostly black and white,
@rem with few mid-tones to be shifted by "+sigmoidal-contrast".
@rem
@rem Updated:
@rem   26-July-2022 for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INFILE=%1

set MID_PT=%2
if "%MID_PT%"=="" set MID_PT=30

set GOAL_SD=%sssGOAL_SD%
if "%GOAL_SD%"=="" set GOAL_SD=0.166667

set CON0=0.0000001
set CON2=30

set PRECISION=%sssPRECISION%
if "%PRECISION%"=="" set PRECISION=-precision 6

set DIRECTION=%sssDIRECTION%
if "%DIRECTION%"=="" set DIRECTION=both

set MAX_PREC=9

set TMP_IN=\temp\sss.miff

if /I "%MID_PT%"=="mean" for /F "usebackq" %%L ^
in (`%IMG7%magick ^
  %INFILE% ^
  +write %TMP_IN% ^
  -precision %MAX_PREC% ^
  -format "MID_PT=%%[fx:100*mean]" info:`) ^
do set %%L

echo %0: MID_PT=%MID_PT%

rem Are we already close?
for /F "usebackq" %%L ^
in (`%IMG7%magick ^
  %TMP_IN% ^
  -precision %MAX_PREC% ^
  ^( +clone -format "SDcur=%%[fx:standard_deviation]\n" -write info: +delete ^) ^
  ^( +clone +sigmoidal-contrast %CON0%x%MID_PT%%% -format "SDdec=%%[fx:standard_deviation]\n" -write info: +delete ^) ^
  -sigmoidal-contrast %CON0%x%MID_PT%%% -format "SDinc=%%[fx:standard_deviation]\n" info:`) ^
do set %%L

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  -format "ALREADY_CLOSE=%%[fx:%SDdec%<=%GOAL_SD%&&%GOAL_SD%<=%SDinc%]\nNEED_DEC=%%[fx:%GOAL_SD%<%SDdec%]" ^
  xc:`) ^
do set %%L

if %NEED_DEC%==1 if /I %DIRECTION%==incOnly set ALREADY_CLOSE=1
if %NEED_DEC%==0 if /I %DIRECTION%==decOnly set ALREADY_CLOSE=1

if %ALREADY_CLOSE%==1 (
  set FINISHED=1
  set FALSE_RESULT=0
  set OPTION=
  goto finished
)

if %NEED_DEC%==1 (
  set SIG_SIGN=+
  set tmpCon=%CON0%
  set CON0=%CON2%
  set CON2=!tmpCon!
) else (
  set SIG_SIGN=-
)

echo %0: SIG_SIGN=%SIG_SIGN%

for /F "usebackq" %%L ^
in (`%IMG7%magick ^
  %TMP_IN% ^
  -precision %MAX_PREC% ^
  ^( +clone %SIG_SIGN%sigmoidal-contrast %CON0%x%MID_PT%%% -format "SD0=%%[fx:standard_deviation]\n" -write info: +delete ^) ^
  %SIG_SIGN%sigmoidal-contrast %CON2%x%MID_PT%%% ^
  -format "SD2=%%[fx:standard_deviation]" ^
  info:`) ^
do set %%L

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  -format "FALSE_RESULT=%%[fx:%SD0%>=%GOAL_SD%||%SD2%<=%GOAL_SD%]" ^
  xc:`) ^
do set %%L

echo %0: SD0=%SD0% SD2=%SD2% FALSE_RESULT=%FALSE_RESULT%

set FINISHED=%FALSE_RESULT%

if %FINISHED%==1 goto finished

set nIter=0

:loop

if %CON0%==%CON2% set FINISHED=1

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  %PRECISION% ^
  -format "CON1=%%[fx:sqrt(%CON0%*%CON2%)]" ^
  xc:`) ^
do set %%L

if %CON0%==%CON1% set FINISHED=1
if %CON1%==%CON2% set FINISHED=1

for /F "usebackq" %%L ^
in (`%IMG7%magick ^
  %TMP_IN% ^
  %PRECISION% ^
  %SIG_SIGN%sigmoidal-contrast %CON1%x%MID_PT%%% ^
  -format "SD1=%%[fx:standard_deviation]" ^
  info:`) ^
do set %%L

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  -format "signDiff=%%[fx:%SD1%<%GOAL_SD%?-1:%SD1%>%GOAL_SD%]" ^
  xc:`) ^
do set %%L

echo %0: CON0=%CON0% CON1=%CON1% CON2=%CON2%
echo %0: SD0=%SD0% SD1=%SD1% SD2=%SD2%

rem echo %0: signDiff=%signDiff%

if %signDiff%==0 (
  set FINISHED=1
) else if %SD0%==%SD1% (
  set FINISHED=1
) else if %SD1%==%SD2% (
  set FINISHED=1
) else if %signDiff%==-1 (
  set CON0=%CON1%
  set SD0=%SD1%
) else (
  set CON2=%CON1%
  set SD2=%SD1%
)

@set /A nIter+=1

if %FINISHED%==0 goto loop

echo %0: nIter=%nIter%

if %ALREADY_CLOSE%==0 (
  set OPTION=%SIG_SIGN%sigmoidal-contrast %CON1%,%MID_PT%%%
)

:finished

call echoRestore

endlocal & ^
set sssFALSE_RESULT=%FALSE_RESULT%& ^
set sssCONTRAST=%CON1%& ^
set sssMID_PT=%MID_PT%& ^
set sssALREADY_CLOSE=%ALREADY_CLOSE%& ^
set sssOPTION=%OPTION%

set sss

exit /B 0

