rem From %1, white lines on black background, fully opaque,
rem makes output %2 skeletonised with short stubs removed.
@rem
@rem If the given output, %2, is XYZ.png,
@rem creates files:
@rem   XYZ.png       skeletonised image, pruned.
@rem   XYZ_ends.png  just the line-ends
@rem   XYZ_ends.csv  list of the line-ends.
@rem
@rem Updated:
@rem   6-August-2022 for IM v7
@rem


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 sps

echo %0: %1 %2

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

for /F %%F in ("%OUTFILE%") do (
  set OUTBASE=%%~dpnF
  set OUTEXT=%%~xF
)

set TEMPEXT=.png
set TEMP_SKEL=%OUTBASE%_sps_temp%TEMPEXT%
set OUT_ENDS_CSV=%OUTBASE%_ends.csv
set OUT_ENDS=%OUTBASE%_ends%OUTEXT%

%IMG7%magick ^
  %INFILE% ^
  -threshold 50%% ^
  -virtual-pixel Edge ^
  -morphology Thinning:-1 Skeleton -clamp ^
  -alpha off ^
  %TEMP_SKEL%

if ERRORLEVEL 1 exit /B 1

:: Calculate approx line width.

for /F "usebackq" %%L in (`%IMG7%magick ^
   %INFILE% ^
  -alpha off ^
  -format "MN_MAIN=%%[fx:mean]" ^
  info:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick ^
  %TEMP_SKEL% ^
  -format "MN_SKEL=%%[fx:mean]" ^
  info:`) do set %%L

if "%MN_SKEL%"=="0" (
  echo %0: skeleton [%TEMP_SKEL%] has zero mean
  exit /B 1
)

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "LINE_THK=%%[fx:%MN_MAIN%/%MN_SKEL%]\nLINE_THK_INT=%%[fx:int(%MN_MAIN%/%MN_SKEL%+0.5)]" ^
  xc:`) do set %%L

echo %0: LINE_THK=%LINE_THK% LINE_THK_INT=%LINE_THK_INT%

call %PICTBAT%pruneStubs %TEMP_SKEL% %OUTFILE% %LINE_THK_INT%
if ERRORLEVEL 1 exit /B 1

%IMG7%magick ^
  %OUTFILE% ^
  -morphology HMT LineEnds ^
  -alpha off ^
  +write %OUT_ENDS% ^
  -transparent Black ^
  sparse-color: |sed -e 's/ /\n/g' >%OUT_ENDS_CSV%

echo %0 end: outfile=%OUTFILE%

call echoRestore

endlocal & set spsOUTFILE=%OUTFILE%& set spsCSVFILE=%OUT_ENDS_CSV%& set spsENDS=%OUT_ENDS%& set spsLINE_THK_INT=%LINE_THK_INT%
