rem From colour image %1,
rem find magnitude of slope of RGB channels, by comet blur method.
rem Output %2 is single image.
rem %3 is (quoted) comma-list of one or two blur sigmas.
rem   First should be at least about 0.12.
rem   Second is orthogonal to first, default half of first
rem %4 is post-processing.
@rem
@rem Assumes magick is HDRI.
@rem
@rem See also slopeXYbl.bat.
@rem
@rem Updated:
@rem   24-December-2017 %3 is one or two sigmas.
@rem   22-July-2022 Upgraded for IM v7.
@rem   26-July-2022 Replace "-evaluate Pow 2 -compose plus -composite -evaluate Divide 2 -evaluate Pow 0.5" by "-fx sqrt((u*u+v*v)/2)"
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 sxybm

if not "%2"=="" set OUTFILE=%2

set BLR_SIG=%~3
call %UTIL%\parseCommaList "%BLR_SIG%" nSIG SIGS

if %nSIG%==0 (
  set BLR_SIG0=1
  set BLR_SIG1=0.5
) else if %nSIG%==1 (
  set BLR_SIG0=%SIGS[0]%

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "BLR_SIG1=%%[fx:%SIGS[0]%/2]"
    xc:`) do set %%L

) else if %nSIG%==2 (
  set BLR_SIG0=%SIGS[0]%
  set BLR_SIG1=%SIGS[1]%
)

echo %0: BLR_SIG0=%BLR_SIG0% BLR_SIG1=%BLR_SIG1%

set POST_PROC=%~4
if "%POST_PROC%"=="." set POST_PROC=

if %BLR_SIG1%==0 (
  set ORTHV=
  set ORTHH=
) else (
  set ORTHV=-morphology Convolve Blur:0x%BLR_SIG1%,90
  set ORTHH=-morphology Convolve Blur:0x%BLR_SIG1%
)

%IMG7%magick ^
  %INFILE% ^
  -alpha off ^
  -virtual-pixel Edge ^
  -define compose:clamp=off ^
  -define convolve:scale="^!" ^
  ( -clone 0 ^
    ( -clone 0 ^
      -morphology Convolve Comet:0x%BLR_SIG0% ^
      %ORTHV% ^
    ) ^
    ( -clone 0 ^
      -morphology Convolve Comet:0x%BLR_SIG0%,180 ^
      %ORTHV% ^
    ) ^
    -delete 0 ^
    -compose MinusDst -composite ^
  ) ^
  ( -clone 0 ^
    ( -clone 0 ^
      -morphology Convolve Comet:0x%BLR_SIG0%,90 ^
      %ORTHH% ^
    ) ^
    ( -clone 0 ^
      -morphology Convolve Comet:0x%BLR_SIG0%,270 ^
      %ORTHH% ^
    ) ^
    -delete 0 ^
    -compose MinusDst -composite ^
  ) ^
  -delete 0 ^
  -channel RGB ^
  -fx "sqrt((u*u+v*v)/2)" ^
  %POST_PROC% ^
  +depth ^
  -depth 32 ^
  -define "quantum:format=floating-point" ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

call echoRestore

endlocal & set sxybmOUTFILE=%OUTFILE%
