rem Root finder: finds value of PP for which expression is roughly zero.
rem %1 expression that contains PP
rem %2 prefix for output environment variables
rem %3 lower guess for PP
rem %4 upper guess for PP
rem %5 epsilon for convergence
@rem
@rem If there is a value of PP between the guesses for which the expression is zero,
@rem this should find it.
@rem
@rem There may be more than one root, but this will find only one.
@rem
@rem Evaluating the expression for the two guesses should give opposite signs,
@rem so the zero crossing is between the guesses.
@rem If that isn't true, the fx expression will write a debug line like:
@rem   [0,0].[64]: Err=1
@rem Even if an error is raised, it may still find a correct solution.
@rem
@rem The code also emits Vmid, the value of the expression at the returned value of PP.
@rem A caller can consider whether the value is close enough to zero.

setlocal

set EXP=%~1
if "%EXP%"=="." set EXP=
if "%EXP%"=="" set EXP=PP*log(PP) + log(1-PP) - PP*log(1-PP) + 0.5*log(2)

set EnvPref=%2
if "%EnvPref%"=="." set EnvPref=

set Guess1=%3
if "%Guess1%"=="." set Guess1=
if "%Guess1%"=="" set Guess1=0

set Guess2=%4
if "%Guess2%"=="." set Guess2=
if "%Guess2%"=="" set Guess2=1

set Eps=%5
if "%Eps%"=="." set Eps=
if "%Eps%"=="" set Eps=1e-9

set fxSolve=^
Pone=%Guess1%; ^
Ptwo=%Guess2%; ^
Warn=0; ^
while (abs(Pone-Ptwo)^>%Eps%, ^
  PP=Pone; ^
  Vone=%EXP%; ^
  PP=Ptwo; ^
  Vtwo=%EXP%; ^
  if (sign(Vone)==sign(Vtwo),Warn=1, ); ^
  PP=(Pone + Ptwo)/2; ^
  Vmid=%EXP%; ^
  if (sign(Vmid)==sign(Vone),Pone=PP,Ptwo=PP) ^
); ^
if (Warn==1, debug(Warn), ); ^
debug(Vmid); ^
PP


for /F "usebackq tokens=1,2,* delims== " %%A in (`%IMG7%magick ^
  xc: ^
  -precision 15 ^
  -format "%%[fx:%fxSolve%]" ^
  info: 2^>^&1`) do (
  echo A=%%A B=%%B C=%%C
  if "%%B"=="Vmid" (
    set Vmid=%%C 
  ) else (
    set PP=%%A
  )
)

echo Vmid=%Vmid%
echo PP=%PP%

endlocal & set %EnvPref%_Vmid=%Vmid%& set %EnvPref%_PP=%PP%