rem %1 input image in xyY space
rem %2 output image in xyY space
rem %3 Primaries and WP: "horseshoe", "sRGB", etc.
rem %4 white point
rem %5 method: clamp, filmic, divide or null

@rem
@rem Also uses:
@rem   sqxCLUTDIR Directory for cluts. [%ICCPROF%]
@rem   sqxHiLimit hiLimit for oogbox [0.7]
@rem   sqxMAXSTAT_FILE name for maximum statistic [none]
@rem   sqxFORCE_MAX assume this maximum instead of statistic [no force]
@rem

@rem FIXME: also check image has embedded XY profile.


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 sqx

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set SHAPE=%~3
if "%SHAPE%"=="." set SHAPE=
if "%SHAPE%"=="" set SHAPE=sRGB

set WhtPnt=%~4
if "%WhtPnt%"=="." set WhtPnt=

set METHOD=%5
if "%METHOD%"=="." set METHOD=
if "%METHOD%"=="" set METHOD=filmic

if /I not %METHOD%==clamp if /I not %METHOD%==divide if /I not %METHOD%==filmic if /I not %METHOD%==null (
  echo %0: Bad method [%METHOD%]
  exit /B 1
)

if "%sqxCLUTDIR%"=="" set sqxCLUTDIR=%ICCPROF%

if "%sqxHiLimit%"=="" set sqxHiLimit=0.7

set TMPDIR=\temp

set CLUTDIM=4096

rem FIXME: CLUT_NAME also needs WP, including for horseshoe.

set CLUT_NAME=%SHAPE: =_%

set MultiParam=1
if %CLUT_NAME%==%SHAPE% set MultiParam=0

set WPx=0.31
set WPy=0.32

:: But what if horseshoe?
:: How does user declare WP for horseshoe?
if "%SHAPE%"=="horseshoe" (
  call :getbary "outPrim sRGB"
  if ERRORLEVEL 1 exit /B 1
) else if %MultiParam%==0 (
  call :getbary "outPrim %SHAPE%"
  if ERRORLEVEL 1 exit /B 1
) else (
  call :getbary "%SHAPE%"
  if ERRORLEVEL 1 exit /B 1
)

if ERRORLEVEL 1 exit /B 1

if not "%WhtPnt%"=="" (
  call :getwp "%WhtPnt%"
)

set CLUT_NAME=%CLUTDIM%_%CLUT_NAME%_%WPx:.=_%_%WPy:.=_%

echo %0: CLUT_NAME=%CLUT_NAME%

set CLUT_FILE=%sqxCLUTDIR%\sqx_clut_%CLUT_NAME%.tiff

echo %0: CLUT_FILE=%CLUT_FILE%

if exist %CLUT_FILE% goto clut_ok


if %MultiParam%==1 goto MultiName
if /I not %SHAPE%==horseshoe goto notHorseshoe

echo %0: Making %CLUT_FILE%

set HSESHOE=m 426.8 358.8 c-106.3 -106.3 -177.2 -174.2 -222.7 -219.7 -39.2 -39.2 -94 -73.4 -114.3 -73.4 -28.7 0 -39.5 45.1 -39.5 98.5 0 59.3 18.4 227.4 66 306.4 0 0 10.3 15.6 21.7 20 l 288.8 -131.8 z

set DR_HSESHOE=translate -47.8,19.2 path '%HSESHOE%'

%IM7DEV%magick ^
  -size %CLUTDIM%x%CLUTDIM% xc:Black ^
  -fill White -stroke None ^
  -draw "scale %%[fx:w/512],%%[fx:h/512] stroke-width %%[fx:512/w] %DR_HSESHOE%" ^
  -rotate 180 ^
  -virtual-pixel Black ^
  -distort depolar %%[fx:hypot(w,h)/sqrt(2)],0,%%[fx:w-w*%WPx%],%%[fx:h*%WPy%] ^
  -scale "x1^!" ^
  -flop ^
  -alpha off ^
  -define quantum:format=floating-point ^
  -depth 32 ^
  %CLUT_FILE%

goto clut_ok

:MultiName

call :drTri "%SHAPE%"
if ERRORLEVEL 1 exit /B 1

goto clut_ok

:notHorseshoe

call :drTri "outPrim %SHAPE%"
if ERRORLEVEL 1 exit /B 1

goto clut_ok


:clut_ok

echo %0: Applying method %METHOD% forwards:

set wrMAXSTAT=
if not "%sqxMAXSTAT_FILE%"=="" (
  set wrMAXSTAT=-format %%[fx:maxima] +write info:%sqxMAXSTAT_FILE%
)

set sTWEAK=
if /I %METHOD%==clamp (
  set sTWEAK=-clamp
) else if /I %METHOD%==divide (
  if "%sqxFORCE_MAX%"=="" (
    set sTWEAK=-evaluate Divide %%[fx:maxima]
  ) else (
    set sTWEAK=-evaluate Divide %sqxFORCE_MAX%
  )
) else if /I %METHOD%==filmic (
  if "%sqxFORCE_MAX%"=="" (
    set sTWEAK=-process 'oogbox loLimit 0 hiLimit %sqxHiLimit% channel 0 v'
  ) else (
    set sTWEAK=-process 'oogbox forceMax %sqxFORCE_MAX% loLimit 0 hiLimit %sqxHiLimit% channel 0 v'
  )
) else if /I %METHOD%==null (
  set sTWEAK=
) else (
  echo %0: Bad method [%METHOD%]
  exit /B 1
)

%IM7DEV%magick ^
  %INFILE% ^
  -set colorspace sRGB ^
  -strip ^
  +profile "*" ^
  -define quantum:format=floating-point ^
  -depth 32 ^
  -process 'rhotheta offset %WPx%,%WPy%' ^
  -channel RGB -separate +channel ^
  ( -clone 1 ^
    -write %TMPDIR%\sxs_theta.miff ^
    %CLUT_FILE% ^
    -clut ^
    -colorspace Gray ^
    -write %TMPDIR%\sxs_rho0.miff ^
    +delete ) ^
  -delete 1 ^
  ( -clone 0 ^
    %TMPDIR%\sxs_rho0.miff ^
    -define compose:clamp=off ^
    -compose DivideSrc -composite ^
    -colorspace Gray ^
    %wrMAXSTAT% ^
    %sTWEAK% ^
    -write %TMPDIR%\sxs_rho_rel2.miff ^
    +delete ) ^
  -delete 0 ^
  %TMPDIR%\sxs_Y.miff

echo %0: Reverse:

if /I not %OUTFILE%==NULL: %IM7DEV%magick ^
  %TMPDIR%\sxs_rho_rel2.miff ^
  %TMPDIR%\sxs_rho0.miff ^
  -define compose:clamp=off ^
  -compose Multiply -composite ^
  -define quantum:format=floating-point ^
  -depth 32 ^
  %TMPDIR%\sxs_theta.miff ^
  %TMPDIR%\sxs_Y.miff ^
  -combine ^
  -set colorspace xyY ^
  -process 'rhotheta offset %WPx%,%WPy% inverse' ^
  %OUTFILE%


call echoRestore

@endlocal

@exit /B 0

::---------------------------------------
:: Subroutines

:drTri

call :getbary %1
if ERRORLEVEL 1 exit /B 1

set i=0
for %%A in (%NUMS%) do (
  if "!prev!"=="" (
    set prev=%%A
  ) else (
    set /A rad=8
    echo ow !prev! and %%A !rad!
    set sDRAW[!i!]=%%[fx:w*!prev!],%%[fx:h-h*%%A]
    set prev=
    set /A i+=1
  )
)

set sDRAW

%IM7DEV%magick ^
  -size %CLUTDIM%x%CLUTDIM% xc:Black ^
  -fill White -stroke None ^
  -draw "polygon %sDRAW[0]%,%sDRAW[1]%,%sDRAW[2]%" ^
  -rotate 180 ^
  -virtual-pixel Black ^
  -distort depolar %%[fx:hypot(w,h)/sqrt(2)],0,%%[fx:w-w*%WPx%],%%[fx:h*%WPy%] ^
  -scale "x1^!" ^
  -flop ^
  -alpha off ^
  -define quantum:format=floating-point ^
  -depth 32 ^
  %CLUT_FILE%

exit /B 0

::---------------------------------------

:getbary

set ARGS=%~1
echo %0: ARGS=%ARGS%

set NUMS=
set WP=
for /F "usebackq tokens=1,2" %%A in (`%IM7DEV%magick ^
  xc: ^
  -process 'barymap inChannels xyY outChannels xyY %ARGS% f stdout v' ^
  NULL: 2^>^&1`) do (
  echo %0: [%%A][%%B]
  if %%A==outPrim set NUMS=%%B
  if %%A==outWP set WP=%%B
)

rem FIXME: no longer outPrim etc.

echo %0: WP=%WP%

if "%NUMS%"=="" (
  echo %0: barymap failed ARGS=%ARGS%
  exit /B 1
)

for /F "tokens=1,2 delims=," %%A in ("%WP%") do (
  set WPx=%%A
  set WPy=%%B
)

exit /B 0


::---------------------------------------

:getwp

set ARGS=%~1
echo %0: ARGS=%ARGS%

set NUMS=
set WP=
for /F "usebackq tokens=1,2" %%A in (`%IM7DEV%magick ^
  xc: ^
  -process 'barymap inChannels xyY outChannels xyY  outWP %ARGS% f stdout v' ^
  NULL: 2^>^&1`) do (
  if %%A==outWP set WP=%%B
)

echo %0: WP=%WP%

if "%WP%"=="" (
  echo %0: barymap failed ARGS=%ARGS%
  exit /B 1
)

for /F "tokens=1,2 delims=," %%A in ("%WP%") do (
  set WPx=%%A
  set WPy=%%B
)

exit /B 0
