rem Straighten two lines: three steps combined.

rem %1 is image with two non-intersecting aliased blue lines.
rem %2 image to be distorted.
rem %3 is output.
rem %4 proportion of effect (typically 0.0 to 1.0) [1.0]
rem %5 shrink or stretch or mean
rem %6 prefix for environment variables [s2l3]
rem %7 channel for separation: R or G or B [default B].
rem %8 interpolation method: linear, linear3, hermite [linear].
rem %9 virtual-pixel method [default None].

@rem Updated:
@rem   8-June-2021 Changed final "magick" to read input from %INDIST% instead of %INFILE%.
@rem   11-July-2021 Added parameters %7 and %8.
@rem   27-April-2023 Allow negative values in %4, FCTPROP.

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 s2l3

set OUTTMPLT=b2c_XX.miff

set INDIST=%2
if "%INDIST%"=="." set INDIST=
if "%INDIST%"=="" set INDIST=%INFILE%

if not "%3"=="" if not "%3"=="." set OUTFILE=%3

set FCTPROP=%4
if "%FCTPROP%"=="." set FCTPROP=
if "%FCTPROP%"=="1.0" set FCTPROP=1
if "%FCTPROP%"=="" set FCTPROP=1

set FCTPROP=(%FCTPROP%)

set ShrStr=%5
if "%ShrStr%"=="." set ShrStr=
if "%ShrStr%"=="" set ShrStr=shrink

set ENVPREF=%6
if "%ENVPREF%"=="." set ENVPREF=
if "%ENVPREF%"=="" set ENVPREF=s2l3

set chSEP=%7
if "%chSEP%"=="." set chSEP=
if "%chSEP%"=="" set chSEP=B

set INTERP=%8
if "%INTERP%"=="." set INTERP=
if "%INTERP%"=="" set INTERP=linear

set VIRTPIX=%9
if "%VIRTPIX%"=="." set VIRTPIX=
if "%VIRTPIX%"=="" set VIRTPIX=None

set TMPDIR=\temp

call %PICTBAT%blue2cluts ^
  %INFILE% ^
  %TMPDIR%\s2l3_blc_XX.miff ^
  %chSEP% ^
  %ShrStr% ^
  %ENVPREF%

if ERRORLEVEL 1 exit /B 1

if not %FCTPROP%==(1) (
  %IMG7%magick ^
    %TMPDIR%\s2l3_blc_1.miff ^
    -function Polynomial "%%[fx:%FCTPROP%],%%[fx:0.5*(1-%FCTPROP%)]" ^
    %TMPDIR%\s2l3_blc_1.miff

  %IMG7%magick ^
    %TMPDIR%\s2l3_blc_2.miff ^
    -function Polynomial "%%[fx:%FCTPROP%],%%[fx:0.5*(1-%FCTPROP%)]" ^
    %TMPDIR%\s2l3_blc_2.miff
)

if /I %INTERP%==linear (
  echo linear
) else if /I %INTERP%==linear3 (
  echo linear3
) else if /I %INTERP%==hermite (
  echo hermite
) else (
  echo %0: Bad INTERP [%INTERP%]
  exit /B 1
)

call %PICTBAT%interpClut ^
  %TMPDIR%\s2l3_blc_1.miff %TMPDIR%\s2l3_blc_2.miff ^
  !%ENVPREF%HH! ^
  %TMPDIR%\s2l3_lt1.miff ^
  !%ENVPREF%XTRATOP! !%ENVPREF%XTRABOT!

if ERRORLEVEL 1 exit /B 1

%IMG7%magick ^
  %INDIST% ^
  %TMPDIR%\s2l3_lt1.miff ^
  -virtual-pixel %VIRTPIX% ^
  -compose Displace ^
    -set option:compose:args 0x100%% ^
    -composite ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1


call echoRestore

@endlocal & set s2l3OUTFILE=%OUTFILE%
