rem Given %1 is an image,
rem %2 is a mask, mostly white, with black at left and right edges,
rem makes %3 that has image stretched to fill the bounding rectangle of the white pixels.
rem %4 direction: h=horizontally, v=vertically. Default h.
rem %5 supersample percentage eg 400. Default 100 (no supersampling).
@rem
@rem Updated:
@rem   27-August-2022 for IM v7.
@rem

setlocal

set INFILE=%1
set MASK=%2
set OUTFILE=%3

set TMP_IN=sh_tmp.miff

set DIRN=%4
if "%DIRN%"=="." set DIRN=
if "%DIRN%"=="" set DIRN=h

set RESAMP_PC=%5
if "%RESAMP_PC%"=="." set RESAMP_PC=
if "%RESAMP_PC%"=="" set RESAMP_PC=100

if /I %DIRN%==h (
  set ROT_START=
  set ROT_END=
) else if /I %DIRN%==v (
  set ROT_START=-rotate -90
  set ROT_END=-rotate 90
) else (
  echo %0: Invalid direction [%DIRN%]
  exit /B 1
)

for /F "usebackq tokens=1-4 delims=x+" %%A in (`%IMG7%magick ^
  %MASK% ^
  -bordercolor Black -border 1 ^
  -format "%%@" ^
  info:`) do (
  set MW=%%A
  set MH=%%B
  set /A MX=%%C-1
  set /A MY=%%D-1
)

set MASK_CROP=-crop %MW%x%MH%+%MX%+%MY% +repage

if %RESAMP_PC%==100 (
  set RESAMP_START=
  set RESAMP_END=
) else (
  set RESAMP_START=-resize %RESAMP_PC%%%
  set RESAMP_END=-resize "%MW%x%MH%^!"
)

set UNUSED_COL=Blue

set WW=
for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  %MASK_CROP% ^
  %ROT_START% ^
  %RESAMP_START% ^
  -format "WW=%%w\nHH=%%h" ^
  +write info: ^
  +depth ^
  %%TMP_IN%%`) do set %%L
if "%WW%"=="" exit /B 1

set EDGE_COL=blue

%IMG7%magick ^
  %MASK% ^
  +depth ^
  %MASK_CROP% ^
  %ROT_START% ^
  %RESAMP_START% ^
  -threshold 50%% ^
  +repage ^
  ( -clone 0 ^
    -sparse-color bilinear ^
0,0,#000,^
%%[fx:w-1],0,#f00,^
0,%%[fx:h-1],#0f0,^
%%[fx:w-1],%%[fx:h-1],#ff0 ^
  ) ^
  ( -clone 0 ^
    -fill %EDGE_COL% -colorize 100 ^
  ) ^
  -swap 0,2 ^
  -compose Over -composite ^
  -crop x1 +repage ^
  -bordercolor %EDGE_COL% -border 1 ^
  -trim +repage ^
  -resize "%WW%x1^!" ^
  -append ^
  %TMP_IN% ^
  +swap ^
  -compose Distort ^
  -set option:compose:args 100%%x100%% -composite ^
  %ROT_END% ^
  %RESAMP_END% ^
  %OUTFILE%

if ERRORLEVEL 1 exit /B 1

endlocal
