rem From image %1, make tile half width and height by darkest path method.
rem Output is same width and height as input, but only 1/4 of the pixels are opaque.
rem %2 is optional output file.
@rem
@rem %3 whether to make cuts to tile horizontally [1] (vertical cuts)
@rem %4 whether to make cuts to tile vertically [1] (horizontal cuts)
@rem %5 method for whitening, either "morph" (accurate) or "flood" (fast). Default: morph.
@rem   Beware: "flood" assumes (0,0) is black, etc.
@rem
@rem Also uses:
@rem
@rem   tdpFTH_SIZ feathering size for top and left. 0=no feathering [2]
@rem   tdpPREF prefix for working files (not output file).
@rem   tdpDEBUG if 1, also creates debugging images.
@rem
@rem Updated:
@rem   14-July-2022 for IM v7. Added %5.
@rem

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 tdp


if not "%2"=="" set OUTFILE=%2

set DO_H=%3
if "%DO_H%"=="." set DO_H=
if "%DO_H%"=="" set DO_H=1

set DO_V=%4
if "%DO_V%"=="." set DO_V=
if "%DO_V%"=="" set DO_V=1

set DO_BOTH=0
if %DO_H%==1 if %DO_V%==1 set DO_BOTH=1

set WH_METH=%5
if "%WH_METH%"=="." set WH_METH=
if "%WH_METH%"=="" set WH_METH=morph

if /I %WH_METH%==morph (
  set sWhMeth=-morphology dilate:-1 2x1+1+0:1,1
) else if /I %WH_METH%==flood (
  set sWhMeth=-draw "color 0,0 floodfill"
) else (
  echo %0: Bad WH_METH [%WH_METH%]
  exit /B 1
)

if "%tdpFTH_SIZ%"=="" set tdpFTH_SIZ=2

echo DO_H=%DO_H% DO_V=%DO_V% DO_BOTH=%DO_BOTH%

if "%tdpPREF%"=="" set tdpPREF=tdp_
set EXT=.miff

set X0=0
set Y0=0

if %DO_H%==1 (
echo %0: find TIL_X
for /F "usebackq delims=, " %%X in (`%IM7DEV%magick ^
  %INFILE% ^
  ^( -clone 0 ^
    -colorspace Gray ^
    -crop 2x1@ +repage ^
    -compose Difference -composite ^
    +write %tdpPREF%lr%EXT% ^
    -scale "x1^!" ^
    -negate ^
    -process onelightest ^
    +delete ^
  ^) ^
  NULL: 2^>^&1`) do (
    set TIL_X=%%X
    set X0=%%X
  )
)

if %DO_V%==1 (
echo %0: find TIL_Y
for /F "usebackq tokens=1 delims=, " %%Y in (`%IM7DEV%magick ^
  %INFILE% ^
  ^( -clone 0 ^
    -colorspace Gray ^
    -crop 1x2@ +repage ^
    -compose Difference -composite ^
    -rotate -90 ^
    +write %tdpPREF%tb%EXT% ^
    -scale "x1^!" ^
    -negate ^
    -process onelightest ^
    +delete ^
  ^) ^
  NULL: 2^>^&1`) do (
    set TIL_Y=%%Y
    set Y0=%%Y
  )
)



::test
%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nX1=%%[fx:int(%X0%+w/2+0.5)]\nY1=%%[fx:int(%Y0%+h/2+0.5)]\n" ^
  %INFILE%

if ERRORLEVEL 1 exit /B 1

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW=%%w\nHH=%%h\nX1=%%[fx:int(%X0%+w/2+0.5)]\nY1=%%[fx:int(%Y0%+h/2+0.5)]\n" ^
  %INFILE%`) do set %%L

echo WW=%WW% HH=%HH% X0=%X0% Y0=%Y0% X1=%X1% Y1=%Y1% 


if %DO_H%==1 (
  echo %0: find vertical path
  rem call %PICTBAT%supWhGate ^
  rem   %tdpPREF%lr%EXT% %X0% %Y0% %tdpPREF%lr_swg.miff
  rem 
  rem call %PICTBAT%supWhGate ^
  rem   %tdpPREF%lr_swg.miff %X0% %Y1% %tdpPREF%lr_swg.miff

  call :WhGates %tdpPREF%lr%EXT% !X0! !Y0! !Y1! %tdpPREF%lr_swg.miff

  if ERRORLEVEL 1 exit /B 1

  if "%tdpDEBUG%"=="1" (
    set sWR_LR_MP=-stroke Yellow -fill None ^
      -draw "translate %X0%,%Y0% circle 0,0 0,10" ^
      -draw "translate %X0%,%Y1% circle 0,0 0,10" ^
      %tdpPREF%lr_swg_mp%EXT%

  ) else (
    set sWR_LR_MP=NULL:
  )

  %IM7DEV%magick ^
    %tdpPREF%lr_swg.miff ^
    -process 'darkestpath' ^
    +write %tdpPREF%lr_swg_ln%EXT% ^
    !sWR_LR_MP!
)

if %DO_V%==1 (
  echo %0: find horizontal path
  set X0R=%Y0%
  set /A Y0R=%WW%-%X1%-1
  set /A Y1R=%WW%-%X0%-1

  echo X0R=!X0R! Y0R=!Y0R! Y1R=!Y1R!

  rem call %PICTBAT%supWhGate ^
  rem   %tdpPREF%tb%EXT% ^
  rem   !X0R! !Y0R! ^
  rem   %tdpPREF%tb_swg.miff
  rem 
  rem call %PICTBAT%supWhGate ^
  rem   %tdpPREF%tb_swg.miff ^
  rem   !X0R! !Y1R! ^
  rem   %tdpPREF%tb_swg.miff

  call :WhGates %tdpPREF%tb%EXT% !X0R! !Y0R! !Y1R! %tdpPREF%tb_swg.miff

  if "%tdpDEBUG%"=="1" (
    set sWR_TB_MP=-stroke Yellow -fill None ^
      -draw "translate !X0R!,!Y0R! circle 0,0 0,10" ^
      -draw "translate !X0R!,!Y1R! circle 0,0 0,10" ^
      %tdpPREF%tb_swg_mp%EXT%
  ) else (
    set sWR_TB_MP=NULL:
  )

  %IM7DEV%magick ^
    %tdpPREF%tb_swg.miff ^
    -process 'darkestpath' ^
    +write %tdpPREF%tb_swg_ln%EXT% ^
    !sWR_TB_MP!
)

rem Check:
echo %0: check
if %DO_BOTH%==1 if "%tdpDEBUG%"=="1" %IMG7%magick ^
  ( %tdpPREF%lr_swg_mp%EXT% ( +clone ) +append +repage ) ^
  ( %tdpPREF%tb_swg_mp%EXT% ( +clone ) +append +repage -rotate 90 ) ^
  -compose Lighten -composite ^
  -fill #f80 -opaque White ^
  -transparent Black ^
  %INFILE% ^
  -compose DstOver -composite ^
  %tdpPREF%chk_swg_mp%EXT%

set DIST_KNL=Euclidean:7

if "%tdpFTH_SIZ%"=="0" (
  set sFEATH=
) else (
  set sFEATH=-morphology Distance "%DIST_KNL%,%tdpFTH_SIZ%^^^!"
)

echo tdpFTH_SIZ=%tdpFTH_SIZ% sFEATH=%sFEATH%

echo %0: make mask
if %DO_BOTH%==1 (
  %IMG7%magick ^
    -fill White ^
    ^( %tdpPREF%lr_swg_ln%EXT% ^
       %sWhMeth% ^
       -alpha off ^
       ^( +clone ^
          %sFEATH% ^
          -negate ^
       ^) ^
       +swap +append +repage ^
    ^) ^
    ^( %tdpPREF%tb_swg_ln%EXT% ^
       %sWhMeth% ^
       -alpha off ^
       -rotate 90 ^
       ^( +clone ^
          %sFEATH% ^
          -negate ^
       ^) ^
       +swap -append +repage ^
    ^) ^
    -compose Multiply -composite ^
    %tdpPREF%mask%EXT%
  if ERRORLEVEL 1 exit /B 1

) else if %DO_H%==1 (
  %IMG7%magick ^
    -fill White ^
    ^( %tdpPREF%lr_swg_ln%EXT% ^
       %sWhMeth% ^
       -alpha off ^
       ^( +clone ^
          %sFEATH% ^
          -negate ^
       ^) ^
       +swap +append +repage ^
    ^) ^
    %tdpPREF%mask%EXT%
  if ERRORLEVEL 1 exit /B 1

) else if %DO_V%==1 (
  %IMG7%magick ^
    -fill White ^
    ^( %tdpPREF%tb_swg_ln%EXT% ^
       %sWhMeth% ^
      -alpha off ^
      -rotate 90 ^
      ^( +clone ^
         %sFEATH% ^
         -negate ^
      ^) ^
      +swap -append +repage ^
    ^) ^
    %tdpPREF%mask%EXT%
  if ERRORLEVEL 1 exit /B 1
)


echo %0: make tile
%IM7DEV%magick ^
  -fill None ^
  %INFILE% ^
  %tdpPREF%mask%EXT% ^
  -alpha off ^
  -compose CopyOpacity -composite ^
  %OUTFILE%

call echoRestore

endlocal & set tdpOUTFILE=%OUTFILE%& set tdpX0=%X0%& set tdpY0=%Y0%& set tdpX1=%X1%& set tdpY1=%Y1%

exit /B 0

:: ------------- Subroutines -------------

:: %1 input image
:: %2 X
:: %3 Y0
:: %4 Y1
:: %5 output file
:: Assumes WW

:WhGates

echo %0: %1 %2 %3 %4 %5

::test
%IMG7%magick identify ^
  -format "Wm1=%%[fx:%WW%-1]\nXm1=%%[fx:%2-1]\nXp1=%%[fx:%2+1]\nHasL=%%[fx:%2>0?1:0]\nHasR=%%[fx:%2<%WW%-1?1:0]\n" ^
  %INFILE%

if ERRORLEVEL 1 exit /B 1

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "Wm1=%%[fx:%WW%-1]\nXm1=%%[fx:%2-1]\nXp1=%%[fx:%2+1]\nHasL=%%[fx:%2>0?1:0]\nHasR=%%[fx:%2<%WW%-1?1:0]\n" ^
  %INFILE%`) do set %%L

if !HasL!==0 (
  set DrawL1=
  set DrawL2=
) else (
  set DrawL1=-draw "line 0,%3 !Xm1!,%3"
  set DrawL2=-draw "line 0,%4 !Xm1!,%4"
)

echo %0: DrawL1=%DrawL1%
echo %0: DrawL2=%DrawL2%

if !HasR!==0 (
  set DrawR1=
  set DrawR2=
) else (
  set DrawR1=-draw "line !Xp1!,%3 !Wm1!,%3"
  set DrawR2=-draw "line !Xp1!,%4 !Wm1!,%4"
)

echo %0: DrawR1=%DrawR1%
echo %0: DrawR2=%DrawR2%

%IM7DEV%magick ^
  %1 ^
  -stroke rgb(100000%%,100000%%,100000%%) ^
  !DrawL1! ^
  !DrawR1! ^
  !DrawL2! ^
  !DrawR2! ^
  -define quantum:format=floating-point ^
  %5

if ERRORLEVEL 1 exit /B 1

exit /B 0
