rem From image %1, a white shape on black background,
rem trace the contour, writing coordinates to text file %2.
rem Tracing starts at nearest white pixel to coord %3,%4 [0,0],
rem   which should be an end-line.
rem %5 is offset to be applied. Use half the line width. [0.5]
rem
rem This assumes shape has more than one white pixel.


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 tc

set OUTFILE=%2

set firstX=%3
if "%firstX%"=="." set firstX=
if "%firstX%"=="" set firstX=0

set firstY=%4
if "%firstY%"=="." set firstY=
if "%firstY%"=="" set firstY=0

set offs=%5
if "%offs%"=="." set offs=
if "%offs%"=="" set offs=0.5

rem FIXME: We need to start at a line-end.

set INTRO=
for /F "usebackq tokens=1-3 delims=:, " %%A in (`%IM7DEV%magick ^
  %INFILE% ^
  -process 'nearestwhite cx %firstX% cy %firstY%' ^
  NULL: 2^>^&1`) do (
  set INTRO=%%A
  set pX=%%B
  set pY=%%C
)

if not "%INTRO%"=="nearestwhite" exit /B 1

if "%pX%"=="none" exit /B 1

set TMPDIR=\temp\
set TMPIN=%TMPDIR%tc.miff
set TMPOUT=%TMPDIR%tc.txt

%IMG7%magick ^
  %INFILE% ^
  -colorspace Gray ^
  -threshold 50%% ^
  -depth 8 ^
  %TMPIN%

set /A prevX=%pX%-1
set /A prevY=%pY%

rem We need to have both pxy and prevxy on boundary,
rem then we can store the original values and start emitting.
rem Beware isolated white pixels,
rem or being surrounded by white.

:: While prevXY is not white, get next

set cnt=0
:getWhite
call :nextClockwise

for /F "usebackq" %%L in (`%IMG7%magick ^
  %TMPIN% ^
  -format "isW=%%[fx:p{%cX%,%cY%}]\n" ^
  info:`) do set %%L

set /A cnt+=1

if %cnt%==9 (
  echo All neighbours are black.
  exit /B 1
)

set prevX=%cX%
set prevY=%cY%

if %isW%==0 goto getWhite


:: While prevXY is white, get next

set cnt=0
:getBlack
call :nextClockwise

for /F "usebackq" %%L in (`%IMG7%magick ^
  %TMPIN% ^
  -format "isW=%%[fx:p{%cX%,%cY%}]\n" ^
  info:`) do set %%L

set /A cnt+=1

if %cnt%==9 (
  echo All neighbours are white.
  exit /B 1
)

if %isW%==1 (
  set prevX=%cX%
  set prevY=%cY%
  goto getBlack
)

rem echo %prevX%,%prevY% is last white

rem Pretend we emitted the previous, for the offsets.
rem (With new scheme, no longer need this.)
set eX=%prevX%
set eY=%prevY%

echo %0: eXY=%eX%,%eY% pXY=%pX%,%pY%

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -format "oX=%%[fx:%pX%+(%pY%-%eY%)*%offs%]\noY=%%[fx:%pY%-(%pX%-%eX%)*%offs%]\n" ^
  info:`) do set %%L

del %TMPOUT% 2>nul
rem echo %oX%,%oY% >%TMPOUT%

set eX=%pX%
set eY=%pY%

set firstpX=%pX%
set firstpY=%pY%
set firstprevX=%prevX%
set firstprevY=%prevY%

call :nextClockwise

echo %0: firstpXY %firstpX%,%firstpY%  firstprevXY %firstprevX%,%firstprevY%

rem eXY is the previously emitted coordinate.
rem If we always start at an end, perhaps we don't need eXY, or that test.
rem Yes, we need eXY for the offsets.

set savedStart=0

set numPts=0
:loop

for /F "usebackq" %%L in (`%IMG7%magick ^
  %TMPIN% ^
  -format "isW=%%[fx:p{%cX%,%cY%}]\n" ^
  info:`) do set %%L

if %isW%==1 (

  for /F "usebackq" %%L in (`%IMG7%magick ^
    xc: ^
    -format "oX=%%[fx:%cX%+(%cY%-%eY%)*%offs%]\noY=%%[fx:%cY%-(%cX%-%eX%)*%offs%]\n" ^
    info:`) do set %%L

  set prevX=%pX%
  set prevY=%pY%
  set pX=%cX%
  set pY=%cY%
  set eX=%cX%
  set eY=%cY%
rem  echo !oX!,!oY! >>%TMPOUT%
rem  set /A numPts+=1
) else (
  call :emitBoundary %pX% %pY% %cX% %cY%

  set prevX=%cX%
  set prevY=%cY%
)

call :nextClockwise

if not %firstpX%==%cX% goto loop
if not %firstpY%==%cY% goto loop
if not %firstprevX%==%eX% goto loop
if not %firstprevY%==%eY% goto loop

rem Repeat the first point.
set cX=%firstpX%
set cY=%firstpY%
set eX=%firstprevX%
set eY=%firstprevY%

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -format "oX=%%[fx:%cX%+(%cY%-%eY%)*%offs%]\noY=%%[fx:%cY%-(%cX%-%eX%)*%offs%]\n" ^
  info:`) do set %%L

rem echo !oX!,!oY! >>%TMPOUT%
rem set /A numPts+=1

if %savedStart%==1 (
  echo %startX%,%startY% >>%TMPOUT%
  set /A numPts+=1
)

echo isLoop >>%TMPOUT%


echo %0: numPts=%numPts%


echo line >%OUTFILE%
echo numPts=%numPts% >>%OUTFILE%
type %TMPOUT% >>%OUTFILE%

call echoRestore

@endlocal & set tlnOUTFILE=%OUTFILE%

@exit /B 0

::--------------------------------
:: Subroutines

:: Of the 8 neighbours of px,py,
:: set cX,cY to next neighbour after prevX,prevY in clockwise direction.

:nextClockwise

set /A dx=%prevX%-%pX%
set /A dy=%prevY%-%pY%

if %dx%==-1 (
  set /A dy-=1
  if /I !dy! LSS -1 (
    set /A dy=-1
    set /A dx+=1
  )
) else if %dx%==1 (
  set /A dy+=1
  if /I !dy! GTR 1 (
    set /A dy=1
    set /A dx-=1
  )
) else if %dy%==-1 (
  set /A dx+=1
) else (
  set /A dx-=1
)

set /A cX=%pX%+(%dx%)
set /A cY=%pY%+(%dy%)

exit /B 0


:emitBoundary

if not %1==%3 if not %2==%4 exit /B 0

for /F "usebackq" %%L in (`%IMG7%magick ^
  xc: ^
  -format "oX=%%[fx:(%1+%3)/2+%offs%]\noY=%%[fx:(%2+%4)/2+%offs%]\n" ^
  info:`) do set %%L

echo %oX%,%oY% >>%TMPOUT%

if %savedStart%==0 (
  set savedStart=1
  set startX=%oX%
  set startY=%oY%
)

set /A numPts+=1

exit /B 0
