rem Given image %1, white points on black background, fully opaque,
rem where groups of 4- or 8-connected points form lines,
rem writes the coordinates of lines to text file %2
rem starting from line-end nearest to coord %3,%4 [0,0]
rem
rem %5 optional template filename for saved debugging images.
rem   Must contain XX, which will be replaced
rem   by skel, lines, jcns, ends, loops, lpbrk.
@rem
@rem Last update:
@rem
@rem   9-Jan-2017 Write width and height.
@rem   14-September-2022 for IM v7.
@rem


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 tln

set OUTFILE=%2

set firstX=%3
if "%firstX%"=="." set firstX=
if "%firstX%"=="" set firstX=0

set firstY=%4
if "%firstY%"=="." set firstY=
if "%firstY%"=="" set firstY=0

set DBG_TMPLT=%5
if "%DBG_TMPLT%"=="." set DBG_TMPLT=


set TMPDIR=\temp\
if not exist %TMPDIR% md %TMPDIR%

set TMPEXT=.miff


set SKEL=%TMPDIR%tl_skel%TMPEXT%

set LINES=%TMPDIR%tl_lines%TMPEXT%
set JCNS=%TMPDIR%tl_jcns%TMPEXT%
set JCNSROT=%TMPDIR%tl_jcns_rot%TMPEXT%
set ENDS=%TMPDIR%tl_ends%TMPEXT%
set LPBRK=%TMPDIR%tl_lpbrk%TMPEXT%

set LOOPS=%TMPDIR%tl_loops%TMPEXT%
set ONEJCN=%TMPDIR%tl_1jcn%TMPEXT%

set ENDS_LIST=%TMPDIR%tl_ends.lis
set PARTLIST=%TMPDIR%tl_part.txt

del %OUTFILE% 2>nul
del %PARTLIST% 2>nul

if "%DBG_TMPLT%"=="" (
  set wrSKEL=
) else (
  set wrSKEL=+write %DBG_TMPLT:XX=skel%
)

%IMG7%magick ^
  %INFILE% ^
  -alpha off ^
  -virtual-pixel Edge ^
  -morphology Thinning:-1 Skeleton -clamp ^
  -threshold 50%% ^
  -alpha off ^
  %wrSKEL% ^
  ( +clone ^
    -virtual-pixel Black ^
    -define morphology:compose=Darken ^
    -morphology Thinning ^
      "LineJunctions;LineJunctions:3>;LineJunctions:5" ^
    -clamp ^
    -morphology Thinning ^
      "3x3:0,0,0,0,1,0,0,0,0" ^
    -clamp -alpha off ^
    +write %LINES% ^
  ) ^
  -compose Difference -composite ^
  %JCNS%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %JCNS% ^
  -rotate 180 ^
  +write %JCNSROT% ^
  -format "WW=%%w\nHH=%%h\nWm1=%%[fx:w-1]\nHm1=%%[fx:h-1]" ^
  info:`) do set %%L

(
  echo width=%WW%
  echo height=%HH%
) >%OUTFILE%

%IMG7%magick ^
  %LINES% ^
  -virtual-pixel Black ^
  -define morphology:compose=Lighten ^
  -morphology HMT LineEnds;3x3:0,0,0,0,1,0,0,0,0 ^
  -clamp -alpha off ^
  %ENDS%

%IM7DEV%magick ^
  %ENDS% ^
  -process allwhite ^
  NULL: 2>%ENDS_LIST%

call %PICTBAT%fillCoords ^
  %LINES% ^
  %ENDS_LIST% ^
  black ^
  %LOOPS%

if ERRORLEVEL 1 exit /B 1

call %PICTBAT%clc2lpbrk ^
  %LOOPS% ^
  %LPBRK%

if ERRORLEVEL 1 exit /B 1

%IMG7%magick ^
  %LINES% ^
  ( %LPBRK% -negate ) ^
  -compose Darken -composite ^
  +write %LINES% ^
  -virtual-pixel Black ^
  -define morphology:compose=Lighten ^
  -morphology HMT LineEnds;3x3:0,0,0,0,1,0,0,0,0 ^
  -clamp -alpha off ^
  %ENDS%

if not "%DBG_TMPLT%"=="" (

  %IMG7%magick ^
    %LINES% -write %DBG_TMPLT:XX=lines% +delete ^
    %JCNS%  -write %DBG_TMPLT:XX=jcns%  +delete ^
    %ENDS%  -write %DBG_TMPLT:XX=ends%  +delete ^
    %LPBRK% -write %DBG_TMPLT:XX=lpbrk% +delete ^
    %LOOPS% -write %DBG_TMPLT:XX=loops% ^
    NULL:
)


set lastX=%firstX%
set lastY=%firstY%
set actEnds=0
set doTestExit=1

:loop

call :findNearest %ENDS% %lastX% %lastY%
if ERRORLEVEL 1 exit /B 1
if "%firstX%"=="none" goto noMore

rem Save first coord in case this is a loop.
set loopStart=%firstX%,%firstY%

set endX=%firstX%
set endY=%firstY%

if %doTestExit%==0 goto skipTestExit

rem If this line leaves a jcn, also include that. (Jcn exit.)
call :findNearest %JCNS% %endX% %endY%
if ERRORLEVEL 1 exit /B 1
set /A dxy=(%endX%-%firstX%)*(%endX%-%firstX%)+(%endY%-%firstY%)*(%endY%-%firstY%)
if /I %dxy% LEQ 2 (
  echo %0: trace jcn exit: end = %endX%,%endY%

  for /F "usebackq tokens=1,2 delims=, " %%X in (`%IM7DEV%magick ^
    %JCNS% ^
    ^( +clone ^
       -fill Black ^
       -draw "color %firstX%,%firstY% floodfill" ^
    ^) ^
    -compose Difference -composite ^
    -process midlightest ^
    %ONEJCN% 2^>^&1`) do (
    set MX=%%X
    set MY=%%Y
  )

  echo %0: trace jcn exit: !MX!,!MY! to %firstX%,%firstY% for %endX%,%endY%

  %IM7DEV%magick ^
    %ONEJCN% ^
    -negate -evaluate Add 1 ^
    -process 'darkestpntpnt s !MX!,!MY! e %firstX%,%firstY% t 0.5' ^
    -negate -evaluate Add 1 ^
    -process 'darkestpntpnt s !MX!,!MY! e %firstX%,%firstY% t 0.5 no_end p f stdout data' ^
    NULL: >>%PARTLIST%

  echo # jcn exit !MX!,!MY! to %firstX%,%firstY% for %endX%,%endY% >>%PARTLIST%
)

:skipTestExit
set doTestExit=1

call %PICTBAT%traceLn %LINES% %endX% %endY% %PARTLIST%

echo %0: Line traced: tlNumPts=%tlNumPts% tlLastX=%tlLastX% tlLastY=%tlLastY%

set lastX=%tlLastX%
set lastY=%tlLastY%

:: Remove both from ENDS.
%IMG7%magick ^
  %ENDS% ^
  -fill Black ^
  -draw "point %endX%,%endY% point %lastX%,%lastY%" ^
  %ENDS%


:: Find next coord from JCNS or LPBRK.

:: FIXME: optimize next stuff:
::   if jcn is neighbour, no need to find nearest loop brk, etc.

set /A xr=%Wm1%-%lastX%
set /A yr=%Hm1%-%lastY%
call :findNearest %JCNSROT% %xr% %yr%
if ERRORLEVEL 1 exit /B 1
if "%firstX%"=="none" (
  set sqJ=99999
) else (
  set /A nrJcnX=%Wm1%-%firstX%
  set /A nrJcnY=%Hm1%-%firstY%
  set /A "sqJ=(%lastX%-!nrJcnX!)*(%lastX%-!nrJcnX!)+(%lastY%-!nrJcnY!)*(%lastY%-!nrJcnY!)"
)

set sqB=99999

if /I %sqJ% GTR 2 (
  call :findNearest %LPBRK% %lastX% %lastY%
  if ERRORLEVEL 1 exit /B 1
  if "!firstX!"=="none" (
    set sqB=99999
  ) else (
    set nrBrkX=!firstX!
    set nrBrkY=!firstY!
    set /A "sqB=(%lastX%-!nrBrkX!)*(%lastX%-!nrBrkX!)+(%lastY%-!nrBrkY!)*(%lastY%-!nrBrkY!)"
  )
)

echo %0: sqJ=%sqJ% sqB=%sqB%

if /I %sqJ% LEQ 2 (
  echo %0: trace junction %nrJcnX%,%nrJcnY%
  call :traceJcn %nrJcnX% %nrJcnY%
  if ERRORLEVEL 1 exit /B 1
) else if /I %sqB% LEQ 2 (
  echo %0: trace loop-break %nrBrkX%,%nrBrkY%
  call %PICTBAT%traceLn %LPBRK% %nrBrkX% %nrBrkY% %PARTLIST%
  if ERRORLEVEL 1 exit /B 1

  echo %0: Loop-break: tlNumPts=!tlNumPts! tlLastX=!tlLastX! tlLastY=!tlLastY!
  set lastX=!tlLastX!
  set lastY=!tlLastY!

  rem Remove loop-break from %LPBRK%. There can only be two points?
  %IMG7%magick ^
    %LPBRK% ^
    -fill Black ^
    -draw "point %nrBrkX%,%nrBrkY% point !lastX!,!lastY!" ^
    %LPBRK%

  rem Repeat first coord of loop
  echo %loopStart% >>%PARTLIST%
  call :cumulCoords
  echo isLoop >>%OUTFILE%

) else (
  echo %0: trace next line %lastX%,%lastY%

  call :cumulCoords
)

if not %lastX%==none goto loop

:noMore

call :cumulCoords

:: Check that %ENDS% and %LPBRK% are entirely black.

for /F "usebackq" %%L in (`%IMG7%magick ^
  %ENDS% ^
  -format "minEnds=%%[fx:minima]\n" ^
  +write info: ^
  +delete ^
  %LPBRK% ^
  -format "minLpbrk=%%[fx:minima]\n" ^
  info:`) do set %%L

if not %minEnds%==0 (
  echo # minEnds=%minEnds% >>%OUTFILE%
  echo %0: minEnds=%minEnds%
  exit /B 1
)

if not %minLpbrk%==0 (
  echo # minLpbrk=%minLpbrk% >>%OUTFILE%
  echo %0: minLpbrk=%minLpbrk%
  exit /B 1
)

del %ENDS%
del %LPBRK%

echo # finished >>%OUTFILE%
echo %0: finished

call echoRestore

@endlocal & set tlnOUTFILE=%OUTFILE%

@exit /B 0

::-------------------------------------------------
:: Subroutines

:: Parameters: image file, centre x, center y.
:findNearest
set INTRO=
for /F "usebackq tokens=1-3 delims=:, " %%A in (`%IM7DEV%magick ^
  %1 ^
  -process 'nearestwhite cx %2 cy %3' ^
  NULL: 2^>^&1`) do (
  set INTRO=%%A
  set firstX=%%B
  set firstY=%%C
)
if ERRORLEVEL 1 exit /B 1

if not "%INTRO%"=="nearestwhite" exit /B 1

if "%firstY%"=="" set firstY=99999

rem echo %0: p1=%1 p2=%2 p3=%3 firstX=%firstX% firstY=%firstY%

exit /B 0

::-------------------------------------------------

:: Parameters: startX, startY
:: Traces path to center.
:: Returns coord on other side of center.
:: Assumes jcn is 4-connected.

:traceJcn

for /F "usebackq tokens=1,2 delims=, " %%X in (`%IM7DEV%magick ^
  %JCNS% ^
  ^( +clone ^
     -fill Black ^
     -draw "color %1,%2 floodfill" ^
  ^) ^
  -compose Difference -composite ^
  -process midlightest ^
  %ONEJCN% 2^>^&1`) do (
  set MX=%%X
  set MY=%%Y
)
echo %0: trace jcn part 1: %1,%2 to !MX!,!MY!
echo # jcn part 1: %1,%2 to !MX!,!MY! >>%PARTLIST%

set /A FX=2*!MX!-%1
set /A FY=2*!MY!-%2

rem echo %0: FX=!FX! FY=!FY!

set doit=1
if %1==!MX! if %2==!MY! set doit=0

if %doit%==0 (
  echo %1,%2 >>%PARTLIST%
) else (
  %IM7DEV%magick ^
    %ONEJCN% ^
    -negate -evaluate Add 1 ^
    -process 'darkestpntpnt s %1,%2 e !MX!,!MY! t 0.5' ^
    -negate -evaluate Add 1 ^
    -process 'darkestpntpnt s %1,%2 e !MX!,!MY! t 0.5 no_end p f stdout data' ^
    NULL: >>%PARTLIST%
)

call :cumulCoords

:: Find nearest end.
call :findNearest %ENDS% !FX! !FY!
set endX=!firstX!
set endY=!firstY!
:: If endXY is neighbour of the junction, trace from MXY to it.
call :findNearest %ONEJCN% !endX! !endY!
set /A dxy=(!endX!-%firstX%)*(!endX!-%firstX%)+(!endY!-%firstY%)*(!endY!-%firstY%)
if /I %dxy% LEQ 2 (

  set doTestExit=0

  set doit=1
  if !firstX!==!MX! if !firstY!==!MY! set doit=0

  if !doit!==0 (
    echo %0: trace jcn part 2: ... to !firstX!,!firstY! for %endX%,%endY%
    echo !firstX!,!firstY! >>%PARTLIST%
  ) else (
    echo %0: trace jcn part 2: %MX%,%MY% to !firstX!,!firstY! for %endX%,%endY%
    %IM7DEV%magick ^
      %ONEJCN% ^
      -negate -evaluate Add 1 ^
      -process 'darkestpntpnt s !MX!,!MY! e !firstX!,!firstY! t 0.5' ^
      -negate -evaluate Add 1 ^
      -process 'darkestpntpnt s !MX!,!MY! e !firstX!,!firstY! t 0.5 no_end p f stdout data' ^
      NULL: >>%PARTLIST%
  )
  echo # jcn part 2 !MX!,!MY! to !firstX!,!firstY! for %endX%,%endY% >>%PARTLIST%
) else (
  echo %0: no jcn part 2: endXY=%endX%,%endY%  firstXY=!firstX!,!firstY!  dxy=%dxy%
)

set lastX=%firstX%
set lastY=%firstY%

exit /B 0

::-------------------------------------------------

:cumulCoords

set numPts=0
if exist %PARTLIST% for /F %%L in (%PARTLIST%) do (
  set LINE=%%L
  if not "!LINE:~0,1!"=="#" set /A numPts+=1
)

if /I %numPts% GTR 0 (
  ( echo line
    echo numPts=%numPts%
    for /F "tokens=*" %%L in (%PARTLIST%) do echo %%L
  ) >>%OUTFILE%
)

del %PARTLIST% 2>nul

exit /B 0
