rem %1 is an image, white with black border.
rem %2 prefix for output environment variables.
rem %3 is WxH, required aspect ratio.
rem %4 mode, either "first" or "cumul".
rem Finds crop parameters.
@rem
@rem Also uses:
@rem   ttPREPROC pre-processing for the image eg "-alpha extract".
@rem   ttDEBUG if not blank, writes debugging image
@rem
@rem This script needs IM v7 for the debugging image.

@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

set INFILE=%1

set EvPref=%2
if "%EvPref%"=="." set EvPref=
if "%EvPref%"=="" set EvPref=tt

set Mode=%4
if "%Mode%"=="." set Mode=
if "%Mode%"=="" set Mode=first

if /I not %Mode%==first if /I not %Mode%==cumul (
  echo Mode [%Mode%] should be first or cumul.
)

set TMP_IN=%TEMP%\tt_tmp.miff

set WW=
for /F "usebackq" %%L in (`%IMG7%magick ^
  %INFILE% ^
  %ttPREPROC% ^
  -format "WW=%%w\nHH=%%h\n" ^
  -write info: ^
  %TMP_IN%`) do set %%L

if "%WW%"=="" exit /B 1

echo WW=%WW% HH=%HH%

set reqAR=%3
if "%reqAR%"=="." set reqAR=
set x0=0
set y0=0
if not "%reqAR%"=="" (
  echo reqAR=%reqAR%
  call parseXxY %WW% %HH% %reqAR%

  set raW=!pxxyX!
  set raH=!pxxyY!

  echo raW=!raW! raH=!raH!

  set /A WWraH=%WW%*!raH!
  set /A HHraW=%HH%*!raW!

  echo WWraH=!WWraH! HHraW=!HHraW!

  if !WWraH! GTR !HHraW! (
    echo required is taller
    set /A rqH=%WW%*!raH!/!raW!
    echo rqH=!rqH!
    set /A x0=^(%WW% - %HH%^*!raW!/!raH!^)/2
  ) else if !WWraH! LSS !HHraW! (
    echo required is wider
    set /A rqW=%HH%*!raW!/!raH!
    set /A extraW=^(!rqW!-%WW%^)/2
    echo rqW=!rqW! extraW=!extraW!
    set /A y0x=!extraW!*%HH%/!rqW!
    set /A y0=^(%HH% - %WW%^*!raH!/!raW!^)/2
  )
)

echo x0=%x0% y0=%y0% y0x=%y0x%

if "%x0%"=="." set x0=
if "%x0%"=="" set x0=0
if "%y0%"=="." set y0=
if "%y0%"=="" set y0=0

set x1=
set y1=
for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "x1=%%[fx:floor((w-1)/2)]\ny1=%%[fx:floor((h-1)/2)]" ^
  %TMP_IN%`) do set %%L

rem echo x1=%x1% y1=%y1%

if "%x1%"=="" exit /B 1

rem Referece: Bresenham

set /A dx=%x1%-%x0%
set /A dy=%y1%-%y0%
set yi=1
if %dy% LSS 0 (
  set yi=-1
  set dy=-dy
)

set xi=1
if %dx% LSS 0 (
  set xi=-1
  set dx=-dx
)

set x0img=%x0%
set y0img=%y0%

set rev=0
if %dy% GTR %dx% (
  set rev=1

  set t=%x0%
  set x0=%y0%
  set y0=!t!

  set t=%x1%
  set x1=%y1%
  set y1=!t!

  set t=%dx%
  set dx=%dy%
  set dy=!t!
)

set /A D=2*%dy%-%dx%
set /A y=%y0%

set fndTL=0
set fndTR=0
set fndBL=0
set fndBR=0
set fndAll=0

echo off

for /L %%X in (%x0%,%xi%,%x1%) do (
  if !fndAll!==0 (
    if %rev%==0 (
      set imgx=%%X
      set imgy=!y!
    ) else (
      set imgx=!y!
      set imgy=%%X
    )
    for /F "usebackq" %%L in (`%IMG7%magick ^
      %TMP_IN% ^
      -format "vTL=%%[fx:p{!imgx!,!imgy!}]\nvTR=%%[fx:p{w-1-!imgx!,!imgy!}]\nvBL=%%[fx:p{!imgx!,h-1-!imgy!}]\nvBR=%%[fx:p{w-1-!imgx!,h-1-!imgy!}]\n" ^
      info:`) do set %%L

    rem echo !D!  %%X,!y!  rev=%rev% vTL=!vTL! vTR=!vTR! vBL=!vBL! vBR=!vBR!

    if !fndTL!==0 if !vTL!==1 (
      set fndTL=1
      set fndTL.x=!imgx!
      set fndTL.y=!imgy!
    )
    if !fndTR!==0 if !vTR!==1 (
      set fndTR=1
      set fndTR.x=!imgx!
      set fndTR.y=!imgy!
    )
    if !fndBL!==0 if !vBL!==1 (
      set fndBL=1
      set fndBL.x=!imgx!
      set fndBL.y=!imgy!
    )
    if !fndBR!==0 if !vBR!==1 (
      set fndBR=1
      set fndBR.x=!imgx!
      set fndBR.y=!imgy!
    )

    set /A fndAll=!fndTL!*!fndTR!*!fndBL!*!fndBR!

    if !D! GTR 0 (
      set /A y+=%yi%
      set /A D-=2*%dx%
    )
    set /A D+=2*%dy%
  )
)

set Bad=0
if %fndTL%==0 (
  echo %0: TL not found
  set Bad=1
)
if %fndTR%==0 (
  echo %0: TR not found
  set Bad=1
)
if %fndBL%==0 (
  echo %0: BL not found
  set Bad=1
)
if %fndBR%==0 (
  echo %0: BR not found
  set Bad=1
)
if %Bad%==1 exit /B 1

set Left=%fndTL.x%
set Top=%fndTL.y%
set Right=%fndBR.x%
set Bottom=%fndBR.y%

if %Left% LSS %fndBL.x% set Left=%fndBL.x%
if %Top% LSS %fndTR.y% set Top=%fndTR.y%
if %Right% LSS %fndTR.x% set Right=%fndTR.x%
if %Bottom% LSS %fndBL.y% set Bottom=%fndBL.y%

echo LTRB = %Left% %Top% %Right% %Bottom%

rem We could extend the crop here.

if /I %Mode%==cumul (
  if %Left% LSS !%EvPref%.Left! set Left=!%EvPref%.Left!
  if %Top% LSS !%EvPref%.Top! set Top=!%EvPref%.Top!
  if %Right% LSS !%EvPref%.Right! set Right=!%EvPref%.Right!
  if %Bottom% LSS !%EvPref%.Bottom! set Bottom=!%EvPref%.Bottom!
)

set /A nW=%WW%-%Left%-%Right%
set /A nH=%HH%-%Top%-%Bottom%

if not "%ttDEBUG%"=="" %IMG7%magick ^
  %INFILE% ^
  -fill #0f04 -stroke None ^
  -draw "rectangle %Left%,%Top%,%%[fx:w-1-%Right%],%%[fx:h-1-%Bottom%] " ^
  -fill None -stroke #0f0 ^
  -draw "line %x0img%,%y0img%,%fndTL.x%,%fndTL.y%" ^
  -draw "line %%[fx:w-1-%x0img%],%y0img%,%%[fx:w-1-%fndTR.x%],%fndTR.y%" ^
  -draw "line %x0img%,%%[fx:h-1-%y0img%],%fndBL.x%,%%[fx:h-1-%fndBL.y%]" ^
  -draw "line %%[fx:w-1-%x0img%],%%[fx:h-1-%y0img%],%%[fx:w-1-%fndBR.x%],%%[fx:h-1-%fndBR.y%]" ^
  %ttDEBUG%

set sCrop=%nW%x%nH%+%Left%+%Top%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %TMP_IN% ^
  -crop %sCrop% +repage ^
  -format "okay=%%[fx:minima==1?1:0]" ^
  info:`) do set %%L

if %nW% LSS 0 set okay=0
if %nH% LSS 0 set okay=0

if /I %Mode%==cumul if !%EvPref%.Okay!==0 set okay=0


call echoRestore

@endlocal & set %EvPref%.Left=%Left%& set %EvPref%.Top=%Top%& set %EvPref%.Right=%Right%& set %EvPref%.Bottom=%Bottom%& set /A %EvPref%.W=%nW%& set /A %EvPref%.H=%nH%& set %EvPref%.Crop=%sCrop%& set %EvPref%.Okay=%okay%
