rem Given %1 is a directory containing just test images,
rem %2 is a destination directory,
rem creates tweaked versions of images in %2.
@rem
@rem Updated:
@rem   3-September-2022 for IM v7.
@rem

set OUTDIR=%2

set EXT=png

for %%F in (%1\*) do (
  echo %%F

  set BASE=%%~nF

  copy /y %%F %OUTDIR%\%%~nxF

  for %%V in (-20 -15 -10 -5 5 10 15 20) do %IMG7%magick ^
    %%F ^
    +depth ^
    ^( +clone ^
       -brightness-contrast %%V,0 ^
       +write %OUTDIR%\!BASE!_bri_%%V.%EXT% ^
       +delete ^
    ^) ^
    -brightness-contrast 0,%%V ^
    %OUTDIR%\!BASE!_con_%%V.%EXT%

  for %%V in (0.75 0.8 0.85 0.9 0.95 1.05 1.1 1.15 1.2 1.25) do %IMG7%magick ^
    %%F ^
    -gamma %%V ^
    -set gamma 0.454545 ^
    +depth ^
    %OUTDIR%\!BASE!_gam_%%V.%EXT%

  for %%V in (0.5 1 2 3) do %IMG7%magick ^
    %%F ^
    -blur 0x%%V ^
    +depth ^
    %OUTDIR%\!BASE!_blr_%%V.%EXT%

  for %%V in (5 10 25 50 75 100) do %IMG7%magick ^
    %%F ^
    -quality %%V ^
    %OUTDIR%\!BASE!_jpg_%%V.jpg

  rem Colour gaussian noise.
  for %%V in (0.5 1 1.5 2 2.5 3) do %IMG7%magick ^
    -seed 1234 ^
    %%F ^
    -attenuate %%V +noise gaussian ^
    +depth ^
    %OUTDIR%\!BASE!_nsc_%%V.%EXT%

  rem Gray gaussian noise.
  for %%V in (0.5 1 1.5 2 2.5 3) do %IMG7%magick ^
    -seed 1234 ^
    %%F ^
    ^( +clone ^
       -fill gray^(50%%^) -colorize 100 ^
       -attenuate %%V +noise gaussian ^
       -modulate 100,0,100 ^
    ^) ^
    -compose hardlight -composite ^
    +depth ^
    %OUTDIR%\!BASE!_nsg_%%V.%EXT%

  for %%V in (25 50 75 150 200 400) do %IMG7%magick ^
    %%F ^
    -resize %%V%% ^
    %OUTDIR%\!BASE!_res_%%V.%EXT%

  for %%V in (5 15 30 45 90) do %IMG7%magick ^
    %%F ^
    -background black ^
    ^( +clone ^
       -rotate %%V +repage ^
       -gravity Center ^
       -crop 50%%x50%%+0+0 +repage ^
       +write %OUTDIR%\!BASE!_rot_%%V.%EXT% ^
       +delete ^
    ^) ^
    -rotate -%%V +repage ^
    -gravity Center ^
    -crop 50%%x50%%+0+0 +repage ^
    %OUTDIR%\!BASE!_rot_-%%V.%EXT%

  for %%V in (90 95 97.5 102.5 105 110) do %IMG7%magick ^
    %%F ^
    -modulate 100,100,%%V ^
    %OUTDIR%\!BASE!_hue_%%V.%EXT%

  for %%V in (70 80 90) do %IMG7%magick ^
    %%F ^
    -gravity Center ^
    ^( +clone ^
       -crop %%V%%x100%%+0+0 +repage ^
       +write %OUTDIR%\!BASE!_crpv_%%V.%EXT% ^
       +delete ^
    ^) ^
    -crop 100%%x%%V%%+0+0 +repage ^
    %OUTDIR%\!BASE!_crph_%%V.%EXT%

  %IMG7%magick ^
    %%F ^
    ^( +clone ^
       -flip ^
       +write %OUTDIR%\!BASE!_flip.%EXT% ^
       +delete ^
    ^) ^
    ^( +clone ^
       -flop ^
       +write %OUTDIR%\!BASE!_flop.%EXT% ^
       +delete ^
    ^) ^
    ^( +clone ^
       -transpose ^
       +write %OUTDIR%\!BASE!_trnsp.%EXT% ^
       +delete ^
    ^) ^
    ^( +clone ^
       -transverse ^
       +write %OUTDIR%\!BASE!_trnsv.%EXT% ^
       +delete ^
    ^) ^
    ^( +clone ^
       -gravity Center -crop 50%%x50%%+0+0 +repage ^
       +write %OUTDIR%\!BASE!-half.%EXT% ^
       +delete ^
    ^) ^
    NULL:

)

rem Note: "half" has hyphen, not underscore.
