rem Given %1 is three text files,
rem  each with x, y, v
rem    x,y are integers
rem    v is floating point, typically 0 to QuantumRange
rem makes three grayscale images,
rem and combines them into %2 output image.
rem %3 desired width "x" height of output, eg 600x400
rem %4 "extend" or "noextend".
rem %5 number of channels, 1 to 3 [default 3]


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave

call %PICTBAT%setInOut %1 c2x

if not "%2"=="" if not "%2"=="." set OUTFILE=%2

set WxH=%3
if "%WxH%"=="." set WxH=

set doExt=%4
if "%doExt%"=="." set doExt=
if "%doExt%"=="" set doExt=noextend

set NumChan=%5
if "%NumChan%"=="." set NumChan=
if "%NumChan%"=="" set NumChan=3

call parseXxY 100 100 %WxH%
set reqW=%pxxyX%
set reqH=%pxxyY%

set INFILE0=%~n1_0%~x1
set INFILE1=%~n1_1%~x1
set INFILE2=%~n1_2%~x1

set TMPDIR=\temp\

set TMPTEXT=\temp\t2c3c.txt

echo %0: ss3cFirstX=%ss3cFirstX%

set /A WW=%ss3cLastX%-%ss3cFirstX%+1
set /A HH=%ss3cLastY%-%ss3cFirstY%+1

if %doExt%==extend (
  set EXTDIST=-virtual-pixel Edge ^
-set option:distort:viewport %reqW%x%reqH%-%ss3cFirstX%-%ss3cFirstY% ^
-distort SRT 1,0 +repage
) else (
  set EXTDIST=
)

for /F "usebackq" %%A in (`%IMG7%magick identify ^
    -precision 19 ^
    -format "%%[fx:QuantumRange]" ^
    xc:`) do set QR=%%A

call :mkGrayImg %INFILE0% x0.miff
if ERRORLEVEL 1 exit /B 1

set IN1=( +clone -fill gray(50%%) -colorize 100 )
set IN2=( +clone -fill gray(50%%) -colorize 100 )

if %NumChan% GTR 1 (
  set IN1=x1.miff
  call :mkGrayImg %INFILE1% !IN1!
  if ERRORLEVEL 1 exit /B 1

  if %NumChan% GTR 2 (
    set IN2=x2.miff
    call :mkGrayImg %INFILE2% !IN2!
    if ERRORLEVEL 1 exit /B 1
  )
)

%IMG7%magick ^
  x0.miff ^
  %IN1% ^
  %IN2% ^
  -combine ^
  %OUTFILE%
if ERRORLEVEL 1 exit /B 1

echo %0: OUTFILE=%OUTFILE%

call echoRestore

@endlocal & set c2xOUTFILE=%OUTFILE%

@exit /B 0

::------------------------------------------
:: Subroutines

:mkGrayImg
set InText=%1
set OutImg=%2

echo %0: InText %InText%

set SORTED=\temp\t2c_%~n1.srt
set SORTED2=\temp\t2c2_%~n1.srt
set SORTED3=\temp\t2c3_%~n1.srt

(
  echo P2
  echo %WW% %HH%
  echo %QR%
) >%SORTED2%

:: Strip comments and empty lines; sort on y then x.
:: grep: we double the ^ for Windows.

grep -v -e '^^[[:space:]]*$' -e '^^#' %InText% |sort -g --key=1 |sort -g --key=2 -s -o %SORTED%

:: Get the third number. Also change "-0" to "0".
sed -e 's/  / /g' -e 's/-0/0/g' %SORTED% |cut -d ' ' -f 3 >>%SORTED2%

:: Clamp negative values to 0.
(
  for /F "tokens=*" %%L in (%SORTED2%) do if %%L LSS 0 (echo 0) else echo %%L
) >%SORTED3%

goto skipSlow

:: Next is slow: we call identify to calculate for every pixel.

(
  echo # ImageMagick pixel enumeration: %WW%,%HH%,%QR%,graya

  for /F "eol=# tokens=1-3 delims=, " %%A in (%SORTED%) do (
    for /F "usebackq" %%L in (`%IMG7%magick identify ^
      -precision 19 ^
      -format "VAL=%%[fx:%%C*QuantumRange/100]" ^
      xc:`) do set %%L

    echo %%A,%%B: ^(!VAL!,%QR%^)
  )
) >%TMPTEXT%

:skipSlow

rem echo %0: TMPTEXT %TMPTEXT%

rem type %TMPTEXT%

rem Don't read from   TXT:%TMPTEXT% ^

%IMG7%magick ^
  PNM:%SORTED3% ^
  -crop %WW%x%HH%+%ss3cFirstX%+%ss3cFirstY% +repage ^
  %EXTDIST% ^
  %OutImg%

if ERRORLEVEL 1 exit /B 1

exit /B 0
