rem Given image %1 was transformed by "+level L1xL2",
rem and we have the same-size result image %2,
rem what are the parameters L1 and L2?
rem Write parameters to environment variables prefixed with %3.
rem %4 quoted x,y coords of pixel in both images.
rem %5 quoted x,y coords of another pixel in both images.
@rem
@rem Also uses:
@rem   wlPreProc  Pre-process for both inputs.
@rem

rem We compare corresponding pixels in the two images.
rem The pixels must be different, and must not have been clamped.

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

rem @call echoOffSave

call %PICTBAT%setInOut %1 wl

set In1=%1
set In2=%2

set Coords1=%4
if [%Coords1%]==[.] set Coords1=
if [%Coords1%]==[] set Coords1="0,0"

set Coords2=%5
if [%Coords2%]==[.] set Coords2=
if [%Coords2%]==[] set Coords2="-1,-1"

call %UTIL%\parseCommaList %Coords1% Cds1 Cds1
if not %Cds1%==2 (
  echo %0: Coords1 [%Coords1%] should be two comma-separated numbers.
  exit /B 1
)
call %UTIL%\parseCommaList %Coords2% Cds2 Cds2
if not %Cds2%==2 (
  echo %0: Coords2 [%Coords2%] should be two comma-separated numbers.
  exit /B 1
)

set X1=%Cds1[0]%
set Y1=%Cds1[1]%
set X2=%Cds2[0]%
set Y2=%Cds1[1]%

for /F "usebackq" %%L in (`%IMG7%magick ^
  -precision 15 ^
  -define "compose:clamp=off" ^
  %In1% %In2% ^
  %wlPreProc% ^
  ^( -clone 0-1 ^
     -crop "1x1+%%[fx:mod(%X1%,w)]+%%[fx:mod(%Y1%,h)]" +repage ^
     ^( -clone 0 ^
        -define ftxt:format^="vin0=\s\n" -write ftxt: ^
        -write mpr:vin0 ^
        +delete ^
     ^) ^
     ^( -clone 1 ^
        -define ftxt:format^="vout0=\s\n" -write ftxt: ^
        -write mpr:vout0 ^
        +delete ^
     ^) ^
     -delete 0--1 ^
  ^) ^
  ^( -clone 0-1 ^
     -crop "1x1+%%[fx:mod(%X2%,w)]+%%[fx:mod(%Y2%,h)]" +repage ^
     ^( -clone 0 ^
        -define ftxt:format^="vin1=\s\n" -write ftxt: ^
        -write mpr:vin1 ^
        +delete ^
     ^) ^
     ^( -clone 1 ^
        -define ftxt:format^="vout1=\s\n" -write ftxt: ^
        -write mpr:vout1 ^
        +delete ^
     ^) ^
     -delete 0--1 ^
  ^) ^
  -delete 0--1 ^
  mpr:vout0 mpr:vout1 ^
  -compose MinusSrc -composite ^
  ^( mpr:vin0 mpr:vin1 ^
     -compose MinusSrc -composite ^
  ^) ^
  -compose DivideSrc -composite ^
  -define ftxt:format^="L2mL1=\s\n" -write ftxt: ^
  +write mpr:L2mL1 ^
  mpr:vin0 ^
  -compose Multiply -composite ^
  mpr:vout0 ^
  -compose MinusDst -composite ^
  -define ftxt:format^="L1=\s\n" -write ftxt: ^
  +write mpr:L1 ^
  mpr:L2mL1 ^
  -compose Plus -composite ^
  -define ftxt:format^="L2=\s\n" -write ftxt: ^
  null:`) do set %%L

echo %0: vin0: %vin0%
echo %0: vout0: %vout0%
echo %0: vin1: %vin1%
echo %0: vout1: %vout1%

exit /B 0


set FMT=^
vindr=%%[fx:(%vin0r%-(%vin1r%))]\n^
vindg=%%[fx:(%vin0g%-(%vin1g%))]\n^
vindb=%%[fx:(%vin0b%-(%vin1b%))]\n^
voutdr=%%[fx:(%vout0r%-(%vout1r%))]\n^
voutdg=%%[fx:(%vout0g%-(%vout1g%))]\n^
voutdb=%%[fx:(%vout0b%-(%vout1b%))]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 15 ^
  -format "%FMT%" ^
  xc:`) do set %%L

echo %0: vind: %vindr% %vindg% %vindb%
echo %0: voutd: %voutdr% %voutdg% %voutdb%

if %voutdr%==0 (
  echo voutdr is zero
  exit /B 1
)
if %voutdg%==0 (
  echo voutdg is zero
  exit /B 1
)
if %voutdb%==0 (
  echo voutdb is zero
  exit /B 1
)

set FMT=^
L2mL1r=%%[fx:100*(%vin0r%-(%vin1r%))/(%vout0r%-(%vout1r%))]\n^
L2mL1g=%%[fx:100*(%vin0g%-(%vin1g%))/(%vout0g%-(%vout1g%))]\n^
L2mL1b=%%[fx:100*(%vin0b%-(%vin1b%))/(%vout0b%-(%vout1b%))]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 15 ^
  -format "%FMT%" ^
  xc:`) do set %%L

echo %0: L2mL1: %L2mL1r% %L2mL1g% %L2mL1b%

set FMT=^
L1r=%%[fx:(%vin0r%*100-(%vout0r%)*(%L2mL1r%))]\n^
L1g=%%[fx:(%vin0g%*100-(%vout0g%)*(%L2mL1g%))]\n^
L1b=%%[fx:(%vin0b%*100-(%vout0b%)*(%L2mL1b%))]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 15 ^
  -format "%FMT%" ^
  xc:`) do set %%L

echo %0: L1: %L1r% %L1g% %L1b%

set FMT=^
L2r=%%[fx:(%L2mL1r%+(%L1r%))]\n^
L2g=%%[fx:(%L2mL1g%+(%L1g%))]\n^
L2b=%%[fx:(%L2mL1b%+(%L1b%))]\n

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 15 ^
  -format "%FMT%" ^
  xc:`) do set %%L

echo %0: L2: %L2r% %L2g% %L2b%


set strLevel=^
-channel R -level %L1r%x%L2r%%% ^
-channel G -level %L1g%x%L2g%%% ^
-channel B -level %L1b%x%L2b%%% ^
+channel

echo %0: strLevel: %strLevel%

rem Verify: if we level with these numbers, do we get the expected result?

%IMG7%magick ^
  %In1% ^
  %strLevel% ^
  %In2% ^
  -metric RMSE ^
  -format "%%[distortion]\n" -compare ^
  info:

call echoRestore

@endlocal & set %3_L1_R=%L1r%& set %3_L1_G=%L1g%& set %3_L1_B=%L1b%&^
set %3_L2_R=%L2r%& set %3_L2_G=%L2g%& set %3_L2_B=%L2b%&^
set %3_strLevel=%strLevel%

