rem Given same-size images %1 and %2,
rem finds angle of rotation for %1 to best match %2, and a score.
@rem
@rem Also uses:
@rem   wrDO_SUPER      if ==0, no supersampling will be done.
@rem   wrDO_SCORE_SUB  if ==0, wrSCORE_SUB will not be found.
@rem   wrMETRIC        default RMSE
@rem   wrDEBUG         if 1, creates images for debugging
@rem   wrDELTEMP       if not 0, will remove temporary file.
@rem   wrCROP_PC       percentage for central crops before unrolling [100].
@rem
@rem Returns:
@rem   wrANGLE     (degrees) rotate first image by this to match second.
@rem   wrANGLE_ERR (degrees) margin of error
@rem   wrSCORE     (0.0 to 1.0, more or less).
@rem   wrSCORE_SUB (0.0 to 1.0, more or less).
@rem   wrDODGY     0 or 1. 1 means the result is unreliable.
@rem
@rem Updated:
@rem   19-August-2016 Added wrCROP_PC.
@rem   10-August-2022 for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave


if "%wrCROP_PC%"=="" set wrCROP_PC=100

set TMPEXT=.miff

call %PICTBAT%setInOut %1 wr
set IN_A=%INFILE%
set TEMP_A=%TEMP%\%~n1_wr_x2%TMPEXT%

call %PICTBAT%setInOut %2 wr
set IN_B=%INFILE%
set TEMP_B=%TEMP%\%~n2_wr%TMPEXT%
if %IN_B%==%IN_A% set TEMP_B=%TEMP%\%~n2_B_wr%TMPEXT%

set DEBUG_FILE=wr_%~n1_%~n2_dbg%EXT%

if not "%wrDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
  del %TEMP_B% 2>nul
)

if "%wrMETRIC%"=="" set wrMETRIC=RMSE

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW1=%%[fx:w*%wrCROP_PC%/100]\nHH1=%%[fx:h*%wrCROP_PC%/100]\nIS_SMALL=%%[fx:max(w,h)*%wrCROP_PC%/100>1000?0:1]" ^
  %IN_A%`) do set %%L

if "%wrDO_SCORE_SUB%"=="" set wrDO_SCORE_SUB=%IS_SMALL%

if "%wrDO_SUPER%"=="" set wrDO_SUPER=%IS_SMALL%

if %wrDO_SUPER%==1 (
  set SUPER_UP=-set option:distort:scale 4
  set SUPER_DN=-resize 100x25%%
  set SUPER_FACT=4
) else (
  set SUPER_UP=
  set SUPER_DN=
  set SUPER_FACT=1
)

echo %0: SUPER_FACT=%SUPER_FACT% WW1=%WW1% HH1=%HH1%


if %wrCROP_PC%==100 (
  set sCROP=
) else (
  set sCROP=-gravity center -crop %wrCROP_PC%%%x%wrCROP_PC%%%+0+0 +repage
)

if not exist %TEMP_A% %IMG7%magick ^
  %IN_A% ^
  %sCROP% ^
  %SUPER_UP% ^
  -distort DePolar 0 ^
  %SUPER_DN% ^
  ^( +clone ^) +append +repage ^
  +depth ^
  %TEMP_A%

if not exist %TEMP_B% %IMG7%magick ^
  %IN_B% ^
  %sCROP% ^
  %SUPER_UP% ^
  -distort DePolar 0 ^
  %SUPER_DN% ^
  +depth ^
  %TEMP_B%

call %PICTBAT%srchImg %TEMP_A% %TEMP_B%
if ERRORLEVEL 1 exit /B 1

set DODGY=0

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 9 ^
  -format "ANGLE=%%[fx:%siCOMP_XW%*360/(%SUPER_FACT%*%WW1%)]\nANGLE_ERR=%%[fx:360/(%SUPER_FACT%*%WW1%)]" ^
  xc:`) do set %%L

if %ANGLE%==360 set ANGLE=0

if %wrDO_SCORE_SUB%==1 (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "DIM_2=%%[fx:int(%WW1%<%HH1%?%WW1%/2:%HH1%/2)]" ^
    xc:`) do set %%L

  for /F "usebackq" %%L in (`%IMG7%magick ^
    %IN_A% ^
    -rotate %ANGLE% +repage ^
    %IN_B% ^
    -gravity Center ^
    -crop !DIM_2!x!DIM_2!+0+0 +repage ^
    -metric %wrMETRIC% -format "ScoreSub=%%[distortion]" -compare ^
    info:`) do set %%L
)


if "%wrDEBUG%"=="1" (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "DIM_2=%%[fx:int(%WW1%<%HH1%?%WW1%/2:%HH1%/2)]" ^
    xc:`) do set %%L

  %IMG7%magick ^
  %IN_A% ^
  ^( +clone -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #f00 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -compose Over -composite ^
  -virtual-pixel None ^
  +distort SRT 1,%ANGLE% ^
  ^( %IN_B% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #0f0 -frame 1x1 ^
       -channel A -evaluate Multiply 0.75 ^
    ^) ^
    -composite ^
  ^) ^
  ^( -clone -1 ^
     -gravity Center ^
     -crop !DIM_2!x!DIM_2!+0+0 ^
     -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #ff0 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -layers merge ^
  +repage ^
  %DEBUG_FILE%
)

if not "%wrDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
  del %TEMP_B% 2>nul
)

call echoRestore

endlocal & set wrANGLE=%ANGLE%& set wrANGLE_ERR=%ANGLE_ERR%& set wrSCORE=%siCOMP_FLT%& set wrSCORE_SUB=%ScoreSub%& set wrDODGY=%DODGY%
