@rem Given same-size images %1 and %2,
@rem finds scale S and rotation angle R in "-distort SRT S,R" for %1 to best match %2, and a score.

@rem Parameters:
@rem   %3 min scale (default 0.5) OR "again" to iterate from previous run.
@rem   %4 max scale (default 1.5)
@rem   %5 number of scale steps (default 100)
@rem   %6
@rem   %7
@rem
@rem Also uses:
@rem   wrsDO_SCORE_SUB  if 0, wrsSCORE_SUB will not be found.
@rem   wrsMETRIC        default RMSE
@rem   wrsDO_SUPER      if 0, no supersampling will be done.
@rem   wrsDEBUG         if 1, creates images for debugging
@rem
@rem Returns:
@rem   wrsANGLE     (degrees)
@rem   wrsSCALE
@rem   wrsFACT      error margin, multiple of wrsSCALE
@rem   wrsSCORE     (0.0 to 1.0, more or less).
@rem   wrsSCORE_SUB (0.0 to 1.0, more or less).
@rem
@rem Updated:
@rem   10-August-2022 for IM v7.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave


if "%wrsMETRIC%"=="" set wrsMETRIC=RMSE

if "%wrsDO_SCORE_SUB%"=="" set wrsDO_SCORE_SUB=1

set wrsMIN_SCALE=%3
if "%wrsMIN_SCALE%"=="." set wrsMIN_SCALE=
if "%wrsMIN_SCALE%"=="" set wrsMIN_SCALE=0.5

set wrsMAX_SCALE=%4
if "%wrsMAX_SCALE%"=="." set wrsMAX_SCALE=
if "%wrsMAX_SCALE%"=="" set wrsMAX_SCALE=1.5

set wrsnSTEPS=%5
if "%wrsnSTEPS%"=="." set wrsnSTEPS=
if "%wrsnSTEPS%"=="" set wrsnSTEPS=100

set wrsOFFS_X=%6
if "%wrsOFFS_X%"=="." set wrsOFFS_X=
if "%wrsOFFS_X%"=="" set wrsOFFS_X=0

set wrsOFFS_Y=%7
if "%wrsOFFS_Y%"=="." set wrsOFFS_Y=
if "%wrsOFFS_Y%"=="" set wrsOFFS_Y=0

if "%wrsDO_SUPER%"=="" set wrsDO_SUPER=1

set wrsDEPTH=+depth


if /I "%wrsMIN_SCALE%" EQU "again" (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 9 ^
    -format "wrsMIN_SCALE=%%[fx:%wrsSCALE%/pow(%wrsFACT%,10)]\nwrsMAX_SCALE=%%[fx:%wrsSCALE%*pow(%wrsFACT%,10)]" ^
    xc:`) do set %%L
)

if "%wrsMIN_SCALE%"=="" exit /B 1

call %PICTBAT%setInOut %1 wrs
set IN_A=%INFILE%
set TEMP_A_BASE=%TEMP%\%~n1_wrs
rem FIXME: also offsets
set TEMP_A=%TEMP%\%~n1_wrs_%wrsMIN_SCALE%_%wrsMAX_SCALE%_%wrsnSTEPS%%EXT%

call %PICTBAT%setInOut %2 wrs
set IN_B=%INFILE%
set TEMP_B=%TEMP%\%~n2_wrs%EXT%
if %IN_B%==%IN_A% set TEMP_B=%TEMP%\%~n2_B_wrs%EXT%

set DEBUG_FILE=wrs_%~n1_%~n2_dbg%EXT%


for /F "usebackq" %%L ^
in (`%IMG7%magick %IN_A% ^
    -format "WW1=%%w\nHH1=%%h\nWW1_2=%%[fx:w/2]\nHH1_2=%%[fx:h/2]\nSRT_OFFS_X=%%[fx:w/2+(%wrsOFFS_X%)]\nSRT_OFFS_Y=%%[fx:h/2+(%wrsOFFS_Y%)]" ^
    INFO:`) ^
do set %%L

if "%WW1%"=="" exit /B 1

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
    -precision 9 ^
    -format "FACT=%%[fx:pow(%wrsMAX_SCALE%/%wrsMIN_SCALE%,1/%wrsnSTEPS%)]" ^
    xc:`) ^
do set %%L


if %wrsDO_SUPER%==1 (
  set SUPER_UP=-set option:distort:scale 4
  set SUPER_FACT=4
) else (
  set SUPER_UP=
  set SUPER_FACT=1
)

if exist %TEMP_A% goto skipTempA

set SCALE=%wrsMIN_SCALE%
set NAMES_A=
for /L %%i in (1,1,%wrsnSTEPS%) do (
  @rem Maybe we should crop all scales so we never have virtual pixels.

  %IMG7%magick ^
  %IN_A% ^
  -distort SRT %WW1_2%,%HH1_2%,!SCALE!,0,%SRT_OFFS_X%,%SRT_OFFS_Y% ^
  %SUPER_UP% ^
  -distort DePolar 0 ^
  -scale "x1^!" ^
  ^( +clone ^) +append +repage ^
  %wrsDEPTH% ^
  %TEMP_A_BASE%_%%i%EXT%

  set NAMES_A=!NAMES_A! %TEMP_A_BASE%_%%i%EXT%

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 9 -format "SCALE=%%[fx:!SCALE!*%FACT%]" ^
    xc:`) do set %%L
)

%IMG7%magick %NAMES_A% -append +repage %TEMP_A%

:skipTempA

%IMG7%magick identify %TEMP_A%


if not exist %TEMP_B% %IMG7%magick ^
  %IN_B% ^
  %SUPER_UP% ^
  -distort DePolar 0 ^
  -scale "x1^!" ^
  %wrDEPTH% %TEMP_B%


for /F "usebackq tokens=1-4 delims=()@, " %%R ^
in (`%IMG7%magick compare ^
  %TEMP_A% %TEMP_B% ^
  -similarity-threshold 0 -dissimilarity-threshold 1 -subimage-search ^
  -metric %wrsMETRIC% -subimage-search ^
  NULL: 2^>^&1`) ^
do (
  set COMP_INT=%%R
  set COMP_FLT=%%S
  set COMP_XW=%%T
  set COMP_YW=%%U
)

if "%COMP_FLT%" GTR "9" exit /B 1

echo COMP_FLT=%COMP_FLT% COMP_XW=%COMP_XW% COMP_YW=%COMP_YW%

set DODGY=0
if %COMP_YW% EQU 0 set DODGY=1
if %COMP_YW% GEQ %wrsnSTEPS% set DODGY=1

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
  -precision 9 ^
  -format "ANGLE=%%[fx:%COMP_XW%*360/(%SUPER_FACT%*%WW1%)]\nANGLE_ERR=%%[fx:360/(%SUPER_FACT%*%WW1%)]" ^
  xc:`) ^
do set %%L

rem What is the magnification here?
set SCALE=%wrsMIN_SCALE%
@for /L %%i in (1,1,%COMP_YW%) do @for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 9 -format "SCALE=%%[fx:!SCALE!*%FACT%]" ^
    xc:`) do @set %%L


echo SCALE=%SCALE%


rem Cleanup: remove most temporary files.
@for /L %%i in (1,1,%wrsnSTEPS%) do @del %TEMP_A_BASE%_%%i%EXT% 2>nul


if %wrsDO_SCORE_SUB%==1 (
  rem Test result.

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "cropX=%%[fx:int(min(%WW1%,%WW1%*%SCALE%)/2)]\ncropY=%%[fx:int(min(%HH1%,%HH1%*%SCALE%)/2)]" ^
    xc:`) do set %%L

  for /F "usebackq" %%L in (`%IMG7%magick ^
    %IN_A% ^
    -distort SRT "%WW1_2%,%HH1_2%,%SCALE%,%ANGLE%,%SRT_OFFS_X%,%SRT_OFFS_Y%" ^
    %IN_B% ^
    -gravity Center ^
    -crop !cropX!x!cropY!+0+0 +repage ^
    -write r.png ^
    -metric %wrsMETRIC% -format "ScoreSub=%%[distortion]" -compare ^
    info:`) do set %%L
)


if "%wrsDEBUG%"=="1" (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "cropX=%%[fx:int(min(%WW1%,%WW1%*%SCALE%)/2)]\ncropY=%%[fx:int(min(%HH1%,%HH1%*%SCALE%)/2)]" ^
    xc:`) do set %%L

  %IMG7%magick ^
  %IN_A% ^
  ^( +clone -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #f00 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -composite ^
  -virtual-pixel None ^
  +distort SRT "%WW1_2%,%HH1_2%,%SCALE%,%ANGLE%,%SRT_OFFS_X%,%SRT_OFFS_Y%" ^
  ^( %IN_B% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #0f0 -frame 1x1 ^
       -channel A -evaluate Multiply 0.75 ^
    ^) ^
    -composite ^
  ^) ^
  ^( -clone -1 ^
     -gravity Center ^
     -crop !cropX!x!cropY!+0+0 ^
     -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #ff0 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -layers merge ^
  +repage ^
  %DEBUG_FILE%
)


call echoRestore

endlocal & set wrsANGLE=%ANGLE%& set wrsANGLE_ERR=%ANGLE_ERR%& set wrsSCALE=%SCALE%& set wrsFACT=%FACT%& set wrsSCORE=%COMP_FLT%& set wrsSCORE_SUB=%ScoreSub%& set wrsDODGY=%DODGY%
