rem Given same-size images %1 and %2, with white shapes on black background,
rem finds angle of rotation for %1 to best match %2, and a score.
rem %3 centre: CXxCY, or furthest or centroid or boundingbox.
rem      Assumes this is in white of mask, and can "see" entire shape boundary.
rem %4 centre for second image
rem %5 scale (supersampling) factor [default 1]
rem %6 normalisation method: none, autolevel, meanSD
rem %7 filename for debugging image [default: no debugging image]
@rem
@rem Also uses:
@rem   wrsMETRIC        default RMSE
@rem   wrsDELTEMP       if not 0, will remove temporary file.
@rem
@rem Also specify centre CX,CY?
@rem
@rem
@rem Returns:
@rem   wrsANGLE     (degrees) rotate first image by this to match second.
@rem   wrsANGLE_ERR (degrees) margin of error
@rem   wrsSCORE     (0.0 to 1.0, more or less).
@rem   wrsDODGY     0 or 1. 1 means the result is unreliable.
@rem

@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave


set TMPEXT=.miff

call %PICTBAT%setInOut %1 wrs
set IN_A=%INFILE%
set TEMP_A=%TEMP%\%~n1_wrs_x2%TMPEXT%

call %PICTBAT%setInOut %2 wrs
set IN_B=%INFILE%
set TEMP_B=%TEMP%\%~n2_wrs%TMPEXT%
if %IN_B%==%IN_A% set TEMP_B=%TEMP%\%~n2_B_wrs%TMPEXT%

set sCent=%3
if "%sCent%"=="." set sCent=
if "%sCent%"=="" set sCent=50cx50c

call %PICTBAT%shpCent %IN_A% %sCent% wrs

if ERRORLEVEL 1 (
  echo %0: Error in shpCent. sCent=%sCent%
  exit /B 1
)

set sCent2=%4
if "%sCent2%"=="." set sCent2=
if "%sCent2%"=="" set sCent2=%wrs_CX%x%wrs_CY%

call %PICTBAT%shpCent %IN_B% %sCent2% wrs2

if ERRORLEVEL 1 (
  echo %0: Error in shpCent. sCent2=%sCent2%
  exit /B 1
)

set sSCALE=%5
if "%sSCALE%"=="." set sSCALE=
if "%sSCALE%"=="" set sSCALE=1

set sNORM=%6
if "%sNORM%"=="." set sNORM=
if "%sNORM%"=="" set sNORM=none

if /I "%sNorm%"=="none" (
  set AUTLEV=
) else if /I "%sNorm%"=="autolevel" (
  set AUTLEV=-auto-level
) else if /I "%sNorm%"=="meansd" (
  set AUTLEV=
) else (
  echo %0: sNorm = %sNorm%
)

set sDEBUG=%7
if "%sDEBUG%"=="." set sDEBUG=

set CX=%wrs_CX%
set CY=%wrs_CY%
set CX2=%wrs2_CX%
set CY2=%wrs2_CY%

echo %0: %CX%,%CY% %CX2%,%CY2%

rem set DEBUG_FILE=wrs_%~n1_%~n2_dbg%EXT%

if not "%wrsDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
  del %TEMP_B% 2>nul
)

if "%wrsMETRIC%"=="" set wrsMETRIC=RMSE

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WW1=%%[fx:w]\nHH1=%%[fx:h]\nIS_SMALL=%%[fx:max(w,h)>1000?0:1]" ^
  %IN_A%`) do set %%L

if "%wrsDO_SUPER%"=="" set wrsDO_SUPER=%IS_SMALL%

if %sSCALE%==1 (
  set SUPER_UP=
  set SUPER_FACT=1
) else (
  set SUPER_UP=-set option:distort:scale %sSCALE%
  set SUPER_FACT=%sSCALE%
)

echo %0: SUPER_FACT=%SUPER_FACT% WW1=%WW1% HH1=%HH1%


if not exist %TEMP_A% %IMG7%magick ^
  %IN_A% ^
  %SUPER_UP% ^
  -distort DePolar -1,0,%CX%,%CY% +repage ^
  -scale "%%[fx:w]x1^!" ^
  %AUTLEV% ^
  ^( +clone ^) +append +repage ^
  -define quantum:format=floating-point -depth 32 ^
  %TEMP_A%

if not exist %TEMP_B% %IMG7%magick ^
  %IN_B% ^
  %SUPER_UP% ^
  -distort DePolar -1,0,%CX2%,%CY2% +repage ^
  -scale "%%[fx:w]x1^!" ^
  %AUTLEV% ^
  -define quantum:format=floating-point -depth 32 ^
  %TEMP_B%

if /I "%sNorm%"=="meanSD" %IMG7%magick ^
  %TEMP_A% %TEMP_B% ^
  -fx "(u-u.mean)*v.standard_deviation/u.standard_deviation + v.mean" ^
  %TEMP_A%

set COMP_XW=
for /F "usebackq tokens=1-4 delims=()@, " %%A in (`%IMG7%magick compare ^
  %TEMP_A% %TEMP_B% ^
  -metric RMSE -similarity-threshold 0 -dissimilarity-threshold 1 -subimage-search ^
  NULL: 2^>^&1`) do (
  set COMP_XW=%%C
  set COMP_FLT=%%B
)
if "%COMP_XW%"=="" (
  echo %0: search failed
  exit /B 1
)


set DODGY=0

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 9 ^
  -format "ANGLE=%%[fx:%COMP_XW%*360/(%SUPER_FACT%*%WW1%)]\nANGLE_ERR=%%[fx:360/(%SUPER_FACT%*%WW1%)]" ^
  xc:`) do set %%L

if %ANGLE%==360 set ANGLE=0

if not "%sDEBUG%"=="" (

  %IMG7%magick ^
    ^( %IN_A% ^
       -colorspace Gray -alpha off ^
       -distort SRT %CX%,%CY%,1,%ANGLE%,%CX2%,%CY2% ^
    ^) ^
    ^( %IN_B% ^
       -colorspace Gray -alpha off ^
    ^) ^
    ^( -clone 0 -fill Black -colorize 100 ^) ^
    -combine ^
    %sDEBUG%
)

if not "%wrsDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
  del %TEMP_B% 2>nul
)


call echoRestore

endlocal & set wrsANGLE=%ANGLE%& set wrsANGLE_ERR=%ANGLE_ERR%& set ^
wrsSCORE=%COMP_FLT%& set wrsDODGY=%DODGY%& set wrsCX=%CX%& set wrsCY=%CY%& set wrsCX2=%CX2%& set wrsCY2=%CY2%
