@rem Given same-size images %1 and %2,
@rem finds scale S in "-distort SRT S,0" for %1 to best match %2, and a score.

@rem Parameters:
@rem   %3 min scale (default 0.5) OR "again" to iterate from previous run.
@rem   %4 max scale (default 1.5)
@rem   %5 number of scale steps (default 100)
@rem   %6 CX } Centre for scaling
@rem   %7 CY }
@rem
@rem Also uses:
@rem   wsDO_SCORE_SUB  if ==0, wsSCORE_SUB will not be found.
@rem   wsMETRIC        default RMSE
@rem   wsDEBUG         if 1, creates images for debugging
@rem
@rem Returns:
@rem   wsDODGY     0 if no problem
@rem   wsSCALE     the found scale factor
@rem   wsFACT      error margin, multiple of wsSCALE
@rem   wsSCORE     (0.0 to 1.0, more or less).
@rem   wsSCORE_SUB (0.0 to 1.0, more or less).
@rem
@rem Revised:
@rem   18-August-2016 Tidied up slightly.
@rem   3-May-2018 Added CX and CY.
@rem   10-August-2022 for IM v7.
@rem


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave


if "%wsMETRIC%"=="" set wsMETRIC=RMSE

if "%wsDO_SCORE_SUB%"=="" set wsDO_SCORE_SUB=1

set wsMIN_SCALE=%3
if "%wsMIN_SCALE%"=="." set wsMIN_SCALE=
if "%wsMIN_SCALE%"=="" set wsMIN_SCALE=0.5

set wsMAX_SCALE=%4
if "%wsMAX_SCALE%"=="." set wsMAX_SCALE=
if "%wsMAX_SCALE%"=="" set wsMAX_SCALE=1.5

set wsnSTEPS=%5
if "%wsnSTEPS%"=="." set wsnSTEPS=
if "%wsnSTEPS%"=="" set wsnSTEPS=100

set CX=%6
if "%CX%"=="." set CX=
if "%CX%"=="" set CX=0

set CY=%7
if "%CY%"=="." set CY=
if "%CY%"=="" set CY=0

echo %0: CX=%CX%  CY=%CY%

if /I "%wsMIN_SCALE%" EQU "again" (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "wsMIN_SCALE=%%[fx:%wsSCALE%/(%wsFACT%*%wsFACT%)]\nwsMAX_SCALE=%%[fx:%wsSCALE%*(%wsFACT%*%wsFACT%)]" ^
    xc:`) do set %%L
)

set TMPEXT=.miff

call %PICTBAT%setInOut %1 ws
set IN_A=%INFILE%
set TEMP_A_BASE=%TEMP%\%~n1_ws
set TEMP_A=%TEMP%\%~n1_ws_%wsMIN_SCALE%_%wsMAX_SCALE%_%wsnSTEPS%%TMPEXT%

set IN_A_TMP=%TEMP%\%~n1_ws_inA%TMPEXT%

call %PICTBAT%setInOut %2 ws
set IN_B=%INFILE%
set TEMP_B=%TEMP%\%~n2_ws%TMPEXT%
if %IN_B%==%IN_A% set TEMP_B=%TEMP%\%~n2_B_ws%TMPEXT%

set DEBUG_FILE=ws_%~n1_%~n2_dbg%EXT%

echo IN_A=%IN_A% IN_B=%IN_B%

%IMG7%magick %IN_A% %IN_A_TMP%

set WW1=
for /F "usebackq" %%L ^
in (`%IMG7%magick identify -format "WW1=%%w\nHH1=%%h" %IN_A_TMP%`) ^
do set %%L

if "%WW1%"=="" exit /B 1

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "FACT=%%[fx:pow(%wsMAX_SCALE%/%wsMIN_SCALE%,1/%wsnSTEPS%)]" ^
    xc:`) ^
do set %%L

echo %0: wsMIN_SCALE=%wsMIN_SCALE% wsMAX_SCALE=%wsMAX_SCALE% wsnSTEPS=%wsnSTEPS% FACT=%FACT%

for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "FINISHED=%%[fx:abs(%FACT%-1)<0.0000001]" ^
    xc:`) ^
do set %%L

if "%FINISHED%"=="1" (
  set SCALE=%wsMIN_SCALE%

  for /F "usebackq" %%L ^
in (`%IMG7%magick identify ^
    -precision 16 ^
    -format "SCALE=%%[fx:sqrt(%wsMIN_SCALE%*%wsMAX_SCALE%)]" ^
    xc:`) ^
do set %%L

  set DODGY=0
  goto end
)

:: For better performance, don't delete these.
::
del /Q %TEMP_A% 2>nul
del /Q %TEMP_B% 2>nul

if exist %TEMP_A% goto skipTempA

echo %0: Building TEMP_A [%TEMP_A%]

set SCALE=%wsMIN_SCALE%
set NAMES_A=
for /L %%i in (1,1,%wsnSTEPS%) do (
  @rem Maybe we should crop all scales so we never have virtual pixels.

  %IMG7%magick ^
  %IN_A_TMP% ^
  -virtual-pixel None ^
  +write mpr:INP ^
  -distort SRT %CX%,%CY%,!SCALE!,0 ^
  -gravity Center ^
  -define distort:viewport=%WW1%x%HH1% ^
  ^( -clone 0 -scale "x1^!" ^
     -define distort:viewport=%WW1%x1 -distort SRT 1,0 +repage ^) ^
  ^( -clone 0 -scale "1x^!" ^
     -define distort:viewport=1x%HH1% -distort SRT 1,0 +repage ^
     -rotate 90 +repage ^) ^
  -delete 0 ^
  +append +repage ^
  -depth 32 ^
  %TEMP_A_BASE%_%%i%TMPEXT%

  set NAMES_A=!NAMES_A! %TEMP_A_BASE%_%%i%TMPEXT%

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 19 -format "SCALE=%%[fx:!SCALE!*%FACT%]" ^
    xc:`) do set %%L
)

rem echo NAMES_A=%NAMES_A%

%IMG7%magick ^
  %NAMES_A% ^
  -append +repage ^
  -channel A -threshold 1%% +channel^
  +write %TEMP_A% ^
  -alpha extract ^
  ws_mask.miff

:skipTempA

%IMG7%magick identify %TEMP_A%

dir %TEMP_B%

if not exist %TEMP_B% (
  echo %0: Building TEMP_B [%TEMP_B%]

  %IMG7%magick ^
  %IN_B% ^
  -gravity Center ^
  -virtual-pixel Mirror ^
  ^( -clone 0 -scale "x1^!" ^) ^
  ^( -clone 0 -scale "1x^!" -rotate 90 +repage ^) ^
  -delete 0 ^
  +append +repage ^
  %TEMP_B%
)

%IMG7%magick identify %TEMP_B%


%IM7DEV%magick ^
  %TEMP_A% %TEMP_B% ^
  -set colorspace sRGB ^
  -process 'rmsealpha v' ^
  NULL:

for /F "usebackq tokens=1-4 delims=()@, " %%R ^
in (`%IMG7%magick compare ^
    -read-mask ws_mask.miff %TEMP_A% +read-mask ^
    %TEMP_B% ^
    -similarity-threshold 0 -dissimilarity-threshold 1 -subimage-search ^
    -metric %wsMETRIC% ^
    NULL: 2^>^&1`) ^
do (
  @set COMP_INT=%%R
  @set COMP_FLT=%%S
  @set COMP_XW=%%T
  @set COMP_YW=%%U
)

echo %0: COMP_FLT=%COMP_FLT% COMP_XW=%COMP_XW% COMP_YW=%COMP_YW%

if "%COMP_FLT%" GTR "9" exit /B 1

set DODGY=0
if %COMP_YW% EQU 0 set DODGY=1
if %COMP_YW% GEQ %wsnSTEPS% set DODGY=1

rem What is the magnification here?
set SCALE=%wsMIN_SCALE%
for /L %%i in (1,1,%COMP_YW%) do (
  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -precision 16 -format "SCALE=%%[fx:!SCALE!*%FACT%]" ^
    xc:`) do set %%L
)

echo COMP_YW=%COMP_YW% SCALE=%SCALE%

rem Cleanup: remove most temporary files.
for /L %%i in (1,1,%wsnSTEPS%) do (
  rem  @del %TEMP_A_BASE%_%%i%TMPEXT%
)


:end

if %wsDO_SCORE_SUB%==1 (
  rem Test result.

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "cropX=%%[fx:int(min(%WW1%,%WW1%*%SCALE%))]\ncropY=%%[fx:int(min(%HH1%,%HH1%*%SCALE%))]" ^
    xc:`) do set %%L

  set ScoreSub=

  for /F "usebackq" %%L in (`%IMG7%magick ^
    %IN_A_TMP% ^
    -distort SRT "%CX%,%CY%,%SCALE%,0" ^
    %IN_B% ^
    -gravity Center ^
    -crop !cropX!x!cropY!+0+0 +repage ^
    -metric %wsMETRIC% -format "ScoreSub=%%[distortion]" -compare ^
    info:`) do set %%L

  if "!ScoreSub!"=="" set DODGY=1
)

if "%wsDEBUG%"=="1" (

  for /F "usebackq" %%L in (`%IMG7%magick identify ^
    -format "cropX=%%[fx:int(min(%WW1%,%WW1%*%SCALE%))]\ncropY=%%[fx:int(min(%HH1%,%HH1%*%SCALE%))]" ^
    xc:`) do set %%L

  %IMG7%magick ^
  %IN_A_TMP% ^
  -virtual-pixel None ^
  ^( %IN_B% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #0f0 -frame 1x1 ^
       -channel A -evaluate Multiply 0.75 ^
    ^) ^
    -compose Over -composite ^
  ^) ^
  ^( -clone -1 ^
     -gravity Center ^
     -crop !cropX!x!cropY!+0+0 ^
     -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #ff0 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  ^( -clone 0 -alpha Transparent ^
     -shave 1x1 ^
     -bordercolor None ^
     -mattecolor #f00 -frame 1x1 ^
     -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -layers merge ^
  +repage ^
  %DEBUG_FILE%

  if ERRORLEVEL 1 exit /B 1
)


call echoRestore

endlocal & set wsSCALE=%SCALE%& set wsFACT=%FACT%& set wsSCORE=%COMP_FLT%& set wsSCORE_SUB=%ScoreSub%& set wsDODGY=%DODGY%& set wsCOMP_YW=%COMP_YW%
