rem Given same-sized images %1 and %2,
rem scales %1 by %3 and rotates by %4, and crops it.
rem Returns location of resultimg image in %2 as offset from centres,
rem ie (0,0) means scaled and rotated %1 is not displaced over %2.
rem %5 is prefix for environment variables for numeric results. [wt]

@rem Also uses:
@rem   wtMETRIC        default RMSE
@rem   wtTUNE_CROP     if 1, crop percentage will be adjusted for the given scale
@rem   wtDEBUG         if 1, creates image for debugging
@rem   wtDEBUG_FILE    name of debugging image file
@rem   wtDELTEMP       if not 0, will remove temporary file.
@rem
@rem Returns:
@rem   wtX, wtY    integer pixel offsets.
@rem                 Positive values mean the first image needs moving
@rem                 down and right in order to register with the second image.
@rem   wtXY        the previous, concatenated in the format +X+Y or -X-Y etc.
@rem   wtXYneg     like wtXY but with negated values.
@rem   wtSCORE     0.0 to 1.0, more or less
@rem   wtDEBUG_OUTFILE  name of debugging image file
@rem
@rem Last updated:
@rem   6-August-2016 Cleaned up.
@rem   10-August-2022 for IM v7.


@if "%2"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal enabledelayedexpansion

@call echoOffSave


set SCALE=%3
if "%SCALE%"=="." set SCALE=
if "%SCALE%"=="" set SCALE=1

set ANGLE=%4
if "%ANGLE%"=="." set ANGLE=
if "%ANGLE%"=="" set ANGLE=0

set ENV_PREF=%5
if "%ENV_PREF%"=="." set ENV_PREF=
if "%ENV_PREF%"=="" set ENV_PREF=wt

set TMPEXT=.miff

if "%wtMETRIC%"=="" set wtMETRIC=RMSE

if "%wtTUNE_CROP%"=="" set wtTUNE_CROP=0

call %PICTBAT%setInOut %1 wt
set IN_A=%INFILE%
set TEMP_A=%TEMP%\%~n1_%SCALE%_%ANGLE%_%wtTUNE_CROP%_wt%TMPEXT%

call %PICTBAT%setInOut %2 wt
set IN_B=%INFILE%

if not "%wtDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
)

if "%wtDEBUG_OUTFILE%"=="" (
  set DEBUG_FILE=wt_%~n1_%~n2_dbg%EXT%
) else (
  set DEBUG_FILE=%wtDEBUG_OUTFILE%
)

for /F "usebackq" %%L ^
in (`%IMG7%magick identify -format "WW2=%%w\nHH2=%%h" %IN_B%`) ^
do set %%L

set /A CROP_W=WW2/2
set /A CROP_H=HH2/2

rem Maybe do next only if SCALE < 1.
rem If done when scale near 2, there is no wriggle-room for search.
if %wtTUNE_CROP%==1 for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "CROP_W=%%[fx:%WW2%*%SCALE%/2]\nCROP_H=%%[fx:%HH2%*%SCALE%/2]" ^
  xc:`) do set %%L

rem Ensure CROP_X has same parity (least significant bit) as WW2,
rem and CROP_Y has same parity as HH2.
set /A CROP_W-=CROP_W%%2^^WW2%%2
set /A CROP_H-=CROP_H%%2^^HH2%%2

echo %0: SCALE=%SCALE% ANGLE=%ANGLE% CROP_W=%CROP_W% CROP_H=%CROP_H%

%IMG7%magick ^
  %IN_A% ^
  -distort SRT %SCALE%,%ANGLE% -gravity center -crop %CROP_W%x%CROP_H%+0+0 +repage ^
  %TEMP_A%

for /F "usebackq" %%L in (`%IMG7%magick ^
  %TEMP_A% ^
  -format "WW1=%%w\nHH1=%%h\nW_2=%%[fx:(%WW2%-w)/2]\nH_2=%%[fx:(%HH2%-h)/2]" ^
  info:`) do set %%L

set /A diffW=WW2-WW1
set /A diffH=HH2-HH1

set siDEBUG=
call %PICTBAT%srchImg %IN_B% %TEMP_A%
if ERRORLEVEL 1 exit /B 1

set /A RESULT_XI=siCOMP_XW-diffW/2
set /A RESULT_YI=siCOMP_YW-diffH/2

echo %0: COMP_FLT=%siCOMP_FLT% COMP_XW=%siCOMP_XW% COMP_YW=%siCOMP_YW%


set DODGY=0
if %siCOMP_XW%==0 set DODGY=1
if %siCOMP_YW%==0 set DODGY=1
if %siCOMP_XW%==%diffW% set DODGY=1
if %siCOMP_YW%==%diffH% set DODGY=1


if "%wtDEBUG%"=="1" (

  for /F "usebackq" %%L in (`%IMG7%magick ^
    %IN_A% ^
    -format "W_2=%%[fx:w/2]\nH_2=%%[fx:h/2]\nnewX=%%[fx:w/2+(%RESULT_XI%)]\nnewY=%%[fx:h/2+(%RESULT_YI%)]" ^
    info:`) do set %%L

  %IMG7%magick ^
  %IN_A% ^
  ^( +clone -alpha Transparent ^
    -shave 1x1 ^
    -bordercolor None ^
    -mattecolor #f00 -frame 1x1 ^
    -channel A -evaluate Multiply 0.75 ^
  ^) ^
  -composite ^
  -virtual-pixel None ^
  +distort SRT !W_2!,!H_2!,%SCALE%,%ANGLE%,!newX!,!newY! ^
  ^( %IN_B% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #0f0 -frame 1x1 ^
       -channel A -evaluate Multiply 0.75 ^
    ^) ^
    -composite ^
  ^) ^
  ^( %TEMP_A% ^
    ^( +clone -alpha Transparent ^
       -shave 1x1 ^
       -bordercolor None ^
       -mattecolor #ff0 -frame 1x1 ^
    ^) ^
    -composite ^
    -alpha set ^
    -channel A -evaluate Multiply 0.5 ^
    -repage +%siCOMP_XW%+%siCOMP_YW% ^
  ^) ^
  -layers merge ^
  +repage ^
  %DEBUG_FILE%
)

if not "%wtDELTEMP%"=="0" (
  del %TEMP_A% 2>nul
)

set SX=
set SY=
if %RESULT_XI% GEQ 0 set SX=+
if %RESULT_YI% GEQ 0 set SY=+
set wtXY=%SX%%RESULT_XI%%SY%%RESULT_YI%

set /A SXn=-%RESULT_XI%
set /A SYn=-%RESULT_YI%
set SX=
set SY=
if %SXn% GEQ 0 set SX=+
if %SYn% GEQ 0 set SY=+
set wtXYneg=%SX%%SXn%%SY%%SYn%

echo %0: RESULT_XI=%RESULT_XI%  RESULT_YI=%RESULT_YI% wtXY=%wtXY%

call echoRestore

endlocal &set %ENV_PREF%X=%RESULT_XI%&^
set %ENV_PREF%Y=%RESULT_YI%&^
set %ENV_PREF%SCORE=%siCOMP_FLT%&^
set %ENV_PREF%DODGY=%DODGY%&^
set %ENV_PREF%DEBUG_FILE=%DEBUG_FILE%&^
set %ENV_PREF%XY=%wtXY%&^
set %ENV_PREF%XYneg=%wtXYneg%
