/* Updated:
     3-April-2018 for v7.0.7-28
*/

#ifndef WHATROT_INC
#define WHATROT_INC


// FIXME: caller might normalize mean and SD.


#include "rmsealpha.inc"
#include "goldsectsrch.inc"
#include "resetpage.inc"
#include "trimone.inc"
#include "centsmcrop.inc"
//#include "virtnone.inc"
#include "writeframe.inc"

typedef struct {
  int
    verbose;

  MagickBooleanType
    recordA,
    recordB;

  char 
    *debug;

  double
    tolerance;

  // Used internally:

  Image
    *scaledA,
    *scaledB2;

  RmseAlphaT
    ra;

  // Returned results:
  double
    angle,  // degrees clockwise, 0.0 <= angle <= 360.0.
    plusOrMinus,
    score;
} WhatRotT;


static void InitWhatRot (WhatRotT * pwr)
{
  pwr->verbose = 0;
  pwr->recordA = pwr->recordB = MagickFalse;
  pwr->scaledA = pwr->scaledB2 = NULL;
  pwr->tolerance = 0.01;
  pwr->score = pwr->angle = pwr->plusOrMinus = 0.0;
  pwr->debug = NULL;

  InitRmseAlpha (&pwr->ra);
}

static void ReInitWhatRot (WhatRotT * pwr)
{
  if (pwr->scaledA)  pwr->scaledA  = DestroyImage (pwr->scaledA);
  if (pwr->scaledB2) pwr->scaledB2 = DestroyImage (pwr->scaledB2);
  ReInitRmseAlpha (&pwr->ra);
}

static void DeInitWhatRot (WhatRotT * pwr)
{
  ReInitWhatRot (pwr);
  DeInitRmseAlpha (&pwr->ra);
}


static Image * AppendRowSelf (
  Image * img,
  ExceptionInfo *exception)
// Returns image twice as wide, just one row,
// made from top row of img appended to itself.
{
  Image * new_img = CloneImage (img, 2 * img->columns, 1,
    MagickTrue, exception);
  if (!new_img) return NULL;

  CacheView * img_view = AcquireVirtualCacheView (img, exception);
  CacheView * new_view = AcquireAuthenticCacheView (new_img, exception);

  const VIEW_PIX_PTR *p = GetCacheViewVirtualPixels(
      img_view,0,0,img->columns,1,exception);
  if (!p) return NULL;

  VIEW_PIX_PTR *q = GetCacheViewAuthenticPixels(
      new_view,0,0,new_img->columns,1,exception);
  if (!q) return NULL;

  ssize_t dq = img->columns * Inc_ViewPixPtr (new_img);
  ssize_t x;
  for (x=0; x < img->columns; x++) {
    SET_PIXEL_RED   (new_img, GET_PIXEL_RED(img,p),   q);
    SET_PIXEL_GREEN (new_img, GET_PIXEL_GREEN(img,p), q);
    SET_PIXEL_BLUE  (new_img, GET_PIXEL_BLUE(img,p),  q);
    SET_PIXEL_ALPHA (new_img, GET_PIXEL_ALPHA(img,p),  q);

    SET_PIXEL_RED   (new_img, GET_PIXEL_RED(img,p),   q+dq);
    SET_PIXEL_GREEN (new_img, GET_PIXEL_GREEN(img,p), q+dq);
    SET_PIXEL_BLUE  (new_img, GET_PIXEL_BLUE(img,p),  q+dq);
    SET_PIXEL_ALPHA (new_img, GET_PIXEL_ALPHA(img,p),  q+dq);

    p += Inc_ViewPixPtr (img);
    q += Inc_ViewPixPtr (new_img);
  }

  new_view = DestroyCacheView (new_view);
  img_view = DestroyCacheView (img_view);

  return new_img;
}


static MagickBooleanType CalcApproxRot (
  WhatRotT * pwr,
  Image * imageA,
  Image * imageB,
  Image * DispMapA,  // May be null.
  Image * DispMapB,  // May be null.
  ExceptionInfo *exception)
//
// Find the rotation (degrees clockwise) that applied to imageB
// makes it best match imageA.
//
// Finds the approximate angle by radial filter,
// ie unroll and scale to one row.
{
  MagickBooleanType bestfit = MagickTrue;

  double minusOne=-1.0;

  if (!pwr->scaledA) {

    Image * clnA = CloneImage (imageA, 0, 0, MagickTrue, exception);
    if (!clnA) return MagickFalse;

//#if IMV6OR7==6
//    SetPixelCacheVirtualMethod (clnA, TransparentVirtualPixelMethod);
//    SetImageAlphaChannel (clnA, ActivateAlphaChannel);
//#else
//    SetPixelCacheVirtualMethod (clnA,
//      TransparentVirtualPixelMethod, exception);
//    SetImageAlphaChannel (clnA, ActivateAlphaChannel, exception);
//#endif

    VIRT_NONE(clnA, exception);

    Image *unrlA = DistortImage (clnA, DePolarDistortion,
      1, &minusOne, bestfit, exception);
    if (!unrlA) return MagickFalse;
    clnA = DestroyImage (clnA);

    pwr->scaledA = ScaleImage (unrlA, unrlA->columns, 1,
        exception);
    if (!pwr->scaledA) return MagickFalse;
    unrlA = DestroyImage (unrlA);
  }

  if (!pwr->scaledB2) {
    Image * clnB = CloneImage (imageB, 0, 0, MagickTrue, exception);
    if (!clnB) return MagickFalse;

//#if IMV6OR7==6
//    SetPixelCacheVirtualMethod (clnB, TransparentVirtualPixelMethod);
//    SetImageAlphaChannel (clnB, ActivateAlphaChannel);
//#else
//    SetPixelCacheVirtualMethod (clnB,
//      TransparentVirtualPixelMethod, exception);
//    SetImageAlphaChannel (clnB, ActivateAlphaChannel, exception);
//#endif

    VIRT_NONE(clnB, exception);

    Image *unrlB = DistortImage (clnB, DePolarDistortion,
      1, &minusOne, bestfit, exception);
    if (!unrlB) return MagickFalse;
    clnB = DestroyImage (clnB);

    // Scale to one row, and width to match A.
    Image * scaledB = ScaleImage (unrlB, pwr->scaledA->columns, 1,
        exception);
    if (!scaledB) return MagickFalse;
    unrlB = DestroyImage (unrlB);

    pwr->scaledB2 = AppendRowSelf (scaledB, exception);
    if (!pwr->scaledB2) return MagickFalse;
    scaledB = DestroyImage (scaledB);
  }

  ssize_t unrWidth = pwr->scaledA->columns;

  pwr->ra.do_verbose = (pwr->verbose > 0);

//WriteFrame ("wr_scaledB2.png", pwr->scaledB2, 0, exception);
//WriteFrame ("wr_scaledA.png", pwr->scaledA, 0, exception);

  // FIXME: Should MS be an option?
  //   It wouldn't currently improve performance.
  if (!subRmseAlpha (&pwr->ra, pwr->scaledB2, pwr->scaledA, exception))
    return MagickFalse;

  pwr->score = pwr->ra.score;
  pwr->angle = pwr->ra.solnX * 360.0 / (double)unrWidth;
  pwr->plusOrMinus = 180.0 / (double)unrWidth;

  if (pwr->verbose) fprintf (stderr,
    "CalcApproxRot: solnX=%li degrees=%g score=%g\n",
    pwr->ra.solnX, pwr->angle, pwr->score);

  if (!pwr->recordB) pwr->scaledB2 = DestroyImage (pwr->scaledB2);
  if (!pwr->recordA) pwr->scaledA  = DestroyImage (pwr->scaledA);

  return MagickTrue;
}


static MagickBooleanType InCircOnly (
  Image * img,
  ExceptionInfo *exception)
// Makes pixels outside an inscribed circle transparent.
{
  CacheView * img_view = AcquireAuthenticCacheView (img, exception);

  // FIXME: enable alpha for the image?

  // FIXME: No anti-aliasing.

  //if (pat_img->columns <= 2 || pat_img->rows <= 2) return MagickTrue;

  // FIXME: following isn't symmetrical.

  double r = img->columns;
  if (r > img->rows) r = img->rows;
  r /= 2.0;

  double a = r;
  double b = r;

  double a2 = a*a;
  double b2 = b*b;
  double a2b2 = a2*b2;

  MagickBooleanType okay = MagickTrue;

  ssize_t y;
#if defined(MAGICKCORE_OPENMP_SUPPORT)
  #pragma omp parallel for schedule(static,4) \
    shared(okay) \
    MAGICK_THREADS(img,img,img->rows,1)
#endif
  for (y = 0; y < img->rows; y++) {
    ssize_t x;

    VIEW_PIX_PTR *sp = GetCacheViewAuthenticPixels(
      img_view,0,y,img->columns,1,exception);
    if (!sp) okay = MagickFalse;

    if (!okay) continue;

    double a2y2   = a2 * (y-b) * (y-b);
    double a2y2m1 = a2 * (y-b-0.5) * (y-b-0.5);
    double a2y2p1 = a2 * (y-b+0.5) * (y-b+0.5);

    for (x = 0; x < img->columns; x++) {
      double b2x2   = b2 * (x-a) * (x-a);
      double b2x2m1 = b2 * (x-a-0.5) * (x-a-0.5);
      double b2x2p1 = b2 * (x-a+0.5) * (x-a+0.5);
      int nIn = 0;

      if (b2x2m1 + a2y2m1 < a2b2) nIn++;
      if (b2x2   + a2y2m1 < a2b2) nIn++;
      if (b2x2p1 + a2y2m1 < a2b2) nIn++;

      if (b2x2m1 + a2y2   < a2b2) nIn++;
      if (b2x2   + a2y2   < a2b2) nIn++;
      if (b2x2p1 + a2y2   < a2b2) nIn++;

      if (b2x2m1 + a2y2p1 < a2b2) nIn++;
      if (b2x2   + a2y2p1 < a2b2) nIn++;
      if (b2x2p1 + a2y2p1 < a2b2) nIn++;

      if (nIn < 9) {
        SET_PIXEL_ALPHA (img,
                         nIn / 9.0 * GET_PIXEL_ALPHA(img,sp),
                         sp);
      }
      sp += Inc_ViewPixPtr (img);
    }
    if (SyncCacheViewAuthenticPixels(img_view,exception) == MagickFalse)
      okay = MagickFalse;
  }

  if (!okay) return MagickFalse;

  img_view = DestroyCacheView (img_view);

  return MagickTrue;
}


typedef struct {
  Image * imageA;
  Image * imageB;
  int verbose;
  ExceptionInfo *exception;
  char * debug;
} GoldSectRotT;


static double cbGetRot (double ang, void * pData, int * okay)
{
  GoldSectRotT * pGsr = (GoldSectRotT *)pData;

  /* For now, a simple method:
     -distort SRT 1,angle the second image
     Make transparent all pixels outside an inscribed circle.
       (See PaintPatches EllipseOnly().)
     Return the alpha-adjusted rmse with the first image.

     As this loses data where some rotated imageB pixels fall on
     the main image but outside imageA, this can give false positives.

     A better method would lose no data, which needs a larger imageA.
     We would make transparent imageA pixels that are outside rotated imageB.
       (Then possibly normalise mean and SD.)
  */

  MagickBooleanType bestfit = MagickTrue;

  double scaleArray[2];
  scaleArray[0] = 1.0;  // Scale factor.
  scaleArray[1] = ang;  // Rotation angle.

  Image *rot_img = DistortImage (pGsr->imageB, ScaleRotateTranslateDistortion,
    2, scaleArray, bestfit, pGsr->exception);
  if (!rot_img) { *okay=0; return 0; }
  ResetPage (rot_img);
  TrimOne (&rot_img, pGsr->exception);

  Image * clnA = CloneImage (pGsr->imageA, 0, 0, MagickTrue, pGsr->exception);
  if (!clnA) { *okay=0; return 0; }

  if (!CentralSmallestCrop (&clnA, &rot_img, (pGsr->verbose > 1), pGsr->exception))
    { *okay=0; return 0; }

  //printf ("clnA: %lix%li  ",    clnA->columns, clnA->rows);
  //printf ("rot_img: %lix%li\n", rot_img->columns, rot_img->rows);

  // FIXME: Do we need to restrict to inscribed circle?
//  if (!InCircOnly (rot_img, pGsr->exception)) { *okay=0; return 0; }

  RmseAlphaT ra;
  InitRmseAlpha (&ra);  // FIXME: Can we do this outside the loop?
  ra.do_verbose = (pGsr->verbose <= 1) ? 0 : pGsr->verbose-1;

  if (pGsr->debug) {
    WriteFrame ("wrcb_rot.png", rot_img, 0, pGsr->exception);
    WriteFrame ("wrcb_clna.png", clnA, 0, pGsr->exception);
  }

  if (!subRmseAlpha (&ra, rot_img, clnA, pGsr->exception))
    { *okay=0; fprintf (stderr, "cbGetRot sra failed\n"); return 0; }

  DeInitRmseAlpha (&ra);

  DestroyImage (rot_img);
  DestroyImage (clnA);

  if (ra.score < 0) {
    fprintf (stderr, "cbGetRot sra %g\n", ra.score);
    *okay = 0;
  }

  if (pGsr->verbose) fprintf (stderr,
    "cbGetRot: ang=%g, score=%g\n", ang, ra.score);

  return ra.score;
}

static MagickBooleanType GoldSectRot (
  WhatRotT * pwr,
  Image * imageA,
  Image * imageB,
  ExceptionInfo *exception)
// Finds rotation by golden section search.
{
  if (pwr->verbose) fprintf (stderr, "GoldSectRot %g +/- %g\n",
    pwr->angle, pwr->plusOrMinus);

  Image * clnB = CloneImage (imageB, 0, 0, MagickTrue, exception);
  if (!clnB) return MagickFalse;

//#if IMV6OR7==6
//  SetPixelCacheVirtualMethod (clnB, TransparentVirtualPixelMethod);
//#else
//  SetPixelCacheVirtualMethod (clnB,
//    TransparentVirtualPixelMethod, exception);
//#endif

  VIRT_NONE(clnB, exception);

  goldSectSrchT gss;

  InitGoldSectSrch (&gss);

  double porm = pwr->plusOrMinus * 2;
  if (porm < 5) porm = 5;

  gss.xLo = pwr->angle - porm;
  gss.xHi = pwr->angle + porm;
  gss.epsX = pwr->tolerance * 2;
  gss.getYcb = cbGetRot;
  gss.verbose = (pwr->verbose <= 1) ? 0 : pwr->verbose-1 ;

  GoldSectRotT gsr;
  gsr.imageA = imageA;
  gsr.imageB = clnB;
  gsr.verbose = gss.verbose;
  gsr.exception = exception;
  gsr.debug = pwr->debug;

  if (! GoldSectSrch (&gss, &gsr)) {
    clnB = DestroyImage (clnB);
    return MagickFalse;
  }

  pwr->angle = gss.fndX;
  pwr->plusOrMinus = gss.xPlusOrMinus;
  pwr->score = gss.calcY;

  clnB = DestroyImage (clnB);

  return MagickTrue;
}


#endif
