/* Updated:
     3-April-2018 for v7.0.7-28
*/

#ifndef WHATROTSCALE_INC
#define WHATROTSCALE_INC

#define wrsDEBUG 1

//#include "virtnone.inc"


typedef struct {
  WhatRotT wr;
  WhatScaleT ws;

  MagickBooleanType
    verbose;

  char 
    *debug;

  double
    ApproxScale,
    ApproxAngle;

  // Returned results:
  double
    scale,
    angle,
    score;

} WhatRotScaleT;

static void InitWhatRotScale (WhatRotScaleT *wrs)
{
  InitWhatRot   (&wrs->wr);
  InitWhatScale (&wrs->ws);

  wrs->verbose = MagickFalse;
  wrs->ApproxScale = 1;
  wrs->ApproxAngle = 0;
  wrs->debug = NULL;

  wrs->scale = wrs->angle = wrs->score = 0;
}

static void DeInitWhatRotScale (WhatRotScaleT *wrs)
{
  DeInitWhatScale (&wrs->ws);
  DeInitWhatRot   (&wrs->wr);
}


static MagickBooleanType ApproxWhatRotScale (
  WhatRotScaleT *wrs,
  Image * main_img,
  Image * sub_img,
  ExceptionInfo *exception)
// Finds approximate scale and rotation.
{
  ConcRingsT cr;
  InitConcRings (&cr);
  cr.verbose = (wrs->verbose > 1);

  wrs->ws.verbose = wrs->verbose;
  wrs->wr.verbose = wrs->verbose;
  wrs->ws.debug = wrs->debug;
  wrs->wr.debug = wrs->debug;


  // Make the concentric rings version of the subimage.
  Image * AvgConcRings = avgconcrings (sub_img, &cr, exception);
  if (!AvgConcRings) {
    fprintf (stderr, "ApproxWhatRotScale: avgconcrings failed\n");
    return MagickFalse;
  }

  if (! CalcApproxScale (&wrs->ws, &cr, 
          main_img, AvgConcRings, NULL, exception))
    return MagickFalse;

  if (! CalcApproxRot (&wrs->wr, main_img, sub_img, NULL, NULL, exception))
    return MagickFalse;

  wrs->scale = wrs->ApproxScale = wrs->ws.scale;
  wrs->angle = wrs->ApproxAngle = wrs->wr.angle;

  wrs->score = (wrs->wr.score + wrs->ws.score)  / 2.0;

  if (wrs->verbose) fprintf (stderr,
    "WhatRotScale approx scale=%g porm=%g ang=%g porm=%g score=%g\n",
    wrs->scale, wrs->ws.porm,
    wrs->angle, wrs->wr.plusOrMinus, wrs->score);

  DestroyImage (AvgConcRings);

  DeInitConcRings (&cr);

  return MagickTrue;
}

static MagickBooleanType RefineWhatRotScale (
  WhatRotScaleT *wrs,
  Image * main_img,
  Image * sub_img,
  ExceptionInfo *exception)
// Given approximate scale and rotation,
// refine the scale and rotation by golden section searches.
// FIXME: Can the inputs be different sizes??
{
  wrs->wr.verbose = (wrs->verbose <= 1) ? 0 : wrs->verbose-1;
  wrs->ws.verbose = wrs->wr.verbose;
  wrs->ws.debug = wrs->debug;
  wrs->wr.debug = wrs->debug;

  wrs->scale = wrs->ApproxScale;
  wrs->angle = wrs->ApproxAngle;

  if (wrs->verbose) fprintf (stderr,
    "RefineWhatRotScale ApproxScale=%g ApproxAng=%g\n",
    wrs->scale, wrs->angle);

  Image * clnSub = CloneImage (sub_img, 0, 0, MagickTrue, exception);
  if (!clnSub) return MagickFalse;

//#if IMV6OR7==6
//  SetPixelCacheVirtualMethod (clnSub, TransparentVirtualPixelMethod);
//  SetImageAlphaChannel (clnSub, ActivateAlphaChannel);
//#else
//  SetPixelCacheVirtualMethod (clnSub,
//    TransparentVirtualPixelMethod, exception);
//  SetImageAlphaChannel (clnSub, ActivateAlphaChannel, exception);
//#endif

  VIRT_NONE(clnSub, exception);

  MagickBooleanType bestfit = MagickTrue;
  MagickBooleanType okay = MagickTrue;

  double srtArray[2];
  int i;
  double prevScore = 9.9;
  for (i=0; i < 10; i++) {
    srtArray[0] = wrs->scale;
    srtArray[1] = wrs->angle;

#if wrsDEBUG==1
    if (wrs->verbose > 1) {
      fprintf (stderr, "RefineWhatRotScale sc=%g rot=%g\n", wrs->scale, wrs->angle);
      WriteFrame ("wrsx_main.png", main_img, 0, exception);
      WriteFrame ("wrsx_sub.png", sub_img, 0, exception);
    }
#endif

    Image *sub_trans = DistortImage (clnSub, ScaleRotateTranslateDistortion,
      2, srtArray, bestfit, exception);
    if (!sub_trans) {
      fprintf (stderr, "WhatRotScale: distort sc and ang failed\n");
      return MagickFalse;
    }
    ResetPage (sub_trans);
    TrimOne (&sub_trans, exception);

#if wrsDEBUG==1
    if (wrs->debug) {
      WriteFrame ("wrsx_trans0.png", sub_trans, 0, exception);
    }
#endif

    int nFailed = 0;

    wrs->wr.angle = 0;
    wrs->wr.plusOrMinus = 5;
    wrs->wr.debug = wrs->debug;
    wrs->ws.debug = wrs->debug;
    okay = GoldSectRot (&wrs->wr, main_img, sub_trans, exception);
    DestroyImage (sub_trans);
    if (!okay) {
      if (wrs->verbose > 1) fprintf (stderr, "WhatRotScale: gss angle failed\n");
      nFailed = 1;
      wrs->wr.angle = 0.0;
      // return MagickFalse;
    }

    wrs->angle += wrs->wr.angle;
    if (wrs->verbose > 1) fprintf (stderr,
      "RefineWhatRotScale ang=%g porm=%g score=%g\n",
      wrs->angle, wrs->wr.plusOrMinus, wrs->wr.score);

    srtArray[1] = wrs->angle;

    sub_trans = DistortImage (clnSub, ScaleRotateTranslateDistortion,
      2, srtArray, bestfit, exception);
    if (!sub_trans) {
      fprintf (stderr, "WhatRotScale: distort sc and new ang failed\n");
      return MagickFalse;
    }
    ResetPage (sub_trans);
    TrimOne (&sub_trans, exception);

#if wrsDEBUG==1
    if (wrs->debug) {
      WriteFrame ("wrsx_trans1.png", sub_trans, 0, exception);
    }
#endif

    wrs->ws.scale = 1.0;
    okay = GoldSectScale (&wrs->ws, main_img, sub_trans, exception);
    DestroyImage (sub_trans);
    if (!okay) {
      if (wrs->verbose > 1) fprintf (stderr, "WhatRotScale: gss scale failed\n");
      nFailed++;
      wrs->ws.scale = 1.0;
      // return MagickFalse;
    }
    if (nFailed == 2) return MagickFalse;

    wrs->scale *= wrs->ws.scale;
    if (wrs->verbose > 1) fprintf (stderr,
      "WhatRotScale scale=%g porm=%g score=%g\n",
      wrs->scale, wrs->ws.porm, wrs->ws.score);

    wrs->score = wrs->ws.score;

    if (wrs->score < 0.01) break;

    if (fabs(prevScore - wrs->score) < 0.001) break;

    prevScore = wrs->score;
  } // end for


  if (wrs->verbose) fprintf (stderr,
    "WhatRotScale refined nIter=%i scale=%g porm=%g ang=%g porm=%g score=%g\n",
    i+1,
    wrs->scale, wrs->ws.porm,
    wrs->angle, wrs->wr.plusOrMinus, wrs->score);

  return (wrs->score > 0);
}


#endif
