#ifndef WHATSCALE_INC
#define WHATSCALE_INC


// Returns the angle and score.

// FIXME: caller might normalize mean and SD.

/* Updated:
     5-April-2018 consider smaller or larger only if non-zero score.
*/

#include "rmsealpha.inc"
#include "goldsectsrch.inc"
#include "avgconcrings.inc"
#include "resetpage.inc"
#include "trimone.inc"
#include "centsmcrop.inc"

#include "writeframe.inc"



typedef struct {
  int
    verbose;

  char 
    *debug;

  double
    inPorm,
    tolerance;

  // Results:
  double
    scale,
    porm,      // error margin, plus or minus
    scaleFact, // error margin
    score;

  MagickBooleanType
    possSmaller,
    possLarger;

} WhatScaleT;


static void InitWhatScale (WhatScaleT * pws)
{
  pws->verbose = 0;
  pws->score = pws->scale = pws->scaleFact = 0;
  pws->inPorm = 1.2;  // >= 1.0
  pws->debug = NULL;
  pws->tolerance = 0.01;

  pws->scale = pws->porm = pws->scaleFact = pws->score = 0.0;
}


static void ReInitWhatScale (WhatScaleT * pws)
{
  ; // Nothing.
}

static void DeInitWhatScale (WhatScaleT * pws)
{
  ; // Nothing.
}


static MagickBooleanType PrepCalcApproxScale (
  ConcRingsT * pcr,
  Image * sub_img,
  Image ** pAvgConcRings, // Creates this.
  Image ** pDispMap,      // If not null, creates this.
  ExceptionInfo *exception)
{
  // Make the concentric rings version of the subimage.
  *pAvgConcRings = avgconcrings (sub_img, pcr, exception);
  if (!*pAvgConcRings) {
    fprintf (stderr, "whatscale: avgconcrings failed\n");
    return MagickFalse;
  }

  if (pDispMap) {
    // Make displacement map, and unroll it.
    double minusOne=-1.0;
    MagickBooleanType bestfit = MagickTrue;

    Image * id_map = mIdentAbsDispMap (sub_img, exception);
#if IMV6OR7==6
    SetImageAlphaChannel (id_map, OpaqueAlphaChannel);
#else
    SetImageAlphaChannel (id_map, OpaqueAlphaChannel, exception);
#endif

    *pDispMap = DistortImage (id_map, DePolarDistortion,
      1, &minusOne, bestfit, exception);
    if (!*pDispMap) return MagickFalse;

    if (pcr->verbose) fprintf (stderr, "whatscale: DispMap %lix%li\n",
      (*pDispMap)->columns, (*pDispMap)->rows);

    //if (!WriteFrameImage (pss->DispMap, "dispmap.png", NULL, exception))
    //  return MagickFalse;

    // Check: width of AvgConcRings should be height of DispMap.
    if ((*pAvgConcRings)->columns != (*pDispMap)->rows) {
      fprintf (stderr, "AvgConcRings->columns != DispMap->rows\n");
    }

  }

  // Possibly shrink pss->AvgConcRings to X columns,
  // and pss->DispMap in same proportion.

/*==
  if (pss->scaleRadMax && pss->AvgConcRings->columns > pss->scaleRadMax) {

    Image * tmp_img1 = RESIZEIMG (pss->AvgConcRings, pss->scaleRadMax, pss->AvgConcRings->rows,
      exception);
    Image * tmp_img2 = RESIZEIMG (pss->DispMap, pss->DispMap->rows, pss->scaleRadMax,
      exception);

    if (!tmp_img1) return MagickFalse;
    ReplaceImageInList (&pss->AvgConcRings, tmp_img1);

    if (!tmp_img2) return MagickFalse;
    ReplaceImageInList (&pss->DispMap, tmp_img2);
  }
==*/

  pcr->ra.do_verbose = pcr->verbose;

  return MagickTrue;
}


static MagickBooleanType CalcApproxScale (
  WhatScaleT * pws,
  ConcRingsT * pcr,
  Image * cropped,      // Must not be null.
  Image * AvgConcRings, // Must not be null.
  Image * DispMap,      // May be null.
  ExceptionInfo *exception)
{
  pws->possSmaller = pws->possLarger = MagickFalse;

  MagickBooleanType bestfit = MagickTrue;

  double minusOne=-1.0;

  Image *unrl_img;

  if (DispMap) {

    // For DePolar, faster alternative is to use pre-built displacement map.
    // Scale image to size of the disp map, and displace ("distort") it.

    // Scale instead of resize. Scale is quicker, and no practical difference.
    unrl_img = ScaleImage (cropped, DispMap->columns, DispMap->rows,
      exception);
    if (!unrl_img) return MagickFalse;

    if (!COMPOSITE(unrl_img, DistortCompositeOp, DispMap, 0,0, exception))
    {
      fprintf (stderr, "CalcApproxScale: Composite failed\n");
      return MagickFalse;
    }

    if (!COMPOSITE(unrl_img, COPY_OPACITY, DispMap, 0,0, exception))
    {
      fprintf (stderr, "CalcApproxScale: Composite2 failed\n");
      return MagickFalse;
    }

    if (pws->debug)
      WriteFrame ("wrx_unrl_img.png", unrl_img, 0, exception);

    //if (pcr->do_verbose) fprintf (stderr, "CalcApproxScale: unrl 2 %lix%li\n",
    //  unrl_img->columns, unrl_img->rows);

  } else {
    unrl_img = DistortImage (cropped, DePolarDistortion,
      1, &minusOne, bestfit, exception);
  }

  if (!unrl_img) return MagickFalse;

  // Scale to 1 column, with height of AvgConcRings width
  // (instead of unrl_img->rows).
  // FIXME: put it back.

  Image * one_col_img = ScaleImage (unrl_img, 1, unrl_img->rows, exception);
  if (!one_col_img) return MagickFalse;
  unrl_img = DestroyImage (unrl_img);

  // Rotate by -90 degrees.

  Image * one_row_img = IntegralRotateImage (one_col_img, 3, exception);
  if (!one_row_img) return MagickFalse;
  one_col_img = DestroyImage (one_col_img);
  ResetPage (one_row_img);

  if (pws->debug)
    WriteFrame ("wsx_one_row_img.png", one_row_img, 0, exception);

  //if (pcr->do_verbose) fprintf (stderr, "CalcApproxScale: %lix%li\n",
  //  one_row_img->columns, one_row_img->rows);

  // Search for one_row_img in AvgConcRings.
  // Return score and scale.

  pcr->ra.doSlideX = MagickFalse;
  if (!subRmseAlpha (&pcr->ra, AvgConcRings, one_row_img, exception)) {
    fprintf (stderr, "CalcApproxScale: subRmseAlpha failed\n");
    return MagickFalse;
  }

  one_row_img = DestroyImage (one_row_img);

  if (pcr->ra.score != 0) {
    if (pcr->ra.solnY==0) pws->possSmaller = MagickTrue;
    else if (pcr->ra.solnY==AvgConcRings->rows-1) pws->possLarger = MagickTrue;
  }

  pws->scale = ConcRingsScale (pcr, pcr->ra.solnY);
  pws->porm  = ConcRingsScale (pcr, pcr->ra.solnY + 1) - pws->scale;
  pws->score = pcr->ra.score;

  if (pws->verbose) {
    fprintf (stderr,
    "CalcApproxScale: solnY=%li scale=%g score=%g",
    pcr->ra.solnY, pws->scale, pws->score);
    if (pws->possSmaller) fprintf (stderr, " (possibly smaller)");
    if (pws->possLarger)  fprintf (stderr, " (possibly larger)");
    fprintf (stderr, "\n");
  }

  return MagickTrue;
}



typedef struct {
  Image * imageA;
  Image * imageB;
  int verbose;
  ExceptionInfo *exception;
  char * debug;
} GoldSectScaleT;

static double cbGetScale (double scale, void * pData, int * okay)
{
  GoldSectScaleT * pGsr = (GoldSectScaleT *)pData;

  /* Resize imageB.
     Crop the larger to the smaller.
     Return the alpha-adjusted rmse.
  */

  MagickBooleanType bestfit = MagickTrue;
  double scaleArray[2];
  scaleArray[0] = scale; // Scale factor.
  scaleArray[1] = 0.0;   // Rotation angle.

  Image *resB = DistortImage (pGsr->imageB, ScaleRotateTranslateDistortion,
    2, scaleArray, bestfit, pGsr->exception);
  if (!resB) { fprintf (stderr, "DistortImage failed\n"); *okay=0; return 0; }
  ResetPage (resB);
  TrimOne (&resB, pGsr->exception);

  Image * clnA = CloneImage (pGsr->imageA, 0, 0, MagickTrue, pGsr->exception);
  if (!clnA) {
    fprintf (stderr, "cbGetScale clone failed\n");
    *okay=0;
    return 0;
  }

  if (!CentralSmallestCrop (&clnA, &resB, (pGsr->verbose > 1), pGsr->exception)) {
    fprintf (stderr, "cbGetScale csc failed\n");
    *okay=0;
    return 0;
  }

  if (pGsr->debug) {
    WriteFrame ("wscb_clna.png", clnA, 0, pGsr->exception);
    WriteFrame ("wscb_resb.png", resB, 0, pGsr->exception);
  }


/*===
  MagickBooleanType CropIt = MagickTrue;
  Image *toCrop, *othImg, *crpd_img;
  if (resB->columns > pGsr->imageA->columns) {
    toCrop = resB;
    othImg = pGsr->imageA;
  } else if (resB->columns < pGsr->imageA->columns) {
    toCrop = pGsr->imageA;
    othImg = resB;
  } else {
    CropIt = MagickFalse;
    crpd_img = resB;
    othImg = pGsr->imageA;
  }

  if (CropIt) {
    RectangleInfo crpRect;
    crpRect.width  = othImg->columns;
    crpRect.height = othImg->rows;
    crpRect.x = (toCrop->columns - othImg->columns) / 2;
    crpRect.y = (toCrop->rows    - othImg->rows) / 2;

    crpd_img = CropCheck (toCrop, &crpRect, pGsr->exception);
    if (!crpd_img) { fprintf (stderr, "CropImage failed\n"); *okay=0; return 0; }

    ResetPage (crpd_img);
  }
===*/

  RmseAlphaT ra;
  InitRmseAlpha (&ra);  // FIXME: Can we do this outside the loop?
  ra.do_verbose = (pGsr->verbose <= 1) ? 0 : pGsr->verbose-1;

//  if (!subRmseAlpha (&ra, crpd_img, othImg, pGsr->exception))
  if (!subRmseAlpha (&ra, clnA, resB, pGsr->exception))
  {
    fprintf (stderr, "cbGetScale: subRmseAlpha failed\n");
    *okay=0;
    return 0;
  }

  DeInitRmseAlpha (&ra);

//  if (CropIt) DestroyImage (crpd_img);
  DestroyImage (clnA);
  DestroyImage (resB);

  if (ra.score < 0) {
    fprintf (stderr, "cbGetScale: subRmseAlpha %g\n", ra.score);
    *okay = 0;
  }

  if (pGsr->verbose) fprintf (stderr,
    "cbGetScale: scale=%g, score=%g\n", scale, ra.score);

  return ra.score;
}


static MagickBooleanType GoldSectScale (
  WhatScaleT * pws,
  Image * imageA,
  Image * imageB,
  ExceptionInfo *exception)
// Finds scale by golden section search.
{

  if (pws->verbose) fprintf (stderr,
    "GoldSectScale inScale=%g inPorm=%g tol=%g\n",
    pws->scale, pws->inPorm, pws->tolerance);

  goldSectSrchT gss;

  InitGoldSectSrch (&gss);

  gss.xLo = pws->scale / pws->inPorm; // FIXME: get error margin from pws.
  gss.xHi = pws->scale * pws->inPorm;
  gss.epsX = pws->tolerance;
  gss.getYcb = cbGetScale;
  gss.verbose = (pws->verbose <= 1) ? 0 : pws->verbose-1 ;

  GoldSectScaleT gsr;
  gsr.imageA = imageA;
  gsr.imageB = imageB;
  gsr.verbose = gss.verbose;
  gsr.exception = exception;
  gsr.debug = pws->debug;

  if (! GoldSectSrch (&gss, &gsr)) return MagickFalse;

  pws->scale = gss.fndX;
  pws->scaleFact = gss.xPlusOrMinus * 2;
  pws->score = gss.calcY;
  pws->porm = gss.xPlusOrMinus;

  return MagickTrue;
}


#endif
