rem From image %1, wraps it into a cylinder.
rem
rem %2 is tilt angle, degrees. Default=0=no tilt. Positive is with camera above cylinder, looking down.
rem %3 is start angle, degrees. Default = -90.
rem %4 is end angle, degrees. Default = 90.
@rem
@rem Updated:
@rem   5-August-2022 for IM v7.
@rem


@if "%1"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

call %PICTBAT%setInOut %1 wc

set CALC=%IMG7%magick identify -precision 9 -format

set tiltAng=%2
if "%tiltAng%"=="" set tiltAng=0

set startAng=%3
if "%startAng%"=="" set startAng=-90

set endAng=%4
if "%endAng%"=="" set endAng=90

if "%wcBackCol%"=="" set wcBackCol=None

rem for /F "usebackq" %%L ^
rem in (`%CALC% "ANG=%startAng%;startAng=%%[fx:while(ANG>180,ANG-=360)]" xc:`) ^
rem do set %%L

for /F "usebackq" %%L ^
in (`%CALC% "startAng=%%[fx:%startAng%<=180?%startAng%:%startAng%-360]" xc:`) ^
do set %%L

echo %0: startAng=%startAng%


rem transAng = (startAng - endAng) mod 360.

for /F "usebackq" %%L ^
in (`%CALC% "negTiltAng=%%[fx:-%tiltAng%]\ntransAng=%%[fx:ta=(%startAng%-(%endAng%))%% 360;ta<0?ta+360:ta]" xc:`) ^
do set %%L

for /F "usebackq" %%L ^
in (`%CALC% "imgAng=%%[fx:360-(%transAng%)]\nbadAng=%%[fx:%transAng%<360?0:1]" xc:`) ^
do set %%L

if %badAng%==1 (
  echo %0: Bad angle
  exit /B 1
)

echo %0: imgAng=%imgAng% transAng=%transAng%

FOR /F "usebackq" %%L ^
IN (`%CALC% "WW=%%w\nHH=%%h\ntransPix=%%[fx:w*(%transAng%)/(%imgAng%)]" %INFILE%`) ^
DO set %%L

rem So we have pix/degree = (WW + transPix) / 360

FOR /F "usebackq" %%L ^
IN (`%CALC% ^
  "pixPerDeg=%%[fx:(%WW%+w*(%transAng%)/(%imgAng%))/360]\npix360=%%[fx:%WW%+(%transPix%)]\npix180=%%[fx:(%WW%+(%transPix%))/2]" ^
  %INFILE%`) ^
DO set %%L

rem In the appended image, what are the pixel coords of -90 and +90?
rem Image starts (pix=0) at startAng and ends (pix=WW) at endAng.
rem -90 is at (-90-startAng)*pixPerDegree ??
rem +90 is at ( 90-startAng)*pixPerDegree ??

FOR /F "usebackq" %%L ^
IN (`%CALC% "pm90=%%[fx:(-90-(%startAng%))*%pixPerDeg%]" xc:`) ^
DO set %%L

FOR /F "usebackq" %%L ^
IN (`%CALC% "pp90=%%[fx:(90-(%startAng%))*%pixPerDeg%]\npmNeg=%%[fx:%pm90%<0?1:0]" xc:`) ^
DO set %%L

FOR /F "usebackq" %%L ^
IN (`%CALC% "ppNeg=%%[fx:%pp90%<0?1:0]" xc:`) ^
DO set %%L

set pixFL=%pm90%
set pixFR=%pp90%
set pixRR=%pp90%

if %pmNeg%==1 FOR /F "usebackq" %%L ^
IN (`%CALC% "pixFL=%%[fx:%pm90%+(%pix360%)]\npixFR=%%[fx:%pp90%+(%pix360%)]" xc:`) ^
DO set %%L

if %ppNeg%==1 FOR /F "usebackq" %%L ^
IN (`%CALC% "pixRR=%%[fx:%pp90%+(%pix360%)]" xc:`) ^
DO set %%L

FOR /F "usebackq" %%L ^
IN (`%CALC% "pixRL=%%[fx:%pixRR%+(%pix360%)/2]" xc:`) ^
DO set %%L

rem ccalc = %pix360% * 2 = exit

%IMG7%magick ^
  %INFILE% ^
  -size %transPix%x%HH% xc:None ^
  +append +repage ^
  ( +clone ) ^
  +append +repage ^
  -write wc_img.png ^
  ( +clone -crop %pix180%x%HH%+%pixFL%+0 +repage -write wc_front.png +delete ) ^
  ( +clone -crop %pix180%x%HH%+%pixRR%+0 +repage -flop -write wc_rear.png +delete ) ^
  NULL:

if ERRORLEVEL 1 (
  echo %0 failed
  exit /B 1
)


rem Verify:
if "%wcDEBUG%"=="1" %IMG7%magick ^
  wc_front.png ^
  wc_rear.png ^
  -append +repage ^
  -bordercolor black -border 1 ^
  wc_comp.png

set pcStartAng=-90
set pcEndAng=90
rem set pcForceRecalc=1
set pcBackCol=None
call %PICTBAT%plane2cyl wc_rear.png %negTiltAng%
call %PICTBAT%plane2cyl wc_front.png %tiltAng%

%IMG7%magick ^
  wc_rear_p2c.png ^
  %wcRearEffect% ^
  ( wc_front_p2c.png -repage +0+%p2cExtraV% ) ^
  -background %wcBackCol% ^
  -layers merge ^
  +repage ^
  %OUTFILE%


