rem Wraps (distorts, warps) an image onto another image.
rem
rem %1 Image to be wrapped.
rem %2 Image to be overlaid.
rem %3 Absolute distortion map.
rem %4 Mask of image after distortion.
rem %5 Highlight (and shadows) mask.
rem %6 Optional supersampling factor, eg 2. [Default = 1 = no supersampling.]
rem %7 Optional output file.
@rem
@rem This is like wrapImage, but with highlight mask.
@rem This is mid-gray for no-change, up to white.
@rem It is applied with overlay, so darkness in the mask darkens the image.
@rem
@rem
@rem Updated:
@rem   5-August-2022 for IM v7.
@rem


@if "%4"=="" findstr /B "rem @rem" %~f0 & exit /B 1

@setlocal

@call echoOffSave

rem call %PICTBAT%setInOut %1 wi

set IMAGE=%1

set MUG=%2

set MAP=%3

set MASK=%4

set HIGHLIGHT=%5
if "%HIGHLIGHT%"=="." set HIGHLIGHT=
if "%HIGHLIGHT%"=="" set HIGHLIGHT=

set nSUPERSAMP=%6
if "%nSUPERSAMP%"=="." set nSUPERSAMP=
if "%nSUPERSAMP%"=="" set nSUPERSAMP=1

if %nSUPERSAMP% GEQ 10 (
  echo nSUPERSAMP is suspiciously large: %nSUPERSAMP%
  exit /B 1
)

if not "%7"=="" if not "%7"=="." set OUTFILE=%7


for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 16 ^
  -format "SUPERSAMPpc=%%[fx:%nSUPERSAMP%*100]" ^
  xc:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WI=%%w\nHI=%%h" ^
  %IMAGE%`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -format "WM=%%w\nHM=%%h" ^
  %MAP%`) do set %%L

rem We want WI >= WM and HI >= HM.
rem If this isn't true, we will resize the image.
rem Even if it is true, we might resize it for super-sampling.

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 16 ^
  -format "resPc=%%[fx:FW=%WM%/%WI%;FH=%HM%/%HI%;MF=FW>FH?FW:FH;MF>1?MF*100:100]" ^
  xc:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 16 ^
  -format "resPc=%%[fx:%resPc%*%SUPERSAMPpc%/100]" ^
  xc:`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 16 ^
  -format "DX=%%[fx:w*%resPc%/200]\nDY=%%[fx:h*%resPc%/200]" ^
  %IMAGE%`) do set %%L

for /F "usebackq" %%L in (`%IMG7%magick identify ^
  -precision 16 ^
  -format "invSUPERSAMPpc=%%[fx:10000/%SUPERSAMPpc%]" ^
  xc:`) do set %%L

if %SUPERSAMPpc%==100 (
  set sSUP=
  set sINVSUP=
) else (
  set sSUP=-resize %SUPERSAMPpc%%%
  set sINVSUP=-resize %invSUPERSAMPpc%%%
)

if %resPc%==100 (
  set sRES=
) else (
  set sRES=-resize %resPc%%%
)

if "%HIGHLIGHT%"=="" (
  set sHILI=
) else (
  set sHILI=%HIGHLIGHT% +swap -compose Overlay -composite
)

echo DX=%DX%  DY=%DY%

%IM7DEV%magick ^
  ( %IMAGE% %sRES% ) ^
  ( %MAP% %sSUP% ) ^
  -gravity Center ^
  -virtual-pixel Edge ^
  -compose Distort -define compose:args=%DX%x%DY% -composite ^
  %sINVSUP% ^
  -crop %WM%x%HM%+0+0 +repage ^
  -compose Over -extent %WM%x%HM% ^
  %MASK% ^
  -alpha off ^
  -compose CopyOpacity -composite ^
  %MUG% ^
  -compose Multiply -composite ^
  ( +clone -blur 0x1 ) ^
  -compose DstOver -composite ^
  %sHILI% ^
  -depth 32 ^
  -compress Zip ^
  %OUTFILE%

rem Alternatives: Multiply, LinearBurn, ColorBurn

if ERRORLEVEL 1 exit /B 1

call echoRestore

endlocal & set wiOUTFILE=%OUTFILE%
