#! /bin/bash

# Distort the input so that,
# when printed and wrapped around a pipe,
# that would look like the input image.

# $1 is input file
# $2 is output file
# $3 is half the angle of the pipe we can see, in degrees, more than 0, less than or equal to 90.
#   The script does NOT test for valid angles.

# See also wrapPipe.bat.

# Updated:
#   25-October-2021
#   27-October-2021 Fixed (?) asymmetrical problem on certain heights and angles.

if [ $# != 3 ]; then
  echo "Usage: $0 input output angle"
  exit 1
fi

infile=$1
outfile=$2
amaxDeg=$3

MAGICK=magick.exe

ww=%[w]
hh=%[h]
hmax=%[fx:h/2]
halfh="%[fx:int(h/2)]"
amax="%[fx:$amaxDeg*pi/180]"

declare $( ${MAGICK} \
  $infile \
  -precision 15 \
  -format "ww=${ww}\nhh=${hh}\nhmax=${hmax}\nhalfh=${halfh}\namax=${amax}\n" \
  info: )

rr=$(${MAGICK} xc: -precision 15 -format "%[fx:$hmax / sin($amax)]" info: )

ex=$(${MAGICK} xc: -precision 15 -format "%[fx:int(($rr * ($amax) * 2 - $hh) / 2)]" info: )

dx=$(${MAGICK} xc: -precision 15 -format "%[fx:${halfh} + $ex - (${halfh}-${hh}/2)]" info: )

dx2=$(${MAGICK} xc: -precision 15 -format "%[fx:${dx}*2]" info: )

echo ww=$ww hh=$hh hmax=$hmax amax=$amax halfh=$halfh rr=$rr ex=$ex dx=$dx dx2=$dx2

${MAGICK} \
  \( $infile \
     -background "#88f" \
     -gravity Center \
     -extent ${ww}x${dx2} \
  \) \
  \( \
     -size ${dx2}x1 xc: \
     -fx "($rr *sin((i-${dx})/$rr)+${dx})/w" \
+write e.png \
     -rotate 90 \
     \( -size 1x$ww gradient:Black-White -rotate -90 \) \
     +swap \
     \( -size 1x1 xc:"gray(50%)" \) \
     -scale "${ww}x${dx2}!" \
     -combine \
+write mymap.miff \
  \) \
  -compose Distort -composite \
  $outfile

magick e.png -format "%[fx:minima] %[fx:maxima] %[fx:1-minima]\n" info:
